(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     16257,        354]
NotebookOptionsPosition[     15340,        320]
NotebookOutlinePosition[     15854,        339]
CellTagsIndexPosition[     15811,        336]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 "P.A.R.A.DIS.E. and ",
 StyleBox["Mathematica",
  FontSlant->"Italic"]
}], "Title",
 CellChangeTimes->{{3.395591024393394*^9, 3.395591038289611*^9}, 
   3.395591124469911*^9, {3.395593663277266*^9, 3.395593679155692*^9}, {
   3.395593744637978*^9, 3.395593767876589*^9}, {3.395593879103283*^9, 
   3.395593954732566*^9}, {3.395594163127825*^9, 3.395594164450371*^9}, {
   3.3969525462899227`*^9, 3.396952549019878*^9}, {3.396952694643244*^9, 
   3.396952729682481*^9}}],

Cell[CellGroupData[{

Cell["Introduction", "Section",
 CellChangeTimes->{{3.3955912107611094`*^9, 3.3955912166441793`*^9}}],

Cell["\<\
P.A.R.A.DIS.E. is a bioinformatics framework written with the Java language. \
It proposes to improve the study of the biological phenomena implying \
non-coding RNAs molecules. It provides:
- a object-oriented model of the concepts necessary to the study of \
non-coding RNAs,
- a communication layer to transparently run external programs and to query \
data repositories,
- a set of interconnected graphical tools to allow a biologist to manipulate \
and display his RNA data.

This document describes how to link P.A.R.A.DIS.E. to Mathematica to have the \
best of both worlds\
\>", "Text",
 CellChangeTimes->{{3.3969520098909187`*^9, 3.396952181264612*^9}, {
   3.396952219363213*^9, 3.3969524109062033`*^9}, {3.3969536134626226`*^9, 
   3.3969536761900578`*^9}, {3.396953972807056*^9, 3.396954016302574*^9}, {
   3.396954888302104*^9, 3.3969549376265306`*^9}, {3.3969550180026417`*^9, 
   3.3969550193947287`*^9}, {3.396955071177308*^9, 3.3969550716412*^9}, 
   3.396955117069201*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Installation", "Section",
 CellChangeTimes->{{3.395591101630209*^9, 3.3955911098862133`*^9}, {
  3.396951995273859*^9, 3.396951997737361*^9}}],

Cell[CellGroupData[{

Cell["Configuration and launch of the Java Virtual Machine", "Subsubsection",
 CellChangeTimes->{{3.395591275981408*^9, 3.395591294250897*^9}}],

Cell[TextData[{
 "The following instructions define the Java virtual machine to use with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". It will add to its classpath all the jar files of the P.A.R.A.DIS.E. \
distribution installed on your computer:"
}], "Text",
 CellChangeTimes->{{3.39559141976103*^9, 3.395591424414942*^9}, {
   3.3955918365874147`*^9, 3.395591879249797*^9}, {3.39559232621414*^9, 
   3.395592328784998*^9}, {3.39695070346347*^9, 3.396950705251154*^9}, 
   3.396950962570677*^9, {3.396953028548408*^9, 3.396953069373788*^9}, {
   3.3970348518914757`*^9, 3.3970348718779497`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ReinstallJava", "[", "]"}], " ", ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<JLink`\>\"", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
  "s2sDirectory", " ", "=", " ", "\"\</Users/fjossinet/S2S/S2S/\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ReinstallJava", "[", 
   RowBox[{
   "CommandLine", "\[Rule]", 
    "\"\</System/Library/Frameworks/JavaVM.framework/Versions/1.5/Home/bin/\
java\>\""}], "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"AddToClassPath", "[", 
   RowBox[{"s2sDirectory", "<>", "\"\<lib/paradise.jar\>\""}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"AddToClassPath", "/@", 
   RowBox[{"FileNames", "[", 
    RowBox[{"\"\<*.jar\>\"", ",", 
     RowBox[{"s2sDirectory", "<>", "\"\<lib/paradise-lib\>\""}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ParadisePlatform", " ", "=", " ", 
   RowBox[{
   "LoadJavaClass", "[", 
    "\"\<fr.unistra.ibmc.paradise.services.ParadisePlatform\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "LoadJavaClass", "[", 
   "\"\<fr.unistra.ibmc.paradise.tools.ParadiseWorkingSession\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "LoadJavaClass", "[", 
   "\"\<fr.unistra.ibmc.paradise.tools.MathematicaAgent\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ParadisePlatform`launchParadisePlatform", "[", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pws", "=", 
   RowBox[{"JavaNew", "[", 
    RowBox[{
    "\"\<fr.unistra.ibmc.paradise.tools.ParadiseWorkingSession\>\"", ",", 
     " ", "\"\<localhost\>\""}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mathematicaAgent", "=", " ", 
   RowBox[{"JavaNew", "[", 
    RowBox[{
    "\"\<fr.unistra.ibmc.paradise.tools.MathematicaAgent\>\"", ",", " ", 
     "pws"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.390392780342647*^9, 3.390392813431497*^9}, {
   3.390392897478945*^9, 3.39039294893659*^9}, {3.390392992972643*^9, 
   3.3903930122980833`*^9}, {3.390393090376095*^9, 3.3903931338908*^9}, 
   3.39039318646487*^9, {3.3903934232192*^9, 3.3903935229317493`*^9}, {
   3.3903935570137653`*^9, 3.390393561636847*^9}, {3.390393892836779*^9, 
   3.390393926460969*^9}, 3.390394005531403*^9, {3.390402787760158*^9, 
   3.390402803143531*^9}, {3.3904028469772377`*^9, 3.390402882914682*^9}, {
   3.390402937495723*^9, 3.390402947732399*^9}, {3.390405571854735*^9, 
   3.390405605674798*^9}, {3.390405738325964*^9, 3.390405900798671*^9}, {
   3.3904059681284723`*^9, 3.390406069152996*^9}, {3.3904063722942133`*^9, 
   3.3904064418746567`*^9}, {3.390406474495163*^9, 3.390406706707636*^9}, 
   3.3904067379461*^9, {3.390406791688716*^9, 3.3904068179775753`*^9}, {
   3.390406956289114*^9, 3.390407001191247*^9}, {3.3904645593187532`*^9, 
   3.390464578568448*^9}, {3.390464689378985*^9, 3.390464760463496*^9}, {
   3.3904728735941896`*^9, 3.390472965264976*^9}, {3.3904730044940357`*^9, 
   3.390473286189986*^9}, {3.390473328657382*^9, 3.3904734688227797`*^9}, {
   3.390473501963765*^9, 3.390473581669129*^9}, {3.3904736915783463`*^9, 
   3.390473711409389*^9}, {3.3904739042230453`*^9, 3.39047394734674*^9}, {
   3.390473992964467*^9, 3.390473996164096*^9}, {3.3904742784063997`*^9, 
   3.390474478357099*^9}, 3.390474529232542*^9, {3.390474693949253*^9, 
   3.390474774082456*^9}, {3.3904748137120867`*^9, 3.390474819335979*^9}, {
   3.3904748629809513`*^9, 3.3904749703965883`*^9}, {3.390475039995698*^9, 
   3.390475040431635*^9}, {3.3904750845489597`*^9, 3.390475099788306*^9}, {
   3.390475142182496*^9, 3.390475357860444*^9}, 3.390475413550703*^9, {
   3.390475488008699*^9, 3.390475567178685*^9}, 3.390475651962656*^9, {
   3.3904756873368797`*^9, 3.390475744720525*^9}, {3.390475781407276*^9, 
   3.390475791390498*^9}, {3.390475852109951*^9, 3.390475873966261*^9}, {
   3.3904762565504637`*^9, 3.3904762567001534`*^9}, {3.390476340173052*^9, 
   3.390476341187439*^9}, {3.390476391150983*^9, 3.390476438072109*^9}, {
   3.390476477759721*^9, 3.390476556440629*^9}, {3.390489397072687*^9, 
   3.390489407109256*^9}, {3.39048945141809*^9, 3.390489452070928*^9}, {
   3.3904894896229353`*^9, 3.390489494651366*^9}, 3.390489548234709*^9, {
   3.390489677092408*^9, 3.3904897122673473`*^9}, {3.3904922157307262`*^9, 
   3.390492304911463*^9}, {3.390492447405319*^9, 3.390492477031157*^9}, {
   3.390492515766479*^9, 3.390492516356152*^9}, {3.3904925543790817`*^9, 
   3.390492554711194*^9}, {3.3904928307332907`*^9, 3.3904928315937767`*^9}, {
   3.390493223231382*^9, 3.390493227265216*^9}, {3.390493331183381*^9, 
   3.390493334429695*^9}, {3.3904940505249157`*^9, 3.390494070122916*^9}, {
   3.3904944076268787`*^9, 3.39049442305729*^9}, {3.3904946105870123`*^9, 
   3.3904946112517977`*^9}, {3.3904946841441174`*^9, 3.390494684796774*^9}, 
   3.390494727719902*^9, {3.3904954995561523`*^9, 3.390495507961171*^9}, 
   3.3904956434112453`*^9, {3.390495857919757*^9, 3.390495873294312*^9}, {
   3.390495905667151*^9, 3.390496046988243*^9}, {3.390496082864112*^9, 
   3.39049609171784*^9}, {3.3904961440824537`*^9, 3.3904961460131807`*^9}, {
   3.390496247830594*^9, 3.390496266974688*^9}, {3.390496325793638*^9, 
   3.390496438441986*^9}, {3.390496726375505*^9, 3.3904967941197567`*^9}, {
   3.390496869533907*^9, 3.390496899483861*^9}, {3.3904970043152647`*^9, 
   3.390497066022934*^9}, {3.390497225266802*^9, 3.390497231837556*^9}, {
   3.390497294782628*^9, 3.3904972979182367`*^9}, {3.390497330231413*^9, 
   3.39049733165622*^9}, {3.390497416681535*^9, 3.390497439598092*^9}, {
   3.390497504612729*^9, 3.390497507096305*^9}, {3.390497791487185*^9, 
   3.390497855850565*^9}, {3.3905302137614813`*^9, 3.390530220378109*^9}, {
   3.3953140986754*^9, 3.39531409912497*^9}, {3.3953141325490513`*^9, 
   3.395314135974288*^9}, {3.395314216302217*^9, 3.395314269905195*^9}, {
   3.395314304992325*^9, 3.3953143967754307`*^9}, {3.395314442055846*^9, 
   3.395314452700735*^9}, {3.395314726067726*^9, 3.395314735187006*^9}, {
   3.395314801594293*^9, 3.395314802848298*^9}, 3.395314845452126*^9, {
   3.395315036956525*^9, 3.3953150485058537`*^9}, {3.395315082105008*^9, 
   3.39531511790853*^9}, {3.395315165562045*^9, 3.395315187199586*^9}, {
   3.395315217631549*^9, 3.3953153489766083`*^9}, {3.395315393458647*^9, 
   3.395315562920554*^9}, {3.395328305068646*^9, 3.395328305403265*^9}, 
   3.395328605402931*^9, {3.3953321585262337`*^9, 3.3953322052623262`*^9}, {
   3.3954277510027018`*^9, 3.395427751382855*^9}, 3.395427823896378*^9, 
   3.395429094787125*^9, {3.3955910173645983`*^9, 3.39559102099859*^9}, {
   3.395591319302924*^9, 3.395591337451642*^9}, {3.395591628280486*^9, 
   3.395591632896351*^9}, {3.395591741363634*^9, 3.395591746089611*^9}, {
   3.3955919344256897`*^9, 3.3955919832104816`*^9}, {3.3955920354079323`*^9, 
   3.395592141135178*^9}, {3.395592188531623*^9, 3.3955921962650537`*^9}, {
   3.39559234656876*^9, 3.395592396095365*^9}, {3.410774247625731*^9, 
   3.4107742488295937`*^9}, {3.4510212557614193`*^9, 3.451021316661407*^9}, 
   3.451021349314722*^9, {3.451021385696423*^9, 3.4510214019423532`*^9}, {
   3.451021456993162*^9, 3.451021457606761*^9}, 3.451021545835376*^9, 
   3.45102165410947*^9, {3.451021735304511*^9, 3.45102174788666*^9}, {
   3.4510319350498457`*^9, 3.4510319351357527`*^9}, {3.451034912054449*^9, 
   3.451034916643643*^9}, {3.451034987341817*^9, 3.451034987438486*^9}, {
   3.4510350376475897`*^9, 3.4510350509573727`*^9}, {3.45104327630295*^9, 
   3.451043291586681*^9}, {3.451047809617515*^9, 3.4510478097144833`*^9}, {
   3.451047929299542*^9, 3.45104792966576*^9}, 3.451106242825547*^9, {
   3.451106757249558*^9, 3.451106783520515*^9}, {3.451106925523695*^9, 
   3.451106925785796*^9}, {3.4511070973399076`*^9, 3.45110710550712*^9}, {
   3.451107362269484*^9, 3.451107362423962*^9}, {3.451107558644106*^9, 
   3.4511075596892138`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
  "LoadJavaClass", "[", "\"\<fr.unistra.ibmc.paradise.core.io.PDBFileIO\>\"", 
   "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"LoadJavaClass", "[", "\"\<java.io.File\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pdbParser", " ", "=", " ", 
   RowBox[{"JavaNew", "[", 
    RowBox[{
    "\"\<fr.unistra.ibmc.paradise.core.io.PDBFileIO\>\"", ",", "pws"}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pdbParser", "@", 
   RowBox[{"parseFile", "[", 
    RowBox[{
     RowBox[{"JavaNew", "[", 
      RowBox[{
      "\"\<java.io.File\>\"", ",", 
       "\"\</Users/fjossinet/samples/ur0012.pdb\>\""}], "]"}], ",", 
     RowBox[{"mathematicaAgent", "@", 
      RowBox[{
       RowBox[{"getParadiseTool", "[", "]"}], "@", 
       RowBox[{"getProgressMonitor", "[", "]"}]}]}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.4510227071231213`*^9, 3.451022717189893*^9}, {
   3.4510338719293833`*^9, 3.4510338750185547`*^9}, {3.451035087229334*^9, 
   3.451035132144515*^9}, {3.451035182443329*^9, 3.4510351835144663`*^9}, {
   3.451035291804203*^9, 3.451035326442491*^9}, {3.451035360051917*^9, 
   3.451035366311758*^9}, {3.451035477870118*^9, 3.4510354827108593`*^9}, {
   3.4510355305891743`*^9, 3.451035593664227*^9}, {3.451041161986329*^9, 
   3.451041171340375*^9}, {3.4510434100105352`*^9, 3.451043443360078*^9}, 
   3.451043588587647*^9, {3.4510472817136374`*^9, 3.451047367256344*^9}, {
   3.4510476838399143`*^9, 3.451047724352572*^9}, {3.451106169622491*^9, 
   3.451106173033017*^9}, {3.45110626570751*^9, 3.451106311307238*^9}, {
   3.451107099865308*^9, 3.4511071080344467`*^9}, {3.451108600065914*^9, 
   3.451108600840084*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ts", " ", "=", " ", 
   RowBox[{"pdbParser", "@", 
    RowBox[{
     RowBox[{"getTertiaryStructures", "[", "]"}], "@",
     RowBox[{"get", "[", "0", "]"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mathematicaAgent", "@", 
   RowBox[{"annotateTertiaryStructure", "[", "ts", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.451108603184766*^9, 3.451108607657914*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SecondaryStructure", " ", "=", " ", 
   RowBox[{
   "LoadJavaClass", "[", 
    "\"\<fr.unistra.ibmc.paradise.core.features.SecondaryStructure\>\"", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"molecule", " ", "=", " ", 
   RowBox[{"pws", "@", 
    RowBox[{
     RowBox[{"getMolecules", "[", "]"}], "@", 
     RowBox[{"get", "[", "0", "]"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mathematicaAgent", "@", 
   RowBox[{"predictSecondaryStructure", "[", "molecule", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Instance", "/@", 
  RowBox[{"molecule", "@", 
   RowBox[{
    RowBox[{"getAnnotations", "[", "]"}], "@", 
    RowBox[{"toArray", "[", "]"}]}]}]}]}], "Input",
 CellChangeTimes->{{3.45110652537156*^9, 3.451106612772602*^9}, {
   3.45110670633453*^9, 3.451106750028852*^9}, {3.4511068074944487`*^9, 
   3.451106815007509*^9}, {3.451106895990225*^9, 3.451106965281402*^9}, {
   3.451107049609392*^9, 3.451107087398368*^9}, {3.451107396113496*^9, 
   3.451107536205676*^9}, {3.4511075720535183`*^9, 3.4511077486732597`*^9}, {
   3.451107792396441*^9, 3.451107843848366*^9}, {3.451108208347875*^9, 
   3.451108234826044*^9}, {3.45110837121319*^9, 3.4511084148330107`*^9}, 
   3.451108629220186*^9, {3.451108716030902*^9, 3.451108795644393*^9}, {
   3.451108841044516*^9, 3.451108897946836*^9}, 3.451108928441348*^9, 
   3.451108963928906*^9, {3.451109011077488*^9, 3.451109028735819*^9}, {
   3.451109085958034*^9, 3.451109158414909*^9}, {3.451109201141686*^9, 
   3.451109206094719*^9}, {3.451109252032298*^9, 3.451109253463224*^9}, {
   3.451109285814995*^9, 3.451109357037016*^9}, {3.4511093871775703`*^9, 
   3.451109390140581*^9}, {3.451109477259871*^9, 3.4511094796212063`*^9}, {
   3.451109589786866*^9, 3.451109652918969*^9}, {3.451109709777775*^9, 
   3.451109745588334*^9}, {3.451109788496168*^9, 3.451109827398795*^9}, {
   3.4511098856861153`*^9, 3.451109929519359*^9}, {3.451109983681011*^9, 
   3.451109999493273*^9}, {3.451110058344494*^9, 3.451110079031742*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1342, 802},
WindowMargins->{{-1474, Automatic}, {-43, Automatic}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
PrintingOptions->{"FirstPageHeader"->True},
FrontEndVersion->"7.0 for Mac OS X x86 (32-bit) (February 18, 2009)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "UTF-8"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 486, 10, 66, "Title"],
Cell[CellGroupData[{
Cell[1078, 36, 101, 1, 72, "Section"],
Cell[1182, 39, 1000, 19, 101, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2219, 63, 148, 2, 72, "Section"],
Cell[CellGroupData[{
Cell[2392, 69, 143, 1, 24, "Subsubsection"],
Cell[2538, 72, 606, 11, 26, "Text"],
Cell[3147, 85, 7926, 137, 203, "Input"],
Cell[11076, 224, 1733, 38, 83, "Input"],
Cell[12812, 264, 422, 10, 53, "Input"],
Cell[13237, 276, 2063, 39, 83, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
