package fr.unistra.ibmc.paradise;

import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.ParadiseFeature;


/**
 * Warn for an architectural problem when constructing someting related to the Molecule, ParadiseFeature and Location concepts
 */
public class ArchitectureException extends RuntimeException {
    private Location location;
    private Molecule molecule;
    private ParadiseFeature feature, subfeature;

    public ArchitectureException(String message) {
        this(message, null, null, null);
    }

    public ArchitectureException(String message, ParadiseFeature feature, ParadiseFeature subfeature) {
        this(message, null, null, feature, subfeature);
    }

    public ArchitectureException(String message, Location l, ParadiseFeature feature) {
        this(message, null, l, feature);
    }

    public ArchitectureException(String message, Molecule m, Location l) {
        this(message, m, l, null);
    }

    public ArchitectureException(String message, Molecule m, Location l, ParadiseFeature feature) {
        this(message, m, l, feature, null);
    }

    public ArchitectureException(String message, Molecule m, Location l, ParadiseFeature feature, ParadiseFeature subfeature) {
        super(message);
        this.location = l;
        this.molecule = m;
        this.feature = feature;
        this.subfeature = subfeature;
    }

    /**
     * If the problem is related to a Location, return it
     *
     * @return null if a Location is not related to the problem
     */
    public Location getLocation() {
        return this.location;
    }

    /**
     * Return the Molecule related to the problem
     *
     * @return null if no precise Molecule has been associated with the problem
     */
    public Molecule getMolecule() {
        return this.molecule;
    }

    /**
     * If the problem is related to a ParadiseFeature, return it
     *
     * @return null if a ParadiseFeature is not related to the problem
     */
    public ParadiseFeature getParadiseFeature() {
        return this.feature;
    }

    /**
     * If the problem is related to a ParadiseFeature which wants to be, which is or which wants to be removed as a subfeature, return it
     *
     * @return null if a ParadiseFeature, as a subfeature, is not related to the problem
     */
    public ParadiseFeature getParadiseSubFeature() {
        return this.subfeature;
    }
}
