package fr.unistra.ibmc.paradise;

import fr.unistra.ibmc.paradise.services.ParadisePlatform;
import fr.unistra.ibmc.paradise.tools.AbstractParadiseTool;
import fr.unistra.ibmc.paradise.tools.ParadiseWorkingSession;
import fr.unistra.ibmc.paradise.tools.component.*;
import fr.unistra.ibmc.paradise.tools.component.ProgressMonitor;
import fr.unistra.ibmc.paradise.utils.ParadiseConfig;
import fr.unistra.ibmc.paradise.utils.IOUtils;
import groovy.lang.Binding;
import groovy.ui.Console;
import jade.wrapper.ControllerException;
import org.jdesktop.swingworker.SwingWorker;
import org.jdom.JDOMException;

import javax.swing.*;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.prefs.Preferences;
import java.util.prefs.BackingStoreException;
import java.net.URL;

public class Paradise extends AbstractParadiseTool {

    public static final String releaseName = "PARADISE engine 2.0";
    private static String currentPlatformAddress;
    public static int mainFeaturesCreatedCount=0;
    private ProgressMonitor monitor;
    public static final int STRUCTURAL_ALIGNMENT = 0, SECONDARY_STRUCTURE_PREDICTION = 1, TERTIARY_STRUCTURE_ANNOTATION = 2,SECONDARY_STRUCTURE_DRAWING = 3, HELICES_REORGANIZATION = 4, TERTIARY_STRUCTURE_PREDICTION = 5, TERTIARY_STRUCTURE_REFINEMENT = 6, SEND_RESIDUES_BY_CHAT =7, NC_RNAS_CANDIDATES_SEARCHING = 8, SEND_MESSAGES_BY_CHAT = 9, SEND_POSITION_BY_CHAT=10, SEND_COLLABORATE_MESSAGE=12;

    // MODIFICATION of the COMMUNITY FUNCTIONNALITIES
    // services that can do the community agent (used on the Assemble part)
    public static final int LOGIN = 20, GET_CONNECTED_USERS = 22, ADD_USER = 23;

    public Paradise(ParadiseWorkingSession workingSession) {
        super(Paradise.releaseName,workingSession);
        this.monitor = new DefaultProgressMonitor();
    }

    public ProgressMonitor getProgressMonitor() {
        return this.monitor;
    }

    public static String getCurrentPlatformAddress() {
        return currentPlatformAddress;
    }

    public static void setCurrentPlatformAddress(String currentPlatformAddress) {
        Paradise.currentPlatformAddress = currentPlatformAddress;
    }

    public static String getInstallPath() {
        return Paradise.class.getProtectionDomain().getCodeSource().getLocation().getPath().split("paradise.jar")[0];
    }

    public static File getTmpDirectory() {
        File tmpDir = new File(new StringBuffer(Paradise.getInstallPath()).append("tmp").toString());
        if (!tmpDir.exists())
            tmpDir.mkdir();
        return tmpDir;
    }


    public static File getTertiaryDataDirectory() {

        File tertiaryDataDir = new File(Paradise.getParadiseDirectory(),"tertiary_data");
        if (!tertiaryDataDir.exists()) {
            tertiaryDataDir.mkdir();
            try {
                File dataZipped = new File(Paradise.getInstallPath(),"tertiary_data.zip");
                if (dataZipped.exists()) //this test is necessary from the RC1 to RC2 transition since the tertiary data are provided as a zip file (since at this moment the update system didn't support the extraction of zip files) 
                    try {
                        IOUtils.extractArchivedFile(Paradise.getParadiseDirectory().getAbsolutePath(),dataZipped);
                        dataZipped.delete();
                    } catch (IOException e) {
                        e.printStackTrace();
                        return null;
                    }
                else
                    IOUtils.copyDirectory(new File(new File(Paradise.getInstallPath()), "tertiary_data"),tertiaryDataDir);
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            }
        }
        return tertiaryDataDir;
    }

    public static File getDatabasesDirectory() {
        File sequencesDir = new File(Paradise.getParadiseDirectory(),"databases");
        if (!sequencesDir.exists())
            sequencesDir.mkdir();
        return sequencesDir;
    }

    public static File getParadiseDirectory() {
        File userDir = new File(new StringBuffer(System.getProperty("user.home")).append(System.getProperty("file.separator")).append(".paradise").append(System.getProperty("file.separator")).toString());
        if (!userDir.exists())
            userDir.mkdir();
        return userDir;
    }

    public static String getScriptsPath() {
        return new StringBuffer(Paradise.getInstallPath()).append("scripts").append(System.getProperty("file.separator")).toString();
    }

    public static String getWorkingDirectory() {
        Preferences preferences = Preferences.userNodeForPackage(Paradise.class);
        return preferences.get("Working Directory", getInstallPath());
    }

    public static void setWorkingDirectory(String workingDirectory) {
        Preferences preferences = Preferences.userNodeForPackage(Paradise.class);
        preferences.put("Working Directory", workingDirectory);
    }

    public static void main(final String[] args) throws IOException, ControllerException, JDOMException, BackingStoreException {
        //ParadiseConfig.parseConfigFile();
        if (args.length == 0) {
            ParadisePlatform.launchParadisePlatform();
            new Connect("localhost").execute();
        }
        else if (args.length == 1) {
            if ("algorithms_path".equals(args[0])) {
                System.out.println(ParadiseConfig.getAlgorithmsPath());
            }
            else //connect to a remote platform
                new Connect(args[0]).execute();
        }
    }

    public void launch() {
        Binding b = new Binding();
        b.setProperty("session",Paradise.this.getWorkingSession());
        b.setProperty("agent",agent);
        Console console = new groovy.ui.Console(b);
        console.setCurrentFileChooserDir(new File(Paradise.getInstallPath()));
        console.setStatusLabel(new JLabel("Connected to "+((ParadiseWorkingSession)this.moleculeFactory).getPlatformAddress()+", ready for work..."));
        console.run();
    }

    static class Connect extends SwingWorker {

        private String platformAddress;

        Connect(String platformAddress) {
            this.platformAddress = platformAddress;
        }

        protected Object doInBackground() throws Exception {
            System.out.println("try to connect to "+platformAddress);
            ParadiseWorkingSession p = new ParadiseWorkingSession(platformAddress);
            new ParadiseAgent(p);
            return null;
        }

    };
}
