package fr.unistra.ibmc.paradise;

import fr.unistra.ibmc.paradise.tools.ParadiseWorkingSession;
import fr.unistra.ibmc.paradise.tools.AbstractParadiseToolAgent;
import fr.unistra.ibmc.paradise.tools.AnswerBehaviour;
import fr.unistra.ibmc.paradise.tools.ParadiseToolAgent;
import fr.unistra.ibmc.paradise.utils.GeneralUtils;
import fr.unistra.ibmc.paradise.core.analysis.Analysis;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.gui.GuiEvent;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.wrapper.StaleProxyException;
import org.jdesktop.swingx.JXErrorPane;
import org.jdesktop.swingx.error.ErrorInfo;

import javax.swing.*;
import java.util.logging.Level;
import java.util.Iterator;


public class ParadiseAgent extends AbstractParadiseToolAgent {

    public ParadiseAgent(ParadiseWorkingSession workingSession) throws StaleProxyException {
        super(new Paradise(workingSession));
        //this.addBehaviour(new InformBehaviour());
        //this.addBehaviour(new FailureBehaviour());
        //this.addBehaviour(new CheckCompatibilityBehaviour());
    }

    protected void onGuiEvent(GuiEvent event) {
        AnswerBehaviour answerBehaviour = null;
        Iterator iterator = event.getAllParameter();
        if (iterator.hasNext())
            answerBehaviour = (AnswerBehaviour)iterator.next();
        ListAgentsBehaviour behaviour = new ListAgentsBehaviour(Analysis.TERTIARY_STRUCTURE_ANNOTATION, answerBehaviour, paradiseTool.getProgressMonitor());
        if (behaviour != null)
            this.addBehaviour(behaviour);
    }

    protected class CheckCompatibilityBehaviour extends OneShotBehaviour {
        public void action() {
            DFAgentDescription description = ParadiseAgent.this.findAgent("system", "CheckCompatibility");
            if (description != null) {
                ACLMessage request = new ACLMessage(ACLMessage.REQUEST);
                request.addReceiver(description.getName());
                //request.setContent(fr.unistra.ibmc.paradise.Paradise.version);
                ParadiseAgent.this.send(request);
            } else {
                ErrorInfo errorInfo = new ErrorInfo("Error",
                        "Not able to check the compatibility for the PARADISE platform", null,
                        null, null, Level.SEVERE, null);
                JXErrorPane.showDialog(null, errorInfo);
            }
        }
    }

    /**
     * A behaviour to display INFORM messages
     */
    protected class InformBehaviour extends CyclicBehaviour {
        public void action() {
            ACLMessage message = myAgent.receive(MessageTemplate.MatchPerformative(ACLMessage.INFORM));
            if (message != null)
                ((ParadiseToolAgent)myAgent).getParadiseTool().getProgressMonitor().printMessage(message.getContent());
            else
                block();
        }
    }

    protected class FailureBehaviour extends CyclicBehaviour {
        public void action() {
            ACLMessage message = myAgent.receive(MessageTemplate.MatchPerformative(ACLMessage.FAILURE));
            if (message != null)
                if (JOptionPane.showConfirmDialog(
                        null,
                        "You need PARADISE "+message.getContent()+" to use the platform hosted at "+ Paradise.getCurrentPlatformAddress()+".\n"
                                +"Do you want to download it?",
                        "Incompatibility with the PARADISE platform version!!",
                        JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
                    GeneralUtils.openBrowser("https://simtk.org/project/xml/downloads.xml?group_id=233");

                else
                    block();
        }
    }
}
