package fr.unistra.ibmc.paradise.core;

import fr.unistra.ibmc.paradise.BiologicalSymbolException;
import fr.unistra.ibmc.paradise.core.features.SequenceFeatureFactory;

/**
 * Created by: fjossinet
 * Date: 9 mars 2007
 */
public abstract class AbstractMoleculeFactory implements MoleculeFactory {

    protected SequenceFeatureFactory sequenceFactory;
    protected String description;

    protected AbstractMoleculeFactory(String description) {
        this.description = description;
        this.sequenceFactory = new SequenceFeatureFactory(this,description);
    }

    public SequenceFeatureFactory getSequenceFeatureFactory() {
        return this.sequenceFactory;
    }

    public RNA createRNA(ParadiseID paradiseID, String name, Source source, String sequence) throws BiologicalSymbolException {
        return new RNA(paradiseID, this, name, source, sequence);
    }

    public Protein createProtein(ParadiseID paradiseID, String name, Source source, String sequence) throws fr.unistra.ibmc.paradise.BiologicalSymbolException {
        return new Protein(paradiseID, this, name, source, sequence);
    }

    public LinearGenomicDNA createLinearGenomicDNA(ParadiseID paradiseID, String name, Source source, String sequence) throws BiologicalSymbolException {
        return new LinearGenomicDNA(paradiseID, this, name, source, sequence);
    }

    public CircularGenomicDNA createCircularGenomicDNA(ParadiseID paradiseID, String name, Source source, String sequence) throws fr.unistra.ibmc.paradise.BiologicalSymbolException {
        return new CircularGenomicDNA(paradiseID, this, name, source, sequence);
    }

    public String getDescription() {
        return this.description;
    }

}
