package fr.unistra.ibmc.paradise.core;

/**
 * Utility class allowing to filter collectionsabd to collect all instances from one class inside a new Collection object
 * Adapted from http://www.javaspecialists.co.za/archive/newsletter.do?issue=127&locale=en_US
 * @author fjossinet
 *
 */

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class CollectionFilter {
    /**
     * Filters the src collection and puts the objects matching the
     * clazz into the dest collection.
     */
    public static <T> void filter(Class<T> clazz,
                                  Collection<?> src,
                                  Collection<T> dest) {
        for (Object o : src) {
            if (clazz.isInstance(o)) {
                dest.add(clazz.cast(o));
            }
        }
    }

    /**
     * Filters the src collection and puts all matching objects into
     * an ArrayList, which is then returned.
     */
    public static <T> List<T> filter(Class<T> clazz,
                                     Collection<?> src) {
        List<T> result = new ArrayList<T>();
        filter(clazz, src, result);
        return result;
    }

    /**
     * Filters the src collection and puts the objects matching the
     * clazz, BUT NOT the not_clazz, into the dest collection.
     */
    public static <T> void filter(Class<T> clazz,
                                  Class<?> not_clazz,
                                  Collection<?> src,
                                  Collection<T> dest) {
        for (Object o : src) {
            if (clazz.isInstance(o) && !not_clazz.isInstance(o)) {
                dest.add(clazz.cast(o));
            }
        }
    }


    /**
     * Filters the src collection and puts the objects matching the
     * clazz, BUT NOT the not_clazz, into an ArrayList which is then returned.
     */
    public static <T> Collection<T> filter(Class<T> clazz,
                                           Class<?> not_clazz,
                                           Collection<?> src) {
        Collection<T> result = new ArrayList<T>();
        filter(clazz, not_clazz, src, result);
        return result;
    }
}
