package fr.unistra.ibmc.paradise.core;

import fr.unistra.ibmc.paradise.BiologicalSymbolException;

import java.util.HashMap;
import java.util.Map;

public abstract class NucleicAcid extends AbstractMolecule {
    protected static Map<String,String> modifiedNucleotides;
    static {
        modifiedNucleotides = new HashMap<String,String>();
        //this map is filled by the modifiedResidues.groovy script provided with PARADISE (in the scripts subdirectory)
        modifiedNucleotides.put("5CM","C");
        modifiedNucleotides.put("5NC","C");
        modifiedNucleotides.put("G36","G");
        modifiedNucleotides.put("C34","C");
        modifiedNucleotides.put("EDA","A");
        modifiedNucleotides.put("6OG","G");
        modifiedNucleotides.put("C38","C");
        modifiedNucleotides.put("MGT","G");
        modifiedNucleotides.put("MIA","A");
        modifiedNucleotides.put("U34","U");
        modifiedNucleotides.put("QUO","G");
        modifiedNucleotides.put("G7M","G");
        modifiedNucleotides.put("G47","G");
        modifiedNucleotides.put("AS","A");
        modifiedNucleotides.put("G31","G");
        modifiedNucleotides.put("A40","G");
        modifiedNucleotides.put("GDP","G");
        modifiedNucleotides.put("A23","A");
        modifiedNucleotides.put("A38","A");
        modifiedNucleotides.put("6HG","G");
        modifiedNucleotides.put("6HT","U");
        modifiedNucleotides.put("6HA","A");
        modifiedNucleotides.put("6HC","C");
        modifiedNucleotides.put("SC","C");
        modifiedNucleotides.put("U31","U");
        modifiedNucleotides.put("C45","C");
        modifiedNucleotides.put("6MA","A");
        modifiedNucleotides.put("C49","C");
        modifiedNucleotides.put("T23","U");
        modifiedNucleotides.put("TAF","U");
        modifiedNucleotides.put("PGP","G");
        modifiedNucleotides.put("CCC","C");
        modifiedNucleotides.put("A47","A");
        modifiedNucleotides.put("5HT","U");
        modifiedNucleotides.put("2MG","G");
        modifiedNucleotides.put("M2G","G");
        modifiedNucleotides.put("OMC","C");
        modifiedNucleotides.put("YYG","G");
        modifiedNucleotides.put("7MG","G");
        modifiedNucleotides.put("1MA","A");
        modifiedNucleotides.put("TLC","U");
        modifiedNucleotides.put("P2U","U");
        modifiedNucleotides.put("TSP","U");
        modifiedNucleotides.put("YG","G");
        modifiedNucleotides.put("5BU","U");
        modifiedNucleotides.put("CBR","C");
        modifiedNucleotides.put("70U","U");
        modifiedNucleotides.put("12A","A");
        modifiedNucleotides.put("2MU","U");
        modifiedNucleotides.put("UFR","U");
        modifiedNucleotides.put("MAD","A");
        modifiedNucleotides.put("TTE","U");
        modifiedNucleotides.put("TLN","U");
        modifiedNucleotides.put("OMG","G");
        modifiedNucleotides.put("UCL","U");
        modifiedNucleotides.put("OMU","U");
        modifiedNucleotides.put("H2U","U");
        modifiedNucleotides.put("PSU","U");
        modifiedNucleotides.put("5MC","C");
        modifiedNucleotides.put("5MU","U");
        modifiedNucleotides.put("BRU","U");
        modifiedNucleotides.put("5IC","C");
        modifiedNucleotides.put("SPT","U");
        modifiedNucleotides.put("TGP","G");
        modifiedNucleotides.put("FHU","U");
        modifiedNucleotides.put("64T","U");
        modifiedNucleotides.put("GCK","C");
        modifiedNucleotides.put("AD2","A");
        modifiedNucleotides.put("CB2","C");
        modifiedNucleotides.put("IU","U");
        modifiedNucleotides.put("AVC","A");
        modifiedNucleotides.put("5CG","G");
        modifiedNucleotides.put("UMS","U");
        modifiedNucleotides.put("BGM","G");
        modifiedNucleotides.put("C36","C");
        modifiedNucleotides.put("SMT","U");
        modifiedNucleotides.put("2DA","A");
        modifiedNucleotides.put("EDC","C");
        modifiedNucleotides.put("MRG","G");
        modifiedNucleotides.put("ATM","U");
        modifiedNucleotides.put("UBB","U");
        modifiedNucleotides.put("GNG","G");
        modifiedNucleotides.put("CAR","C");
        modifiedNucleotides.put("UMP","U");
        modifiedNucleotides.put("3ME","U");
        modifiedNucleotides.put("A3P","A");
        modifiedNucleotides.put("FA2","A");
        modifiedNucleotides.put("PQ1","G");
        modifiedNucleotides.put("DGP","G");
        modifiedNucleotides.put("CBV","C");
        modifiedNucleotides.put("AET","A");
        modifiedNucleotides.put("GMU","U");
        modifiedNucleotides.put("LC","C");
        modifiedNucleotides.put("LHU","U");
        modifiedNucleotides.put("LG","G");
        modifiedNucleotides.put("3DA","A");
        modifiedNucleotides.put("TTD","U");
        modifiedNucleotides.put("MTU","G");
        modifiedNucleotides.put("E1X","G");
        modifiedNucleotides.put("TCP","U");
        modifiedNucleotides.put("TPC","C");
        modifiedNucleotides.put("GNP","G");
        modifiedNucleotides.put("7DA","A");
        modifiedNucleotides.put("7GU","G");
        modifiedNucleotides.put("2PR","G");
        modifiedNucleotides.put("I5C","C");
        modifiedNucleotides.put("5FC","C");
        modifiedNucleotides.put("GMS","G");
        modifiedNucleotides.put("2BT","U");
        modifiedNucleotides.put("2DT","U");
        modifiedNucleotides.put("8FG","G");
        modifiedNucleotides.put("NMT","U");
        modifiedNucleotides.put("NMS","U");
        modifiedNucleotides.put("2NT","U");
        modifiedNucleotides.put("EIT","U");
        modifiedNucleotides.put("TFE","U");
        modifiedNucleotides.put("P2T","U");
        modifiedNucleotides.put("2AT","U");
        modifiedNucleotides.put("2GT","U");
        modifiedNucleotides.put("2OT","U");
        modifiedNucleotides.put("BOE","U");
        modifiedNucleotides.put("1MG","G");
        modifiedNucleotides.put("T6A","A");
        modifiedNucleotides.put("RIA","A");
        modifiedNucleotides.put("XCY","C");
        modifiedNucleotides.put("CSL","C");
        modifiedNucleotides.put("PPW","G");
        modifiedNucleotides.put("A5M","C");
        modifiedNucleotides.put("ZDU","U");
        modifiedNucleotides.put("UD5","U");
        modifiedNucleotides.put("S2M","U");
        modifiedNucleotides.put("O2C","C");
        modifiedNucleotides.put("GNE","G");
        modifiedNucleotides.put("8OG","G");
        modifiedNucleotides.put("DOC","C");
        modifiedNucleotides.put("SUR","U");
        modifiedNucleotides.put("A1P","G");
        modifiedNucleotides.put("CMP","A");
        modifiedNucleotides.put("AMP","A");
        modifiedNucleotides.put("6IA","A");
        modifiedNucleotides.put("CTG","U");
        modifiedNucleotides.put("5FU","U");
        modifiedNucleotides.put("MA7","A");
        modifiedNucleotides.put("UAR","U");
        modifiedNucleotides.put("2AD","A");
        modifiedNucleotides.put("U33","U");
        modifiedNucleotides.put("T2T","U");
        modifiedNucleotides.put("XUG","G");
        modifiedNucleotides.put("2ST","U");
        modifiedNucleotides.put("2EG","G");
        modifiedNucleotides.put("ONE","U");
        modifiedNucleotides.put("5IU","U");
        modifiedNucleotides.put("NEA","A");
        modifiedNucleotides.put("U2N","U");
        modifiedNucleotides.put("DU","U");
        modifiedNucleotides.put("DDG","G");
        modifiedNucleotides.put("GH3","G");
        modifiedNucleotides.put("GFH","G");
        modifiedNucleotides.put("ADP","A");
        modifiedNucleotides.put("SDG","G");
        modifiedNucleotides.put("QSI","A");
        modifiedNucleotides.put("TMP","U");
        modifiedNucleotides.put("GSU","A");
        modifiedNucleotides.put("TCY","A");
        modifiedNucleotides.put("DCZ","C");
        modifiedNucleotides.put("CTP","C");
        modifiedNucleotides.put("SSU","U");
        modifiedNucleotides.put("C42","C");
        modifiedNucleotides.put("G38","G");
        modifiedNucleotides.put("A43","A");
        modifiedNucleotides.put("NYM","U");
        modifiedNucleotides.put("1SC","C");
        modifiedNucleotides.put("3AD","A");
        modifiedNucleotides.put("A2M","A");
        modifiedNucleotides.put("T5S","U");
        modifiedNucleotides.put("GTP","G");
        modifiedNucleotides.put("DG3","G");
        modifiedNucleotides.put("DUP","U");
        modifiedNucleotides.put("F2A","A");
        modifiedNucleotides.put("D3T","U");
        modifiedNucleotides.put("2FI","G");
        modifiedNucleotides.put("2FE","A");
        modifiedNucleotides.put("FMG","G");
        modifiedNucleotides.put("IGU","A");
        modifiedNucleotides.put("T38","U");
        modifiedNucleotides.put("T39","U");
        modifiedNucleotides.put("126","U");
        modifiedNucleotides.put("G49","G");
        modifiedNucleotides.put("C43","C");
        modifiedNucleotides.put("G48","G");
        modifiedNucleotides.put("A44","A");
        modifiedNucleotides.put("U36","U");
        modifiedNucleotides.put("AZT","U");
        modifiedNucleotides.put("DCT","C");
        modifiedNucleotides.put("DC","C");
        modifiedNucleotides.put("DA","A");
        modifiedNucleotides.put("DG","G");
        modifiedNucleotides.put("DT","U");
        modifiedNucleotides.put("T","U");
    }
    
    protected NucleicAcid(ParadiseID paradiseID, MoleculeFactory moleculeFactory, String name, Source source) throws BiologicalSymbolException {
         super(paradiseID, moleculeFactory, name, source);
    }

    public void addModifiedResidue(String modifiedResidue, String unmodifiedResidue) {
        modifiedNucleotides.put(modifiedResidue,unmodifiedResidue);       
    }
}
