package fr.unistra.ibmc.paradise.core;

import fr.unistra.ibmc.paradise.core.features.ParadiseFeatureFactory;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.paradise.ArchitectureException;

import java.util.Collection;
import java.util.List;

public interface ParadiseFeature extends Documentable, ParadiseProjectItem {

    /**
     * Return the Factory which has created this ParadiseFeature
     *
     * @return
     */
    public ParadiseFeatureFactory getFactory();

    public List<ParadiseFeature> getSubFeatures();

    public <T extends ParadiseFeature> List<T> getSubFeatures(Class<T> c);

    public <T extends ParadiseFeature> List<T> getSubFeatures(Class<T> c, Molecule molecule);

    public <T extends ParadiseFeature> List<T> getSubFeatures(Class<T> c, Molecule molecule, Location location);

    public <T extends ParadiseFeature> List<T> getSubFeaturesRecursively(Class<T> c);

    public List<ParadiseFeature> getSubFeaturesRecursively();

    public List<ParadiseFeature> getParentFeatures();

    public <T extends ParadiseFeature> List<T> getParentFeatures(Class<T> c);

    /**
     * Return the location of the annotation for this ParadiseFeature and for the molecule given as argument
     * Return the union of its own location and the location for the molecule given as argument for each sub-features
     * WARNING: returned a a new Location, so if modified by the calling method, the original location is not modified.
     * To modufy the original location of the ParadiseFeature, use the getLocation() method provided by the Molecule.Annotation concept
     * @param molecule
     * @return an empty location if this feature doesn't annotate the molecule given as argument.
     */
    public Location getFullLocation(Molecule molecule);

    /**
     * Add a new Annotation wrapping this ParadiseFeature. Since a ParadiseFeature can be linked only once to a Molecule through an Annotation object
     * this new Annotation will be linked to a Molecule never annotated before by this ParadiseFeature
     *
     * @param a
     */
    public void addAnnotation(Molecule.Annotation a) throws ArchitectureException;

    public void removeAnnotation(Molecule.Annotation a);

    public List<Molecule.Annotation> getAnnotations();

    public boolean annotates(Molecule molecule);

    public boolean hasSubFeature(ParadiseFeature subFeature);

    public boolean hasNoSubFeatures();

    public boolean hasNoParentFeatures();

    public boolean annotatesNothing();

    public Collection<Molecule> getMolecules();

    public void addToLocation(Location l, Molecule m) throws ArchitectureException;

    public void removeFromLocation(Location l, Molecule m);

    /**
     * Return all the residues composing the ParadiseFeature
     *
     * @return
     */
    public Residues getResidues();

    /**
     * Test if a residue is inside the ParadiseFeature
     *
     * @param r
     * @return
     */
    public boolean contains(Residue r);

    public ParadiseID getParadiseID();

}
