package fr.unistra.ibmc.paradise.core;

import fr.unistra.ibmc.paradise.core.utils.Residue;

import java.util.*;

/**
 * A collection of Residue objects. In this collection, the Residue objects are ordered according to:
 * - their molecular chain
 * - their absolute position in this molecular chain
 */
public class Residues {
    private Map<ParadiseID, Set<Residue>> molecularChains;

    public Residues() {
        this.molecularChains = new HashMap<ParadiseID, Set<Residue>>();
    }

    public void add(Residue r) {
        if (this.molecularChains.containsKey(r.getMoleculeId()))
            this.molecularChains.get(r.getMoleculeId()).add(r);
        else {
            // the residues have to be without duplicate elements and ordered according to their absolute position
            TreeSet<Residue> residues = new TreeSet<Residue>(new Comparator() {

                public int compare(Object o1, Object o2) {
                    return new Integer(((Residue) o1).getAbsolutePosition()).compareTo(new Integer(((Residue) o2).getAbsolutePosition()));
                }
            });
            residues.add(r);
            this.molecularChains.put(r.getMoleculeId(), residues);
        }
    }

    public Collection<Residue> getResidues() {
        List<Residue> residues = new ArrayList<Residue>();
        for (Set<Residue> s : this.molecularChains.values())
            residues.addAll(s);
        return residues;
    }

    public Iterator<Residue> iterator() {
        return this.getResidues().iterator();
    }

    /**
     * Return all the residues attached to a molecular chain whose paradise id is given as argument
     *
     * @param paradiseLSID
     * @return
     */
    public Collection<Residue> getResidues(ParadiseID paradiseLSID) {
        List<Residue> residues = new ArrayList<Residue>();
        if (this.molecularChains.containsKey(paradiseLSID))
            residues.addAll(this.molecularChains.get(paradiseLSID));
        return residues;
    }

    public void addAll(Residues residues) {
        for (Residue r : residues.getResidues())
            this.add(r);
    }
}
