package fr.unistra.ibmc.paradise.core;

/**
 * A link of affiliation between two molecules. It links two contiguous blocks of residues, each one belonging to a different Molecule ( a precursor and a product)
 */
public class Transformation {
    private Molecule precursor, product;
    private Location precursorLocation, productLocation;

    /**
     * An Transformation object is constructed with the start and end values to be sure that the Location is contiguous (that we don't have a Location composed with several Block objects)
     *
     * @param precursor
     * @param precursorStart
     * @param precursorEnd
     * @param product
     * @param productStart
     * @param productEnd
     */
    public Transformation(Molecule precursor, int precursorStart, int precursorEnd, Molecule product, int productStart, int productEnd) {
        this.precursor = precursor;
        this.product = product;
        this.precursorLocation = new Location(precursorStart, precursorEnd);
        this.productLocation = new Location(productStart, productEnd);
    }

    public Molecule getPrecursor() {
        return precursor;
    }

    public Molecule getProduct() {
        return product;
    }

    public Location getPrecursorLocation() {
        return precursorLocation;
    }

    public Location getProductLocation() {
        return productLocation;
    }
}
