package fr.unistra.ibmc.paradise.core.analysis;

import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.MoleculeFactory;
import fr.unistra.ibmc.paradise.core.features.AbstractParadiseFeatureFactoryFactory;
import fr.unistra.ibmc.paradise.core.features.ParadiseFeatureFactory;

import java.util.ArrayList;
import java.util.List;

public abstract class AbstractAnalysis extends AbstractParadiseFeatureFactoryFactory implements Analysis {

    protected Parameters parameters;
    protected List<PreRequisite> prerequisites;
    protected List<String> categories;
    private List<ParadiseFeatureFactory> featureFactories;
    protected List<Molecule> newMolecules;
    protected String status = "Start of analysis";   //the initial status for any analysis
    private boolean isCancelled;

    protected AbstractAnalysis(String description, MoleculeFactory moleculeFactory) {
        super(moleculeFactory, description);
        this.parameters = new Parameters();
        this.prerequisites = new ArrayList<PreRequisite>();
        this.featureFactories = new ArrayList<ParadiseFeatureFactory>();
        this.newMolecules = new ArrayList<Molecule>();
        this.categories = new ArrayList<String>();
    }

    public String getStatus() {
        return this.status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public void cancel() {
        this.isCancelled = true;
    }

    public boolean isCancelled() {
        return this.isCancelled;
    }

    public List<Molecule> getNewMolecules() {
        return new ArrayList<Molecule>(this.newMolecules);
    }

    public MoleculeFactory getMoleculeFactory() {
        return this.moleculeFactory;
    }

    public void addParadiseFeatureFactory(ParadiseFeatureFactory factory) {
        //add the new ParadiseFeatureFactory only if no such factory already registered
        if (this.getParadiseFeatureFactory(factory.getClass()) == null)
            this.featureFactories.add(factory);
    }

    public <T> T getParadiseFeatureFactory(Class<T> factoryType) {
        for (ParadiseFeatureFactory f : this.featureFactories)
            if (factoryType.isInstance(f))
                return factoryType.cast(f);
        return null;
    }

    public void addCategory(String category) {
        this.categories.add(category);
    }

    public String[] getCategories() {
        return this.categories.toArray(new String[]{});
    }

    public String toString() {
        return this.description;
    }


    public Parameters getParameters() {
        return this.parameters;
    }

    public ArrayList<PreRequisite> getPreRequisites() {
        return new ArrayList<PreRequisite>(prerequisites);
    }

    public Parameters getParameters(String name) {
        return this.parameters.getParameters(name);
    }

    protected Parameter addParameter(String name, String[] choices, String defaultValue) {
        Parameter p = new Parameter(name, choices, defaultValue);
        this.parameters.addParameter(p);
        return p;
    }

    protected Parameter addParameter(String name, String[] choices) {
        Parameter p = new Parameter(name, choices);
        this.parameters.addParameter(p);
        return p;
    }

    protected Parameter addParameter(String name, String defaultValue) {
        Parameter p = new Parameter(name, defaultValue);
        this.parameters.addParameter(p);
        return p;
    }

    protected Parameter addParameter(String name) {
        Parameter p = new Parameter(name);
        this.parameters.addParameter(p);
        return p;
    }

}
