package fr.unistra.ibmc.paradise.core.analysis;

import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.MoleculeFactory;
import fr.unistra.ibmc.paradise.core.features.ParadiseFeatureFactory;
import fr.unistra.ibmc.paradise.core.features.ParadiseFeatureFactoryFactory;

import java.util.ArrayList;
import java.util.List;

public interface Analysis extends ParadiseFeatureFactoryFactory {

    public final String SECONDARY_STRUCTURE_PREDICTION="2D prediction",
                        SECONDARY_STRUCTURE_DRAWING = "2D drawing",
                        STRUCTURAL_ALIGNMENT = "structural alignment",
                        TERTIARY_STRUCTURE_PREDICTION = "3D prediction",
                        TERTIARY_MOTIFS_FOLDING = "3D motifs folding",
                        TERTIARY_STRUCTURE_REFINEMENT = "3D refinement",
                        TERTIARY_STRUCTURE_ANNOTATION = "3D annotation",
                        NC_RNAS_CANDIDATES_SEARCHING = "non-coding RNAs candidates searching";

    public final String[] ANALYSIS_CATEGORIES= {
            SECONDARY_STRUCTURE_PREDICTION, SECONDARY_STRUCTURE_DRAWING,
            STRUCTURAL_ALIGNMENT, TERTIARY_STRUCTURE_PREDICTION,
            TERTIARY_MOTIFS_FOLDING, TERTIARY_STRUCTURE_REFINEMENT,
            TERTIARY_STRUCTURE_ANNOTATION,NC_RNAS_CANDIDATES_SEARCHING 
    };

    public String getDescription();

    /**
     * Return the current status of the Analysis
     *
     * @return
     */
    public String getStatus();

    public void cancel();

    public boolean isCancelled();

    public String[] getCategories();

    public void addParadiseFeatureFactory(ParadiseFeatureFactory factory);

    public <T> T getParadiseFeatureFactory(Class<T> factoryType);

    public MoleculeFactory getMoleculeFactory();

    /**
     * @return an empty list if no parameters for this Analysis
     */
    public Parameters getParameters();

    public ArrayList<PreRequisite> getPreRequisites();

    /**
     * Return all the Parameter objects with the name given as argument
     *
     * @param name the name of the parameters to recover
     * @return empty list if no parameters with the name given as argument
     */
    public Parameters getParameters(String name);

    /**
     * Do the job. Can throw two kinds of exception:
     * <ul>
     *    <li>AnalysisException: not a usual problem</li>
     *    <li>NoSolutionException: "sual" problem. The data sent don't fit the requirements of the algorithm</li>
     * </ul>
     *
     * @return
     * @throws AnalysisException
     * @throws NoSolutionException
     */
    public void run(List<Molecule> molecules, Parameters parameters) throws AnalysisException, NoSolutionException;

    /**
     * Return the molecules created by the analysis
     *
     * @return
     */
    public List<Molecule> getNewMolecules();
}
