package fr.unistra.ibmc.paradise.core.analysis;

import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.ParadiseFeature;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * User: fjossinet
 * Date: 5 janv. 2007
 */
public class Parameter implements Serializable {

    private String name;
    private String[] choices;
    private String value;
    private boolean optional;
    private Molecule molecule;
    private ParadiseFeature feature;

    public Parameter(String name, String[] choices, String defaultChoice) {
        this(name, choices);
        this.value = defaultChoice;
    }

    public Parameter(String name, String[] choices) {
        this.name = name;
        this.choices = choices;
    }

    public Parameter(String name, String defaultChoice) {
        this(name, new String[]{}, defaultChoice);
    }

    public Parameter(String name) {
        this(name, new String[]{});
    }

    /**
     * Return the id for this parameter
     *
     * @return
     */
    public String getName() {
        return this.name;
    }

    /**
     * Return all the choices available for this parameter
     *
     * @return
     */
    public List<String> getChoices() {
        List<String> copy = new ArrayList<String>(this.choices.length);
        for (String c : this.choices)
            copy.add(c);
        return copy;
    }

    public void setValue(String value) {
        if (this.choices.length == 0)
            this.value = value;
        //if some choices are registered, the value must be one of these choices
        for (String choice : this.choices) {
            if (choice.equals(value))
                this.value = value;
        }
    }

    public String getValue() {
        return this.value;
    }

    public boolean isFilled() {
        return this.value!=null;
    }

    public boolean isOptional() {
        return optional;
    }

    public void setOptional(boolean optional) {
        this.optional = optional;
    }

    /**
     * Return the ParadiseFeature associated to this paramater
     * @return null if the parameter is not linked to a particular ParadiseFeature
     */
    public ParadiseFeature getFeature() {
        return feature;
    }

    /**
     * Indicate that the parameter is associated to the ParadiseFeature given as argument
     * @param feature
     */
    public void setFeature(ParadiseFeature feature) {
        this.feature = feature;
    }

    /**
     * Return the Molecule associated to this paramter
     * @return  null if the parameter is not linked to a particular Molecule
     */
    public Molecule getMolecule() {
        return molecule;
    }

    /**
     * Indicate that the parameter is associated to the Molecule given as argument
     * @param molecule
     */
    public void setMolecule(Molecule molecule) {
        this.molecule = molecule;
    }
}
