package fr.unistra.ibmc.paradise.core.features;

import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.ArchitectureException;


public class CisHHInteraction extends BaseBaseInteraction {
    private int dominant;

    protected CisHHInteraction(StructuralFeatureFactory factory, int dominantResidue, Molecule m, Location l) {
        super(factory, m, l);
        this.dominant = dominantResidue;
    }

    protected CisHHInteraction(StructuralFeatureFactory factory, int dominantResidue, Molecule m1, Location l1, Molecule m2, Location l2) {
        super(factory, m1, l1, m2, l2);
        this.dominant = dominantResidue;
    }

    protected CisHHInteraction(StructuralFeatureFactory factory, Molecule m, Location l) {
        super(factory, m, l);
        this.dominant = BaseBaseInteraction.RESIDUE_UNDEFINED;
    }

    protected CisHHInteraction(StructuralFeatureFactory factory, Molecule m1, Location l1, Molecule m2, Location l2) {
        super(factory, m1, l1, m2, l2);
        this.dominant = BaseBaseInteraction.RESIDUE_UNDEFINED;
    }

    public void generateAtomAtomInteractions() {
        if (this.getAnnotations().size() == 1) {
            Molecule m = this.getAnnotations().get(0).getMolecule();
            Location l = this.getAnnotations().get(0).getLocation();
            char f = m.getResidueAt(l.getStart()).charAt(0);
            char s = m.getResidueAt(l.getEnd()).charAt(0);

            AtomAtomInteraction aai;
            switch (f) {
                case 'A':
                    switch (s) {
                        case 'A':
                            //Nothing
                            break;
                        case 'C':
                            //nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "N6", "O6", AtomAtomInteraction.INTERACTION_HBOND, m, l);
                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
                case 'C':
                    switch (s) {
                        case 'A':
                            //Nothing
                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "C6", "O6", AtomAtomInteraction.INTERACTION_CH, m, l);
                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
                case 'G':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "O6", "N6", AtomAtomInteraction.INTERACTION_HBOND, m, l);
                            break;
                        case 'C':
                            aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "O6", "C6", AtomAtomInteraction.INTERACTION_CH, m, l);
                            break;
                        case 'G':
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_UNDEFINED)
                                throw new ArchitectureException("This interaction is ambiguous, you must specify the dominant residue");
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_1) {
                                aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "O6", "C8", AtomAtomInteraction.INTERACTION_CH, m, l);
                            } else {
                                aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "C8", "O6", AtomAtomInteraction.INTERACTION_CH, m, l);
                            }
                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
                case 'U':
                    switch (s) {
                        case 'A':
                            //Nothing
                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            //Nothing
                            break;
                        case 'U':
                            //nothing
                            break;
                    }
                    break;
            }
        }
        if (this.getAnnotations().size() == 2) {
            Molecule m1 = this.getAnnotations().get(0).getMolecule();
            Molecule m2 = this.getAnnotations().get(1).getMolecule();
            Location l1 = this.getAnnotations().get(0).getLocation();
            Location l2 = this.getAnnotations().get(1).getLocation();
            char f = m1.getResidueAt(l1.getStart()).charAt(0);
            char s = m2.getResidueAt(l2.getEnd()).charAt(0);

            AtomAtomInteraction aai;
            switch (f) {
                case 'A':
                    switch (s) {
                        case 'A':
                            //Nothing
                            break;
                        case 'C':
                            //nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "N6", "O6", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);
                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
                case 'C':
                    switch (s) {
                        case 'A':
                            //Nothing
                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "C6", "O6", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);
                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
                case 'G':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "O6", "N6", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);
                            break;
                        case 'C':
                            aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "O6", "C6", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);
                            break;
                        case 'G':
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_UNDEFINED)
                                throw new ArchitectureException("This interaction is ambiguous, you must specify the dominant residue");
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_1) {
                                aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "O6", "C8", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);
                            } else {
                                aai = ((StructuralFeatureFactory) this.factory).createAtomAtomInteraction(this, "C8", "O6", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);
                            }
                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
                case 'U':
                    switch (s) {
                        case 'A':
                            //Nothing
                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            //Nothing
                            break;
                        case 'U':
                            //nothing
                            break;
                    }
                    break;
            }
        }
    }

    public char getEdge1() {
        return EDGE_HOOGSTEEN;
    }

    public char getEdge2() {
        return EDGE_HOOGSTEEN;
    }

    public char getOrientation() {
        return ORIENTATION_CIS;
    }

    public int getDominant() {
        return dominant;
    }
}