package fr.unistra.ibmc.paradise.core.features;

import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.Molecule;

import java.util.HashMap;
import java.util.Map;

public class StructuralIdentity extends Identity {

    private String name;
    private int length;
    private Map<Molecule, Location> gapsLocations;

    protected StructuralIdentity(IdentityFeatureFactory factory, String name) {
        super(factory);
        this.name = name;
        this.gapsLocations = new HashMap<Molecule, Location>();
    }

    public void addAnnotation(Molecule.Annotation a) {
        super.addAnnotation(a);
        if (this.length < a.getLocation().getLength())
            this.length = a.getLocation().getLength();
    }

    public void removeAnnotation(Molecule.Annotation a) {
        Location l = a.getLocation();
        super.removeAnnotation(a);
        if (l.getLength() == this.length) {
            int newLength = 0;
            //was it the only annotation with this max length ??
            for (Molecule.Annotation _a:this.getAnnotations())
                if (_a.getLocation().getLength() == this.length)
                    return;
                else if (newLength < _a.getLocation().getLength())
                    newLength = _a.getLocation().getLength();
            this.length = newLength;
        }
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return this.name;
    }

    protected boolean isBefore(StructuralIdentity identity) {
        for (Molecule.Annotation an : this.getAnnotations()) {
            Location lri = identity.getFullLocation(an.getMolecule());
            if (lri != null) {
                Location lth = an.getLocation();
                return (lth.getStart() < lri.getStart());
            }
        }
        //tludwig here the problem is if this annotates m1 and m2, and identity annotates m3 and m4
        //there is no way to know which one is "before" since they have no molecule in common
        //however imho, this punctual inconsistency should be corrected as soon as
        //- this or identity, annotates one of the other one's molecule
        //- this or identity are both compared to a third RI that have a common molecule with them...
        return false; //by default return false, but true should work as well
    }

    public int getLength() {
        //the length of a structural identity is the sum of the location length and of the gaps length for any molecuke annotated by this structural identity
        Molecule m = this.getMolecules().iterator().next();
        Location gapsLocation = this.getGapsLocation(m);
        return gapsLocation != null ? this.getFullLocation(m).getLength()+gapsLocation.getLength() : this.getFullLocation(m).getLength();
    }

    public Location getGapsLocation(Molecule m) {
        return this.gapsLocations.get(m);
    }

    public void addGapsLocation(Molecule m, Location l) {
        this.gapsLocations.put(m,l);
    }

}
