package fr.unistra.ibmc.paradise.core.features;

import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.ArchitectureException;


public class TransSSInteraction extends BaseBaseInteraction {
    private int dominant;

    protected TransSSInteraction(StructuralFeatureFactory factory, int dominantResidue, Molecule m, Location l) {
        super(factory, m, l);
        this.dominant = dominantResidue;
    }

    protected TransSSInteraction(StructuralFeatureFactory factory, Molecule m, Location l) {
        super(factory, m, l);
        this.dominant = BaseBaseInteraction.RESIDUE_UNDEFINED;
    }

    protected TransSSInteraction(StructuralFeatureFactory factory, int dominantResidue, Molecule m1, Location l1, Molecule m2, Location l2) {
        super(factory, m1, l1, m2, l2);
        this.dominant = dominantResidue;
    }

    protected TransSSInteraction(StructuralFeatureFactory factory, Molecule m1, Location l1, Molecule m2, Location l2) {
        super(factory, m1, l1, m2, l2);
        this.dominant = BaseBaseInteraction.RESIDUE_UNDEFINED;
    }

    public char getEdge1() {
        return EDGE_SUGAR_EDGE;
    }

    public char getEdge2() {
        return EDGE_SUGAR_EDGE;
    }

    public char getOrientation() {
        return ORIENTATION_TRANS;
    }

    public int getDominant() {
        return dominant;
    }

    public void generateAtomAtomInteractions() {
        if (this.getAnnotations().size() == 1) {
            Molecule m = this.getAnnotations().get(0).getMolecule();
            Location l = this.getAnnotations().get(0).getLocation();
            char f = m.getResidueAt(l.getStart()).charAt(0);
            char s = m.getResidueAt(l.getEnd()).charAt(0);
            AtomAtomInteraction aai;
            switch (f) {
                case 'A':
                    switch (s) {
                        case 'A':
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_UNDEFINED)
                                throw new ArchitectureException("This interaction is ambiguous, you must specify the dominant residue");
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_1) {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C2", AtomAtomInteraction.INTERACTION_CH, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "N3", AtomAtomInteraction.INTERACTION_CH, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            } else {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "N3", AtomAtomInteraction.INTERACTION_CH, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C2", AtomAtomInteraction.INTERACTION_CH, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N1", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            }
                            break;
                        case 'C':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "O2", AtomAtomInteraction.INTERACTION_CH, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'G':
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_UNDEFINED)
                                throw new ArchitectureException("This interaction is ambiguous, you must specify the dominant residue");
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_1) {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "N3", AtomAtomInteraction.INTERACTION_CH, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            } else {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "N3", AtomAtomInteraction.INTERACTION_CH, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            }
                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "O2", AtomAtomInteraction.INTERACTION_CH, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                    }
                    break;
                case 'C':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "C2", AtomAtomInteraction.INTERACTION_CH, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N1", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "N2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "O2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                    }
                    break;
                case 'G':
                    switch (s) {
                        case 'A':
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_UNDEFINED)
                                throw new ArchitectureException("This interaction is ambiguous, you must specify the dominant residue");
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_1) {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C2", AtomAtomInteraction.INTERACTION_CH, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "N3", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            } else {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "N3", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C2", AtomAtomInteraction.INTERACTION_CH, m, l);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N1", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            }
                            break;
                        case 'C':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "N3", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                    }
                    break;
                case 'U':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "C2", AtomAtomInteraction.INTERACTION_CH, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N1", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "N2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N2", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
            }
        }
        if (this.getAnnotations().size() == 2) {
            Molecule m1 = this.getAnnotations().get(0).getMolecule();
            Molecule m2 = this.getAnnotations().get(1).getMolecule();
            Location l1 = this.getAnnotations().get(0).getLocation();
            Location l2 = this.getAnnotations().get(1).getLocation();
            char f = m1.getResidueAt(l1.getStart()).charAt(0);
            char s = m2.getResidueAt(l2.getEnd()).charAt(0);
            AtomAtomInteraction aai;
            switch (f) {
                case 'A':
                    switch (s) {
                        case 'A':
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_UNDEFINED)
                                throw new ArchitectureException("This interaction is ambiguous, you must specify the dominant residue");
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_1) {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C2", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "N3", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            } else {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "N3", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C2", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N1", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            }
                            break;
                        case 'C':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "O2", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'G':
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_UNDEFINED)
                                throw new ArchitectureException("This interaction is ambiguous, you must specify the dominant residue");
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_1) {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "N3", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            } else {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "N3", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            }
                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "C2", "O2", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                    }
                    break;
                case 'C':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "C2", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N1", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "N2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "O2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                    }
                    break;
                case 'G':
                    switch (s) {
                        case 'A':
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_UNDEFINED)
                                throw new ArchitectureException("This interaction is ambiguous, you must specify the dominant residue");
                            if (this.getDominant() == BaseBaseInteraction.RESIDUE_1) {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C2", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "N3", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            } else {
                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "N3", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C2", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                                aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N1", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            }
                            break;
                        case 'C':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "N3", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O2'", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                    }
                    break;
                case 'U':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "C2", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N1", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "N2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2'", "N2", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
            }
        }
    }
}