package fr.unistra.ibmc.paradise.core.features;

import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.Molecule;

public class TransWHInteraction extends BaseBaseInteraction {
    protected TransWHInteraction(StructuralFeatureFactory factory, Molecule m, Location l) {
        super(factory, m, l);
    }

    protected TransWHInteraction(StructuralFeatureFactory factory, Molecule m1, Location l1, Molecule m2, Location l2) {
        super(factory, m1, l1, m2, l2);
    }

    public char getEdge1() {
        return EDGE_WATSON_CRICK;
    }

    public char getEdge2() {
        return EDGE_HOOGSTEEN;
    }

    public char getOrientation() {
        return ORIENTATION_TRANS;
    }

    public void generateAtomAtomInteractions() {
        if (this.getAnnotations().size() == 1) {
            Molecule m = this.getAnnotations().get(0).getMolecule();
            Location l = this.getAnnotations().get(0).getLocation();
            char f = m.getResidueAt(l.getStart()).charAt(0);
            char s = m.getResidueAt(l.getEnd()).charAt(0);

            AtomAtomInteraction aai;
            switch (f) {
                case 'A':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N6", "N7", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "N6", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'C':
                            //nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N6", "N7", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O6", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'U':
                            //nothing
                            break;
                    }
                    break;
                case 'C':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N4", "N7", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N6", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'C':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C5", AtomAtomInteraction.INTERACTION_CH, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "N4", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N4", "N7", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "O6", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
                case 'G':
                    switch (s) {
                        case 'A':
                            //Nothing
                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "N7", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O6", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O6", "C5", AtomAtomInteraction.INTERACTION_CH, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O4", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                    }
                    break;
                case 'U':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N7", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "N6", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O4", "C8", AtomAtomInteraction.INTERACTION_CH, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N7", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O4", "C5", AtomAtomInteraction.INTERACTION_CH, m, l);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "O4", AtomAtomInteraction.INTERACTION_HBOND, m, l);

                            break;
                    }
                    break;
            }
        }
        if (this.getAnnotations().size() == 2) {
            Molecule m1 = this.getAnnotations().get(0).getMolecule();
            Molecule m2 = this.getAnnotations().get(1).getMolecule();
            Location l1 = this.getAnnotations().get(0).getLocation();
            Location l2 = this.getAnnotations().get(1).getLocation();
            char f = m1.getResidueAt(l1.getStart()).charAt(0);
            char s = m2.getResidueAt(l2.getEnd()).charAt(0);
            AtomAtomInteraction aai;
            switch (f) {
                case 'A':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N6", "N7", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "N6", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'C':
                            //nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N6", "N7", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O6", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'U':
                            //nothing
                            break;
                    }
                    break;
                case 'C':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N4", "N7", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N6", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'C':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "C5", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "N4", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N4", "N7", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "O6", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'U':
                            //Nothing
                            break;
                    }
                    break;
                case 'G':
                    switch (s) {
                        case 'A':
                            //Nothing
                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "N7", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N2", "O6", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O6", "C5", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N1", "O4", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                    }
                    break;
                case 'U':
                    switch (s) {
                        case 'A':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N7", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O2", "N6", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'C':
                            //Nothing
                            break;
                        case 'G':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O4", "C8", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "N7", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                        case 'U':
                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "O4", "C5", AtomAtomInteraction.INTERACTION_CH, m1, l1, m2, l2);

                            aai = ((StructuralFeatureFactory) factory).createAtomAtomInteraction(this, "N3", "O4", AtomAtomInteraction.INTERACTION_HBOND, m1, l1, m2, l2);

                            break;
                    }
                    break;
            }
        }
    }
}