package fr.unistra.ibmc.paradise.core.utils;

import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.ParadiseID;

import java.io.Serializable;

/**
 * Same role as the Point2D class for x,y coordinates. The Residue class is a utility class to wrap
 * the position of a residue and the lsid of its molecule. It also proposes some utility methods
 */
public class Residue implements Serializable {

    public static final char GUANINE = 'G', CYTOSINE = 'C', URIDINE = 'U', THYMINE = 'T', ADENINE = 'A';

    private int absolutePosition;
    private Molecule molecule;

    public Residue(int absolutePosition, Molecule molecule) {
        this.absolutePosition = absolutePosition;
        this.molecule = molecule;
    }

    public int getAbsolutePosition() {
        return absolutePosition;
    }

    public ParadiseID getMoleculeId() {
        return this.molecule.getParadiseID();
    }

    /**
     * Return the next Residue in the sequence
     *
     * @return null if the current residue is the last one in the sequence
     */
    public Residue getNextResidue() {
        if (this.absolutePosition == this.molecule.getLength())
            return null;
        else
            return new Residue(this.absolutePosition + 1, this.molecule);
    }

    /**
     * return the previous residue in the sequence
     *
     * @return null if the current residue is the first one in the sequence
     */
    public Residue getPreviousResidue() {
        if (this.absolutePosition == 1)
            return null;
        else
            return new Residue(this.absolutePosition - 1, this.molecule);
    }

    /**
     * Return the character in the sequence for this residue
     *
     * @return
     */
    public char getSymbol() {
        return this.molecule.printSequence(new Location(this.absolutePosition)).toCharArray()[0];
    }

    public Molecule getMolecule() {
        return this.molecule;
    }

    public boolean equals(Object o) {
        if (!Residue.class.isInstance(o))
            return false;
        return ((Residue) o).getAbsolutePosition() == this.getAbsolutePosition() && ((Residue) o).getMolecule().equals(this.getMolecule());
    }
}
