/*
 * ParadisePlatform.java
 *
 * Created on 16 octobre 2006, 13:50
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package fr.unistra.ibmc.paradise.services;

import fr.unistra.ibmc.paradise.core.AbstractMoleculeFactory;
import fr.unistra.ibmc.paradise.services.analysis.ParadiseAnalysisAgent;
import fr.unistra.ibmc.paradise.services.analysis.pseudoviewer.PseudoviewerAgent;
import fr.unistra.ibmc.paradise.services.analysis.ibmc.varna.IBMCVarnaAgent;
import fr.unistra.ibmc.paradise.services.analysis.ibmc.contrafold.IBMCContrafoldAgent;
import fr.unistra.ibmc.paradise.services.analysis.ebi.vrnaplot.EBIVrnaplotAgent;
import fr.unistra.ibmc.paradise.services.analysis.ebi.vrnafold.EBIVrnafoldAgent;
import fr.unistra.ibmc.paradise.services.analysis.ebi.vrnadistance.EBIVrnadistanceAgent;
import fr.unistra.ibmc.paradise.services.analysis.rnamotif.RNAMotifAgent;
import fr.unistra.ibmc.paradise.services.analysis.ibmc.nahelix.IBMCNahelixAgent;
import fr.unistra.ibmc.paradise.services.analysis.ibmc.rnart.IBMCRnartAgent;
import fr.unistra.ibmc.paradise.services.analysis.ibmc.rnaview.IBMCRnaviewAgent;
import fr.unistra.ibmc.paradise.services.analysis.ibmc.rnafold.IBMCRnafoldAgent;
import fr.unistra.ibmc.paradise.services.analysis.ibmc.rnaplot.IBMCRnaplotAgent;
import fr.unistra.ibmc.paradise.services.analysis.ibmc.rnadistance.IBMCRnadistanceAgent;
import fr.unistra.ibmc.paradise.services.community.ParadiseCommunityAgent;
import fr.unistra.ibmc.paradise.Paradise;
import fr.unistra.ibmc.paradise.utils.IOUtils;
import jade.core.AID;
import jade.core.Agent;
import jade.core.Profile;
import jade.core.ProfileImpl;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.wrapper.*;
import org.apache.log4j.Logger;
import org.apache.log4j.Priority;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author fjossinet
 */
public final class ParadisePlatform extends AbstractMoleculeFactory {

    private static ParadisePlatform platform;
    private Logger logger;
    private PlatformManager platformManager;
    private Map<String, Class> agentsLaunched;
    private Map<String, List<Conversation>> currentConversations;
    private AgentContainer container;
    private String address;

    public static void launchParadisePlatform() {
        IOUtils.clearDirectory(Paradise.getTmpDirectory());
        String hostAddress;
        try {
            hostAddress = InetAddress.getLocalHost().getHostAddress();
        } catch (UnknownHostException e) {
            e.printStackTrace();
            return;
        }
        if (platform == null)
            platform = new ParadisePlatform(hostAddress);
    }

    public static ParadisePlatform getPlatform() {
        return platform;
    }

    private ParadisePlatform(final String address) {
        super(fr.unistra.ibmc.paradise.Paradise.releaseName);
        this.address = address;
        this.agentsLaunched = new HashMap<String, Class>();
        this.currentConversations = new HashMap<String, List<Conversation>>();
        jade.core.Runtime r = jade.core.Runtime.instance();
        Profile p = new ProfileImpl();
        p.setParameter(Profile.MAIN_HOST, address);
        //the next parameter will give the name of the platform. This name will be recovered by a ParadiseToolAgent to check is the tool is compatible with the platform.
        //if not compatible, will throw an IncompatibilityException
        p.setParameter(Profile.PLATFORM_ID, fr.unistra.ibmc.paradise.Paradise.releaseName);
        this.container = r.createMainContainer(p);
        if (this.container == null) {
            System.err.println("WARNING: it seems that a PARADISE platform has already been deployed at " + p.getParameter(Profile.MAIN_HOST, "Undefined"));
            return;
        }
        try {
            this.container.start();
            platformManager = new PlatformManager(address, this.container);
            this.container.addPlatformListener(new PlatformController.Listener() {

                public void bornAgent(final PlatformEvent platformEvent) {
                    writeLogMessage(this.getClass(), Priority.INFO, "Agent added to the platform", null, platformEvent.getAgentGUID(), null);
                    platformManager.addAgent(platformEvent.getAgentGUID());
                }

                public void deadAgent(PlatformEvent platformEvent) {
                    final String agentName = platformEvent.getAgentGUID().split("@")[0];
                    platformManager.removeAgent(platformEvent.getAgentGUID());
                    if (agentsLaunched.containsKey(agentName))
                        writeLogMessage(this.getClass(), Priority.FATAL, agentName + " crashed", "", null, null);
                    List<Conversation> conversations = currentConversations.get(agentName);
                    //if the dead agent has still conversations to close
                    if (conversations != null) {
                        for (int i = 0; i < conversations.size(); i++) {
                            final Conversation c = conversations.get(i);
                            try {
                                ParadisePlatform.this.writeLogMessage(this.getClass(), Priority.INFO, "The platform tries to clean the conversion #" + c.conversationId + " between dead agent " + agentName + " and " + c.partner.getLocalName(), c.conversationId, null, null);
                                AgentController controller = container.acceptNewAgent("Cleaner", new Agent() {
                                    protected void setup() {
                                        this.addBehaviour(new CloseDiscussionBehaviour());
                                    }

                                    class CloseDiscussionBehaviour extends OneShotBehaviour {

                                        public void action() {
                                            ACLMessage query = new ACLMessage(ACLMessage.FAILURE);
                                            query.addReceiver(c.partner);
                                            query.setContent("The service " + agentName + " cannot process your request efficiently");
                                            query.setConversationId(c.conversationId);
                                            myAgent.send(query);
                                            myAgent.doDelete();
                                        }
                                    }
                                });
                                controller.start();
                            } catch (StaleProxyException e) {
                                e.printStackTrace();
                            }
                        }
                        currentConversations.remove(agentName);
                    }
                    Class service = ParadisePlatform.this.agentsLaunched.get(agentName);
                    if (service != null) {
                        ParadisePlatform.this.agentsLaunched.remove(agentName);
                        ParadiseAnalysisAgent a = null;
                        AgentController controller = null;
                        final Constructor[] constructors = service.getConstructors();
                        for (int i = 0; i < constructors.length; i++) {
                            try {
                                //constructor of ParadiseAnalysisAgent subclass with the ParadisePlatform as argument
                                a = (ParadiseAnalysisAgent) constructors[i].newInstance(new Object[]{ParadisePlatform.this});
                                if (a != null) {
                                    controller = container.acceptNewAgent(a.getParadiseAgentName(), a);
                                    controller.start();
                                    writeLogMessage(this.getClass(), Priority.INFO, "Agent relaunched efficiently", "", a.getName(), null);
                                    //register it to relaunch it if terminated due to any problem
                                    agentsLaunched.put(a.getAID().getLocalName(), service);
                                }
                            } catch (InstantiationException e) {
                                e.printStackTrace();
                            } catch (IllegalAccessException e) {
                                e.printStackTrace();
                            } catch (InvocationTargetException e) {
                                e.printStackTrace();
                            } catch (StaleProxyException e) {
                                e.printStackTrace();
                            }
                        }
                    } else {
                        writeLogMessage(this.getClass(), Priority.WARN, "Agent seems not registered, couldn't launch it", null, agentName, null);
                    }
                }

                public void startedPlatform(PlatformEvent platformEvent) {
                }

                public void suspendedPlatform(PlatformEvent platformEvent) {
                }

                public void resumedPlatform(PlatformEvent platformEvent) {
                }

                public void killedPlatform(PlatformEvent platformEvent) {
                }
            });
        } catch (ControllerException e) {
            e.printStackTrace();
            return;
        }
        ParadiseAnalysisAgent a = null;
        //################ Agents for IBMC Web Services ##################
        try {
            a = new IBMCRnaviewAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), IBMCRnaviewAgent.class);
        }
        catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new IBMCRnafoldAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), IBMCRnafoldAgent.class);
        }
        catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new IBMCRnaplotAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), IBMCRnaplotAgent.class);
        }
        catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new IBMCRnadistanceAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), IBMCRnaplotAgent.class);
        }
        catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new IBMCNahelixAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), IBMCNahelixAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new IBMCRnartAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), IBMCRnartAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new IBMCVarnaAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), IBMCVarnaAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new RNAMotifAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), RNAMotifAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new IBMCContrafoldAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), IBMCContrafoldAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        }

        //################ Agents for EBI Web Services ##################
        try {
            a = new EBIVrnaplotAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), EBIVrnaplotAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new EBIVrnafoldAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), EBIVrnafoldAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        }
        try {
            a = new EBIVrnadistanceAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), EBIVrnadistanceAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        }
        /*
        try {
            a = new PseudoviewerAgent(this);
            this.agentsLaunched.put(a.getAID().getLocalName(), PseudoviewerAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        } */
        try {
            ParadiseCommunityAgent pca = new ParadiseCommunityAgent(this,"CommunityAgent", ParadiseCommunityAgent.categories);
            this.agentsLaunched.put(pca.getAID().getLocalName(), ParadiseCommunityAgent.class);
        } catch (StaleProxyException e) {
            e.printStackTrace();
        }
    }

    public AgentContainer getAgentContainer() {
        return container;
    }

    public void writeLogMessage(Class clazz, Priority level, String message, String conversationID, String agent, Throwable t) {
        /*
        String s = "";
        if (t != null) {
            for (StackTraceElement ste : t.getStackTrace()) {
                s += ste.toString() + "\n";
            }
        }
        logger.log(level, new LogMessage(System.currentTimeMillis(), level, message, conversationID, agent, s));*/
    }

    public void registerConversation(String agentName, String conversationId, AID partner) {
        if (this.currentConversations.containsKey(agentName))
            this.currentConversations.get(agentName).add(new Conversation(conversationId, partner));
        else {
            List<Conversation> conversations = new ArrayList<Conversation>();
            conversations.add(new Conversation(conversationId, partner));
            this.currentConversations.put(agentName, conversations);
        }
        this.writeLogMessage(this.getClass(), Priority.INFO, "beginning of conversation #" + conversationId + " between agent " + agentName + " and " + partner.getLocalName(), conversationId, null, null);
    }

    public void unregisterConversation(String agentName, String conversationId) {
        if (this.currentConversations.containsKey(agentName)) {
            List<Conversation> conversations = this.currentConversations.get(agentName);
            Conversation hit = null;
            for (Conversation c : conversations)
                if (c.conversationId.equals(conversationId)) {
                    hit = c;
                    break;
                }
            if (hit != null) {
                conversations.remove(hit);
                this.writeLogMessage(this.getClass(), Priority.INFO, "End of conversation between agent " + agentName + " and " + hit.partner.getLocalName(), conversationId, agentName, null);
            }
        }
    }

    public String getAddress() {
        return address;
    }

    private class Conversation {
        private String conversationId;
        private AID partner;


        public Conversation(String conversationId, AID partner) {
            this.conversationId = conversationId;
            this.partner = partner;
        }
    }

}
