package fr.unistra.ibmc.paradise.services;

import jade.core.Agent;
import jade.domain.DFService;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.domain.FIPAException;
import jade.wrapper.AgentController;
import jade.wrapper.StaleProxyException;
import org.apache.log4j.Priority;

import java.net.URL;

public abstract class ParadisePlatformAgent extends Agent {

    protected String name;
    protected String[] categories;
    //protected FileWriter logWriter;
    protected ParadisePlatform platform;

    protected ParadisePlatformAgent(ParadisePlatform platform, String name, String[] categories) throws StaleProxyException {
        this.platform = platform;
        this.name = name;
        this.categories = categories;
        AgentController controller = platform.getAgentContainer().acceptNewAgent(this.getParadiseAgentName() + "[at]" + platform.getAddress(), this);
        controller.start();
    }

    public String getParadiseAgentName() {
        return this.name;
    }

    public String[] getParadiseAgentCategories() {
        return this.categories;
    }

    public ParadisePlatform getParadisePlatform() {
        return this.platform;
    }

    protected void setup() {
        DFAgentDescription dfd;
        dfd = new DFAgentDescription();
        dfd.setName(this.getAID());
        for (String type : this.categories) {
            ServiceDescription sd = new ServiceDescription();
            sd.setName(this.name);
            sd.setType(type);
            dfd.addServices(sd);
        }
        try {
            DFService.register(this, dfd);
        } catch (FIPAException e) {
            e.printStackTrace();
        }
    }

    protected void takeDown() {
        try {
            DFService.deregister(this);
        } catch (FIPAException e) {
            e.printStackTrace();
        }
        ParadisePlatform.getPlatform().writeLogMessage(this.getClass(), Priority.INFO, "Agent terminated", "", this.getName(), null);
    }

    /*  public String getAgentTmpDirectory() {
        return this.getJarLocation(this.getClass(),this.getJarName(this.getClass()))+"tmp" + System.getProperty("file.separator");
    }*/

    public String getAgentDirectory() {
        return this.getJarLocation(this.getClass(), this.getJarName(this.getClass()));
    }

    private String getJarName(Class c) {
        final URL location;
        final String classLocation = c.getName().replace('.', '/') + ".class";
        final ClassLoader loader = c.getClassLoader();
        if (loader == null)
            location = ClassLoader.getSystemResource(classLocation);
        else
            location = loader.getResource(classLocation);
        String[] tokens = location.toString().split("!")[0].split("/");
        return tokens[tokens.length - 1];
    }

    private String getJarLocation(Class c, String jarName) {
        return c.getProtectionDomain().getCodeSource().getLocation().getPath().split(jarName)[0];
    }

    public void writeLogMessage(String message, Priority priority, String conversationID, String agent, Exception ex) {
        ParadisePlatform.getPlatform().writeLogMessage(this.getClass(), priority, message, conversationID, agent, ex);
        /*try {

            this.logWriter.write(new Date()+": "+ex.getMessage()+"\n");
            for(StackTraceElement ste : ex.getStackTrace()){
                this.logWriter.write("\t "+ste.toString()+"\n");
            }
			Throwable cause = ex.getCause();
			if(cause != null && cause instanceof Exception){
				Exception ca = (Exception)cause;
				this.logWriter.write(new Date()+": "+ca.getMessage()+"\n");
            	for(StackTraceElement ste : ca.getStackTrace()){
	                this.logWriter.write("\t "+ste.toString()+"\n");
				}
			}
			this.logWriter.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } */
    }
}
