package fr.unistra.ibmc.paradise.services;

import fr.unistra.ibmc.paradise.utils.ParadiseConfig;
import fr.unistra.ibmc.paradise.Paradise;
import jade.wrapper.AgentContainer;
import jade.wrapper.ControllerException;
import jade.wrapper.StaleProxyException;
import org.apache.log4j.Appender;
import org.apache.log4j.Layout;
import org.apache.log4j.spi.ErrorHandler;
import org.apache.log4j.spi.Filter;
import org.apache.log4j.spi.LoggingEvent;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.prefs.BackingStoreException;

public class PlatformManager extends JFrame {
    private LogTable table;
    private AgentList agentList;
    private ExceptionViewer exception;
    private Appender appender;
    SimpleDateFormat format;

    public PlatformManager(final String address, final AgentContainer container) {
        this.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(final java.awt.event.WindowEvent e) {
                if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(PlatformManager.this, "Do you want to stop the platform?")) {
                    try {
                        ParadiseConfig.save();
                        container.kill();
                    } catch (BackingStoreException e1) {
                        e1.printStackTrace();
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    } catch (StaleProxyException e1) {
                        e1.printStackTrace();
                    } catch (ControllerException e1) {
                        e1.printStackTrace();
                    }
                    System.exit(0);
                }
            }
        });
        JMenuBar menuBar = new JMenuBar();
        JMenu menu = new JMenu("Options");
        menuBar.add(menu);
        JMenuItem item = new JMenuItem("Stop platform");
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(PlatformManager.this, "Do you want to stop the platform?")) {
                    try {
                        ParadiseConfig.save();
                        container.kill();
                    } catch (BackingStoreException e1) {
                        e1.printStackTrace();
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    } catch (StaleProxyException e1) {
                        e1.printStackTrace();
                    } catch (ControllerException e1) {
                        e1.printStackTrace();
                    }
                    System.exit(0);
                }
            }
        });
        menu.add(item);
        this.setJMenuBar(menuBar);
        table = new LogTable();
        exception = new ExceptionViewer();
        agentList = new AgentList();
        appender = new ParadiseAppender();
        format = new SimpleDateFormat("yyyy/MM/dd - HH:mm:ss.SSS");

        JPanel zone = new JPanel();
        zone.setLayout(new BorderLayout());

        JPanel title = new JPanel(new BorderLayout());
        JPanel agents = new JPanel(new BorderLayout());
        JPanel messages = new JPanel(new BorderLayout());
        title.add(new TitlePanel(address, format.format(System.currentTimeMillis())), BorderLayout.CENTER);
        agents.add(new JScrollPane(agentList), BorderLayout.CENTER);
        messages.add(new JScrollPane(table), BorderLayout.CENTER);
        title.setBorder(BorderFactory.createTitledBorder("P.A.R.A.DIS.E Platform information"));
        agents.setBorder(BorderFactory.createTitledBorder("Connected Agents"));
        messages.setBorder(BorderFactory.createTitledBorder("Message Log"));
        zone.add(title, BorderLayout.NORTH);
        zone.add(agents, BorderLayout.WEST);
        zone.add(messages, BorderLayout.CENTER);
        zone.add(exception, BorderLayout.SOUTH);
        this.getContentPane().add(zone);
        this.pack();
        //this.setVisible(true);
        this.setTitle("PARADISE Platform Manager");
    }

    public Appender getAppender() {
        return appender;
    }

    private class ParadiseAppender implements Appender {
        public ParadiseAppender() {

        }

        public void doAppend(LoggingEvent event) {
            table.addRecord((LogMessage) event.getMessage());
            validate();
        }

        public String getName() {
            return null;
        }

        public void setErrorHandler(ErrorHandler errorHandler) {
        }

        public ErrorHandler getErrorHandler() {
            return null;
        }

        public void setLayout(Layout layout) {
        }

        public Layout getLayout() {
            return null;
        }

        public void setName(String s) {
        }

        public boolean requiresLayout() {
            return false;
        }

        public void addFilter(Filter filter) {
        }

        public Filter getFilter() {
            return null;
        }

        public void clearFilters() {
        }

        public void close() {
        }
    }

    private class LogTable extends JTable implements TableModel, ListSelectionListener, TableCellRenderer, MouseListener {
        RecordList records;
        public static final int ORDER_TIME = 1;
        public static final int ORDER_CONVERSATION_ID = 2;
        public static final int ORDER_MESSAGE = 3;
        public static final int ORDER_PRIORITY = 4;
        public static final int ORDER_TIME_REVERT = 5;
        public static final int ORDER_CONVERSATION_ID_REVERT = 6;
        public static final int ORDER_MESSAGE_REVERT = 7;
        public static final int ORDER_PRIORITY_REVERT = 8;
        public static final int ORDER_AGENT = 9;
        public static final int ORDER_AGENT_REVERT = 10;
        int order = ORDER_TIME;
        JButton timeButton;
        JButton priorityButton;
        JButton agentButton;
        JButton conversationIDButton;
        JButton messageButton;

        public LogTable() {
            super();
            timeButton = new JButton("Time");
            priorityButton = new JButton("Priority");
            agentButton = new JButton("Agent");
            conversationIDButton = new JButton("Conversation ID");
            messageButton = new JButton("Message");
            this.setModel(this);
            records = new RecordList();
            this.setDefaultRenderer(JLabel.class, this);
            this.setCellSelectionEnabled(false);
            this.setColumnSelectionAllowed(false);
            this.setRowSelectionAllowed(true);
            this.createDefaultColumnsFromModel();
            for (int i = 0; i < this.getColumnModel().getColumnCount(); i++) {
                TableColumn tc = this.getColumnModel().getColumn(i);
                tc.setHeaderRenderer(this);
            }
            this.getTableHeader().addMouseListener(this);
            this.getSelectionModel().addListSelectionListener(this);
        }

        public void valueChanged(ListSelectionEvent event) {
            String s = "No StackTrace associated to this Message";
            int i = this.getSelectedRow();
            Color color = Color.WHITE;
            if (i != -1) {
                if (!records.get(i).getException().equals(""))
                    s = records.get(i).getException();
                color = records.get(i).getColor();

            }
            exception.setException(s, color);
        }

        public void addRecord(LogMessage mssg) {
            records.add(mssg);
            table.revalidate();
        }

        public int getRowCount() {
            return records.size();
        }

        public int getColumnCount() {
            return 5;
        }

        public String getColumnName(int i) {
            switch (i) {
                case 0:
                    return "Time";
                case 1:
                    return "Priority";
                case 2:
                    return "Agent";
                case 3:
                    return "ConversationID";
                case 4:
                    return "Message";
            }
            return null;
        }

        public Class<?> getColumnClass(int i) {
            return JLabel.class;
        }

        public boolean isCellEditable(int row, int column) {
            return false;
        }

        public Object getValueAt(int row, int column) {
            switch (column) {
                case 0:
                    return format.format(records.get(row).getTime());
                case 1:
                    return records.get(row).getPriority().toString();
                case 2:
                    if (records.get(row).getAgent() == null)
                        return "No Agent";
                    return records.get(row).getAgent();
                case 3:
                    return records.get(row).getConversationID();
                case 4:
                    return records.get(row).getMessage();
            }
            return null;
        }

        public void setValueAt(Object o, int row, int column) {
        }

        public void addTableModelListener(TableModelListener tableModelListener) {
        }

        public void removeTableModelListener(TableModelListener tableModelListener) {
        }

        public Component getTableCellRendererComponent(JTable jTable, Object o, boolean b, boolean b1, int row, int col) {
            if (row == -1) {
                switch (col) {
                    case 0:
                        return timeButton;
                    case 1:
                        return priorityButton;
                    case 2:
                        return agentButton;
                    case 3:
                        return conversationIDButton;
                    case 4:
                        return messageButton;
                }
            }
            JLabel label = new JLabel();
            if (o != null)
                label.setText(o.toString());
            Color color = records.get(row).getColor();
            String ag = records.get(row).getAgent();
            for (int idx : agentList.getSelectedIndices()) {
                if (ag != null && ag.equals(agentList.model.getElementAt(idx)))
                    color = new Color(0xaa, 0xaa, 0xff);
            }
            label.setBackground(color);
            label.setOpaque(true);
            return label;
        }

        public void mousePressed(MouseEvent e) {
        }

        public void mouseReleased(MouseEvent e) {
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mouseClicked(MouseEvent e) {
            JTableHeader th = getTableHeader();
            if (th.getHeaderRect(0).contains(e.getPoint())) {
                if (order == ORDER_TIME)
                    order = ORDER_TIME_REVERT;
                else
                    order = ORDER_TIME;
            }
            if (th.getHeaderRect(1).contains(e.getPoint())) {
                if (order == ORDER_PRIORITY)
                    order = ORDER_PRIORITY_REVERT;
                else
                    order = ORDER_PRIORITY;
            }
            if (th.getHeaderRect(2).contains(e.getPoint())) {
                if (order == ORDER_AGENT)
                    order = ORDER_AGENT_REVERT;
                else
                    order = ORDER_AGENT;
            }
            if (th.getHeaderRect(3).contains(e.getPoint())) {
                if (order == ORDER_CONVERSATION_ID)
                    order = ORDER_CONVERSATION_ID_REVERT;
                else
                    order = ORDER_CONVERSATION_ID;
            }
            if (th.getHeaderRect(4).contains(e.getPoint())) {
                if (order == ORDER_MESSAGE)
                    order = ORDER_MESSAGE_REVERT;
                else
                    order = ORDER_MESSAGE;
            }
        }

        private class RecordList {
            ArrayList<LogMessage> records;
            java.util.List<Integer> orderByPriority;
            java.util.List<Integer> orderByConversationID;
            java.util.List<Integer> orderByMessage;
            java.util.List<Integer> orderByAgent;

            public RecordList() {
                records = new ArrayList<LogMessage>();

                orderByPriority = new ArrayList<Integer>();
                orderByConversationID = new ArrayList<Integer>();
                orderByMessage = new ArrayList<Integer>();
                orderByAgent = new ArrayList<Integer>();
            }

            public void add(LogMessage record) {
                records.add(record);
                int nb = records.size() - 1;

                boolean added = false;
                for (int i = 0; i < nb - 1; i++) {
                    if (record.getPriority().toInt() > records.get(orderByPriority.get(i)).getPriority().toInt()) {
                        added = true;
                        orderByPriority.add(i, nb);
                        break;
                    }
                }
                if (!added)
                    orderByPriority.add(nb);

                added = false;
                for (int i = 0; i < nb - 1; i++) {
                    if (before(record.getConversationID(), records.get(orderByConversationID.get(i)).getConversationID())) {
                        added = true;
                        orderByConversationID.add(i, nb);
                        break;
                    }
                }
                if (!added)
                    orderByConversationID.add(nb);

                added = false;
                for (int i = 0; i < nb - 1; i++) {
                    if (before(record.getMessage(), records.get(orderByMessage.get(i)).getMessage())) {
                        added = true;
                        orderByMessage.add(i, nb);
                        break;
                    }
                }
                if (!added)
                    orderByMessage.add(nb);

                added = false;
                for (int i = 0; i < nb - 1; i++) {
                    if (before(record.getAgent(), records.get(orderByAgent.get(i)).getAgent())) {
                        added = true;
                        orderByAgent.add(i, nb);
                        break;
                    }
                }
                if (!added)
                    orderByAgent.add(nb);
            }

            boolean before(String s1, String s2) {
                return s1 != null && (s2 == null || s1.compareToIgnoreCase(s2) < 0);
            }

            public int size() {
                return records.size();
            }

            public LogMessage get(int i) {
                switch (order) {
                    case ORDER_TIME:
                        return records.get(i);
                    case ORDER_PRIORITY:
                        return records.get(orderByPriority.get(i));
                    case ORDER_CONVERSATION_ID:
                        return records.get(orderByConversationID.get(i));
                    case ORDER_MESSAGE:
                        return records.get(orderByMessage.get(i));
                    case ORDER_TIME_REVERT:
                        return records.get((records.size() - 1) - i);
                    case ORDER_PRIORITY_REVERT:
                        return records.get(orderByPriority.get((records.size() - 1) - i));
                    case ORDER_CONVERSATION_ID_REVERT:
                        return records.get(orderByConversationID.get((records.size() - 1) - i));
                    case ORDER_MESSAGE_REVERT:
                        return records.get(orderByMessage.get((records.size() - 1) - i));
                    case ORDER_AGENT:
                        return records.get(orderByAgent.get(i));
                    case ORDER_AGENT_REVERT:
                        return records.get(orderByAgent.get((records.size() - 1) - i));
                }
                return records.get(i);
            }
        }
    }

    public void addAgent(String agentName) {
        agentList.addAgent(agentName);
        this.validate();
    }

    public void removeAgent(String agentName) {
        agentList.removeAgent(agentName);
        this.validate();
    }

    private class TitlePanel extends JPanel {
        public TitlePanel(String address, String time) {
            this.add(new JLabel("PARADISE platform has been efficiently deployed at " + address + " and is running since " + time));
        }
    }

    private class AgentList extends JList implements ListCellRenderer, ListSelectionListener {
        Icon iconServer;
        Icon iconClient;
        DefaultListModel model;

        public AgentList() {
            super(new DefaultListModel());
            this.setCellRenderer(this);
            iconServer = new ImageIcon(Paradise.class.getResource("pictures/edison.png"));
            iconClient = new ImageIcon(Paradise.class.getResource("pictures/spy.png"));
            model = (DefaultListModel) this.getModel();
            this.setLayoutOrientation(JList.VERTICAL);
            this.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
            this.setVisibleRowCount(-1);
            this.addListSelectionListener(this);
        }

        public void addAgent(String a) {
            model.addElement(a);
        }

        public void removeAgent(String a) {
            model.removeElement(a);
        }

        public Component getListCellRendererComponent(JList jList, Object o, int i, boolean selected, boolean b1) {
            JPanel p = new JPanel();
            if (i % 2 == 0)
                p.setBackground(new Color(0xcc, 0xcc, 0xcc));
            else
                p.setBackground(new Color(0xaa, 0xaa, 0xaa));
            if (selected) {
                p.setBackground(new Color(0xaa, 0xaa, 0xff));
                table.revalidate();
            }
            p.setOpaque(true);
            p.setLayout(new BorderLayout());
            JPanel right = new JPanel();
            right.setLayout(new BorderLayout());
            right.setOpaque(false);
            String agent = (String) o;
            String name = agent;
            String address = "";
            int idAt = agent.indexOf("[at]");
            int idFrom = agent.indexOf("[from]");
            if (idAt != -1) {
                name = agent.substring(0, idAt);
                address = agent.substring(idAt + 4);
                p.add(new JLabel(iconServer), BorderLayout.WEST);
            } else {
                name = agent.substring(0, idFrom);
                address = agent.substring(idFrom + 6);
                p.add(new JLabel(iconClient), BorderLayout.WEST);
            }
            JLabel lName = new JLabel(name);
            lName.setOpaque(false);
            right.add(lName, BorderLayout.NORTH);
            JLabel lAddress = new JLabel(address);
            lAddress.setOpaque(false);
            right.add(lAddress, BorderLayout.SOUTH);
            p.add(right, BorderLayout.EAST);
            return p;
        }

        public void valueChanged(ListSelectionEvent event) {
            table.repaint();
        }
    }

    private class ExceptionViewer extends JPanel {
        JTextArea jta;

        public ExceptionViewer() {
            jta = new JTextArea(10, 80);
            JScrollPane sp = new JScrollPane(jta);
            sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
            this.add(sp);
            this.setBorder(BorderFactory.createTitledBorder("Related Java Exception :"));
        }

        public void setException(String s, Color color) {
            jta.setBackground(color);
            jta.setText(s);
        }
    }
}
