package fr.unistra.ibmc.paradise.services.analysis;

import fr.unistra.ibmc.paradise.core.analysis.Analysis;
import fr.unistra.ibmc.paradise.core.analysis.Parameter;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import fr.unistra.ibmc.paradise.services.ParadisePlatform;
import fr.unistra.ibmc.paradise.services.ParadisePlatformAgent;
import fr.unistra.ibmc.paradise.Paradise;
import jade.wrapper.StaleProxyException;
import org.apache.log4j.Priority;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.io.File;

public abstract class ParadiseAnalysisAgent extends ParadisePlatformAgent {

    protected Map<String, Analysis> currentAnalysis;
    protected Parameters parameters;

    /**
     * A ParadiseAnalysisAgent needs a ParadisePlatform to produce Molecule objets and an Analysis to produce new ParadiseFeature.
     *
     * @param platform the MoleculeFactory of the ParadiseAnalysisAgent
     * @param analysis
     */
    protected ParadiseAnalysisAgent(ParadisePlatform platform, Analysis analysis) throws StaleProxyException {
        super(platform, analysis.getDescription(), analysis.getCategories());
        this.currentAnalysis = new HashMap<String, Analysis>();
        this.parameters = new Parameters();
    }

    /**
     * Return the Analysis associated with the convesation whose id is given as argument
     *
     * @param conversationId
     * @return null if no Analysis registered with this conversation id
     */
    public Analysis getAnalysis(String conversationId) {
        return this.currentAnalysis.get(conversationId);
    }

    public void closeConversation(String conversationId) {
        this.writeLogMessage("Job finished", Priority.INFO, conversationId, this.getName(), null);
        this.currentAnalysis.remove(conversationId);
        this.getParadisePlatform().unregisterConversation(this.getLocalName(), conversationId);
    }

    /**
     * Add a new Analysis associated with a conversation id to the Analysis queue
     *
     * @param conversationId
     */
    abstract public Analysis addAnalysis(String conversationId);

    protected void setup() {
        super.setup();
        this.addBehaviour(new SubscribeBehaviour());
        this.addBehaviour(new CancelBehaviour());
        this.addBehaviour(new RequestBehaviour());
    }

}
