package fr.unistra.ibmc.paradise.services.analysis;

import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.analysis.Analysis;
import fr.unistra.ibmc.paradise.core.analysis.Parameter;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import jade.core.behaviours.CyclicBehaviour;
import jade.domain.FIPAAgentManagement.Property;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;
import org.apache.log4j.Priority;
import org.jdesktop.swingworker.SwingWorker;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * When a ParadiseAnalysisAgent is contacted to run the analysis
 * User: fjossinet
 * Date: 12 mars 2007
 */
public class RequestBehaviour extends CyclicBehaviour {

    public void action() {
        final ACLMessage message = myAgent.receive(MessageTemplate.MatchPerformative(ACLMessage.REQUEST));
        if (message != null) {
            final Analysis analysis = ((ParadiseAnalysisAgent) myAgent).getAnalysis(message.getConversationId());
            try {
                if (analysis != null) {
                    ((ParadiseAnalysisAgent) myAgent).writeLogMessage("Received request from " + message.getSender(), Priority.INFO, message.getConversationId(), myAgent.getName(), null);
                    final ArrayList<Molecule> molecules;
                    try {
                        molecules = (ArrayList<Molecule>) message.getContentObject();
                    } catch (UnreadableException e) {
                        e.printStackTrace();
                        return;
                    }
                    final Parameters parameters = new Parameters();
                    Iterator properties = message.getEnvelope().getAllProperties();
                    while (properties.hasNext()) {
                        Property prop = (Property) properties.next();
                        if (prop.getName().equals("parameters")) {
                            for (Parameter p : ((Parameters) prop.getValue()).getParameters()) {
                                parameters.addParameter(p);
                            }
                            break;
                        }
                    }
                    if (molecules == null) {
                        Exception e = new Exception("The server needs a list of molecules (even if empty)");
                        ((ParadiseAnalysisAgent) myAgent).writeLogMessage(e.getMessage(), Priority.ERROR, message.getConversationId(), myAgent.getName(), e);
                        ACLMessage reply = message.createReply();
                        reply.setContentObject(e);
                        reply.setPerformative(ACLMessage.FAILURE);
                        ((ParadiseAnalysisAgent) myAgent).closeConversation(message.getConversationId());
                        myAgent.send(reply);
                    } else if (parameters == null) {
                        Exception e = new Exception("The server needs a list of parameters (even if empty)");
                        ((ParadiseAnalysisAgent) myAgent).writeLogMessage(e.getMessage(), Priority.ERROR, message.getConversationId(), myAgent.getName(), e);
                        ACLMessage reply = message.createReply();
                        reply.setContentObject(e);
                        reply.setPerformative(ACLMessage.FAILURE);
                        ((ParadiseAnalysisAgent) myAgent).closeConversation(message.getConversationId());
                        myAgent.send(reply);
                    } else {
                        // the following class is to check is the status of the analysis has changed. If so, an INFORM message is sent to the tool to inform the user
                        class AnalysisWorker extends SwingWorker implements ActionListener {
                            private String currentStatus = "";
                            private Timer timer;

                            protected Object doInBackground() throws Exception {
                                timer = new Timer(1, this);
                                timer.start();
                                try {
                                    analysis.run(molecules, parameters);
                                    if (!analysis.isCancelled()) {
                                        ACLMessage reply = message.createReply();
                                        reply.setPerformative(ACLMessage.PROPOSE);
                                        reply.setContentObject(molecules);
                                        ((ParadiseAnalysisAgent) myAgent).closeConversation(message.getConversationId());
                                        myAgent.send(reply);
                                        timer.stop();
                                    }
                                } catch (Exception e) {
                                    ((ParadiseAnalysisAgent) myAgent).writeLogMessage(e.getMessage(), Priority.ERROR, message.getConversationId(), myAgent.getName(), e);
                                    ACLMessage reply = message.createReply();
                                    reply.setContentObject(e);
                                    reply.setPerformative(ACLMessage.FAILURE);
                                    ((ParadiseAnalysisAgent) myAgent).closeConversation(message.getConversationId());
                                    myAgent.send(reply);
                                }
                                return null;
                            }

                            public void actionPerformed(ActionEvent e){
                                if (!analysis.isCancelled() && !currentStatus.equals(analysis.getStatus())) {
                                    currentStatus = analysis.getStatus();
                                    ACLMessage reply = message.createReply();
                                    reply.setPerformative(ACLMessage.INFORM);
                                    reply.setContent(currentStatus);
                                    myAgent.send(reply);
                                }
                            }
                        }
                        new AnalysisWorker().execute();
                    }
                } else {
                    ACLMessage reply = message.createReply();
                    Exception e = new Exception("Received request from " + message.getSender() + " but no registered Analysis found");
                    ((ParadiseAnalysisAgent) myAgent).writeLogMessage("Received request from " + message.getSender() + " but no registered Analysis found", Priority.ERROR, message.getConversationId(), myAgent.getName(), e);
                    reply.setContentObject(e);
                    reply.setPerformative(ACLMessage.FAILURE);
                    ((ParadiseAnalysisAgent) myAgent).closeConversation(message.getConversationId());
                    myAgent.send(reply);

                }
            }
            catch (IOException ioe) {
                ioe.printStackTrace();
            }
        } else {
            block();
        }
    }
}
