package fr.unistra.ibmc.paradise.services.analysis.ebi.vrnadistance;

import fr.unistra.ibmc.paradise.core.analysis.*;
import fr.unistra.ibmc.paradise.core.*;
import fr.unistra.ibmc.paradise.core.features.*;
import fr.unistra.ibmc.paradise.utils.IOUtils;

import javax.xml.ws.Holder;
import java.util.List;
import java.util.Arrays;
import java.io.File;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.FileReader;

import org.apache.commons.io.FileUtils;

public class EBIVrnadistanceAnalysis extends AbstractAnalysis {

    protected EBIVrnadistanceAnalysis(MoleculeFactory factory) {
        super("RNADistance (EBI Web Service)", factory);
        this.addCategory(Analysis.STRUCTURAL_ALIGNMENT);
    }

    public void run(List<Molecule> molecules, Parameters parameters) throws AnalysisException, NoSolutionException {
        try {
            File inputFile = IOUtils.createTemporaryFile("rnadistance_input"),
                    rnadistanceOutputFile = IOUtils.createTemporaryFile("rnadistance_output");

            //we output the information as an rnafold output
            PrintWriter pw = new PrintWriter(inputFile);
            for (Molecule m:molecules)
                    pw.println(m.getSelectedFeatures(SecondaryStructure.class).get(0).printAsBracketNotation(m));
            pw.close();

            VrnadistanceService service = new VrnadistanceService();
            Vrnadistance vrnadistance = service.getVrnadistancePort();

            Holder<String> outfile = new Holder<String>(), report = new Holder<String>();
            Holder<Long> detailedStatus = new Holder<Long>();

            this.status = "RNAdistance is running";
            long start = System.currentTimeMillis();

            vrnadistance.runAndWaitFor(FileUtils.readFileToString(inputFile),"", Arrays.asList(new String[]{"f"}),"p",false,true,report,detailedStatus,outfile,new Holder<String>());

            FileUtils.writeStringToFile(rnadistanceOutputFile,outfile.value);
            this.status ="RNAdistance done in "+(System.currentTimeMillis()-start)/1000+"s";
            BufferedReader buff= new BufferedReader(new FileReader(rnadistanceOutputFile));
            String line = null;
            //with the rnadistance ouput we're creating a new structural alignment
            IdentityFeatureFactory identityFeatureFactory = this.createIdentityFeatureFactory();
            StructuralFeatureFactory structuralFeatureFactory = this.createStructuralFeatureFactory();
            StructuralAlignment alignment = identityFeatureFactory.createStructuralAligment(new ParadiseID(), "Alignment generated by RNADistance", Source.getAnalysisAsSource(this.description));
            int sequenceIndex = 0;
            StructuralIdentity si = null;
            SecondaryStructure ss = null;
            for (Molecule m:molecules) {
                m.removeAllSelectedFeatures();
                m.addSelectedFeature(alignment);
            }
            while ((line = buff.readLine())!= null)
                if (line.startsWith(".") || line.startsWith("(") || line.startsWith("_")) {
                    char[] characters = line.trim().toCharArray();
                    sequenceIndex++;
                    int alignmentPos = 0, moleculePos = 1;
                    Location gapsLocation = new Location(), molecularLocation = new Location();
                    ss = structuralFeatureFactory.createRNASecondaryStructure(new ParadiseID(),"Secondary Structure", Source.getAnalysisAsSource(this.description));
                    molecules.get(sequenceIndex-1).addSelectedFeature(ss);
                    molecules.get(sequenceIndex-1).addSelectedFeature(alignment);
                    Location l = null;
                    for (char c:characters) {
                        if (c != '_' ) {
                            if (c == '.') {
                                if (l == null)
                                    l = new Location(moleculePos);
                                else
                                    l.add(moleculePos);
                            }
                            else {
                                if ( l != null) {
                                    structuralFeatureFactory.createSingleStrand("SS",ss,molecules.get(sequenceIndex-1),l);
                                    l = null;
                                }
                            }
                            molecularLocation.add(moleculePos++);
                        }
                        else {
                            gapsLocation.add(alignmentPos+1);
                            if ( l != null) {
                                structuralFeatureFactory.createSingleStrand("SS",ss,molecules.get(sequenceIndex-1),l);
                                l = null;
                            }
                        }
                        alignmentPos++;
                    }
                    if (si == null)
                        si = identityFeatureFactory.createStructuralIdentity(alignment,"Structural Identity generated by RNADistance",molecules.get(sequenceIndex-1),molecularLocation);
                    else
                        molecules.get(sequenceIndex-1).addFeature(si, molecularLocation);
                    si.addGapsLocation(molecules.get(sequenceIndex-1), gapsLocation);
                }
        }
        catch (Exception e) {
            throw new AnalysisException(e);
        }
    }
}
