package fr.unistra.ibmc.paradise.services.analysis.ebi.vrnafold;

import fr.unistra.ibmc.paradise.core.analysis.*;
import fr.unistra.ibmc.paradise.core.MoleculeFactory;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.ParadiseID;
import fr.unistra.ibmc.paradise.core.Source;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.io.CTFileIO;
import fr.unistra.ibmc.paradise.tools.component.DefaultProgressMonitor;
import fr.unistra.ibmc.paradise.utils.IOUtils;

import javax.xml.ws.Holder;
import java.util.List;
import java.util.Arrays;
import java.io.File;


/**
 * This class use the webservice vrnafold from EBI (http://www.ebi.ac.uk/soaplab/typed/services/nucleic_rna_folding.vrnafold?wsdl)
 * The classes used by this one have been automatically generated using wimport from JAX-WS:
 * wsimport.sh -d ~/paradise/src/ -p fr.unistra.ibmc.paradise.services.analysis.vrnafold -keep http://www.ebi.ac.uk/soaplab/typed/services/nucleic_rna_folding.vrnafold?wsdl
 * For an unknown reason, using Apache Axis doesn't work.
 */
public class EBIVrnafoldAnalysis extends AbstractAnalysis {

    protected EBIVrnafoldAnalysis(MoleculeFactory factory) {
        super("RNAFold (EBI Web Service)", factory);
        this.addCategory(Analysis.SECONDARY_STRUCTURE_PREDICTION);
    }

    public void run(List<Molecule> molecules, Parameters parameters) throws AnalysisException, NoSolutionException {
        try {
            VrnafoldService service = new VrnafoldService();
            Vrnafold vrnafold = service.getVrnafoldPort();
            Holder<String> outfile = new Holder<String>(), report = new Holder<String>();
            Holder<Long> detailedStatus = new Holder<Long>();
            SequenceInput seqInput = new SequenceInput();
            seqInput.setDirectData(">"+molecules.get(0).getName()+"\n"+molecules.get(0).printSequence());
            this.status = "Query to the webservice";
            vrnafold.runAndWaitFor(seqInput,"","","","",37.0f,false,true,true,true,true,"",true,"0",1.07f,"1",report,detailedStatus,outfile,new Holder<String>(),new Holder<String>(),new Holder<String>());
            String[] tokens = outfile.value.split(" ")[0].split("\\n"); //the first split remove the energy value (which is the second token) and the second split the sequence and the bracket notation (the sequence is the first token)
            File vrnafoldOutput = IOUtils.createTemporaryFile("vrnafold");
            CTFileIO.writeCTFile(vrnafoldOutput,tokens[0].trim(),tokens[1].trim());
            CTFileIO parser = new CTFileIO(this.moleculeFactory);
            parser.parseFile(vrnafoldOutput, new DefaultProgressMonitor());
            molecules.get(0).removeAllSelectedFeatures();
            if (parser.getMolecules().size() != 0) {
                SecondaryStructure ss = (SecondaryStructure)parser.getMolecules().get(0).getAnnotations(SecondaryStructure.class).get(0).getFeature();
                SecondaryStructure copySS = ss.getFactory().copySecondaryStructure(Arrays.asList(molecules.get(0)),ss, false);
                //[fjossinet] it's more logical that the origin of the secondary structure is the current analysis rather than the CTFileIO parser
                copySS.setParadiseID(new ParadiseID());
                copySS.setName("Secondary Structure");
                copySS.setSource(Source.getAnalysisAsSource(this.description));
                molecules.get(0).addSelectedFeature(copySS);
            }
        }
        catch (Exception e) {
            throw new AnalysisException(e);
        }
    }
}

