package fr.unistra.ibmc.paradise.services.analysis.ebi.vrnaplot;

import fr.unistra.ibmc.paradise.core.analysis.*;
import fr.unistra.ibmc.paradise.core.MoleculeFactory;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.features.Residue2D;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructureDisplay;

import java.util.List;
import java.util.Iterator;
import java.io.StringReader;

import javax.xml.ws.Holder;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.filter.ElementFilter;
import org.jdom.input.SAXBuilder;

/**
 * This class use the webservice vrnaplot from EBI (http://www.ebi.ac.uk/soaplab/typed/services/nucleic_rna_folding.vrnafold?wsdl)
 * The classes used by this one have been automatically generated using wimport from JAX-WS:
 * wsimport.sh -d ~/paradise/src/ -p fr.unistra.ibmc.paradise.services.analysis.vrnaplot -keep http://www.ebi.ac.uk/soaplab/typed/services/nucleic_rna_folding.vrnaplot?wsdl
 * For an unknown reason, using Apache Axis doesn't work.
 */
public class EBIVrnaplotAnalysis extends AbstractAnalysis {

    protected EBIVrnaplotAnalysis(MoleculeFactory factory) {
        super("RNAPlot (EBI Web Service)", factory);
        this.addCategory(Analysis.SECONDARY_STRUCTURE_DRAWING);
    }

    public void run(List<Molecule> molecules, Parameters parameters) throws AnalysisException, NoSolutionException {
        try {
            SecondaryStructureDisplay display = null;
            SecondaryStructure ss = null;
            for (Molecule m:molecules) {
                if (ss == null)
                    ss = m.getSelectedFeatures(SecondaryStructure.class).get(0);
                if (display == null)
                    display = ss.getFactory().createSecondaryStructureDisplay(ss,m, new Location(1,m.getLength()));
                VrnaplotService service = new VrnaplotService();
                Vrnaplot vrnaplot = service.getVrnaplotPort();
                Holder<String> outfile = new Holder<String>(), report = new Holder<String>();
                Holder<Long> detailedStatus = new Holder<Long>();

                this.status = "Query to the webservice";
                vrnaplot.runAndWaitFor(">"+m.getName()+"\n"+m.printSequence()+"\n"+ss.printAsBracketNotation(m)+" 0.0)",
                        "",
                        "naview","svg",
                        "",
                        "",
                        report,detailedStatus,outfile,new Holder<String>());

                String svgOutput = outfile.value;
                this.status = "Parsing of VRNAplot output";
                Document doc = new SAXBuilder().build(new StringReader(svgOutput));
                Iterator it = doc.getRootElement().getDescendants(new ElementFilter("text"));
                int pos=0;
                float minX = 0, minY = 0;
                while (it.hasNext()) {
                    Element e = (Element)it.next();
                    Residue2D r = ss.getFactory().createResidue2D(display, Float.parseFloat(e.getAttributeValue("x")),Float.parseFloat(e.getAttributeValue("y")),m, ++pos);
                    if (r.getX() < minX )
                        minX = r.getX();
                    if (r.getY() < minY)
                        minY = r.getY();
                }
                //the drawing should be located after the (0,0) point
                for (Residue2D r:display.getResidues2D()) {
                    r.setX(r.getX()+Math.abs(minX));
                    r.setY(r.getY()+Math.abs(minY));
                }
                m.removeAllSelectedFeatures();
                m.addSelectedFeature(display);
            }

        } catch (Exception e) {
            throw new AnalysisException(e);
        }
    }

}
