package fr.unistra.ibmc.paradise.services.analysis.ibmc.contrafold;

import fr.unistra.ibmc.paradise.core.analysis.*;
import fr.unistra.ibmc.paradise.core.MoleculeFactory;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.ParadiseID;
import fr.unistra.ibmc.paradise.core.Source;
import fr.unistra.ibmc.paradise.core.io.ParadiseFileIO;
import fr.unistra.ibmc.paradise.core.io.BPSeqFileIO;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.utils.IOUtils;
import fr.unistra.ibmc.paradise.tools.component.DefaultProgressMonitor;

import java.util.List;
import java.util.Arrays;
import java.io.File;

import org.apache.commons.io.FileUtils;


public class IBMCContrafoldAnalysis extends AbstractAnalysis {

    public IBMCContrafoldAnalysis(MoleculeFactory factory) {
        super("Contrafold (ARN/IBMC Web Service)",factory);
        this.addCategory(Analysis.SECONDARY_STRUCTURE_PREDICTION);
    }

    public void run(List<Molecule> molecules, Parameters parameters) throws AnalysisException, NoSolutionException {
        try {
            Molecule m = molecules.get(0);
            ContrafoldService service = new ContrafoldServiceLocator();
            Contrafold_PortType contrafold = service.getContrafold();
            String bpseqResult = contrafold.predict2D(">"+m.getName()+"\n"+m.printSequence());
            File contrafoldOutput = IOUtils.createTemporaryFile("contrafold_output");
            FileUtils.writeStringToFile(contrafoldOutput, bpseqResult);
            ParadiseFileIO parser = new BPSeqFileIO(this.moleculeFactory);
            parser.parseFile(contrafoldOutput, new DefaultProgressMonitor());
            m.removeAllSelectedFeatures();
            SecondaryStructure ss = (SecondaryStructure)parser.getMolecules().get(0).getAnnotations(SecondaryStructure.class).get(0).getFeature();
            SecondaryStructure copySS = ss.getFactory().copySecondaryStructure(Arrays.asList(m),ss, false);
            //[fjossinet] it's more logical that the origin of the secondary structure is the current analysis rather than the CTFileIO parser
            copySS.setParadiseID(new ParadiseID());
            copySS.setName("Secondary Structure");
            copySS.setSource(Source.getAnalysisAsSource(this.description));
            m.addSelectedFeature(copySS);
        }
        catch (Exception e) {
            throw new AnalysisException(e);
        }
    }
}
