package fr.unistra.ibmc.paradise.services.analysis.ibmc.rnafold;

import fr.unistra.ibmc.paradise.core.analysis.*;
import fr.unistra.ibmc.paradise.core.MoleculeFactory;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.ParadiseID;
import fr.unistra.ibmc.paradise.core.Source;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.io.CTFileIO;
import fr.unistra.ibmc.paradise.utils.IOUtils;
import fr.unistra.ibmc.paradise.tools.component.DefaultProgressMonitor;

import java.util.List;
import java.util.Arrays;
import java.io.File;

import org.apache.commons.io.FileUtils;


public class IBMCRnafoldAnalysis extends AbstractAnalysis {

    protected IBMCRnafoldAnalysis(MoleculeFactory factory) {
        super("RNAFold (ARN/IBMC Web Service)", factory);
        this.addCategory(Analysis.SECONDARY_STRUCTURE_PREDICTION);
    }

    public void run(List<Molecule> molecules, Parameters parameters) throws AnalysisException, NoSolutionException {
        try {
            RnafoldService service = new RnafoldServiceLocator();
            Rnafold_PortType rnafold = service.getRnafold();
            this.status = "Query to the webservice";
            String result = rnafold.predict2D(">"+molecules.get(0).getName()+"\n"+molecules.get(0).printSequence());
            File rnafoldOutput = IOUtils.createTemporaryFile("rnafold");
            FileUtils.writeStringToFile(rnafoldOutput,result);
            CTFileIO parser = new CTFileIO(this.moleculeFactory);
            parser.parseFile(rnafoldOutput, new DefaultProgressMonitor());
            molecules.get(0).removeAllSelectedFeatures();
            if (parser.getMolecules().size() != 0) {
                SecondaryStructure ss = (SecondaryStructure)parser.getMolecules().get(0).getAnnotations(SecondaryStructure.class).get(0).getFeature();
                SecondaryStructure copySS = ss.getFactory().copySecondaryStructure(Arrays.asList(molecules.get(0)),ss, false);
                //[fjossinet] it's more logical that the origin of the secondary structure is the current analysis rather than the CTFileIO parser
                copySS.setParadiseID(new ParadiseID());
                copySS.setName("Secondary Structure");
                copySS.setSource(Source.getAnalysisAsSource(this.description));
                molecules.get(0).addSelectedFeature(copySS);
            }
        }
        catch (Exception e) {
            throw new AnalysisException(e);
        }
    }
}
