package fr.unistra.ibmc.paradise.services.analysis.ibmc.rnart;

import fr.unistra.ibmc.paradise.utils.*;

import java.util.StringTokenizer;

/**
 * Java version of mrgnuc.f 13/09/2005 (Thomas Ludwig)<br/>
 * Not translated but rewritten !!!!<br/>
 * FYI : the original mrgnuc.f is at the end of this file<br/>
 * <br/>
 * Program MRGNUC<br/>
 * PROGRAM TO ADD THE SHIFTS FROM THE PROGRAM NUCLSQ<br/>
 * TO THE  COORDINATES CONTAINED IN LSQ.INP.  THE OUTPUTTED COORDINATES
 * ARE IN THE SO-CALLED MIT FORMAT.<br/>
 * <br/>
 * E.Westhof
 * <ul>
 * <li>05/1986 : Changed so that bmin = sc(2)</li>
 * <li>11/1987 : More than 99 residues</li>
 * </ul>
 * <p>Title: Fortran To Java</p>
 * <p/>
 * <p>Description: Java version of old Fortran Tools (nahelix, fragment, nuclin,
 * nuclsq...)</p>
 * <p/>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p/>
 * <p>Company: IBMC/CNRS - Strasbourg</p>
 *
 */

public class MgrNuc {
    public String tmpPath;
    public String inPath;
    public String outPath;

    public MgrNuc(String tmpPath, NucLSQ nucLSQ) throws RnartException {
        this.tmpPath = tmpPath;
        inPath = tmpPath;
        outPath = tmpPath;

        Printing.setDebugMode(OutputMode.off);
        Printing.setVerboseMode(OutputMode.off);
        Printing.setMessageMode(OutputMode.off);

        double[] sc = new double[8];
        double shtp[] = new double[1000];
        double damb[] = new double[1000];
        double damq[] = new double[1000];
        double[] dv = new double[50000];
        double[] d = new double[50000];

        int nbets = 0;
        int kb = 0;
        int iqzero = 0;
        int kno = 0;
        String lsqda = inPath + "LSQ.DAT";
        FileHandler lsqdat = new FileHandler(lsqda, FileHandler.READING);
        String line = lsqdat.readLine();
        line = lsqdat.readLine();
        StringTokenizer tok = new StringTokenizer(line);
        for (int i = 0; i < 8; i++) {
            tok.nextToken();
        }
        int idaliz = Tool.parseInt(tok.nextToken());
        line = lsqdat.readLine();
        tok = new StringTokenizer(line);
        int na = Tool.parseInt(tok.nextToken());
        for (int i = 0; i < 9; i++) {
            tok.nextToken();
        }
        int nocc = Tool.parseInt(tok.nextToken());
        int itemp = Tool.parseInt(tok.nextToken());
        int nsgr = Tool.parseInt(tok.nextToken());
        if (idaliz == 1) {
            nocc = 0;
            itemp = 0;
        }
        int nv = 0;
        if (itemp == 0) {
            nv = 3 * na + nocc + 2;
        }
        if (itemp == 1) {
            nv = 4 * na + nocc + 1;
        }
        int nbet = na;
        if (itemp == 0) {
            nbet = 1;
        }
        int iatmn = 3 * na + 1;
        int ibet1 = iatmn + 1;
        int ibetn = iatmn + nbet;
        for (int i = 0; i < 9; i++) {
            line = lsqdat.readLine();
        }
        line = lsqdat.readLine();
        tok = new StringTokenizer(line);
        double to = Tool.parseDouble(tok.nextToken());
        int no = Tool.parseInt(tok.nextToken());
        for (int i = 0; i < no; i++) {
            sc[i] = Tool.parseDouble(tok.nextToken());
        }
        line = lsqdat.readLine();
        tok = new StringTokenizer(line);
        int jabn = Tool.parseInt(tok.nextToken());
        int nb = tok.countTokens();
        for (int i = 0; i < nb; i++) {
            shtp[i] = Tool.parseDouble(tok.nextToken());
        }
        if (jabn > 15) {
            line = lsqdat.readLine();
            tok = new StringTokenizer(line);
            nb = tok.countTokens();
            for (int i = 15; i < 15 + nb; i++) {
                shtp[i] = Tool.parseDouble(tok.nextToken());
            }
        }
        if (itemp != 0) {
            line = lsqdat.readLine();
            tok = new StringTokenizer(line);
            nb = tok.countTokens();
            for (int i = 0; i < nb; i++) {
                damb[i] = Tool.parseDouble(tok.nextToken());
            }
            if (jabn > 15) {
                line = lsqdat.readLine();
                tok = new StringTokenizer(line);
                nb = tok.countTokens();
                for (int i = 15; i < 15 + nb; i++) {
                    damb[i] = Tool.parseDouble(tok.nextToken());
                }
            }
        }
        if (nocc != 0) {
            line = lsqdat.readLine();
            tok = new StringTokenizer(line);
            nb = tok.countTokens();
            for (int i = 0; i < nb; i++) {
                damq[i] = Tool.parseDouble(tok.nextToken());
            }
            if (jabn > 15) {
                line = lsqdat.readLine();
                tok = new StringTokenizer(line);
                nb = tok.countTokens();
                for (int i = 15; i < 15 + nb; i++) {
                    damq[i] = Tool.parseDouble(tok.nextToken());
                }
            }
        }
        lsqdat.close();
        String shftsbi = tmpPath + "SHFTS.BIN";

        nucLSQ.ishftw.close();

        System.gc();

        FileHandler shftsbin = new FileHandler(shftsbi, FileHandler.READING);
        for (int i = 0; i < nv; i++) {
            dv[i] = 0;
        }
        for (int jab1 = 1; jab1 <= jabn; jab1++) {
            line = shftsbin.readLine();
            if (line == null) {
                throw new RnartException("No more data");
            }
            tok = new StringTokenizer(line);
            for (int ic = 0; ic < nv; ic++) {
                if (tok.countTokens() > 0) {
                    d[ic] = Tool.parseDouble(tok.nextToken());
                } else {
                    System.out.println("failed to read token " + (ic + 1) + " / " + nv);
                }
            }
            double sksh = shtp[jab1 - 1];
            if (sksh > 0) {
                for (int i = 1; i < iatmn; i++) {
                    dv[i] += sksh * d[i];
                    Printing.verbose("dv[" + i + "] = " + dv[i] + " = " + sksh + " * " + d[i]);
                }
                if (itemp != 0) {
                    sksh = damb[jab1 - 1];
                    if (sksh > 0) {
                        nbets = na;
                        if (nocc > 0 && damq[jab1 - 1] > 0) {
                            nbets = na - nocc;
                        }
                        for (int i = 0; i < nbets; i++) {
                            int j = iatmn + i;
                            dv[j] += sksh * d[j];
                            Printing.verbose("dv[" + j + "] = " + dv[j] + " = " + sksh + " * " + d[j]);
                        }
                    }
                }
                if (nocc != 0) {
                    sksh = damq[jab1 - 1];
                    if (sksh > 0) {
                        for (int i = 0; i < nocc; i++) {
                            int j = ibetn + i;
                            dv[j] += sksh * d[j];
                            Printing.verbose("dv[" + j + "] = " + dv[j] + " = " + sksh + " * " + d[j]);
                        }
                    }
                }
            }
        }
        shftsbin.close();
        String lsqin = inPath + "LSQ.INP";
        FileHandler inlsqinp = new FileHandler(lsqin, FileHandler.READING);
        String atomsne = outPath + "ATOMS.hd";
        FileHandler outatomsnew = new FileHandler(atomsne, FileHandler.WRITING);
        for (int i = 1; i <= na; i++) {
            line = inlsqinp.readLine();
            HD hd;
            try {
                hd = new HD(line);
            }
            catch (ParsingException e) {
                throw new RnartException(e);
            }
            int nn = hd.atomNumber;
            char residueName = hd.residueName;
            int residueNumber = hd.residueNumber;
            String atom = hd.atomName;
            int kss = hd.atomCode;
            double x = hd.x;
            double y = hd.y;
            double z = hd.z;
            double bt = hd.b;
            double q = hd.q;

            x += dv[3 * i - 2];
            y += dv[3 * i - 1];
            z += dv[3 * i];
            int i4 = i;
            int k4 = kss;
            if (itemp != 0 && kb <= nbets) {
                kb++;
                bt += dv[iatmn + i - 1];
                if (sc[1] > bt) {
                    bt = sc[1];
                }
            }
            if (nocc != 0 && i > iqzero && q >= 0 && kno <= nocc) {
                kno++;
                int m = i - iqzero;
                q += dv[ibetn + m - 1];
                if (q < 0.01) {
                    q = 0.01;
                }
                if (q > 1) {
                    q = 1;
                }
            }
            try {
                outatomsnew.println(HD.getAtomString(i4, residueName, residueNumber, atom, x, y, z, bt, q));
                Printing.verbose(HD.getAtomString(i4, residueName, residueNumber, atom, x, y, z, bt, q));
            }
            catch (ParsingException e) {
                throw new RnartException(e);
            }
        }
        inlsqinp.close();
        outatomsnew.close();
    }
}
