package fr.unistra.ibmc.paradise.services.analysis.ibmc.varna;

import fr.unistra.ibmc.paradise.core.analysis.*;
import fr.unistra.ibmc.paradise.core.*;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructureDisplay;
import fr.unistra.ibmc.paradise.core.features.Residue2D;

import java.util.List;
import java.util.Iterator;
import java.io.StringReader;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.filter.ElementFilter;
import org.jdom.input.SAXBuilder;


public class IBMCVarnaAnalysis extends AbstractAnalysis {

    public IBMCVarnaAnalysis(MoleculeFactory factory) {
        super("VARNA (ARN/IBMC Web Service)",factory);
        this.addCategory(Analysis.SECONDARY_STRUCTURE_DRAWING);
    }

    public void run(List<Molecule> molecules, Parameters parameters) throws AnalysisException, NoSolutionException {
        try {
            SecondaryStructureDisplay display = null;
            SecondaryStructure ss = null;
            for (Molecule m:molecules) {
                if (ss == null)
                    ss = m.getSelectedFeatures(SecondaryStructure.class).get(0);
                if (display == null)
                    display = ss.getFactory().createSecondaryStructureDisplay(ss,m, new Location(1,m.getLength()));
                VarnaService service = new VarnaServiceLocator();
                Varna_PortType varna = service.getVarna();

                this.status = "Query to the webservice";
                String svgOutput = varna.drawRNA(">"+m.getName()+"\n"+m.printSequence()+"\n"+ss.printAsBracketNotation(m));
                this.status = "Parsing of VARNA output";
                Document doc = new SAXBuilder().build(new StringReader(svgOutput));
                Iterator it = doc.getRootElement().getDescendants(new ElementFilter("text"));
                int pos=0;
                float minX = 0, minY = 0;
                while (it.hasNext()) {
                    Element e = (Element)it.next();
                    String value = e.getValue();
                    if ("A".equals(value) || "U".equals(value) || "G".equals(value) || "C".equals(value) || "T".equals(value)) {
                        Residue2D r = ss.getFactory().createResidue2D(display, Float.parseFloat(e.getAttributeValue("x")),Float.parseFloat(e.getAttributeValue("y")),m, ++pos);
                        if (r.getX() < minX )
                            minX = r.getX();
                        if (r.getY() < minY)
                            minY = r.getY();
                    }
                }
                //the drawing should be located after the (0,0) point
                for (Residue2D r:display.getResidues2D()) {
                    r.setX(r.getX()+Math.abs(minX));
                    r.setY(r.getY()+Math.abs(minY));
                }
                m.removeAllSelectedFeatures();
                m.addSelectedFeature(display);
            }
        }
        catch (Exception e) {
            throw new AnalysisException(e);
        }
    }
}
