package fr.unistra.ibmc.paradise.services.analysis.pseudoviewer;

import fr.unistra.ibmc.paradise.core.analysis.*;
import fr.unistra.ibmc.paradise.core.MoleculeFactory;
import fr.unistra.ibmc.paradise.core.Molecule;

import java.util.List;
import java.net.URL;

import org.apache.axis.client.Service;
import org.apache.axis.client.Call;

import javax.xml.namespace.QName;

import kr.ac.inha.wilab.WSPseudoViewer.encodedTypes.*;


public class PseudoviewerAnalysis extends AbstractAnalysis {

    protected PseudoviewerAnalysis(MoleculeFactory factory) {
        super("Pseudoviewer (Inha University Web Service)", factory);
        this.addCategory(Analysis.SECONDARY_STRUCTURE_DRAWING);
    }

    public void run(List<Molecule> molecules, Parameters parameters) throws AnalysisException, NoSolutionException {
        try {
            URL wsdl =
				new URL("http://165.246.44.42/WSPseudoViewer/WSPseudoViewer.asmx?WSDL");
            Service ser =
				new Service(
					wsdl,
					new QName(
						"http://wilab.inha.ac.kr/WSPseudoViewer/",
						"WSPseudoViewer"));

            WSPVNumbering numbering=new WSPVNumbering();
			numbering.setBase_numbers("");
			numbering.setInterval(10);
			numbering.setNumbering_option("default");

            WSPVOption option = new WSPVOption();
            option.setOutput_type("coordinations");
            option.setOutput_name("test");
			option.setDrawing_option("default");
			option.setNumbering(numbering);

            WSPVRequest pvrq=new WSPVRequest();

            WSPVInData pvinput=new WSPVInData();
            WSPVSequence sequence = new WSPVSequence();
            sequence.setName("toto");
            sequence.setStart_Base(1);
            sequence.setSequence_data("GGGAAACCC");
            pvinput.setSequence(new WSPVSequence[]{sequence});
            pvinput.setSequence_data("GGGAAACCC");

            BracketView bv = new BracketView("(((...)))");
            pvinput.setStructure_data(bv);
            pvinput.setStrands(new StemLoopStrands());
            pvinput.setOption(option);
            
            pvrq.setStructures(new WSPVInput[]{pvinput});
            pvrq.setOption(option);

            /*
            StringBuffer pvfile = new StringBuffer();
            pvfile.append("GGGAAACCC\n");
            pvfile.append("(((...)))");

            WSPVNumbering numbering=new WSPVNumbering();
			numbering.setBase_numbers("1");
			numbering.setInterval(10);
			numbering.setNumbering_option("default");

			WSPVOption option=new WSPVOption();
			//request the url of the result image
			option.setOutput_type("url gif");
			option.setOutput_name("test");
			option.setDrawing_option("default");
			option.setNumbering(numbering);
            option.setScale(1.0f);

			WSPVRequest pvrq=new WSPVRequest();
			pvrq.setOption(option);

			WSPVInFileData pvinput=new WSPVInFileData();
			pvinput.setPV_file(pvfile.toString());
			pvinput.setOption(option);

			pvrq.setStructures(new WSPVInput[]{pvinput});*/

            Call call= (Call)ser.createCall(new QName("WSPseudoViewerSoap"),"WSPVRunRpc");
			//register the type of the inheritance classes.
			WSPVRegisterTypeMapping.registerTypeMapping(call);

			call.setReturnType(new QName("http://wilab.inha.ac.kr/WSPseudoViewer/","WSPVResponse"), WSPVResponse.class);
			WSPVResponse str=(WSPVResponse)call.invoke(new Object[] {pvrq});
			System.out.println(str);
        }
         catch (Exception e) {
            throw new AnalysisException(e);
        }
    }
}
