package fr.unistra.ibmc.paradise.services.analysis.pseudoviewer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;

import javax.xml.namespace.QName;
import javax.xml.rpc.ServiceException;

import kr.ac.inha.wilab.WSPseudoViewer.encodedTypes.*;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;

//sample web service client for PseudoViewer web service
//PseudoViewer web page: pseudoviwer.inha.ac.kr
public class WSPVClient {
	public static void main(java.lang.String[] args) {

		try {

			//Read input file, args[0] should be a input file in this example.
			if(args[0] == null)
			{
				System.err.println(
				"java WSPVClient <pv input file>\n");
				System.exit(0);
			}
			File infile = new File(args[0]);
			BufferedReader reader = new BufferedReader(new FileReader(infile));
			String line;
			StringBuffer pvfile = new StringBuffer();
			while ((line = reader.readLine()) != null) {
				pvfile.append(line + "\n");
			}
			reader.close();
			//Pseudoviewer web service wsdl file
			URL wsdl =
				new URL("http://165.246.44.42/WSPseudoViewer/WSPseudoViewer.asmx?WSDL");

			//There are two kinds of web method whitch WSPV supports.
			//1. 'document/literal' type method. (import kr.ac.inha.wilab.WSPseudoViewer.*)
			//2. 'rpc/encoded' type method. (import kr.ac.inha.wilab.WSPseudoViewer.encodedTypes.*)
			//It would be more reliable to use 'rpc/encoded' method in JAVA

			//'document/literal' methods are 'WSPVRun', 'WSPVServerTest' and 'WSPVServerRequestTest'
			//'rpc/encoded' methods are 'WSPVRunRpc', 'WSPVServerTestRpc' and 'WSPVServerRequestTestRpc'
			
			//set the input
			WSPVNumbering numbering=new WSPVNumbering();
			numbering.setBase_numbers("");
			numbering.setInterval(10);
			numbering.setNumbering_option("default");
			
			WSPVOption option=new WSPVOption();
			//request the url of the result image
			option.setOutput_type("url gif");
			option.setOutput_name("test");
			option.setDrawing_option("default");
			option.setNumbering(numbering);
			
			WSPVRequest pvrq=new WSPVRequest();
			pvrq.setOption(option);
			
			WSPVInFileData pvinput=new WSPVInFileData();
			pvinput.setPV_file(pvfile.toString());
			pvinput.setOption(option);
			
			pvrq.setStructures(new WSPVInput[]{pvinput});			
			
			//Make service
			Service ser =
				new Service(
					wsdl,
					new QName(
						"http://wilab.inha.ac.kr/WSPseudoViewer/",
						"WSPseudoViewer"));
			
			//There is 'WSPVServerTestRpc' method for checking the server condition.
			//It needs no request message and 'Server ok' is the response when the server is working.
			/* server test method call
			Call call= (Call)ser.createCall(new QName("WSPseudoViewerSoap"),"WSPVServerTestRpc");
			WSPVRegisterTypeMapping.registerTypeMapping(call);
			java.lang.String pvResponse=(java.lang.String)call.invoke(new Object[]{});
			System.out.println(pvResponse);
			*/
			
			//If you want to check your request message is correct,
			// call 'WSPVServerRequestTestRpc(pvrq)' method. You can get 'Request ok' message, if the request message is correct.
			/* request test method call
			Call call= (Call)ser.createCall(new QName("WSPseudoViewerSoap"),"WSPVServerRequestTestRpc");
			WSPVRegisterTypeMapping.registerTypeMapping(call);
			java.lang.String pvResponse=(java.lang.String)call.invoke(new Object[]{pvrq});
			System.out.println(pvResponse);
			*/			
			
			//Make 'call' for 'WSPVRunRpc' web method
			Call call= (Call)ser.createCall(new QName("WSPseudoViewerSoap"),"WSPVRunRpc");
			//register the type of the inheritance classes.
			WSPVRegisterTypeMapping.registerTypeMapping(call);
			
			call.setReturnType(new QName("http://wilab.inha.ac.kr/WSPseudoViewer/","WSPVResponse"),WSPVResponse.class);
			WSPVResponse str=(WSPVResponse)call.invoke(new Object[] {pvrq});
			System.out.println(((WSPVOutURL)str.getPVResult()[0].getImages()[0]).getImage_URL());			
			
		} catch (RemoteException e) {
			System.err.println("failed: " + e.toString());
			System.exit(1);
		} catch (MalformedURLException e) {
			System.err.println("failed: " + e.toString());
			System.exit(1);
		} catch (ServiceException e) {
			System.err.println("Service unavailable: " + e.toString());
			System.exit(1);
		} catch (IOException e) {
			System.err.println("Input file error: " + args[0]);
			System.exit(1);
		}
	}
}
