package fr.unistra.ibmc.paradise.services.analysis.rnamotif;

import fr.unistra.ibmc.paradise.core.analysis.*;
import fr.unistra.ibmc.paradise.core.MoleculeFactory;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.io.CTFileIO;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.features.Helix;
import fr.unistra.ibmc.paradise.core.features.SingleStrand;
import fr.unistra.ibmc.paradise.core.features.StructuralDomain;
import fr.unistra.ibmc.paradise.Paradise;
import fr.unistra.ibmc.paradise.tools.component.DefaultProgressMonitor;
import fr.unistra.ibmc.paradise.utils.ParadiseConfig;
import fr.unistra.ibmc.paradise.utils.IOUtils;

import java.util.List;
import java.util.ArrayList;
import java.io.*;


public class RNAMotifAnalysis extends AbstractAnalysis {

    public RNAMotifAnalysis(MoleculeFactory factory) {
        super("RNAMotif 3.0.5 (Local Service)",factory);
        this.addCategory(Analysis.NC_RNAS_CANDIDATES_SEARCHING);
    }

    public void run(List<Molecule> molecules,Parameters parameters) throws AnalysisException, NoSolutionException {
        try {
            String databaseSelected = null;
            Parameters p = parameters.getParameters("database");
            if (p.size() == 0)
                throw new AnalysisException("No database selected for the RNAMotif service");
            else
                databaseSelected = p.getParameter(0).getValue();
            Molecule m = molecules.get(0);
            SecondaryStructure ss = m.getSelectedFeatures(SecondaryStructure.class).get(0);
            //we create the descriptor file corresponding to this secondary structure
            File descrFile =  IOUtils.createTemporaryFile("descr");
            PrintWriter pw = new PrintWriter(descrFile);
            pw.println("descr");

            List<StructuralDomain> domains=new ArrayList<StructuralDomain>();
            for (int i=1; i<=m.getLength();) {
                StructuralDomain sd = ss.getEnclosingStructuralDomain(new Residue(i,m));
                domains.add(sd); //in this case we want the helices twice to be able to export the h3 line in the description file (see just below)
                i = i+sd.getLength();
            }

            List<StructuralDomain> helicesAlreadyOutputted = new ArrayList<StructuralDomain>();
            for (StructuralDomain sd:domains) {
                Parameters params = parameters.getParameters(sd);
                if (Helix.class.isInstance(sd)) {
                    if (!helicesAlreadyOutputted.contains(sd)) {
                        pw.println("h5( minlen="+params.getParameters("minlength").getParameter(0).getValue()+", maxlen="+params.getParameters("maxlength").getParameter(0).getValue()+")");
                        helicesAlreadyOutputted.add(sd);
                    }
                    else
                        pw.println("h3");
                }
                else if (SingleStrand.class.isInstance(sd)) {
                    pw.println("ss( minlen="+params.getParameters("minlength").getParameter(0).getValue()+", maxlen="+params.getParameters("maxlength").getParameter(0).getValue()+")");
                }
            }
            pw.close();
            for (File fastaFile:new File(Paradise.getDatabasesDirectory(),databaseSelected).listFiles(new FileFilter() {
                public boolean accept(File file) {
                    return file.getName().endsWith(".fasta");
                }}))
            {
                /*
                OneShotApplicationDriver rnamotif = new OneShotApplicationDriver(new StringBuffer(ParadiseConfig.getAlgorithmsPath()).append("").append(System.getProperty("file.separator")).append("src").append(System.getProperty("file.separator")).append("rnamotif").toString());
                File rnamotifOutputFile = IOUtils.createTemporaryFile("rnamotif");
                rnamotif.run(new String[]{"-descr", descrFile.getAbsolutePath(), fastaFile.getAbsolutePath()},null, rnamotifOutputFile);

                //the hits list (if any) is transformed as a CT file (with rm2ct)
                OneShotApplicationDriver rm2ct = new OneShotApplicationDriver(new StringBuffer(ParadiseConfig.getAlgorithmsPath()).append("").append(System.getProperty("file.separator")).append("src").append(System.getProperty("file.separator")).append("rm2ct").toString());
                File rm2ctOutputFile = IOUtils.createTemporaryFile("rm2ct");
                rm2ct.run(new String[]{rnamotifOutputFile.getAbsolutePath()},null, rm2ctOutputFile);
                //now we need to split the single ct file into several ones (one per hit)
                BufferedReader buff = new BufferedReader(new FileReader(rm2ctOutputFile));
                int i=0;
                List<File> ctFiles = new ArrayList<File>();
                PrintWriter ctFile = null;
                CTFileIO parser = new CTFileIO(moleculeFactory);
                DefaultProgressMonitor monitor = new DefaultProgressMonitor();
                molecules.clear();
                String line = null;
                int counter =0;
                while ((line = buff.readLine())!= null) {
                    if (counter == 0) {
                        counter = Integer.parseInt(line.trim().split(" ")[0]);
                        if (ctFile != null) {
                            ctFile.close();
                            File f = ctFiles.get(ctFiles.size()-1);
                            parser.parseFile(f,monitor);
                            SecondaryStructure _ss = parser.getSecondaryStructures().get(0);
                            Molecule _m = _ss.getMolecules().iterator().next();
                            _m.addSelectedFeature(_ss);
                            molecules.add(_m);
                            ctFiles.remove(f);
                            f.delete();
                        }
                        File f =IOUtils.createTemporaryFile("ctCandidate");
                        ctFiles.add(f);
                        ctFile = new PrintWriter(f);
                        ctFile.println(line);
                    }
                    else {
                        ctFile.println(line);
                        counter--;
                    }
                }
                if (ctFile != null) {
                    ctFile.close();
                    File f = ctFiles.get(ctFiles.size()-1);
                    parser.parseFile(f,monitor);
                    SecondaryStructure _ss = parser.getSecondaryStructures().get(0);
                    Molecule _m = _ss.getMolecules().iterator().next();
                    _m.addSelectedFeature(_ss);
                    molecules.add(_m);
                    ctFiles.remove(f);
                    f.delete();
                }*/
            }
        }
        catch (AnalysisException e) {
            throw e;
        }
        catch (Exception e) {
            throw new AnalysisException(e);
        }
    }

}
