package fr.unistra.ibmc.paradise.services.community;

import fr.unistra.ibmc.paradise.core.analysis.Parameter;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import jade.core.behaviours.CyclicBehaviour;
import jade.domain.FIPAAgentManagement.Property;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import org.apache.log4j.Priority;

import java.io.IOException;
import java.util.Iterator;


/**
 * When a ParadiseCommunityAgent is contacted to answer to a request :
 * LOGIN / UNLOG / GET_CONNECTED_USERS / REGISTER that are implemented in parameters of the message
 * 
 * User: fpozzobon
 * Date: 15/07/2009
 */
public class RequestCommunityBehaviour extends CyclicBehaviour {

	private static final long serialVersionUID = 7619612493793044064L;

	public static String USER_NAME="User_name",
	PASSWORD="Mdp",
	ANONYMOUS="Anonymous";
	
	public static String REQUEST_TYPE="Request";

	/**
	 * The action of the behaviour if the message perform with REQUEST
	 */
    public void action() {
        final ACLMessage message = myAgent.receive(MessageTemplate.MatchPerformative(ACLMessage.REQUEST));
        if (message != null) {
            try {
                    ((ParadiseCommunityAgent) myAgent).writeLogMessage("Received login request from " + message.getSender(), Priority.INFO, message.getConversationId(), myAgent.getName(), null);

                    // Getting the parameters
                    final Parameters parameters = new Parameters();
                    Iterator properties = message.getEnvelope().getAllProperties();
                    while (properties.hasNext()) {
                        Property prop = (Property) properties.next();
                        if (prop.getName().equals("parameters")) {
                            for (Parameter p : ((Parameters) prop.getValue()).getParameters()) {
                                parameters.addParameter(p);
                            }
                            break;
                        }
                    }
                    
                    if (parameters == null || parameters.size()<1) {
                    	
                        Exception e = new Exception("Invalid request");
                        ((ParadiseCommunityAgent) myAgent).writeLogMessage(e.getMessage(), Priority.ERROR, message.getConversationId(), myAgent.getName(), e);
                        ACLMessage reply = message.createReply();
                        reply.setContentObject(e);
                        reply.setPerformative(ACLMessage.FAILURE);
                        ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
                        myAgent.send(reply);
                        
                    }
                    else
                    {
                        ACLMessage reply = message.createReply();

                        String request = parameters.getParameters(RequestCommunityBehaviour.REQUEST_TYPE).getParameter(0).getValue();
                        
                        if(request.equals(ParadiseCommunityAgent.LOGIN))
                        {
                        	this.login(parameters, reply, message);
                        }
                        else if (request.equals(ParadiseCommunityAgent.GET_CONNECTED_USERS))
                        {
                        	this.getConnectedUsers(parameters, reply, message);
                        }
                        else if (request.equals(ParadiseCommunityAgent.UNLOG))
                        {
                        	this.unLog(parameters, reply, message);
                        }
                        else if (request.equals(ParadiseCommunityAgent.REGISTER))
                        {
                        	this.register(parameters, reply, message);
                        }
                        else
                        {
                        	//unrecognized ask
                            Exception e = new Exception("Unrecognized ask");
                            reply.setContentObject(e);
                            reply.setPerformative(ACLMessage.FAILURE);
                            ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
                            myAgent.send(reply);
                        }
                        
                    }

            }
            catch (IOException ioe) {
                ioe.printStackTrace();
            }
        } else {
            block();
        }
    }
    
    /**
     * Called when the message is a login message
     * Used in order to try a login
     * @param parameters
     * @param reply
     * @param message
     * @throws IOException
     */
    protected void login(Parameters parameters, ACLMessage reply, ACLMessage message) throws IOException
    {
        
        Parameters tmpUsers = parameters.getParameters(RequestCommunityBehaviour.USER_NAME);
        String nameUser= tmpUsers.getParameter(0).getValue();
        Parameters tmpPass = parameters.getParameters(RequestCommunityBehaviour.PASSWORD);
        String passUser= tmpPass.getParameter(0).getValue();
        Parameters tmpAnonymous = parameters.getParameters(RequestCommunityBehaviour.ANONYMOUS);
        String anonymous= tmpAnonymous.getParameter(0).getValue();
        
        // we verify if the userName is not already connected
        if (((ParadiseCommunityAgent) myAgent).isNameConnected(nameUser))
        {
            String e = "User already connected";
            reply.setContentObject(e);
            reply.setPerformative(ACLMessage.FAILURE);
            ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
            myAgent.send(reply);
        	
        }
        else
        {
        	
        	//we verify if the user try to login in anonymous but a the name is registered
        	if(((ParadiseCommunityAgent) myAgent).isNameRegistered(nameUser) && anonymous.equals("true"))
        	{
                String e = "User registered, you can't use this name in anonymous.";
                reply.setContentObject(e);
                reply.setPerformative(ACLMessage.FAILURE);
                ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
                myAgent.send(reply);
        		
        	}
        	else
        	{

    	        if(((ParadiseCommunityAgent) myAgent).logIn(nameUser, passUser, anonymous, message.getSender()))
    	        {
    	            reply.setPerformative(ACLMessage.INFORM);
    	            String response = "Login successfull";
    	            reply.setContentObject(response);
    	            ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
    	            myAgent.send(reply);
    	        }
    	        else
    	        {
    	            String e = "Invalid login";
    	            reply.setContentObject(e);
    	            reply.setPerformative(ACLMessage.FAILURE);
    	            ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
    	            myAgent.send(reply);
    	        }
        	}
        }
    }
    
    
    /**
     * Send the list of connected users requested by the client
     * @param parameters
     * @param reply
     * @param message
     * @throws IOException
     */
    protected void getConnectedUsers(Parameters parameters, ACLMessage reply, ACLMessage message) throws IOException
    {

        reply.setPerformative(ACLMessage.PROPOSE);
        ContactList users = new ContactList(((ParadiseCommunityAgent) myAgent).getConnectedUsers());
        reply.setContentObject(users);
        ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
        myAgent.send(reply);
    }
    
    /**
     * Unlog an user that requested this action
     * @param parameters
     * @param reply
     * @param message
     * @throws IOException
     */
    protected void unLog(Parameters parameters, ACLMessage reply, ACLMessage message) throws IOException
    {
    	((ParadiseCommunityAgent) myAgent).unLog(message.getSender());
    	reply.setPerformative(ACLMessage.INFORM);
    	String response = "You are now unlogged";
    	reply.setContentObject(response);
    	((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
    	myAgent.send(reply);
    }
    
    /**
     * Write in the "database" (the xml file) a new user if the name is not already registered
     * @param parameters
     * @param reply
     * @param message
     * @throws IOException
     */
    protected void register(Parameters parameters, ACLMessage reply, ACLMessage message) throws IOException
    {

    	if(parameters.size()<3)
    	{
            Exception e = new Exception("Invalid request");
            ((ParadiseCommunityAgent) myAgent).writeLogMessage(e.getMessage(), Priority.ERROR, message.getConversationId(), myAgent.getName(), e);
            reply = message.createReply();
            reply.setContentObject(e);
            reply.setPerformative(ACLMessage.FAILURE);
            ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
            myAgent.send(reply);
    	}
    	else
    	{
    		Parameters tmpUsers = parameters.getParameters(RequestCommunityBehaviour.USER_NAME);
            String nameUser= tmpUsers.getParameter(0).getValue();
            Parameters tmpPass = parameters.getParameters(RequestCommunityBehaviour.PASSWORD);
            String passUser= tmpPass.getParameter(0).getValue();
            
            if(((ParadiseCommunityAgent) myAgent).writeUser(nameUser, passUser))
            {
                reply.setPerformative(ACLMessage.INFORM);
                String response = "Successful register";
                reply.setContentObject(response);
                ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
                myAgent.send(reply);
            }
            else
            {
                String e = "User name already registered";
                reply.setContentObject(e);
                reply.setPerformative(ACLMessage.FAILURE);
                ((ParadiseCommunityAgent) myAgent).closeConversation(message.getSender(),message.getConversationId());
                myAgent.send(reply);
            }
    	}
    }
        
}
