package fr.unistra.ibmc.paradise.services.community.tools;

import org.w3c.dom.*;
import org.apache.xerces.parsers.DOMParser;
import org.xml.sax.InputSource;

import java.io.*;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

/**
 *
 * This class is used for xml documents in order to use it easily
 * methods are really closed with the ParadiseCommunityAgent
 * @author F. Pozzobon
 *
 */
public class XMLTools {

    /**
     * Read an XML file
     * @param url
     * @return Document : the XML file
     */
    public static Document readXMLFile(String url)
    {
        try{
            DOMParser parser = new DOMParser();
            parser.parse(new InputSource(url));
            Document doc = parser.getDocument();
            return doc;
        }
        catch (Exception e){
            e.getMessage();
            System.err.println(e.getMessage());
        }
        return null;
    }

    /**
     * Get the text between the nodeName
     * @param node
     * @param nodeName
     * @return
     */
    public static String getText(Node node,String nodeName)
    {

        NodeList userNL = ((Element)node).getElementsByTagName(nodeName);

        Element element = ((Element)userNL.item(0));

        return element.getFirstChild().getNodeValue();
    }

    /**
     * Set the text in the node
     * @param doc
     * @param node
     * @param texte
     */
    public static void setText(Document doc, Node node,String texte)
    {
        node.appendChild(doc.createTextNode(texte));
    }

    /**
     * Update the XML file (delete and recreate the content)
     * @param path
     * @param doc
     * @return
     */
    public static boolean updateXMLFile(String path, Document doc)
    {
        Transformer transformer;
        try {
            transformer = TransformerFactory.newInstance().newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");

            //initialize StreamResult with File object to save to file
            StreamResult result = new StreamResult(new StringWriter());
            DOMSource source = new DOMSource(doc);

            transformer.transform(source, result);
            String xmlString = result.getWriter().toString();

            return createXMLDocument(path,xmlString);
        } catch (Exception e1) {
            e1.printStackTrace();
            return false;
        }

    }

    /**
     * Create an XML file if it doesn't exist
     * @param path
     * @param contain
     * @return
     */
    public static boolean createXMLDocument(String path, String contain)
    {

        try {
            File f=new File(path);

            if(!f.exists())
                f.createNewFile();

            try {
                PrintWriter out = new PrintWriter(new FileOutputStream(path));
                out.println(contain);

                out.close();
                return true;
            }
            catch(FileNotFoundException e) {
                System.err.println("File not found");
                return false;
            }
        } catch (IOException e) {
            System.err.println("io exception");
            return false;
        }
    }

    /**
     * add a child to a node with a text
     * @param doc
     * @param father
     * @param tag
     * @param text
     * @return
     */
    public static Node addChild(Document doc, Node father, String tag, String text)
    {

        Node child = doc.createElement(tag);
        setText(doc,child,text);
        father.appendChild(child);

        return child;
    }

    /**
     * Get a node that have name of attribute
     * @param userDoc
     * @param tagName
     * @param attribute
     * @return
     */
    public static Node getNodeFromAttribute(Document userDoc, String tagName, String attribute)
    {
        if(userDoc != null)
        {
            NodeList list = userDoc.getElementsByTagName(tagName);
            if(list.getLength() == 0){

            }
            else
            {
                for(int i = 0; i < list.getLength();++i)
                {
                    String name = list.item(i).getFirstChild().getNodeValue();;
                    if(name.equals(attribute))
                    {
                        return list.item(i);
                    }

                }
            }

        }
        return null;
    }


}
