package fr.unistra.ibmc.paradise.tools;

import fr.unistra.ibmc.paradise.core.features.*;
import org.jdesktop.application.Application;
import org.jdesktop.application.ApplicationContext;
import org.jdesktop.application.ResourceMap;

import javax.swing.*;
import java.awt.*;
import java.util.Collection;


public abstract class AbstractParadiseGUITool extends Application implements ParadiseTool, ParadiseFeatureFactoryFactory {
    protected Window window;
    protected String description;
    protected ParadiseToolAgent agent;
    protected ParadiseWorkingSession workingSession;

    protected AbstractParadiseGUITool(String description, ParadiseWorkingSession workingSession) {
        this.description = description;
        this.workingSession = workingSession;
    }

    public void launch() {
        Runnable doCreateAndShowGUI = new Runnable() {
            public void run() {
                try {
                    /* Initialize the ApplicationContext application properties
                    */
                    ApplicationContext ctx = AbstractParadiseGUITool.this.getContext();
                    ctx.setApplicationClass(AbstractParadiseGUITool.class);
                    ctx.setApplication(AbstractParadiseGUITool.this);

                    /* Load the application resource map, notably the
          * Application.* properties.
          */
                    ResourceMap appResourceMap = ctx.getResourceMap();

                    appResourceMap.putResource("platform", platform());

                    /* Initialize the UIManager lookAndFeel property with the
          * Application.lookAndFeel resource.  If the the resource
          * isn't defined we default to "system".
          */
                    /*
                    String key = "Application.lookAndFeel";
                    String lnfResource = appResourceMap.getString(key);
                    String lnf = (lnfResource == null) ? "system" : lnfResource;
                    if (lnf.equalsIgnoreCase("system")) {
                        String name = UIManager.getSystemLookAndFeelClassName();
                        UIManager.setLookAndFeel(name);
                    }
                    else if (!lnf.equalsIgnoreCase("default")) {
                        UIManager.setLookAndFeel(lnf);
                    }*/
                    AbstractParadiseGUITool.this.initialize(new String[]{});
                    AbstractParadiseGUITool.this.startup();
                    AbstractParadiseGUITool.this.waitForReady();
                }
                catch (IllegalStateException e) {
                    //TODO [fjossinet] test why we have this exception launched
                }

            }
        };
        SwingUtilities.invokeLater(doCreateAndShowGUI);
    }

    public void setAgent(ParadiseToolAgent agent) {
        this.agent = agent;
    }

    public ParadiseToolAgent getAgent() {
        return this.agent;
    }

    public ParadiseWorkingSession getWorkingSession() {
        return workingSession;
    }

    public IdentityFeatureFactory createIdentityFeatureFactory() {
        return new IdentityFeatureFactory(this.workingSession,this.description);
    }

    public StructuralFeatureFactory createStructuralFeatureFactory() {
        return new StructuralFeatureFactory(this.workingSession,this.description);
    }

    public SequenceFeatureFactory createSequenceFeatureFactory() {
        return new SequenceFeatureFactory(this.workingSession,this.description);
    }

    public String getDescription() {
        return this.description;
    }

    public void interactionsRemoved(Collection<BaseBaseInteraction> interactions) {

    }

    public void interactionsAdded(Collection<BaseBaseInteraction> interactions) {

    }

}
