package fr.unistra.ibmc.paradise.tools;

import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.analysis.NeedMolecules;
import fr.unistra.ibmc.paradise.core.analysis.Parameter;
import fr.unistra.ibmc.paradise.core.analysis.PreRequisite;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.Envelope;
import jade.domain.FIPAAgentManagement.Property;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.domain.FIPAException;
import jade.gui.GuiAgent;
import jade.imtp.rmi.ServiceManagerRMI;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;
import jade.wrapper.AgentController;
import jade.wrapper.StaleProxyException;

import javax.swing.*;
import java.io.IOException;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Arrays;


public abstract class AbstractParadiseToolAgent extends GuiAgent implements ParadiseToolAgent {

    protected ParadiseTool paradiseTool;

    protected AbstractParadiseToolAgent(ParadiseTool tool) throws StaleProxyException {
        this.paradiseTool = tool;
        this.paradiseTool.setAgent(this);
        this.connectToPlatform();
    }

    public void connectToPlatform() throws StaleProxyException {
        AgentController controller = this.paradiseTool.getWorkingSession().getAgentContainer().acceptNewAgent(this.paradiseTool.getDescription()+"_"+System.nanoTime()+"[from]" + this.paradiseTool.getWorkingSession().getAddress(), this);
        controller.start();
    }

    public DFAgentDescription findAgent(String category, String name) {
        DFAgentDescription[] hits = new DFAgentDescription[]{};
        DFAgentDescription template = new DFAgentDescription();
        ServiceDescription templateSd = new ServiceDescription();
        if (category.length() != 0)
            templateSd.setType(category);
        if (name.length() != 0)
            templateSd.setName(name);
        template.addServices(templateSd);
        try {
            hits = DFService.search(this, template);
        }
        catch (FIPAException e) {
            JOptionPane.showMessageDialog(null, e.getMessage(), "Warning", JOptionPane.WARNING_MESSAGE);
        }
        //in principle only one hit
        if (hits.length > 0)
            return hits[0];
        else return null;
    }

    public DFAgentDescription[] findAgents(String category) {
        DFAgentDescription[] hits = new DFAgentDescription[]{};
        DFAgentDescription template = new DFAgentDescription();
        ServiceDescription templateSd = new ServiceDescription();
        if (category.length() != 0)
            templateSd.setType(category);
        template.addServices(templateSd);
        try {
            hits = DFService.search(this, template);
        }
        catch (FIPAException e) {
            JOptionPane.showMessageDialog(null, e.getMessage(), "Warning", JOptionPane.WARNING_MESSAGE);
        }
        //hits sorted by name
        Arrays.sort(hits,new Comparator() {
            public int compare(Object o, Object o1) {
                return ((DFAgentDescription)o).getName().compareTo(((DFAgentDescription)o1).getName());
            }
        });
        return hits;
    }

    protected void setup() {
        this.paradiseTool.launch();
    }

    public ParadiseTool getParadiseTool() {
        return this.paradiseTool;
    }

    protected abstract class QueryBehaviour extends OneShotBehaviour {
        protected String conversationId;

        protected QueryBehaviour() {
            this.conversationId = System.currentTimeMillis() + "" + Math.random();
        }

        protected QueryBehaviour(String conversationId) {
            this.conversationId = conversationId;
        }

        public String getConversationId() {
            return this.conversationId;
        }
    }

    protected class ListAgentsBehaviour extends QueryBehaviour {

        private String serviceCategory, agentName;
        private ArrayList<Molecule> selectedMolecules;
        private Parameters selectedParameters;
        private AnswerBehaviour finalAnswerBehaviour;
        private fr.unistra.ibmc.paradise.tools.component.ProgressMonitor monitor;

        /**
         * USe this constructor if you don't know which agent to use for a given category
         * @param category
         * @param answerBehaviour
         * @param monitor
         */
        public ListAgentsBehaviour(String category, AnswerBehaviour answerBehaviour, fr.unistra.ibmc.paradise.tools.component.ProgressMonitor monitor) {
            this.serviceCategory = category;
            this.finalAnswerBehaviour = answerBehaviour;
            this.selectedMolecules = answerBehaviour.getSelectedMolecules();
            this.selectedParameters = answerBehaviour.getSelectedParameters();
            this.conversationId = answerBehaviour.getConversationId();
            this.monitor = monitor;
        }

        /**
         * Use this constructor if you know exactly which agent to target
         * @param answerBehaviour
         * @param agentName
         * @param monitor
         */
        public ListAgentsBehaviour(String agentName, String category, AnswerBehaviour answerBehaviour,  fr.unistra.ibmc.paradise.tools.component.ProgressMonitor monitor) {
            this(category,answerBehaviour,monitor);
            this.agentName = agentName;
        }

        public void action() {
            if (AbstractParadiseToolAgent.this.isPlatformAvailable()) {
                monitor.startAnimation();
                if (this.agentName == null) {
                    monitor.printMessage("Check services available");
                    DFAgentDescription[] descriptions = AbstractParadiseToolAgent.this.findAgents(this.serviceCategory);
                    //show a JDialog with the list of agents available if more than one agent
                    if (descriptions.length > 1) {
                        Object[] agentNames = new Object[descriptions.length];
                        for (int i=0; i<descriptions.length;i++)
                            agentNames[i]=descriptions[i].getName().getLocalName().split("\\[at\\]")[0];
                        this.monitor.stopAnimation();
                        this.agentName = (String)JOptionPane.showInputDialog(null, "Please choose a service" , this.serviceCategory,JOptionPane.PLAIN_MESSAGE,null,agentNames,agentNames[0]);
                    }
                    else if (descriptions.length == 0) {
                        JOptionPane.showMessageDialog(null, "No service found for the category: "+this.serviceCategory);
                        this.monitor.stopAnimation();
                    }
                    else
                        this.agentName = descriptions[0].getName().getLocalName().split("\\[at\\]")[0];
                }
                if (this.agentName != null && this.agentName.length() > 0) {
                    this.monitor.startAnimation();
                    AbstractParadiseToolAgent.this.addBehaviour(new SubscribeBehaviour(this.conversationId, this.serviceCategory, this.agentName, this.selectedMolecules, this.selectedParameters, this.finalAnswerBehaviour, monitor));
                }
                else
                    this.monitor.stopAnimation();
            }
        }
    }

    protected class SubscribeBehaviour extends QueryBehaviour {
        protected String serviceCategory, serviceName;
        protected AnswerBehaviour finalAnswerBehaviour;
        protected ArrayList<Molecule> selectedMolecules;
        protected Parameters selectedParameters;
        protected fr.unistra.ibmc.paradise.tools.component.ProgressMonitor monitor;

        /**
         * This constructor is used if a conversation starts with directly with a subscription to a defined service
         * @param serviceCategory
         * @param serviceName
         * @param selectedMolecules
         * @param selectedParameters
         */
        public SubscribeBehaviour(String serviceCategory, String serviceName, ArrayList<Molecule> selectedMolecules, Parameters selectedParameters) {
            this.serviceCategory = serviceCategory;
            this.serviceName = serviceName;
            this.selectedMolecules = selectedMolecules;
            this.selectedParameters = selectedParameters;
        }


        /**
         * This constructor is used if the conversation has started with a ListAgentsBehaviour
         * @param conversationId
         * @param serviceCategory
         * @param serviceName
         * @param selectedMolecules
         * @param selectedParameters
         * @param finalAnswerBehaviour
         * @param monitor
         */
        public SubscribeBehaviour(String conversationId, String serviceCategory, String serviceName, ArrayList<Molecule> selectedMolecules, Parameters selectedParameters, AnswerBehaviour finalAnswerBehaviour, fr.unistra.ibmc.paradise.tools.component.ProgressMonitor monitor) {
            super(conversationId);
            this.serviceCategory = serviceCategory;
            this.serviceName = serviceName;
            this.selectedMolecules = selectedMolecules;
            this.selectedParameters = selectedParameters;
            this.finalAnswerBehaviour = finalAnswerBehaviour;
            this.monitor = monitor;
        }

        public void action() {
            if (AbstractParadiseToolAgent.this.isPlatformAvailable()) {
                DFAgentDescription description = AbstractParadiseToolAgent.this.findAgent(this.serviceCategory, this.serviceName);
                if (description != null) {
                    ACLMessage query = new ACLMessage(ACLMessage.SUBSCRIBE);
                    query.addReceiver(description.getName());
                    query.setConversationId("" + this.conversationId);
                    monitor.printMessage("Waiting for " + this.serviceName + " Service subscribe answer");
                    this.finalAnswerBehaviour.setProgressMonitor(monitor);
                    this.finalAnswerBehaviour.setServiceName(this.serviceName);
                    this.finalAnswerBehaviour.setServiceType(this.serviceCategory);
                    AbstractParadiseToolAgent.this.addBehaviour(new SubscribeAnswerBehaviour(this.conversationId, this.serviceCategory, this.serviceName, this.finalAnswerBehaviour, selectedMolecules, selectedParameters, monitor));
                    AbstractParadiseToolAgent.this.send(query);
                } else {
                    monitor.printMessage("Cannot find the service named " + this.serviceName + " on the PARADISE platform");
                    monitor.stopAnimation();
                }
            }
        }

        public void setFinalAnswerBehaviour(AnswerBehaviour finalAnswerBehaviour) {
            this.finalAnswerBehaviour = finalAnswerBehaviour;
        }

    }

    protected class SubscribeAnswerBehaviour extends AnswerBehaviour {

        protected AnswerBehaviour finalAnswerBehaviour;

        private SubscribeAnswerBehaviour(String conversationId, String serviceType, String serviceName, AnswerBehaviour finalAnswerBehaviour, ArrayList<Molecule> selectedMolecules, Parameters selectedParameters, fr.unistra.ibmc.paradise.tools.component.ProgressMonitor monitor) {
            super(conversationId, selectedMolecules, selectedParameters);
            this.monitor = monitor;
            this.serviceName = serviceName;
            this.serviceType = serviceType;
            this.finalAnswerBehaviour = finalAnswerBehaviour;
        }

        public void action() {
            ACLMessage message = myAgent.receive(MessageTemplate.MatchConversationId("" + this.conversationId));
            if (message != null && message.getPerformative() == ACLMessage.CONFIRM) {
                //this confirm message can contain some prerequisites
                //if so, construction of a JDialog
                try {
                    ArrayList<PreRequisite> prerequisites = (ArrayList<PreRequisite>) message.getContentObject();
                    if (prerequisites.size() != 0) {
                        for (PreRequisite prerequisite:prerequisites) {
                            if (NeedMolecules.class.isInstance(prerequisite)) {
                                java.util.List<Molecule> molecules = paradiseTool.getWorkingSession().getMolecules();
                                Collections.sort(molecules, new Comparator() {
                                    public int compare(Object o1, Object o2) {
                                        return o1.toString().compareTo(o2.toString());
                                    }
                                });
                                for (Molecule m:selectedMolecules)
                                    molecules.remove(m);
                                if (molecules.size() == 0)
                                    monitor.printMessage("This service needs another molecule but no molecules are available");
                                else {
                                    Molecule m = (Molecule)JOptionPane.showInputDialog(null, "This service needs another molecule" , "Load a molecule",JOptionPane.PLAIN_MESSAGE,null,molecules.toArray(new Molecule[]{}),molecules.get(0));
                                    if (m != null)
                                        selectedMolecules.add(m);
                                    else
                                        monitor.printMessage("Analysis cancelled");
                                }
                            }
                        }
                    }
                } catch (UnreadableException e) {
                    monitor.printException(e);
                    monitor.stopAnimation();
                    this.done = true;
                    return;
                }
                ACLMessage reply = message.createReply();
                reply.setPerformative(ACLMessage.REQUEST);
                try {
                    reply.setContentObject(selectedMolecules);
                    if (selectedParameters != null) {
                        Envelope envelope = new Envelope();
                        envelope.addProperties(new Property("parameters", selectedParameters));
                        reply.setEnvelope(envelope);
                    }
                }
                catch (IOException e) {
                    monitor.printException(e);
                    monitor.stopAnimation();
                    this.done = true;
                    return;
                }
                monitor.printMessage("Waiting for " + this.serviceName + " Service request answer");
                AbstractParadiseToolAgent.this.addBehaviour(this.finalAnswerBehaviour);
                AbstractParadiseToolAgent.this.send(reply);
                this.done = true;
            } else if (message != null && message.getPerformative() == ACLMessage.FAILURE) {
                try {
                    monitor.printException((Exception)message.getContentObject());
                } catch (UnreadableException e) {
                    e.printStackTrace();
                }
                monitor.stopAnimation();
                this.done = true;
            }
            else
                block();
        }
    }

    protected boolean isPlatformAvailable() {
        String url = "rmi://" + this.paradiseTool.getWorkingSession().getPlatformAddress() + ":" + 1099 + "/" + jade.core.IMTPManager.SERVICE_MANAGER_NAME;
        ServiceManagerRMI obj = null;
        boolean problem = false;
        try {
            obj = (ServiceManagerRMI) Naming.lookup(url);
            obj.getPlatformName();
        } catch (NotBoundException e1) {
            problem = true;
        } catch (MalformedURLException e1) {
            problem = true;
        } catch (RemoteException e1) {
            problem = true;
        }
        if (problem) {
            JOptionPane.showMessageDialog(null, "Platform problem, try to reconnect!!", "Platform problem", JOptionPane.WARNING_MESSAGE);
        }
        return !problem;
    }
}
