package fr.unistra.ibmc.paradise.tools.component;

import fr.unistra.ibmc.paradise.core.io.*;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.tools.ParadiseWorkingSession;

import java.io.File;
import java.util.List;
import java.util.ArrayList;
import org.jdesktop.application.Application;

public class FileParsingTask extends ParadiseTask {

        private File f;
        private ParadiseFileIO parser;
        private List<Molecule> moleculesBeforeParsing;
        private ParadiseWorkingSession session;

        public FileParsingTask(Application app, File f, ParadiseFileIO parser, ProgressMonitor monitor, ParadiseWorkingSession session) {
            super(app,monitor);
            this.f = f;
            this.parser = parser;
            this.session = session;
            this.moleculesBeforeParsing = new ArrayList<Molecule>(session.getMolecules());
        }

        @Override
        protected Void doInBackground() {
            try {
                this.setMessage("Parsing of " + f.getName());
                monitor.startAnimation();
                long start = System.currentTimeMillis();
                this.parser.parseFile(f, monitor);
                this.setMessage("End of Parsing. Parsing time: "+((System.currentTimeMillis()-start)/1000)+"s");
            }
            catch (InterruptedException e) {
                //some cleaning
                MOLECULES:
                for (Molecule m : this.session.getMolecules()) {
                    for (Molecule _m : this.moleculesBeforeParsing)
                        if (_m.getParadiseID().equals(m.getParadiseID()))
                            continue MOLECULES;
                    this.session.removeMolecule(m);
                }
                return null;
            }
            catch (FileParsingException e) {
                monitor.printException(e);
                //some cleaning
                for (Molecule m : this.parser.getMolecules())
                    this.session.removeMolecule(m);
                return null;
            }
            return null;
        }

    }

