package fr.unistra.ibmc.paradise.utils;


import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public class CTFile {
    /**
     * le nombre de bases
     */
    int nbBases;

    /**
     * l'energie libre minimum
     */
    float energy;

    /**
     * le nom de la sequence (le nom du fichier, si aucun nom n'est pr�cis�)
     */
    String name;
    /**
     * le nom du fichier .ct
     */
    String fnamebase;
    /**
     * la liste des bases du fichier
     */
    CTAtom[] lesBases;

    /**
     * Cr�e une repres�ntation d'un fichier .ct
     *
     * @param fileName String : le nom du fichier .ct
     */
    public CTFile(String fileName) throws ParsingException {
        fnamebase = fileName.substring(0, fileName.lastIndexOf("."));

        String ligne;
        try {
            BufferedReader in = new BufferedReader(new FileReader(fileName));
            ligne = in.readLine();
            if (ligne == null) {
                try {
                    throw new Exception("File is empty");
                }
                catch (Exception e) {
                    e.printStackTrace();
                }
            }
            StringTokenizer tok = new StringTokenizer(ligne, " \t");
            if (tok.countTokens() == 5) {
                nbBases = Parser.parseInt(tok.nextToken());
                tok.nextToken();
                tok.nextToken();
                energy = Parser.parseFloat(tok.nextToken());
                name = tok.nextToken();
            } else {
                nbBases = Parser.parseInt(tok.nextToken());
                energy = (float) 0;
                name = fileName;
            }

            lesBases = new CTAtom[nbBases + 1];

            for (int i = 1; i <= nbBases; i++) {
                ligne = in.readLine();
                if (ligne == null) {
                    try {
                        throw new Exception("Missing base(s)");
                    }
                    catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                tok = new StringTokenizer(ligne, " \t");
                if (tok.countTokens() < 6) {
                    try {
                        throw new Exception("File not in standard CT format");
                    }
                    catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                lesBases[i] = new CTAtom(Parser.parseInt(tok.nextToken()), tok.nextToken().charAt(0), Parser.parseInt(tok.nextToken()), Parser.parseInt(tok.nextToken()), Parser.parseInt(tok.nextToken()), Parser.parseInt(tok.nextToken()));
            }
            in.close();
        }
        catch (IOException ieo) {
            System.err.println(ieo.getMessage());
        }
        computePseudoKnot();
    }

    public CTFile() {

    }


    public CTFile(String sequence, String bracket) {
        nbBases = sequence.length();
        lesBases = new CTAtom[nbBases + 1];

        for (int i = 0; i < nbBases; i++) {
            lesBases[i + 1] = new CTAtom(i + 1, sequence.charAt(i), i, i + 2, 0, i + 1);
        }
        for (int i = 0; i < nbBases; i++) {
            if (bracket.charAt(i) == '(') {
                int match = searchClosingBracket(bracket, i, '(');
                lesBases[i + 1].pair = match + 1;
                lesBases[match + 1].pair = i + 1;
            }
            if (bracket.charAt(i) < 124 && bracket.charAt(i) > 96) {
                int match = searchClosingBracket(bracket, i, bracket.charAt(i));
                lesBases[i + 1].pseudo = match + 1;
                lesBases[match + 1].pseudo = i + 1;
                lesBases[i + 1].psgroup = bracket.charAt(i) - 96;
                lesBases[match + 1].psgroup = bracket.charAt(i) - 96;
            }
        }
        if (nbBases > 0) {
            lesBases[1].prev = 0;
            lesBases[nbBases].next = 0;
        }
    }

    private int searchClosingBracket(String s, int p, char c) {
        int ret = 0;
        int level = 0;

        if (c == '(') {
            for (int i = p; i < s.length(); i++) {
                if (s.charAt(i) == '(') {
                    level++;
                }
                if (s.charAt(i) == ')') {
                    level--;
                }
                if (level == 0) {
                    return i;
                }
            }
        }

        if (c <= 'z' && c >= 'a') {
            for (int i = p; i < s.length(); i++) {
                if (s.charAt(i) == c) {
                    level++;
                }
                if (s.charAt(i) == Character.toUpperCase(c)) {
                    level--;
                }
                if (level == 0) {
                    return i;
                }
            }
        }

        return ret;
    }

    /**
     * Crée un fichier .ct a partir d'un fichier .fakect (o� chaque base est un 'A')
     *
     * @param fileName String : le nom du fichier .fakect
     */
    static void ctFromfakect(String fileName) throws ParsingException {
        CTFile t = new CTFile();
        String ligne;
        try {
            BufferedReader in = new BufferedReader(new FileReader(fileName));
            ligne = in.readLine();
            if (ligne == null) {
                try {
                    throw new Exception("File is empty");
                }
                catch (Exception e) {
                    e.printStackTrace();
                }
            }
            StringTokenizer tok = new StringTokenizer(ligne, " \t");
            t.nbBases = Parser.parseInt(tok.nextToken());
            t.energy = (float) 0;
            t.name = fileName;
            t.lesBases = new CTAtom[t.nbBases + 1];

            for (int i = 1; i <= t.nbBases; i++) {
                t.lesBases[i] = new CTAtom(i, 'A', i - 1, i, 0, i);
            }
            t.lesBases[t.nbBases].next = 0;
            while ((ligne = in.readLine()) != null) {
                tok = new StringTokenizer(ligne, " \t");
                int a = Parser.parseInt(tok.nextToken());
                int b = Parser.parseInt(tok.nextToken());
                t.lesBases[a].pair = b;
                t.lesBases[b].pair = a;
            }
            in.close();
        }
        catch (IOException ieo) {
            System.err.println(ieo.getMessage());
        }
        t.saveFile(fileName.substring(0, fileName.length() - 7) + ".ct", fileName.substring(0, fileName.length() - 7));
    }

    /**
     * Met à jour structures de contr�le des pseudoknot
     */
    void computePseudoKnot() {
        int pseudo = 0;
        int psgroup = 0;
        int i, j, k, l;
        int oldHelix = -1;
        ArrayList<Integer> ps = new ArrayList<Integer>();


        for (i = 1; i <= nbBases; i++) {
            if (lesBases[i].pseudo == 0) {
                k = lesBases[i].pair;
                if (oldHelix == i + k)
                    continue;
                oldHelix = i + k;
                if (k != 0) {
                    if (k > i) {
                        for (j = i + 1; j < k; j++) {
                            if ((l = lesBases[j].pair) > k) /* en fait l!=0 && l>k */ {
                                int psid = l + lesBases[l].pair;
                                if (!ps.contains(psid))
                                    ps.add(psid);
                                psgroup = ps.size();
                                pseudo++;

                                lesBases[j].pseudo = pseudo;
                                lesBases[l].pseudo = pseudo;
                                lesBases[j].psgroup = psgroup;
                                lesBases[l].psgroup = psgroup;
                                lesBases[j].pair = 0;
                                lesBases[l].pair = 0;
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Imprime le status des liaisons (helice ou pseudonoeud)
     */
    void printStatus() {
        System.out.println("Nombre de bases : " + this.nbBases);
        System.out.println("Liste des appariement :");
        for (int i = 1; i <= this.nbBases; i++) {
            if (lesBases[i].pair > lesBases[i].id) {
                System.out.print(lesBases[i].id + " - " + lesBases[i].pair + " : ");
                if (lesBases[i].pseudo > 0) {
                    System.out.println("pseudonoeud : " + lesBases[i].pseudo + " group : " + lesBases[i].psgroup);
                } else {
                    System.out.println("helice");
                }
            }
        }
    }

    /**
     * Sauvegarde la structure dans un fichier .ct
     *
     * @param fileName String : le nom du fichier
     */
    public void saveFile(String fileName, String moleculeName) {
        try {
            PrintWriter out = new PrintWriter(new FileWriter(fileName));
            out.println(nbBases + "\tENERGY =\t" + energy + "\t" + moleculeName);
            for (int i = 1; i <= nbBases; i++) {
                if (lesBases[i].pseudo == 0)
                    out.println(lesBases[i].id + "\t" + lesBases[i].base + "\t" + lesBases[i].prev + "\t" + lesBases[i].next + "\t" + lesBases[i].pair + "\t" + lesBases[i].name);
                else
                    out.println(lesBases[i].id + "\t" + lesBases[i].base + "\t" + lesBases[i].prev + "\t" + lesBases[i].next + "\t" + lesBases[i].pseudo + "\t" + lesBases[i].name);

            }
            out.close();
        }
        catch (IOException ioe) {
            System.err.println(ioe.getMessage());
        }

    }

    /**
     * Affiche le fichier .ct sur la sortie standard
     */
    void printToScreen() {
        System.out.println(nbBases + "\tENERGY =\t" + energy + "\t" + name);
        for (int i = 1; i <= nbBases; i++) {
            System.out.println(lesBases[i].id + "\t" + lesBases[i].base + "\t" + lesBases[i].prev + "\t" + lesBases[i].next + "\t" + lesBases[i].pair + "\t" + lesBases[i].name);
        }
    }

    public String getName() {
        return this.name;
    }

    public String getSequence() {
        String sequence = "";
        for (int i = 1; i <= nbBases; i++) {
            sequence += lesBases[i].base;
        }
        return sequence.toUpperCase();
    }

    public String getBracket() throws ParsingException {
        List<String> pk = new ArrayList<String>();

        String bracket = "";
        for (int i = 1; i <= nbBases; i++) {
            int pair = lesBases[i].pair;
            int pseudo = lesBases[i].pseudo;
            int pg = lesBases[i].psgroup;

            if (pair != 0) {
                if (pair < i)
                    bracket += ')';
                else if (pair > i)
                    bracket += '(';
                else
                    throw new ParsingException("There was a problem while parsing your ct file");
            } else if (pseudo != 0) {
                String sign = pg + "-" + pseudo;

                char min = (char) ('a' + pg - 1);
                char max = (char) ('A' + pg - 1);

                if (pk.contains(sign)) {
                    bracket += max;
                    pk.remove(sign);
                } else {
                    bracket += min;
                    pk.add(sign);
                }
            } else {
                bracket += '.';
            }
        }
        return bracket;
    }

    static class CTAtom {
        /**
         * l'index de la base
         */
        int id;

        /**
         * la lettre de la base
         */
        char base;

        /**
         * l'index de la base pr�c�dante (sens 5'-3') : 0 si aucune
         */
        int prev;

        /**
         * l'index de la base suivante : 0 si aucune
         */
        int next;

        /**
         * l'index de la base appari� : 0 si aucune
         */
        int pair;

        /**
         * le num�ro original de la base dans la s�quence
         */
        int name;

        /**
         * indique si la liaison forme un pseudonoeud :
         * 0 : pas de pseudonoeud,
         * x : indice de la liaison dans le pseudonoeud
         */

        int pseudo = 0;
        /**
         * indique si la liaison forme un pseudonoeud :
         * 0 : pas de pseudo noeud,
         * x : indice du pseudonoeud
         */
        int psgroup = 0;

        public CTAtom(int cid, char cbase, int cprev, int cnext, int cpair, int cname) {
            id = cid;
            base = cbase;
            prev = cprev;
            next = cnext;
            pair = cpair;
            name = cname;
        }
    }

}

