package fr.unistra.ibmc.paradise.utils;

import javax.swing.*;
import javax.swing.Timer;
import java.awt.*;
import java.lang.reflect.Method;
import java.util.*;


public class GeneralUtils {

    static private class CommandLineProgressBar {
        private java.util.Timer timer;

        private CommandLineProgressBar() {
        }

        private void start() {
            this.timer = new java.util.Timer();
            this.timer.schedule(new ProgressionTask(), 0, 2000);
        }

        private void stop() {
            this.timer.cancel();
        }

        private class ProgressionTask extends TimerTask {
            public void run() {
                System.out.print(".");
            }
        }
    }

    /**
     * Reverse a String
     *
     * @param s
     * @return
     */
    public static String reverse(String s) {
        StringBuffer out = new StringBuffer();
        for (int i = s.length() - 1; i >= 0; i--)
            out.append(s.charAt(i));
        return out.toString();
    }

    /**
     * Center a window on the current screen
     *
     * @param window the window to center
     */
    public static void centerOnScreen(final java.awt.Window window) {
        final java.awt.Dimension win = window.getSize();
        final java.awt.Dimension screen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();

        final int newX = (screen.width - win.width) / 2;
        final int newY = (screen.height - win.height) / 2;

        window.setLocation(newX, newY);
    }

    public static void rightSideOfTargetWindow(final java.awt.Window targetWindow,final java.awt.Window newWindow, int transX, int transY) {
        Point location = targetWindow.getLocationOnScreen();
        newWindow.setLocation(location.x+(int)targetWindow.getSize().getWidth()+transX, location.y+transY);
    }

    public static void leftSideOfTargetWindow(final java.awt.Window targetWindow,final java.awt.Window newWindow, int transX, int transY) {
        Point location = targetWindow.getLocationOnScreen();
        newWindow.setLocation(location.x-(int)newWindow.getSize().getWidth()+transX, location.y+transY);
    }

    public static void aboveTargetWindow(final java.awt.Window targetWindow,final java.awt.Window newWindow, int transX, int transY) {
        Point location = targetWindow.getLocationOnScreen();
        newWindow.setLocation(location.x+transX, location.y-(int)newWindow.getSize().getHeight()+transY);
    }

    public static void belowTargetWindow(final java.awt.Window targetWindow,final java.awt.Window newWindow, int transX, int transY) {
        Point location = targetWindow.getLocationOnScreen();
        newWindow.setLocation(location.x+transX, location.y+(int)targetWindow.getSize().getHeight()+transY);
    }

    public static void openBrowser(String url) {
        String osName = System.getProperty("os.name");
        try {
            if (osName.startsWith("Mac OS")) {
                Class fileMgr = Class.forName("com.apple.eio.FileManager");
                Method openURL = fileMgr.getDeclaredMethod("openURL",
                        new Class[]{String.class});
                openURL.invoke(null, new Object[]{url});
            } else if (osName.startsWith("Windows")) {
                Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + url);
            } else {
                String[] browsers = {"firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape"};
                String browser = null;
                for (int count = 0; count < browsers.length && browser == null; count++) {
                    if (Runtime.getRuntime().exec(new String[]{"which", browsers[count]}).waitFor() == 0) {
                        browser = browsers[count];
                    }
                }
                if (browser == null) {
                    throw new Exception("Could not find web browser");
                } else {
                    Runtime.getRuntime().exec(new String[]{browser, url});
                }
            }
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Error attempting to launch web browser:\n" + e.getLocalizedMessage());
        }
    }
}
