package fr.unistra.ibmc.paradise.utils;

public class Printing {
    private static OutputMode WARNING = OutputMode.onErr;
    private static OutputMode VERBOSE = OutputMode.onOut;
    private static OutputMode DEBUG = OutputMode.onOut;
    private static OutputMode MESSAGE = OutputMode.onOut;
    private static OutputMode ENTERED = OutputMode.off;
    private static OutputMode READ = OutputMode.onOut;
    private static OutputMode WRITE = OutputMode.onOut;

    /**
     * set the output mode for warning message (onErr by default)
     *
     * @param m int - the output mode
     */
    public static void setWarningMode(OutputMode m) {
        WARNING = m;
    }

    /**
     * set the output mode for verbose message (onOut by default)
     *
     * @param m int - the output mode
     */
    public static void setVerboseMode(OutputMode m) {
        VERBOSE = m;
    }

    public static void setEnteredMode(OutputMode m) {
        ENTERED = m;
    }

    /**
     * set the output mode for debug message (onOut by default during developement, off during use)
     *
     * @param m int - the output mode
     */
    public static void setDebugMode(OutputMode m) {
        DEBUG = m;
    }

    /**
     * set the output mode for standard message (onOut by default)
     *
     * @param m int - the output mode
     */
    public static void setMessageMode(OutputMode m) {
        MESSAGE = m;
    }

    /**
     * set the output mode for "read" message (onOut by default)
     *
     * @param m OutputMode - the output mode
     */
    public static void setReadMode(OutputMode m) {
        READ = m;
    }

    /**
     * set the output mode for "write" message (onOut by default)
     *
     * @param m OutputMode - the output mode
     */
    public static void setWriteMode(OutputMode m) {
        WRITE = m;
    }

    public static OutputMode getReadMode() {
        return READ;
    }

    public static OutputMode getWriteMode() {
        return WRITE;
    }

    /**
     * outputs a given warning message
     *
     * @param s String - the message
     */
    public static void warning(String s) {
        if (WARNING == OutputMode.onOut) {
            System.out.println("WARNING : " + s);
        }
        if (WARNING == OutputMode.onErr) {
            System.err.println("WARNING : " + s);
        }
    }

    /**
     * outputs a given verbose message
     *
     * @param s String - the message
     */
    public static void verbose(String s) {
        Throwable toto = new Throwable();
        StackTraceElement t = toto.getStackTrace()[1];
        String cn = t.getClassName();
        if (cn.lastIndexOf(".") > -1) {
            cn = cn.substring(cn.lastIndexOf(".") + 1);

        }

        if (VERBOSE == OutputMode.onOut) {
            System.out.println("Verbose (" + cn + "." + t.getMethodName() + "() line :" + t.getLineNumber() + ") ---> " + s);
        }
        if (VERBOSE == OutputMode.onErr) {
            System.err.println("Verbose (" + cn + "." + t.getMethodName() + "() line :" + t.getLineNumber() + ") ---> " + s);
        }
    }


    public static void excel(double d) {
        System.out.println((d + "").replace('.', ','));
    }


    /**
     * outputs a given debug message
     *
     * @param s String - the message
     */
    public static void debug(String s) {
        Throwable toto = new Throwable();
        StackTraceElement t = toto.getStackTrace()[1];
        String cn = t.getClassName();
        if (cn.lastIndexOf(".") > -1) {
            cn = cn.substring(cn.lastIndexOf(".") + 1);

        }

        if (DEBUG == OutputMode.onOut) {
            System.out.println("Debug (" + cn + "." + t.getMethodName() + "() line :" + t.getLineNumber() + ") ---> " + s);
        }
        if (DEBUG == OutputMode.onErr) {
            System.err.println("Debug (" + cn + "." + t.getMethodName() + "() line :" + t.getLineNumber() + ") ---> " + s);
        }
    }

    /**
     * outputs a given standard message
     *
     * @param s String - the message
     */
    public static void message(String s) {
        if (MESSAGE == OutputMode.onOut) {
            System.out.println("Message : " + s);
        }
        if (MESSAGE == OutputMode.onErr) {
            System.err.println("Message : " + s);
        }
    }

    /**
     * outputs a given fatal message on standard error, and exits the program
     *
     * @param s String - the message
     */
    public static void die(String s) {
        try {
            throw (new Exception("Program stopping : " + s));
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * outputs on standard output the last line read in a file (and the the line number in source code vers the call as made)
     *
     * @param line     String - the line read
     * @param fileName String - the filename
     */
    public static void justRead(String line, String fileName) {
        Throwable toto = new Throwable();
        StackTraceElement t = toto.getStackTrace()[1];
        String cn = t.getClassName();
        if (cn.lastIndexOf(".") > -1) {
            cn = cn.substring(cn.lastIndexOf(".") + 1);
        }
        if (READ == OutputMode.onOut) {
            System.out.println("Read : " + cn + "." + t.getMethodName() + "() line :" + t.getLineNumber() + " | in " + fileName + " | " + line);
        }
        if (READ == OutputMode.onErr) {
            System.err.println("Read : " + cn + "." + t.getMethodName() + "() line :" + t.getLineNumber() + " | in " + fileName + " | " + line);
        }
    }

    public static void justWritten(String line, String fileName) {
        Throwable toto = new Throwable();
        StackTraceElement t = toto.getStackTrace()[1];
        String cn = t.getClassName();
        if (cn.lastIndexOf(".") > -1) {
            cn = cn.substring(cn.lastIndexOf(".") + 1);
        }
        if (WRITE == OutputMode.onOut) {
            System.out.println("Write : " + cn + "." + t.getMethodName() + "() line :" + t.getLineNumber() + " | in " + fileName + " | " + line);
        }
        if (WRITE == OutputMode.onErr) {
            System.err.println("Write : " + cn + "." + t.getMethodName() + "() line :" + t.getLineNumber() + " | in " + fileName + " | " + line);
        }
    }

    /**
     * Calls entered() on top of a method "toto()" to print "Entered : toto()"
     */
    public static void entered() {
        if (ENTERED == OutputMode.onOut) {
            Throwable toto = new Throwable();
            StackTraceElement t = toto.getStackTrace()[1];
            String cn = t.getClassName();
            if (cn.lastIndexOf(".") > -1) {
                cn = cn.substring(cn.lastIndexOf("."));
            }
            System.out.println("Entered : " + cn + "." + t.getMethodName());
        }
        if (ENTERED == OutputMode.onErr) {
            Throwable toto = new Throwable();
            StackTraceElement t = toto.getStackTrace()[1];
            String cn = t.getClassName();
            if (cn.lastIndexOf(".") > -1) {
                cn = cn.substring(cn.lastIndexOf("."));
            }
            System.err.println("Entered : " + cn + "." + t.getMethodName());
        }
    }

    public static void trace(String text) {
        System.out.println(text + " : \n");
        Throwable toto = new Throwable();
        for (StackTraceElement t : toto.getStackTrace())
            System.out.println(t);
    }

    private static int timer;

    public static void start() {
        timer = SystemTools.dailyEpoc();
    }

    public static void stop(String s) {
        System.out.println(s + " took " + (SystemTools.dailyEpoc() - timer) / 1000d + " seconds");
    }

}
