package fr.unistra.ibmc.paradise.utils;

import java.io.*;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class SystemTools {
    /**
     * Moves/renames a file
     *
     * @param sourceName - the source name
     * @param destName   - the destination name
     */
    public static void mv(String sourceName, String destName) {

        File d = new File(destName);
        if (!d.delete()) {
            //      Printing.warning("Could not delete file : "+d.getName());
        }
        /*    else
                {
                    System.out.println("succesfuly deleted file : "+d.getName());
                }
        */
        System.gc();

        cp(sourceName, destName);

        File s = new File(sourceName);
        if (!s.delete()) {
            Printing.warning("Could not delete file : " + s.getName());
        }
        /*   else
        {
            System.out.println("succesfuly deleted file : "+s.getName());
        }*/
    }

    public static boolean isFileEmpty(String filename) {
        boolean ret = true;
        try {
            BufferedReader in = new BufferedReader(new FileReader(filename));
            ret = (in.readLine() == null);
        }
        catch (IOException e) {
            //e.printStackTrace();
        }
        return ret;
    }

    public static String createTempDirectory(String name) {
        try {
            File tmp = File.createTempFile("paradise-" + name + "-" + SystemTools.epoc(), "");
            tmp.delete();
            tmp.mkdir();
            tmp.deleteOnExit();
            return tmp.getCanonicalPath() + System.getProperty("file.separator");
        }
        catch (IOException e) {
            System.err.println("Can't create temporary directory");
        }
        return "." + System.getProperty("file.separator");
    }

    public static void renameTo(String sourceName, String destName) {

        File d = new File(destName);
        if (!d.delete()) {
            //     Printing.warning("Could not delete file : "+d.getName());
        }
        /*      else
        {
            System.out.println("succesfuly deleted file : "+d.getName());
        }*/


        boolean success = false;

        File source = new File(sourceName);
        File dest = new File(destName);

        for (int i = 0; i < 20; i++) {
            success = source.renameTo(dest);
            if (success) {
                break;
            }
            System.gc();
            try {
                Thread.sleep(50);
            }
            catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        if (!success) {
            Printing.warning("The file " + sourceName + " could not been renamed to " + destName);
            System.exit(1);
        }
        /*  else
        {
            System.out.println("The file " + sourceName + " succesfuly renamed to " + destName);
        }*/

    }

    /**
     * Copies a file
     *
     * @param sourceName - the source name
     * @param destName   - the destination name
     */
    public static void cp(String sourceName, String destName) {
        try {
            BufferedInputStream in = new BufferedInputStream(new FileInputStream(sourceName));
            BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(destName));
            int b;
            while ((b = in.read()) != -1) {
                out.write(b);
            }
            in.close();
            out.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void cp(BufferedReader in, PrintWriter out, long length) {
        char[] buf = new char[(int) length];
        int read = 0;
        try {
            read = in.read(buf, 0, (int) length);
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        if (read != (int) length) {
            Printing.warning("READ " + read + " / " + length + " bytes");
        }
        /*    else
        {
            System.out.println("Successfully read "+read+" bytes");
        }*/
        //out.write(buf, 0, (int)length);
        out.print(buf);
        //      out.print(" ");
        out.flush();
    }

    public static void rm(String filename) {
        File tmp = new File(filename);
        tmp.delete();
    }

    public static String unifySpaces(String s) {
        String ret = s.replace('\n', ' ');
        ret = ret.replace('\t', ' ');
        ret = ret.replace('\r', ' ');
        return ret.replace('\f', ' ');
    }

    public static long epoc() {
        return Calendar.getInstance().getTimeInMillis();
    }

    public static int dailyEpoc() {
        return (int) (Calendar.getInstance().getTimeInMillis() % 86400000);
    }

    public static void deleteDirectory(File dir) {
        for (File child : dir.listFiles()) {
            if (child.isDirectory()) {
                deleteDirectory(child);
            } else {
                child.delete();
            }
        }
        dir.delete();
    }

    public static int countLines(String filename) {
        int c = 0;
        try {
            BufferedReader in = new BufferedReader(new FileReader(filename));
            while (in.readLine() != null) {
                c++;
            }
            in.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        return c;
    }

    public static void emptyDirectory(File dir) {
        File[] children = dir.listFiles();
        for (File aChildren : children) {
            if (aChildren.isDirectory()) {
                deleteDirectory(aChildren);
            } else {
                aChildren.delete();
            }
        }
    }

    public static String getIP() {
        try {
            return java.net.InetAddress.getLocalHost().getHostAddress();
        }
        catch (UnknownHostException e) {
            e.printStackTrace();
        }
        return "0.0.0.0";
    }

    public static String getDate() {
        DateFormat df = new SimpleDateFormat("dd/MM/yyyy");
        Date now = new Date();
        return df.format(now);
    }

    public static String getTime() {
        DateFormat df = new SimpleDateFormat("hh:mmm:ss");
        Date now = new Date();
        return df.format(now);
    }


}
