package fr.unistra.ibmc.paradise.utils.matrix;

/**
 * Created by IntelliJ IDEA.
 * User: tludwig
 * Date: 30 mars 2007
 * Time: 10:54:40
 * To change this template use File | Settings | File Templates.
 */
public class StackingMatrix {

    private static final float[] cC4 = {6.614f, -7.068f, 3.100f};
    private static final float[] cC3 = {6.646f, -5.859f, 4.020f};
    private static final float[] cC2 = {7.380f, -4.830f, 3.170f};
    private static final float[] cO3 = {7.297f, -6.145f, 5.250f};

    private static final float[] nC4 = {9.384f, -2.375f, 5.910f};
    private static final float[] nC3 = {8.758f, -1.340f, 6.830f};
    private static final float[] nC2 = {8.820f, -0.077f, 5.980f};
    private static final float[] nO3 = {9.461f, -1.229f, 8.061f};

    private static final Matrix current = getMatrixFor(cC2, cC3, cC4, cO3);
    private static final Matrix next = getMatrixFor(nC2, nC3, nC4, nO3);
    private static final Matrix toNext = current.solveTranspose(next).transpose();

    public static Matrix getReferenceToResidueMatrix(float[] oC2O3, float[] oC3O3, float[] oC4O3, float[] oO3) {
        Matrix residue = getMatrixFor(oC2O3, oC3O3, oC4O3, oO3);
        return current.solveTranspose(residue).transpose();
    }

    public static Matrix getResidueToReferenceMatrix(float[] oC2O3, float[] oC3O3, float[] oC4O3, float[] oO3) {
        return getReferenceToResidueMatrix(oC2O3, oC3O3, oC4O3, oO3).inverse();
    }

    public static Matrix getToNextMatrix() {
        return toNext;
    }

    public static Matrix getToPreviousMatrix() {
        return getToNextMatrix().inverse();
    }

    public static float[] applyMatrixToPoint(Matrix M, float[] point) {
        return applyMatrixToPoint(M, point[0], point[1], point[2]);
    }

    public static float[] applyMatrixToPoint(Matrix M, float x, float y, float z) {
        float[] ret = new float[3];
        ret[0] = (float) (M.get(0, 0) * x + M.get(0, 1) * y + M.get(0, 2) * z + M.get(0, 3));
        ret[1] = (float) (M.get(1, 0) * x + M.get(1, 1) * y + M.get(1, 2) * z + M.get(1, 3));
        ret[2] = (float) (M.get(2, 0) * x + M.get(2, 1) * y + M.get(2, 2) * z + M.get(2, 3));
        return ret;
    }

    public static Matrix getMatrixFor(float[] a, float[] b, float[] c, float[] d) {
        double[][] m = {{a[0], b[0], c[0], d[0]},
                {a[1], b[1], c[1], d[1]},
                {a[2], b[2], c[2], d[2]},
                {1, 1, 1, 1}};
        return new Matrix(m);
    }
}
