package fr.unistra.ibmc.s2s;

import fr.unistra.ibmc.paradise.Paradise;
import fr.unistra.ibmc.paradise.utils.GeneralUtils;
import fr.unistra.ibmc.paradise.utils.VTextIcon;
import fr.unistra.ibmc.paradise.utils.IOUtils;
import fr.unistra.ibmc.paradise.core.*;
import fr.unistra.ibmc.paradise.core.analysis.Parameter;
import fr.unistra.ibmc.paradise.core.analysis.NoSolutionException;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import fr.unistra.ibmc.paradise.core.analysis.Analysis;
import fr.unistra.ibmc.paradise.core.io.*;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.paradise.core.features.*;
import fr.unistra.ibmc.paradise.tools.ParadiseWorkingSession;
import fr.unistra.ibmc.paradise.tools.ParadiseTool;
import fr.unistra.ibmc.paradise.tools.AbstractParadiseToolAgent;
import fr.unistra.ibmc.paradise.tools.AbstractParadiseGUITool;
import fr.unistra.ibmc.paradise.tools.component.*;
import fr.unistra.ibmc.paradise.BiologicalSymbolException;
import fr.unistra.ibmc.s2s.rnalign.graphics.*;
import fr.unistra.ibmc.s2s.rnalign.features.Alignment;
import fr.unistra.ibmc.s2s.rnalign.features.BiologicalSequence;
import fr.unistra.ibmc.s2s.rnalign.features.Symbol;
import fr.unistra.ibmc.s2s.rnalign.utils.RessourcesUtils;
import fr.unistra.ibmc.s2s.rnalign.utils.IoUtils;
import fr.unistra.ibmc.s2s.ssviewer.graphics.Rna2DViewer;
import fr.unistra.ibmc.s2s.behaviours.*;
import fr.unistra.ibmc.s2s.ssviewer.features.Interaction2D;
import fr.unistra.ibmc.s2s.ssviewer.features.BaseBaseInteraction2D;
import fr.unistra.ibmc.s2s.tertiaryviewers.TertiaryViewerDriver;
import fr.unistra.ibmc.s2s.tertiaryviewers.PyMOLDriver;
import fr.unistra.ibmc.s2s.tertiaryviewers.ChimeraDriver;

import java.util.*;
import java.util.List;
import java.util.logging.Level;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.text.SimpleDateFormat;

import groovy.lang.GroovyShell;
import groovy.lang.GroovyObject;

import javax.swing.*;
import javax.swing.Timer;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.filechooser.FileView;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.imageio.ImageIO;

import org.jdesktop.swingx.*;
import org.jdesktop.swingx.decorator.Highlighter;
import org.jdesktop.swingx.decorator.AlternateRowHighlighter;
import org.jdesktop.swingx.error.ErrorInfo;
import org.jdesktop.swingworker.SwingWorker;
import org.jdesktop.application.Application;
import org.noos.xing.mydoggy.plaf.MyDoggyToolWindowManager;
import org.noos.xing.mydoggy.*;
import org.jdom.JDOMException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import jade.gui.GuiEvent;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import com.sun.syndication.io.FeedException;
import com.sun.syndication.io.SyndFeedInput;
import com.sun.syndication.io.XmlReader;
import com.sun.syndication.feed.synd.SyndFeed;
import com.sun.syndication.feed.synd.SyndEntry;


public class S2S extends AbstractParadiseGUITool {

    public static final String releaseName="S2S 2.0, Release Candidate 3";
    private Mediator mediator;
    private SequencesList sequencesList;
    private PreferencesPanel preferencesPanel;
    private Rna2DViewer rna2dViewer;
    private TertiaryViewersToolBar tertiaryViewersToolBar;
    private List<TertiaryViewerDriver> drivers;
    private GroovyShell groovyShell;
    private MessageBar messageBar;
    private PluginsMenu plugins;
    private ToolWindowManager toolWindowManager;
    private static File backup = new File(new File(S2S.getUserDirPath()),"rnalign.backup");
    private IdentityFeatureFactory identityFeatureFactory;
    public static final byte SELECT_STRUCTURAL_DOMAIN= 0, SELECT_INTERACTION = 1, SELECT_RESIDUE = 2;
    private ToolWindow ssExplorer, ssViewer;

    public static boolean updateS2S() {
        try {
            URL updates = new URL("http://ftp.bioinformatics.org/pub/s2s/updates.xml");
            URLConnection urlConnection = updates.openConnection();
            urlConnection.setUseCaches(false);
            urlConnection.connect();
            InputStream stream = urlConnection.getInputStream();
            SAXBuilder sxb = new SAXBuilder();
            Document doc = sxb.build(stream);
            List versions = doc.getRootElement().getChildren("version");
            if (!versions.isEmpty()) {
                if (S2S.releaseName.equals(((Element)versions.get(0)).getChild("name").getText())) { //S2S is up to date
                    JOptionPane.showMessageDialog(null,"S2S is up to date");
                    return false;
                }
                else {
                    new UpdateFrame(versions);
                    return true;
                }
            }
            else {
                JOptionPane.showMessageDialog(null,"S2S is up to date");
                return false;
            }
        }
        catch (Exception e) {
            ErrorInfo errorInfo = new ErrorInfo(
                    "Error",
                    "Problem during the update",
                    null,
                    null,
                    e,
                    Level.SEVERE,
                    null);
            JXErrorPane.showDialog(null,errorInfo);
        }
        return false;
    }

    public S2S(ParadiseWorkingSession workingSession) {
        super(S2S.releaseName, workingSession);
    }

    public Window getWindow() {
        return this.window;
    }

    public void activeSecondaryStructureNavigator() {
        this.ssExplorer.setActive(true);
    }

    public void activeRna2DViewer() {
        this.ssViewer.setActive(true);
    }

    public fr.unistra.ibmc.paradise.tools.component.ProgressMonitor getProgressMonitor() {
        return this.messageBar;
    }

    public void synchronize() {
        Alignment a = mediator.getAlignmentCanvas().getMainAlignment();
        if (a!= null)
            a.updateStructuralAlignment();
    }

    public MainParadiseFeature getCurrentFeature() {
        if (this.mediator == null || this.mediator.getAlignmentCanvas() == null || this.mediator.getAlignmentCanvas().getMainAlignment() == null)
            return null;
        else
            return this.mediator.getAlignmentCanvas().getMainAlignment().getStructuralAlignment();
    }

    public void printState(String state) {
        this.messageBar.printState(state);
    }

    public boolean isConnected() {
        return this.agent != null;
    }

    public void setS2SLastSavePath(File f) {
        Preferences prefs = Preferences.userRoot().node("S2S");
        prefs.put("S2SLastSavePath", f.getPath());
    }

    public String getS2SLastSavePath() {
        Preferences prefs = Preferences.userRoot().node("S2S");
        return prefs.get("S2SLastSavePath", "myFiles");
    }

    public void setS2SLastLoadPath(File f) {
        Preferences prefs = Preferences.userRoot().node("S2S");
        prefs.put("S2SLastLoadPath", f.getPath());
    }

    public String getS2SLastLoadPath() {
        Preferences prefs = Preferences.userRoot().node("S2S");
        return prefs.get("S2SLastLoadPath", "myFiles");
    }

    public static String getPluginsPath() {
        String path =  new StringBuffer(S2S.getUserDirPath()).append("plugins").append(System.getProperty("file.separator")).toString();
        File f = new File(path);
        if (!f.exists())
            f.mkdir();
        return path;
    }

    @Override
    protected void startup() {
        this.window = new JXFrame(this.getDescription());
        ((JXFrame)this.window).setDefaultCloseOperation(((JXFrame)this.window).DO_NOTHING_ON_CLOSE);

        this.window.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(final java.awt.event.WindowEvent e) {
                if (JOptionPane.showConfirmDialog(null, "Are you sure you want to exit S2S?", "Confirm exit", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                    try {
                        S2SConfig.save();
                    } catch (BackingStoreException e1) {
                        e1.printStackTrace();
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    }
                    System.exit(0);
                }
            }
        });
        this.groovyShell = new GroovyShell();
        MyDoggyToolWindowManager myDoggyToolWindowManager = new MyDoggyToolWindowManager();
        this.toolWindowManager = myDoggyToolWindowManager;
        this.toolWindowManager.getToolWindowManagerDescriptor().setPushAwayMode(PushAwayMode.HORIZONTAL);

        this.mediator = new Mediator(this);
        this.rna2dViewer = new Rna2DViewer(this.mediator,workingSession);

        ((JFrame) this.window).setTitle("S2S: Sequence TO Structure");
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        //###### the message bar ############

        this.messageBar = new MessageBar();

        this.sequencesList = new SequencesList(mediator, workingSession);
        mediator.setSequencesList(sequencesList);

        final JScrollPane sequencesListScrollPane = new JScrollPane(this.sequencesList);
        sequencesListScrollPane.getViewport().setBackground(Color.WHITE);
        sequencesListScrollPane.setPreferredSize(new Dimension(300,(int)(screenSize.getHeight()*0.8)));

        AlignmentCanvas alignmentCanvas = new AlignmentCanvas(this.mediator,workingSession,(int)screenSize.getWidth(),(int)screenSize.getHeight());
        this.rna2dViewer.addSelectionListener(alignmentCanvas);
        alignmentCanvas.addSelectionListener(this.rna2dViewer);
        final JScrollPane alignmentCanvasScrollPane = new JScrollPane(alignmentCanvas);

        this.preferencesPanel = new PreferencesPanel();
        final JScrollPane preferencesPanelScrollPane = new JScrollPane(preferencesPanel);
        preferencesPanelScrollPane.getViewport().setBackground(Color.WHITE);
        preferencesPanelScrollPane.setPreferredSize(new Dimension(300,(int)(screenSize.getHeight()*0.8)));
        final LWMatricesPanel lwMatricesPanel = new LWMatricesPanel();
        final JScrollPane lwMatricesPanelScrollPane = new JScrollPane(lwMatricesPanel);
        lwMatricesPanelScrollPane.getViewport().setBackground(Color.WHITE);
        lwMatricesPanelScrollPane.setPreferredSize(new Dimension(300,(int)(screenSize.getHeight()*0.8)));

        final ComputedAlignmentPanel computedAlignmentPanel = new ComputedAlignmentPanel();
        final JScrollPane computedAlignmentPanelScrollPane = new JScrollPane(computedAlignmentPanel);
        computedAlignmentPanelScrollPane.getViewport().setBackground(Color.WHITE);
        computedAlignmentPanelScrollPane.setPreferredSize(new Dimension(300,(int)(screenSize.getHeight()*0.8)));

        SecondaryStructureNavigator navigator = new SecondaryStructureNavigator(this.mediator);
        navigator.addSelectionListener(alignmentCanvas);
        alignmentCanvas.addSelectionListener(navigator);
        navigator.addSelectionListener(this.rna2dViewer);
        this.rna2dViewer.addSelectionListener(navigator);

        final JScrollPane navigatorScrollPane = new JScrollPane(navigator);
        navigatorScrollPane.getViewport().setBackground(Color.WHITE);

        toolWindowManager.getContentManager().addContent("Alignment", "alignment", null,alignmentCanvasScrollPane);
        this.ssExplorer = toolWindowManager.registerToolWindow("2DExplorer","2D Explorer",null, navigatorScrollPane , ToolWindowAnchor.LEFT);
        DockedTypeDescriptor descriptor = (DockedTypeDescriptor) this.ssExplorer.getTypeDescriptor(ToolWindowType.DOCKED);
        descriptor.setDockLength(200);
        descriptor.setPreviewEnabled(false);

        ToolWindow toolWindow = toolWindowManager.registerToolWindow("Sequences","Sequences",null,sequencesListScrollPane,ToolWindowAnchor.RIGHT);
        descriptor = (DockedTypeDescriptor) toolWindow.getTypeDescriptor(ToolWindowType.DOCKED);
        descriptor.setDockLength(300);
        descriptor.setPreviewEnabled(false);

        toolWindow  = toolWindowManager.registerToolWindow("Preferences","Preferences",null, preferencesPanelScrollPane, ToolWindowAnchor.RIGHT);
        descriptor = (DockedTypeDescriptor) toolWindow.getTypeDescriptor(ToolWindowType.DOCKED);
        descriptor.setDockLength(300);
        descriptor.setPreviewEnabled(false);

        toolWindow = toolWindowManager.registerToolWindow("LWMatrices","LWMatrices",null, lwMatricesPanelScrollPane, ToolWindowAnchor.RIGHT);
        descriptor = (DockedTypeDescriptor) toolWindow.getTypeDescriptor(ToolWindowType.DOCKED);
        descriptor.setDockLength(300);
        descriptor.setPreviewEnabled(false);

        this.ssViewer = toolWindowManager.registerToolWindow("2DViewer","2DViewer",null, this.rna2dViewer, ToolWindowAnchor.BOTTOM);
        descriptor = (DockedTypeDescriptor) this.ssViewer.getTypeDescriptor(ToolWindowType.DOCKED);
        descriptor.setDockLength(300);
        descriptor.setPreviewEnabled(false);

        toolWindow = toolWindowManager.registerToolWindow("News","News",null, new NewsPanel() , ToolWindowAnchor.BOTTOM);
        descriptor = (DockedTypeDescriptor) toolWindow.getTypeDescriptor(ToolWindowType.DOCKED);
        descriptor.setDockLength(50);
        descriptor.setPreviewEnabled(false);


        for (ToolWindow window : toolWindowManager.getToolWindows())
            window.setAvailable(true);

        JPanel toolbars = new JPanel();
        toolbars.setLayout(new BorderLayout());
        toolbars.add(new S2SToolBar(mediator), BorderLayout.NORTH);
        toolbars.add(this.rna2dViewer.getToolbar(),BorderLayout.CENTER);
        toolbars.add(new TertiaryViewersToolBar(),BorderLayout.SOUTH);
        this.window.add(toolbars, BorderLayout.NORTH);
        this.window.add(myDoggyToolWindowManager, BorderLayout.CENTER);
        this.window.add(this.messageBar, BorderLayout.SOUTH);
        ((JXFrame)this.window).setJMenuBar(new S2SMenu());

        this.window.pack();
        this.window.setLocation((int)(screenSize.width*0.1),(int)(screenSize.height*0.025));
        this.window.setVisible(true);
        this.drivers = new ArrayList<TertiaryViewerDriver>();

        if (backup.exists()) {
            if (JOptionPane.showConfirmDialog(null,"Do you want to recover your last working session?", "Previous Session Recovery", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE) == JOptionPane.OK_OPTION) {
                S2S.this.workingSession.restorePreviousWorkingSession();
                S2S.this.getProgressMonitor().printMessage("Project "+workingSession.getCurrentParadiseProject().getName()+" opened");
                ((JFrame) S2S.this.window).setTitle(((JFrame) S2S.this.window).getTitle()+" - project "+workingSession.getCurrentParadiseProject().getName());
                JOptionPane.showMessageDialog(null,"Previous working session recovered");
            }
            else
                backup.delete();
        }
        class Backup extends SwingWorker implements ActionListener {
            private Timer timer;
            protected Object doInBackground() throws Exception {
                timer = new Timer(900000,this);  //each 15 minutes
                timer.start();
                return null;
            }

            public void actionPerformed(ActionEvent e) {
                if (workingSession.getCurrentParadiseProject() != null) {
                    S2S.this.synchronize();
                    if (!backup.exists())
                        try {
                            backup.createNewFile();
                        } catch (IOException e1) {
                            e1.printStackTrace();
                        }
                    ObjectOutputStream stream = null;
                    /*
                    try {
                        stream = new ObjectOutputStream(new FileOutputStream(backup));
                        stream.writeObject(workingSession.getCurrentParadiseProject());
                        List<Molecule> molecules=S2S.this.workingSession.getMolecules();
                        for (Molecule m:molecules)
                            m.makeAllAnnotationsPersistent();
                        stream.writeObject(molecules);
                        for (Molecule m:molecules)
                            m.makeAllAnnotationsNotPersistent();
                        printMessage("Backup file created");
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    }
                    finally {
                        if (stream!=null)
                            try {
                                stream.close();
                            } catch (IOException e1) {
                                e1.printStackTrace();
                            }
                    }*/
                }
            }
        }
        new Backup().execute();
        this.identityFeatureFactory = S2S.this.createIdentityFeatureFactory();

    }

    public Mediator getMediator() {
        return mediator;
    }

    /**
     * Return the current AlignmentCanvas
     *
     * @return the current AlignmentCanvas. Return null if no AlignmentCanvas available
     */
    public AlignmentCanvas getAlignmentCanvas() {
        return this.mediator.getAlignmentCanvas();
    }

    public void residuesSelected(ParadiseTool source, Collection<Residue> residues) {
        if (source == this || !mediator.isLinked() || this.mediator.getAlignmentCanvas().getMainAlignment() == null )
            return;
        this.mediator.getAlignmentCanvas().clearSelectedPositions();
        for (Residue r:residues)
            for (BiologicalSequence s:this.mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequences())
                if (s.getMolecule().getParadiseID().equals(r.getMoleculeId()))
                    s.addSelectedPosition(r.getAbsolutePosition());
        if (residues.size() > 0 )
            this.mediator.getAlignmentCanvas().repaint();
    }


    public void loadRNASecondaryStructure(SecondaryStructure ss) {
        SecondaryStructureDisplay display = ss.getSecondaryStructureDisplay();
        //if no secondarystructure display associated with this secondarystructure
        if (display == null) {
            GuiEvent ev = new GuiEvent(this, fr.unistra.ibmc.paradise.Paradise.SECONDARY_STRUCTURE_DRAWING);
            ArrayList<Molecule> molecules = new ArrayList<Molecule>();
            for (Molecule m : ss.getMolecules()) {
                if (RNA.class.isInstance(m)) {
                    m.removeAllSelectedFeatures();
                    molecules.add(m);
                    m.addSelectedFeature(ss);
                }
            }
            ev.addParameter(new SecondaryStructureDrawingRequestAnswerBehaviour(molecules,new Parameters()));
            ((AbstractParadiseToolAgent) agent).postGuiEvent(ev);
        }
        else {
            List<StructuralAlignment> alignments = ss.getLinkedStructuralAlignments();
            if (!alignments.isEmpty()) {
                StructuralAlignment al = alignments.get(0);
                Collection<Molecule> alignmentMolecules = al.getMolecules();
                for (Molecule m:ss.getMolecules())
                    if (alignmentMolecules.contains(m)) {
                        this.mediator.setMainAlignment(al, m,ss);
                        this.mediator.setModel(ss, display);
                        break;
                    }

            }
            else {
                List<Molecule> molecules = new ArrayList<Molecule>(ss.getMolecules());
                Molecule m = null;
                if (molecules.size() > 1)
                    m = (Molecule)JOptionPane.showInputDialog(S2S.this.window, "Choose a molecule", "Choose a molecule",JOptionPane.PLAIN_MESSAGE, null,molecules.toArray(),molecules.get(0));
                else
                    m = molecules.get(0);
                if (m != null) {
                    StructuralAlignment sa = identityFeatureFactory.createStructuralAligment(new ParadiseID(), "Structural Alignment", Source.getGraphicalToolAsSource(mediator.getS2s().getDescription()));
                    identityFeatureFactory.createStructuralIdentity(sa,"full molecule",m, new Location(1,m.getLength())); //TODO rather create StructuralIDentities according to the reference structure
                    sa.setName("Structural Alignment "+ Paradise.mainFeaturesCreatedCount++);
                    ss.linkStructuralAlignment(sa);
                    this.mediator.setMainAlignment(sa,m,ss);
                }
                this.mediator.setModel(ss, display);
            }
        }
    }

    private class S2SMenu extends javax.swing.JMenuBar {

        private S2SMenu() {
            JMenu menu = null, submenu = null;
            JMenuItem item = null;
            this.add(new FileMenu());

            JMenu compareMenu = new JMenu("Compare with");
            JMenu algorithmsMenu = new JMenu("new alignment generated by");
            item = new JMenuItem("CARNAC");
            algorithmsMenu.add(item);
            compareMenu.add(algorithmsMenu);
            item = new JMenuItem("with an alignment in memory");
            compareMenu.add(item);
            //menu.add(compareMenu);

            submenu = new JMenu("Display");
            item = new JMenuItem("Quick");
            item.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    mediator.getAlignmentCanvas().setQuickDraw(true);
                }
            });
            submenu.add(item);
            item = new JMenuItem("Best");
            item.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    mediator.getAlignmentCanvas().setQuickDraw(false);
                }
            });
            submenu.add(item);
            //menu.add(submenu);

            this.add(new PluginsMenu());

            menu = new JMenu("Help");

            menu.add(new OpenWebPageItem("S2S Website", "http://bioinformatics.org/s2s/"));
            menu.add(new UpdateMenuItem());

            this.add(menu);

        }

        private class UpdateMenuItem extends JMenuItem implements ActionListener {

            private UpdateMenuItem() {
                super("Update S2S");
                this.addActionListener(this);
            }

            public void actionPerformed(ActionEvent actionEvent) {
                new SwingWorker() {
                    protected Object doInBackground() throws Exception {
                        S2S.updateS2S();
                        return null;
                    }
                }.execute();
            }
        }

        private class FileMenu extends JMenu {

            public FileMenu() {
                super("File");
                JMenu load = new JMenu("Load...");
                load.setIcon(RessourcesUtils.getIcon("open.png"));
                this.add(load);
                JMenuItem item = new JMenuItem("S2S Alignment");
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null && JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog(S2S.this.window,"Are you sure to remove your current Alignment?"))
                            return;
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null)
                            mediator.clearSession();
                        final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                        fileChooser.setFileView(new FileView() {
                            public Icon getIcon(File f) {
                                if (IOUtils.isParadiseProject(f) || f.getName().endsWith(".s2s"))
                                    return new ImageIcon(RessourcesUtils.getImage("paradise-project.png"));
                                else return null;
                            }

                            public Boolean isTraversable(File f) {
                                return f.isDirectory() && !IOUtils.isParadiseProject(f);
                            }
                        });
                        fileChooser.setFileHidingEnabled(true);
                        fileChooser.setAcceptAllFileFilterUsed(false);
                        fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                        fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
                            public boolean accept(File file) {
                                return file.isDirectory() && !IOUtils.isParadiseProject(file) || file.getName().endsWith(".s2s");
                            }

                            public String getDescription() {
                                return "S2S alignments (binary data)";
                            }
                        });
                        fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
                            public boolean accept(File file) {
                                return file.isDirectory() && !IOUtils.isParadiseProject(file) || IOUtils.containsStructuralAlignmentData(file);
                            }

                            public String getDescription() {
                                return "S2S alignments (textual data)";
                            }
                        });
                        if (fileChooser.showOpenDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                            final File f = fileChooser.getSelectedFile();
                            if (f.getName().endsWith(".s2s")) {
                                Paradise.setWorkingDirectory(f.getAbsolutePath());
                                final ParadiseFileIO parser = new S2SAlignmentParser(workingSession);
                                new FileParsingTask(S2S.this, f, parser,messageBar,workingSession) {
                                    @Override
                                    protected void finished() {
                                        List<StructuralAlignment> alignments = parser.getStructuralAlignments();
                                        StructuralAlignment alignment = null;
                                        if (alignments.size() != 1) {
                                            String[] names = new String[alignments.size()];
                                            for (int i=0; i< names.length;i++)
                                                names[i] = alignments.get(i).getName();
                                            String name = (String)JOptionPane.showInputDialog(S2S.this.window,"Choose an alignment", "Choose an alignment", JOptionPane.PLAIN_MESSAGE,null, names, names[0]);
                                            if (name != null) {
                                                for (StructuralAlignment al:alignments)
                                                    if (al.getName().equals(name)) {
                                                        alignment = al;
                                                        break;
                                                    }
                                            }
                                            else return;
                                        }
                                        else
                                            alignment =  alignments.get(0);
                                        SecondaryStructure ss = alignment.getParentFeatures(SecondaryStructure.class).get(0);
                                        Collection<Molecule> molecules= alignment.getMolecules();
                                        for (Molecule m:ss.getMolecules())
                                            if (molecules.contains(m)) {
                                                S2S.this.mediator.setMainAlignment(alignment, m,ss);
                                                S2S.this.mediator.setModel(ss, ss.getSecondaryStructureDisplay());
                                                break;
                                            }
                                        super.finished();
                                    }
                                }.execute();
                            }
                            else {
                                Paradise.setWorkingDirectory(f.getParentFile().getAbsolutePath());
                                final ParadiseFileIO parser = new ParadiseProjectIO(workingSession);
                                new FileParsingTask(S2S.this, f, parser,messageBar,workingSession) {
                                    @Override
                                    protected void finished() {
                                        List<StructuralAlignment> alignments = parser.getStructuralAlignments();
                                        StructuralAlignment alignment = null;
                                        if (alignments.size() != 1) {
                                            String[] names = new String[alignments.size()];
                                            for (int i=0; i< names.length;i++)
                                                names[i] = alignments.get(i).getName();
                                            String name = (String)JOptionPane.showInputDialog(S2S.this.window,"Choose an alignment", "Choose an alignment", JOptionPane.PLAIN_MESSAGE,null, names, names[0]);
                                            if (name != null) {
                                                for (StructuralAlignment al:alignments)
                                                    if (al.getName().equals(name)) {
                                                        alignment = al;
                                                        break;
                                                    }
                                            }
                                            else return;
                                        }
                                        else
                                            alignment =  alignments.get(0);
                                        SecondaryStructure ss = alignment.getParentFeatures(SecondaryStructure.class).get(0);
                                        Collection<Molecule> alignmentMolecules = alignment.getMolecules();
                                        for (Molecule m:ss.getMolecules())
                                            if (alignmentMolecules.contains(m)){
                                                S2S.this.mediator.setMainAlignment(alignment, m,ss);
                                                S2S.this.mediator.setModel(ss, ss.getSecondaryStructureDisplay());
                                            }

                                        super.finished();
                                    }
                                }.execute();
                            }
                        }
                    }
                });
                load.add(item);

                item = new JMenuItem("RNA alignment");
                if (!S2S.this.isConnected()) {
                    item.setEnabled(false);
                    item.setToolTipText("Need RNA algorithms");
                }
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null && JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog(S2S.this.window,"Are you sure to remove your current Alignment?"))
                            return;
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null)
                            mediator.clearSession();
                        final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                        fileChooser.setFileHidingEnabled(true);
                        fileChooser.setAcceptAllFileFilterUsed(false);
                        final StockholmFileIO parser = new StockholmFileIO(workingSession);
                        fileChooser.setFileFilter(new OpenFileFilter(parser));
                        if (fileChooser.showOpenDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                            File f = fileChooser.getSelectedFile();
                            Paradise.setWorkingDirectory(f.getAbsolutePath());
                            new FileParsingTask(S2S.this, f, parser,messageBar,workingSession) {
                                @Override
                                protected void finished() {
                                    super.finished();
                                    List<StructuralAlignment> alignments = parser.getStructuralAlignments();
                                    StructuralAlignment alignment = null;
                                    if (alignments.size() == 0) {
                                        JOptionPane.showMessageDialog(S2S.this.window,"No alignments available");
                                        return;
                                    }
                                    alignment = alignments.get(0);
                                    if (alignment != null)
                                        S2S.this.loadRNASecondaryStructure(parser.getSecondaryStructures().get(0));
                                }
                            }.execute();

                        }
                    }
                });
                load.add(item);

                item = new JMenuItem("RNA molecule");
                if (!S2S.this.isConnected()) {
                    item.setEnabled(false);
                    item.setToolTipText("Need RNA algorithms");
                }
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null && JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog(S2S.this.window,"Are you sure to remove your current Alignment?"))
                            return;
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null)
                            mediator.clearSession();
                        final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                        fileChooser.setFileHidingEnabled(true);
                        fileChooser.setAcceptAllFileFilterUsed(false);
                        final FastaFileIO parser = new FastaFileIO(workingSession);
                        fileChooser.setFileFilter(new OpenFileFilter(parser));
                        if (fileChooser.showOpenDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                            File f = fileChooser.getSelectedFile();
                            Paradise.setWorkingDirectory(f.getAbsolutePath());
                            new FileParsingTask(S2S.this, f, parser,messageBar,workingSession) {
                                @Override
                                protected void finished() {
                                    super.finished();
                                    List<Molecule> molecules = parser.getMolecules();
                                    Molecule m = null;
                                    if (molecules.size() == 0) {
                                        JOptionPane.showMessageDialog(S2S.this.window,"No molecules available");
                                        return;
                                    }
                                    else if (molecules.size() > 1)
                                        m = (Molecule)JOptionPane.showInputDialog(S2S.this.window, "Choose a molecule", "Choose a molecule",JOptionPane.PLAIN_MESSAGE, null,molecules.toArray(),molecules.get(0));
                                    else
                                        m = molecules.get(0);
                                    if (m != null) {
                                        GuiEvent ev = new GuiEvent(this, fr.unistra.ibmc.paradise.Paradise.SECONDARY_STRUCTURE_PREDICTION);
                                        ArrayList<Molecule> _molecules = new ArrayList<Molecule>();
                                        _molecules.add(molecules.get(0));
                                        ev.addParameter(new SecondaryStructurePredictionRequestAnswerBehaviour(_molecules,new Parameters()));
                                        ((S2SAgent)S2S.this.getAgent()).postGuiEvent(ev);
                                    }
                                }
                            }.execute();

                        }
                    }
                });
                load.add(item);

                item = new JMenuItem("RNA Secondary Structure");
                if (!S2S.this.isConnected()) {
                    item.setEnabled(false);
                    item.setToolTipText("Need RNA algorithms");
                }
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null && JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog(S2S.this.window,"Are you sure to remove your current Alignment?"))
                            return;
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null)
                            mediator.clearSession();
                        final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                        fileChooser.setFileHidingEnabled(true);
                        fileChooser.setAcceptAllFileFilterUsed(false);
                        fileChooser.setFileFilter(new OpenFileFilter(new BPSeqFileIO(workingSession)));
                        fileChooser.setFileFilter(new OpenFileFilter(new CTFileIO(workingSession)));
                        fileChooser.setFileFilter(new OpenFileFilter(new FastaFileIO(workingSession)));
                        if (fileChooser.showOpenDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                            File f = fileChooser.getSelectedFile();
                            Paradise.setWorkingDirectory(f.getAbsolutePath());
                            final ParadiseFileIO parser = ((OpenFileFilter)fileChooser.getFileFilter()).getParser();
                            new FileParsingTask(S2S.this, f, parser,messageBar,workingSession) {
                                @Override
                                protected void finished() {
                                    if (FastaFileIO.class.isInstance(parser)) {
                                        Molecule m = parser.getMolecules().get(0);
                                        String bn = ((FastaFileIO)parser).getBracketNotation(m);
                                        if (bn != null) {
                                            GuiEvent ev = new GuiEvent(this, Paradise.SECONDARY_STRUCTURE_DRAWING);
                                            ArrayList<Molecule> molecules = new ArrayList<Molecule>();
                                            if (RNA.class.isInstance(m)) {
                                                m.removeAllSelectedFeatures();
                                                molecules.add(m);
                                            }
                                            try {
                                                //TODO: workaround, the paradise engine should be able to transform a bracket notation as a Secondary Structure object directly
                                                File f = IOUtils.createTemporaryFile("bracketNotation");
                                                CTFileIO.writeCTFile(f,m.printSequence(),bn);
                                                CTFileIO parser2 = new CTFileIO(workingSession) ;
                                                parser2.parseFile(f, new DefaultProgressMonitor());
                                                SecondaryStructure ss = parser2.getSecondaryStructures().get(0);
                                                SecondaryStructure copySS = ss.getFactory().copySecondaryStructure(Arrays.asList(new Molecule[]{m}),ss,false);
                                                m.addSelectedFeature(copySS);
                                            } catch (Exception e1) {
                                                e1.printStackTrace();
                                            }
                                            ev.addParameter(new SecondaryStructureDrawingRequestAnswerBehaviour(molecules,new Parameters()));
                                            ((AbstractParadiseToolAgent) S2S.this.getAgent()).postGuiEvent(ev);
                                        }
                                        else
                                            JOptionPane.showMessageDialog(S2S.this.window,"No Secondary Structure described in the FASTA file");
                                    }
                                    else
                                        S2S.this.loadRNASecondaryStructure(parser.getSecondaryStructures().get(0));
                                }
                            }.execute();
                        }
                    }
                });
                load.add(item);

                item = new JMenuItem("RNA Tertiary Structure");
                if (!S2S.this.isConnected()) {
                    item.setEnabled(false);
                    item.setToolTipText("Need RNA algorithms");
                }
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null && JOptionPane.OK_OPTION != JOptionPane.showConfirmDialog(S2S.this.window,"Are you sure to remove your current Alignment?"))
                            return;
                        if (mediator.getAlignmentCanvas().getMainAlignment() != null)
                            mediator.clearSession();
                        final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(fr.unistra.ibmc.paradise.Paradise.getWorkingDirectory());
                        fileChooser.setFileHidingEnabled(true);
                        fileChooser.setAcceptAllFileFilterUsed(false);
                        final PDBFileIO pdbParser = new PDBFileIO(workingSession);
                        fileChooser.setFileFilter(new OpenFileFilter(pdbParser));
                        if (fileChooser.showOpenDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                            File f = fileChooser.getSelectedFile();
                            Paradise.setWorkingDirectory(f.getAbsolutePath());

                            new FileParsingTask(S2S.this, f, pdbParser,messageBar,workingSession) {
                                @Override
                                protected void finished() {
                                    super.finished();
                                    TertiaryStructure ts = pdbParser.getTertiaryStructures().get(0);
                                    GuiEvent ev = new GuiEvent(this, fr.unistra.ibmc.paradise.Paradise.TERTIARY_STRUCTURE_ANNOTATION);
                                    ArrayList<Molecule> molecules = new ArrayList<Molecule>();
                                    for (Molecule m:ts.getMolecules()) {
                                        m.removeAllSelectedFeatures();
                                        //[fjossinet]] we send only the Residue3D annotating RNA molecules
                                        for (Residue3D r : ts.getSubFeatures(Residue3D.class, m)) {
                                            m.addSelectedFeature(r);
                                        }
                                        molecules.add(m);
                                    }
                                    ev.addParameter(new TertiaryStructureAnnotationRequestAnswerBehaviour(molecules,new Parameters()));
                                    ((S2SAgent)S2S.this.getAgent()).postGuiEvent(ev);
                                }
                            }.execute();

                        }
                    }
                });
                load.add(item);

                JMenu importMenu = new JMenu("Import RNA molecules...") {
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(mediator.getAlignmentCanvas().getMainAlignment() != null);
                        super.paintComponent(graphics);
                    }
                };
                if (!S2S.this.isConnected()) {
                    importMenu.setEnabled(false);
                    importMenu.setToolTipText("Need RNA algorithms");
                }
                importMenu.setIcon(RessourcesUtils.getIcon("open.png"));
                this.add(importMenu);
                item = new JMenuItem("Search new candidates");
                item.addActionListener(new ActionListener() {

                    public void actionPerformed(ActionEvent actionEvent) {
                        new SelectDatabasesFrame();
                    }

                });
                //importMenu.add(item);

                item = new JMenuItem("from local files");
                item.addActionListener(new ActionListener() {

                    public void actionPerformed(ActionEvent actionEvent) {
                        if (mediator.getAlignmentCanvas()!=null) {
                            final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                            fileChooser.setAcceptAllFileFilterUsed(false);
                            fileChooser.setFileHidingEnabled(true);
                            fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {

                                public boolean accept(File file) {
                                    if (file.isDirectory())
                                        return true;
                                    for (String suffix:FastaFileIO.suffixes)
                                        if (file.getName().endsWith(suffix))
                                            return true;
                                    return false;
                                }

                                public String getDescription() {
                                    return "Fasta Files";
                                }
                            });
                            fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {

                                public boolean accept(File file) {
                                    if (file.isDirectory())
                                        return true;
                                    for (String suffix:CTFileIO.suffixes)
                                        if (file.getName().endsWith(suffix))
                                            return true;
                                    return false;
                                }

                                public String getDescription() {
                                    return "Ct Files";
                                }
                            });
                            fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {

                                public boolean accept(File file) {
                                    if (file.isDirectory())
                                        return true;
                                    for (String suffix:BPSeqFileIO.suffixes)
                                        if (file.getName().endsWith(suffix))
                                            return true;
                                    return false;
                                }

                                public String getDescription() {
                                    return "BPseq Files";
                                }
                            });
                            if (fileChooser.showOpenDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                                final java.io.File file = fileChooser.getSelectedFile();

                                final boolean automaticAlignment = JOptionPane.showConfirmDialog(null,"Do you want to align your new sequences automatically?") == JOptionPane.OK_OPTION;
                                final StringBuffer structuralAlignmentAgentName = new StringBuffer();
                                if (automaticAlignment) {
                                    //which structural alignment tool to use
                                    //we do this previous step to avoid to ask the same question to the user for each fasta sequence in the original file (imagine that if the fasta file constains thousands of sequences)
                                    DFAgentDescription[] descriptions = S2S.this.agent.findAgents(Analysis.STRUCTURAL_ALIGNMENT);
                                    //show a JDialog with the list of agents available if more than one agent
                                    if (descriptions.length > 1) {
                                        Object[] agentNames = new Object[descriptions.length];
                                        for (int i=0; i<descriptions.length;i++)
                                            agentNames[i]=descriptions[i].getName().getLocalName().split("\\[at\\]")[0];

                                        structuralAlignmentAgentName.append((String)JOptionPane.showInputDialog(null, "Please choose a service" , Analysis.STRUCTURAL_ALIGNMENT,JOptionPane.PLAIN_MESSAGE,null,agentNames,agentNames[0]));
                                    }
                                    else if (descriptions.length == 0)
                                        JOptionPane.showMessageDialog(null, "No service found for the category: "+Analysis.STRUCTURAL_ALIGNMENT);
                                    else
                                        structuralAlignmentAgentName.append(descriptions[0].getName().getLocalName().split("\\[at\\]")[0]);

                                }
                                if (fileChooser.getFileFilter().getDescription().endsWith("Ct Files"))
                                    new ParadiseTask(mediator.getS2s(), mediator.getS2s().getProgressMonitor()) {
                                        protected Object doInBackground() throws Exception {
                                            CTFileIO parser = new CTFileIO(workingSession);
                                            parser.parseFile(file,mediator.getS2s().getProgressMonitor());
                                            Molecule m = parser.getMolecules().get(0);
                                            if (automaticAlignment) {
                                                //we call an algorithm to pre-align this new molecule to the reference molecule
                                                GuiEvent ev = new GuiEvent(this, Paradise.STRUCTURAL_ALIGNMENT);
                                                ArrayList<Molecule> molecules = new ArrayList<Molecule>();
                                                Molecule refMol = mediator.getAlignmentCanvas().getMainAlignment().getBiologicalReferenceSequence().getMolecule();
                                                refMol.addSelectedFeature(mediator.getAlignmentCanvas().getMainAlignment().getReferenceStructure());
                                                m.addSelectedFeature(parser.getSecondaryStructures().get(0));
                                                molecules.add(refMol);
                                                molecules.add(m);
                                                m.addSelectedFeature(parser.getSecondaryStructures().get(0));
                                                ev.addParameter(new StructuralAlignmentRequestAnswerBehaviour(molecules,new Parameters()));
                                                ev.addParameter(structuralAlignmentAgentName.toString());
                                                ((AbstractParadiseToolAgent)agent).postGuiEvent(ev);
                                            }
                                            else {
                                                BiologicalSequence seq = new BiologicalSequence(m);
                                                mediator.getAlignmentCanvas().getMainAlignment().addBiologicalSequence(seq);
                                                mediator.getSequencesList().addRow(seq.getMolecule());
                                                mediator.getAlignmentCanvas().repaint();
                                            }
                                            this.finished();
                                            return null;
                                        }
                                    }.execute();
                                else if (fileChooser.getFileFilter().getDescription().endsWith("BPseq Files"))
                                    new ParadiseTask(mediator.getS2s(), mediator.getS2s().getProgressMonitor()) {
                                        protected Object doInBackground() throws Exception {
                                            BPSeqFileIO parser = new BPSeqFileIO(workingSession);
                                            parser.parseFile(file,mediator.getS2s().getProgressMonitor());
                                            Molecule m = parser.getMolecules().get(0);
                                            if (automaticAlignment) {
                                                //we call an algorithm to pre-align this new molecule to the reference molecule
                                                GuiEvent ev = new GuiEvent(this, Paradise.STRUCTURAL_ALIGNMENT);
                                                ArrayList<Molecule> molecules = new ArrayList<Molecule>();
                                                Molecule refMol = mediator.getAlignmentCanvas().getMainAlignment().getBiologicalReferenceSequence().getMolecule();
                                                refMol.addSelectedFeature(mediator.getAlignmentCanvas().getMainAlignment().getReferenceStructure());
                                                m.addSelectedFeature(parser.getSecondaryStructures().get(0));
                                                molecules.add(refMol);
                                                molecules.add(m);
                                                m.addSelectedFeature(parser.getSecondaryStructures().get(0));
                                                ev.addParameter(new StructuralAlignmentRequestAnswerBehaviour(molecules,new Parameters()));
                                                ev.addParameter(structuralAlignmentAgentName.toString());
                                                ((AbstractParadiseToolAgent)agent).postGuiEvent(ev);
                                            }
                                            else {
                                                BiologicalSequence seq = new BiologicalSequence(m);
                                                mediator.getAlignmentCanvas().getMainAlignment().addBiologicalSequence(seq);
                                                mediator.getSequencesList().addRow(seq.getMolecule());
                                                mediator.getAlignmentCanvas().repaint();
                                            }
                                            this.finished();
                                            return null;
                                        }
                                    }.execute();
                                else  if (fileChooser.getFileFilter().getDescription().endsWith("Fasta Files"))
                                    new ParadiseTask(mediator.getS2s(), mediator.getS2s().getProgressMonitor()) {
                                        protected Object doInBackground() throws Exception {
                                            messageBar.startAnimation();
                                            FastaFileIO parser = new FastaFileIO(workingSession);
                                            parser.parseFile(file,mediator.getS2s().getProgressMonitor());
                                            if (automaticAlignment) {
                                                //which 2D prediction tool to use before the alignment??
                                                //we do this previous step to avoid to ask the same question to the user for each fasta sequence in the original file (imagine that if the fasta file constains thousands of sequences)
                                                DFAgentDescription[] descriptions = S2S.this.agent.findAgents(Analysis.SECONDARY_STRUCTURE_PREDICTION);
                                                String agentName = null;
                                                //show a JDialog with the list of agents available if more than one agent
                                                if (descriptions.length > 1) {
                                                    Object[] agentNames = new Object[descriptions.length];
                                                    for (int i=0; i<descriptions.length;i++)
                                                        agentNames[i]=descriptions[i].getName().getLocalName().split("\\[at\\]")[0];
                                                    this.monitor.stopAnimation();
                                                    agentName = (String)JOptionPane.showInputDialog(null, "Please choose a service" , Analysis.SECONDARY_STRUCTURE_PREDICTION,JOptionPane.PLAIN_MESSAGE,null,agentNames,agentNames[0]);
                                                }
                                                else if (descriptions.length == 0) {
                                                    JOptionPane.showMessageDialog(null, "No service found for the category: "+Analysis.SECONDARY_STRUCTURE_PREDICTION);
                                                    this.monitor.stopAnimation();
                                                }
                                                else
                                                    agentName = descriptions[0].getName().getLocalName().split("\\[at\\]")[0];
                                                //we call an algorithm to pre-align this new molecule to the reference molecule
                                                for (Molecule m:parser.getMolecules()) {
                                                    GuiEvent ev = new GuiEvent(this, Paradise.SECONDARY_STRUCTURE_PREDICTION);
                                                    ArrayList<Molecule> molecules = new ArrayList<Molecule>();
                                                    molecules.add(m);
                                                    ev.addParameter(new SecondaryStructurePredictionBeforeStructuralAlignmentAnswerBehaviour(molecules,new Parameters(),structuralAlignmentAgentName.toString()));
                                                    ev.addParameter(agentName);
                                                    ((AbstractParadiseToolAgent)agent).postGuiEvent(ev);
                                                }
                                            }
                                            else {
                                                for (Molecule m:parser.getMolecules()) {
                                                    BiologicalSequence seq = new BiologicalSequence(m);
                                                    mediator.getAlignmentCanvas().getMainAlignment().addBiologicalSequence(seq);
                                                    mediator.getSequencesList().addRow(seq.getMolecule());
                                                    mediator.getAlignmentCanvas().repaint();
                                                }
                                            }
                                            this.finished();
                                            return null;
                                        }
                                    }.execute();
                            }
                        }
                    }
                });
                importMenu.add(item);

                item = new JMenuItem("Save Alignment") {
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(getCurrentFeature()!= null && getCurrentFeature().getAssociatedFile() != null);
                        super.paintComponent(graphics);
                    }
                };
                item.setIcon(RessourcesUtils.getIcon("saveas.png"));
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                        final File saveFile = mediator.getAlignmentCanvas().getMainAlignment().getStructuralAlignment().getAssociatedFile();
                        new ParadiseTask(S2S.this,messageBar) {
                            protected Object doInBackground()  {
                                try {
                                    messageBar.startAnimation();
                                    synchronize();
                                    if (saveFile.getName().endsWith(".rnaml")) {
                                        ParadiseProjectIO.saveWorkingSession(saveFile.getParentFile().getParentFile(),Arrays.asList(new MainParadiseFeature[]{mediator.getAlignmentCanvas().getMainAlignment().getStructuralAlignment()}));
                                        setS2SLastSavePath(saveFile.getParentFile().getParentFile().getParentFile());
                                    }
                                    else if (saveFile.getName().endsWith(".assemble")) {
                                        ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(saveFile));
                                        oos.writeObject(mediator.getAlignmentCanvas().getMainAlignment().getStructuralAlignment());
                                        oos.flush();
                                        oos.close();
                                        setS2SLastSavePath(saveFile.getParentFile());
                                    }
                                    monitor.printMessage("Saving successful");
                                }
                                catch (Exception e) {
                                    monitor.printException(e);
                                }
                                this.finished();
                                return null;
                            }
                        }.execute();
                    }
                });
                this.add(item);
                item = new JMenuItem("Save Alignment As...") {
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(getCurrentFeature()!= null);
                        super.paintComponent(graphics);
                    }
                };
                item.setIcon(RessourcesUtils.getIcon("saveas.png"));
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                        final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(S2S.this.getS2SLastSavePath());
                        fileChooser.setFileView(new FileView() {
                            public Icon getIcon(File f) {
                                if (IOUtils.isParadiseProject(f))
                                    return new ImageIcon(RessourcesUtils.getImage("paradise-project.png"));
                                else if (f.getName().endsWith(".s2s"))
                                    return new ImageIcon(RessourcesUtils.getImage("paradise-project.png"));
                                else return null;
                            }

                            public Boolean isTraversable(File f) {
                                return f.isDirectory() && !IOUtils.isParadiseProject(f);
                            }
                        });
                        fileChooser.setFileHidingEnabled(true);
                        fileChooser.setAcceptAllFileFilterUsed(false);
                        fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
                        fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
                            public boolean accept(File file) {
                                return file.isDirectory() && !IOUtils.isParadiseProject(file) || file.getName().endsWith(".s2s");
                            }

                            public String getDescription() {
                                return "S2S alignments (binary data)";
                            }
                        });
                        fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
                            public boolean accept(File file) {
                                return file.isDirectory() && !IOUtils.isParadiseProject(file) || IOUtils.containsStructuralAlignmentData(file);
                            }

                            public String getDescription() {
                                return "S2S alignments (textual data)";
                            }
                        });
                        if (fileChooser.showSaveDialog(S2S.this.window) == JFileChooser.APPROVE_OPTION) {
                            final File saveFile = fileChooser.getSelectedFile();
                            new ParadiseTask(S2S.this,messageBar) {
                                protected Object doInBackground()  {
                                    try {
                                        messageBar.startAnimation();
                                        synchronize();
                                        if (saveFile.getName().endsWith(".s2s")) {
                                            ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(saveFile));
                                            oos.writeObject(mediator.getAlignmentCanvas().getMainAlignment().getStructuralAlignment());
                                            oos.flush();
                                            oos.close();
                                            mediator.getAlignmentCanvas().getMainAlignment().getStructuralAlignment().setAssociatedFile(saveFile);
                                            setS2SLastSavePath(saveFile.getParentFile());
                                        }
                                        else {
                                            //I'm saving the structural alignment since the parent features are automatically saved by the ParadiseProjectIO class (so the 2D and so, perhaps, the 3D)
                                            ParadiseProjectIO.saveWorkingSession(saveFile,Arrays.asList(new MainParadiseFeature[]{mediator.getAlignmentCanvas().getMainAlignment().getStructuralAlignment()}));
                                            setS2SLastSavePath(saveFile.getParentFile());
                                        }
                                        monitor.printMessage("Saving successful");
                                    }
                                    catch (Exception e) {
                                        monitor.printException(e);
                                    }
                                    this.finished();
                                    return null;
                                }
                            }.execute();
                        }
                    }
                });
                this.add(item);

                JMenu exportMenu = new JMenu("Export..."){
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(getCurrentFeature()!= null);
                        super.paintComponent(graphics);
                    }
                };

                exportMenu.setIcon(RessourcesUtils.getIcon("saveas.png"));
                this.add(exportMenu);

                item = new JMenuItem("Alignment as FASTA file") {
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(getCurrentFeature()!= null);
                        super.paintComponent(graphics);
                    }
                };
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                        final JFileChooser saveFile = new JFileChooser(getS2SLastSavePath());
                        saveFile.setAcceptAllFileFilterUsed(false);
                        saveFile.setFileHidingEnabled(true);
                        saveFile.setFileFilter(new javax.swing.filechooser.FileFilter() {
                            public boolean accept(File file) {
                                if (file.isDirectory())
                                    return true;
                                for (String suffix:FastaFileIO.suffixes)
                                    if (file.getName().endsWith(suffix))
                                        return true;
                                return false;
                            }

                            public String getDescription() {
                                return "FASTA files";
                            }
                        });
                        if (saveFile.showSaveDialog(S2S.this.window) == JFileChooser.APPROVE_OPTION) {
                            setS2SLastSavePath(saveFile.getSelectedFile());
                            new ParadiseTask(S2S.this,messageBar) {
                                protected Object doInBackground()  {
                                    try {
                                        messageBar.startAnimation();
                                        synchronize();
                                        List<String> names = new ArrayList<String>(), sequences = new ArrayList<String>();
                                        for (BiologicalSequence s:mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequences()) {
                                            names.add(s.getMolecule().getName());
                                            sequences.add(s.getSequence());
                                        }
                                        FastaFileIO.exportMoleculesAsFastaFile(saveFile.getSelectedFile(),names, sequences);
                                        monitor.printMessage("Export successful");
                                    }
                                    catch (Exception e) {
                                        monitor.printException(e);
                                    }
                                    return null;
                                }
                            }.execute();
                        }
                    }
                });
                exportMenu.add(item);

                item = new JMenuItem("2D Model as CT file") {
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(getCurrentFeature()!= null);
                        super.paintComponent(graphics);
                    }
                };
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                        final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                        fileChooser.setAcceptAllFileFilterUsed(false);
                        fileChooser.setFileHidingEnabled(true);
                        fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
                            public boolean accept(File file) {
                                if (file.isDirectory())
                                    return true;
                                for (String suffix:CTFileIO.suffixes)
                                    if (file.getName().endsWith(suffix))
                                        return true;
                                return false;
                            }

                            public String getDescription() {
                                return "CT files";
                            }
                        });
                        if (fileChooser.showSaveDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                            final java.io.File file = fileChooser.getSelectedFile();
                            new ParadiseTask(S2S.this,messageBar) {
                                protected Object doInBackground()  {
                                    try {
                                        messageBar.startAnimation();
                                        if (!file.exists())
                                            file.createNewFile();
                                        List<fr.unistra.ibmc.paradise.core.utils.Residue> residues = new ArrayList<fr.unistra.ibmc.paradise.core.utils.Residue>();
                                        Set<BaseBaseInteraction>  interactions = new HashSet<BaseBaseInteraction>();
                                        for (fr.unistra.ibmc.s2s.ssviewer.features.Residue2D r2D: mediator.getRna2dViewer().getSecondaryCanvas().getModel2D().getResidues()) {
                                            residues.add(r2D.getResidue2DFeature().getResidues().iterator().next());
                                            for (Interaction2D interaction:r2D.getInteractions())
                                                if (BaseBaseInteraction2D.class.isInstance(interaction) && ((BaseBaseInteraction2D)interaction).getBaseBaseInteractionFeature().isSecondaryInteraction()) /*a ct file stores only secondary interactions, not tertiary ones)*/
                                                    interactions.add(((BaseBaseInteraction2D)interaction).getBaseBaseInteractionFeature());
                                        }
                                        Collections.sort(residues, new Comparator() {

                                            public int compare(Object o1, Object o2) {
                                                return ((fr.unistra.ibmc.paradise.core.utils.Residue)o1).getAbsolutePosition() - ((fr.unistra.ibmc.paradise.core.utils.Residue)o2).getAbsolutePosition();
                                            }
                                        });
                                        CTFileIO.writeCTFile(residues,new ArrayList(interactions),file);
                                        monitor.printMessage("Export successful");
                                    }
                                    catch (Exception e) {
                                        monitor.printException(e);
                                    }
                                    this.finished();
                                    return null;
                                }
                            }.execute();

                        }
                    }
                });
                exportMenu.add(item);

                item = new JMenuItem("2D Model as SVG file") {
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(getCurrentFeature()!= null);
                        super.paintComponent(graphics);
                    }
                };
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                        final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                        fileChooser.setAcceptAllFileFilterUsed(false);
                        fileChooser.setFileHidingEnabled(true);
                        fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
                            public boolean accept(File file) {
                                return file.isDirectory() || file.getName().endsWith("svg");
                            }

                            public String getDescription() {
                                return "SVG files";
                            }
                        });
                        if (fileChooser.showSaveDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                            final java.io.File file = fileChooser.getSelectedFile();
                            new ParadiseTask(S2S.this,messageBar) {
                                protected Object doInBackground()  {
                                    try {
                                        messageBar.startAnimation();
                                        if (!file.exists())
                                            file.createNewFile();
                                        PrintWriter pw = new PrintWriter(file);
                                        pw.print(mediator.getRna2dViewer().getSecondaryCanvas().toSVG());
                                        pw.close();
                                        monitor.printMessage("Export successful");
                                    }
                                    catch (Exception e) {
                                        monitor.printException(e);
                                    }
                                    this.finished();
                                    return null;
                                }
                            }.execute();

                        }
                    }
                });
                exportMenu.add(item);

                JMenu selectionMenu = new JMenu("Selection...") {
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(getCurrentFeature()!= null);
                        super.paintComponent(graphics);
                    }
                };
                exportMenu.add(selectionMenu);

                item = new JMenuItem("Alignment Selection as FASTA file") {
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(getCurrentFeature()!= null);
                        super.paintComponent(graphics);
                    }
                };
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                        final JFileChooser saveFile = new JFileChooser(getS2SLastSavePath());
                        saveFile.setAcceptAllFileFilterUsed(false);
                        saveFile.setFileHidingEnabled(true);
                        saveFile.setFileFilter(new javax.swing.filechooser.FileFilter() {
                            public boolean accept(File file) {
                                if (file.isDirectory())
                                    return true;
                                for (String suffix:FastaFileIO.suffixes)
                                    if (file.getName().endsWith(suffix))
                                        return true;
                                return false;
                            }

                            public String getDescription() {
                                return "Fasta files";
                            }
                        });
                        if (saveFile.showSaveDialog(S2S.this.window) == JFileChooser.APPROVE_OPTION) {
                            setS2SLastSavePath(saveFile.getSelectedFile());
                            new ParadiseTask(S2S.this,messageBar) {
                                protected Object doInBackground()  {
                                    try {
                                        messageBar.startAnimation();
                                        synchronize();
                                        List<String> names = new ArrayList<String>(), sequences = new ArrayList<String>();
                                        int start = -1, currentPosition = -1;
                                        StringBuffer sequence = null;
                                        String currentName = null;
                                        for (BiologicalSequence s:mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequences()) {
                                            for (int i = 0; i< s.size();i++) {
                                                Symbol symbol = s.getSymbol(i);
                                                int positionInSequence = symbol.getPositionInSequence();
                                                if (s.isSelectedPosition(positionInSequence)) {
                                                    if (positionInSequence -1 != currentPosition ) {
                                                        if (sequence != null) {
                                                            names.add(currentName+ " ("+start+"-"+currentPosition+")");
                                                            sequences.add(sequence.toString());
                                                        }
                                                        currentName = s.getMolecule().getName();
                                                        currentPosition = positionInSequence;
                                                        start = positionInSequence;
                                                        sequence = new StringBuffer();
                                                        sequence.append(symbol.getSymbol());
                                                    }
                                                    else {
                                                        sequence.append(symbol.getSymbol());
                                                        currentPosition = positionInSequence;
                                                    }
                                                }
                                            }
                                        }
                                        //the last one
                                        if (sequence != null) {
                                            names.add(currentName+ " ("+start+"-"+currentPosition+")");
                                            sequences.add(sequence.toString());
                                        }
                                        FastaFileIO.exportMoleculesAsFastaFile(saveFile.getSelectedFile(),names, sequences);
                                        monitor.printMessage("Export successful");
                                    }
                                    catch (Exception e) {
                                        monitor.printException(e);
                                    }
                                    return null;
                                }
                            }.execute();
                        }
                    }
                });
                selectionMenu.add(item);

                item = new JMenuItem("2D Model Selection as CT file") {
                    public void paintComponent(Graphics graphics) {
                        this.setEnabled(getCurrentFeature()!= null);
                        super.paintComponent(graphics);
                    }
                };
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                        final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                        fileChooser.setAcceptAllFileFilterUsed(false);
                        fileChooser.setFileHidingEnabled(true);
                        fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
                            public boolean accept(File file) {
                                if (file.isDirectory())
                                    return true;
                                for (String suffix:CTFileIO.suffixes)
                                    if (file.getName().endsWith(suffix))
                                        return true;
                                return false;
                            }

                            public String getDescription() {
                                return "CT files";
                            }
                        });
                        if (fileChooser.showSaveDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                            final java.io.File file = fileChooser.getSelectedFile();
                            new ParadiseTask(S2S.this,messageBar) {
                                protected Object doInBackground()  {
                                    try {
                                        messageBar.startAnimation();
                                        if (!file.exists())
                                            file.createNewFile();
                                        List<fr.unistra.ibmc.paradise.core.utils.Residue> residues = new ArrayList<fr.unistra.ibmc.paradise.core.utils.Residue>();
                                        Set<BaseBaseInteraction>  interactions = new HashSet<BaseBaseInteraction>();
                                        for (fr.unistra.ibmc.s2s.ssviewer.features.Residue2D r2D: mediator.getRna2dViewer().getSecondaryCanvas().getCurrentResidues()) {
                                            residues.add(r2D.getResidue2DFeature().getResidues().iterator().next());
                                            for (Interaction2D interaction:r2D.getInteractions())
                                                if (BaseBaseInteraction2D.class.isInstance(interaction) && ((BaseBaseInteraction2D)interaction).getBaseBaseInteractionFeature().isSecondaryInteraction()) /*a ct file stores only secondary interactions, not tertiary ones)*/
                                                    interactions.add(((BaseBaseInteraction2D)interaction).getBaseBaseInteractionFeature());
                                        }
                                        Collections.sort(residues, new Comparator() {

                                            public int compare(Object o1, Object o2) {
                                                return ((fr.unistra.ibmc.paradise.core.utils.Residue)o1).getAbsolutePosition() - ((fr.unistra.ibmc.paradise.core.utils.Residue)o2).getAbsolutePosition();
                                            }
                                        });
                                        CTFileIO.writeCTFile(residues,new ArrayList(interactions),file);
                                        monitor.printMessage("Export successful");
                                    }
                                    catch (Exception e) {
                                        monitor.printException(e);
                                    }
                                    this.finished();
                                    return null;
                                }
                            }.execute();
                        }
                    }
                });
                selectionMenu.add(item);

                item = new JMenuItem("Quit");
                item.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent actionEvent) {
                        if (JOptionPane.showConfirmDialog(null, "Are you sure you want to exit S2S?", "Confirm exit", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                            try {
                                S2SConfig.save();
                                backup.delete();
                            } catch (BackingStoreException e1) {
                                e1.printStackTrace();
                            } catch (IOException e1) {
                                e1.printStackTrace();
                            }
                            System.exit(0);
                        }
                    }
                });
                this.add(item);
            }
        }

    }

    private class PluginsMenu extends JMenu {

        private PluginsMenu() {
            super("Plugins");
            File pluginsDir = new File(S2S.getPluginsPath());
            for (File pluginDir:pluginsDir.listFiles(new FileFilter() {
                public boolean accept(File file) {
                    return file.isDirectory();
                }
            })) {
                File script = new File(pluginDir,"Plugin.groovy");
                if (script.exists()) {
                    try {
                        Class pluginClass = groovyShell.getClassLoader().parseClass(script);
                        this.add(new PluginMenuItem((GroovyObject)pluginClass.newInstance()));
                    }
                    catch (Exception e) {
                        ErrorInfo errorInfo = new ErrorInfo(
                                "Error",
                                "Problem to load the plugin "+pluginDir,
                                null,
                                null,
                                e,
                                Level.SEVERE,
                                null);
                        JXErrorPane.showDialog(null, errorInfo);
                    }
                }
            }
        }

        private class PluginMenuItem extends JMenuItem {

            private GroovyObject plugin;

            private PluginMenuItem(final GroovyObject plugin) {
                super((String)plugin.invokeMethod("getTitle",null));
                this.setToolTipText((String)plugin.invokeMethod("getDescription",null));
                this.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent actionEvent) {
                        new SwingWorker() {
                            protected Object doInBackground(){
                                try {
                                    plugin.invokeMethod("run", new Object[]{S2S.this});
                                }
                                catch (Exception e) {
                                    ErrorInfo errorInfo = new ErrorInfo(
                                            "Error",
                                            "Problem to execute the plugin",
                                            null,
                                            null,
                                            e,
                                            Level.SEVERE,
                                            null);
                                    JXErrorPane.showDialog(null, errorInfo);
                                }
                                return null;
                            }
                        }.execute();

                    }
                });
            }
        }
    }

    private class ComputedAlignmentPanel extends JXPanel {

        private JComboBox moleculesChoice;

        private ComputedAlignmentPanel() {
            this.setBackground(Color.WHITE);
            this.setLayout(new VerticalLayout(5));
            this.moleculesChoice = new JComboBox();
            this.moleculesChoice.addItemListener(new ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    mediator.getAlignmentCanvas().setMoleculeToCompare(((BiologicalSequenceItem)e.getItem()).seq.getMolecule());
                    mediator.getAlignmentCanvas().repaint();
                }
            });
            JXPanel subPanel = new JXPanel(new GridBagLayout());
            subPanel.setBackground(Color.WHITE);
            subPanel.add(new JLabel("Compare"), new GridBagConstraints(0,1,1,1,1.0,0.0,GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
            subPanel.add(this.moleculesChoice, new GridBagConstraints(1,1,1,1,0.0,0.0,GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));
            this.add(subPanel);
        }

        private void update() {
            moleculesChoice.removeAllItems();
            if (mediator.getAlignmentCanvas().getMainAlignment() != null)
                for (BiologicalSequence s: mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequences())
                    moleculesChoice.addItem(new BiologicalSequenceItem(s));
        }

        private class BiologicalSequenceItem {
            private BiologicalSequence seq;

            public BiologicalSequenceItem(BiologicalSequence seq) {
                this.seq = seq;
            }

            public String toString() {
                return "S"+(mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequences().indexOf(seq)+1);
            }
        }
    }

    private class TertiaryViewersToolBar extends JToolBar {

        public static final byte PYMOL = 1, VMD = 2, CHIMERA = 3;

        private TertiaryViewersToolBar() {
            this.setBackground(Color.WHITE);
            this.setFloatable(false);
            JLabel toolBarName = new JLabel();
            toolBarName.setIcon(new VTextIcon(toolBarName, "3D", VTextIcon.ROTATE_LEFT));
            this.add(toolBarName);
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(new TertiaryViewerState());

            int height = new ImageIcon(RessourcesUtils.getImage("unselect.png")).getIconHeight();
            JSeparator separator = new JSeparator(SwingConstants.VERTICAL);
            separator.setBackground(Color.GRAY);
            separator.setMaximumSize(new Dimension(1, height));
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(separator);
            this.add(javax.swing.Box.createHorizontalStrut(5));

            ActionButton b = new ActionButton(null,"Show All residues",new ImageIcon(RessourcesUtils.getImage("molecule.png")),new ActionListener() {
                public void actionPerformed(ActionEvent actionEvent) {
                    for (TertiaryViewerDriver driver:drivers)
                        driver.evaluate("show all");
                }
            });
            this.add(b);
            b = new ActionButton(null,"Hide All residues",new ImageIcon(RessourcesUtils.getImage("unselect.png")) , new ActionListener() {
                public void actionPerformed(ActionEvent actionEvent) {
                    for (TertiaryViewerDriver driver:drivers)
                        driver.evaluate("hide all");
                }
            });
            this.add(b);

            separator = new JSeparator(SwingConstants.VERTICAL);
            separator.setBackground(Color.GRAY);
            separator.setMaximumSize(new Dimension(1, height));
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(separator);
            this.add(javax.swing.Box.createHorizontalStrut(5));

            b = new ActionButton(null, "Load tertiary structure with PyMOL", new ImageIcon(RessourcesUtils.getImage("pymol.jpg")), new ActionListener() {
                public void actionPerformed(ActionEvent actionEvent) {

                }
            });
            b.addMouseListener(new MouseAdapter() {
                @Override
                public void mouseClicked(MouseEvent mouseEvent) {
                    if (S2S.this.getAlignmentCanvas().getMainAlignment() != null) {
                        List<TertiaryStructure> tss = S2S.this.getAlignmentCanvas().getMainAlignment().getReferenceStructure().getParentFeatures(TertiaryStructure.class);
                        if (!tss.isEmpty()) {
                            TertiaryStructure ts = tss.get(0);
                            TertiaryViewerDriver driver = null;
                            if (mouseEvent.isShiftDown() || S2SConfig.getPyMOLPath().length()==0) {
                                String path = JOptionPane.showInputDialog("Enter the absolute path of your PyMOL application", S2SConfig.getPyMOLPath());
                                if (path != null && path.length() != 0) {
                                    S2SConfig.setPyMOLPath(path);
                                    driver = new PyMOLDriver(mediator);
                                }
                            }
                            else
                                driver = new PyMOLDriver(mediator);
                            if (driver != null) {
                                drivers.add(driver);
                                mediator.getAlignmentCanvas().addSelectionListener(driver);
                                driver.addSelectionListener(mediator.getAlignmentCanvas());
                                mediator.getRna2dViewer().addSelectionListener(driver);
                                driver.addSelectionListener(mediator.getRna2dViewer());
                                mediator.getSecondaryStructureNavigator().addSelectionListener(driver);
                                driver.addSelectionListener(mediator.getSecondaryStructureNavigator());
                                new LoadTertiaryStructureTask(S2S.this,ts).execute();
                            }
                        }
                        else
                            JOptionPane.showMessageDialog(S2S.this.window,"Your structural alignement is not linked to a tertiary structure");
                    }
                    else
                        JOptionPane.showMessageDialog(S2S.this.window,"No structural alignement loaded in S2S");
                }
            });
            this.add(b);

            b = new ActionButton(null, "Load tertiary structure with Chimera", new ImageIcon(RessourcesUtils.getImage("chimera.jpg")), new ActionListener() {
                public void actionPerformed(ActionEvent actionEvent) {

                }
            });
            b.addMouseListener(new MouseAdapter() {
                @Override
                public void mouseClicked(MouseEvent mouseEvent) {
                    if (S2S.this.getAlignmentCanvas().getMainAlignment() != null) {
                        List<TertiaryStructure> tss = S2S.this.getAlignmentCanvas().getMainAlignment().getReferenceStructure().getParentFeatures(TertiaryStructure.class);
                        if (!tss.isEmpty()) {
                            TertiaryStructure ts = tss.get(0);
                            TertiaryViewerDriver driver = null;
                            if (mouseEvent.isShiftDown() || S2SConfig.getChimeraPath().length()==0) {
                                String path = JOptionPane.showInputDialog("Enter the absolute path of your Chimera application", S2SConfig.getChimeraPath());
                                if (path != null && path.length() != 0) {
                                    S2SConfig.setChimeraPath(path);
                                    driver = new ChimeraDriver(mediator);
                                }
                            }
                            else
                                driver = new ChimeraDriver(mediator);
                            if (driver != null) {
                                drivers.add(driver);
                                mediator.getAlignmentCanvas().addSelectionListener(driver);
                                driver.addSelectionListener(mediator.getAlignmentCanvas());
                                mediator.getRna2dViewer().addSelectionListener(driver);
                                driver.addSelectionListener(mediator.getRna2dViewer());
                                mediator.getSecondaryStructureNavigator().addSelectionListener(driver);
                                driver.addSelectionListener(mediator.getSecondaryStructureNavigator());
                                new LoadTertiaryStructureTask(S2S.this,ts).execute();
                            }
                        }
                        else
                            JOptionPane.showMessageDialog(S2S.this.window,"Your structural alignement is not linked to a tertiary structure");
                    }
                    else
                        JOptionPane.showMessageDialog(S2S.this.window,"No structural alignement loaded in S2S");
                }
            });
            //this.add(b);

            this.add(javax.swing.Box.createHorizontalGlue());
        }

        private class TertiaryViewerState extends JComboBox implements ActionListener {

            ImageIcon[] images = {new ImageIcon(RessourcesUtils.getImage("select.png")),new ImageIcon(RessourcesUtils.getImage("center-view.png")),new ImageIcon(RessourcesUtils.getImage("zoom.png")),new ImageIcon(RessourcesUtils.getImage("show.png")),new ImageIcon(RessourcesUtils.getImage("unselect.png"))};


            TertiaryViewerState() {
                super(new Integer[]{TertiaryViewerDriver.SELECT, TertiaryViewerDriver.CENTER, TertiaryViewerDriver.ZOOM, TertiaryViewerDriver.SHOW, TertiaryViewerDriver.HIDE});
                this.setBackground(Color.WHITE);
                ComboBoxRenderer renderer= new ComboBoxRenderer();
                this.setRenderer(renderer);
                this.setToolTipText("Current state of the 3D viewers");
                this.addActionListener(this);
            }

            @Override
            public Dimension getPreferredSize() {
                return new Dimension(42,22);
            }

            @Override
            public Dimension getMaximumSize() {
                return this.getPreferredSize();
            }

            public void actionPerformed(ActionEvent e) {
                for (TertiaryViewerDriver driver:drivers)
                    driver.setCurrentState(this.getSelectedIndex());
            }


            private class ComboBoxRenderer extends JLabel implements ListCellRenderer {
                private Font uhOhFont;

                public ComboBoxRenderer() {
                    setOpaque(true);
                    setHorizontalAlignment(CENTER);
                    setVerticalAlignment(CENTER);
                }

                /*
                * This method finds the image and text corresponding
                * to the selected value and returns the label, set up
                * to display the text and image.
                */
                public Component getListCellRendererComponent(
                        JList list,
                        Object value,
                        int index,
                        boolean isSelected,
                        boolean cellHasFocus) {
                    //Get the selected index. (The index param isn't
                    //always valid, so just use the value.)
                    int selectedIndex = ((Integer)value).intValue();

                    if (isSelected) {
                        setBackground(list.getSelectionBackground());
                        setForeground(list.getSelectionForeground());
                    } else {
                        setBackground(list.getBackground());
                        setForeground(list.getForeground());
                    }

                    //Set the icon and text.  If icon was null, say so.
                    ImageIcon icon = images[selectedIndex];
                    setIcon(icon);

                    return this;
                }

                //Set the font and text when no image was found.
                protected void setUhOhText(String uhOhText, Font normalFont) {
                    if (uhOhFont == null) { //lazily create this font
                        uhOhFont = normalFont.deriveFont(Font.ITALIC);
                    }
                    setFont(uhOhFont);
                    setText(uhOhText);
                }
            }

        }

        private class LoadTertiaryStructureTask extends ParadiseTask {

            private TertiaryStructure ts;


            private LoadTertiaryStructureTask(Application app, TertiaryStructure ts) {
                super(app, messageBar);
                this.ts = ts;
            }

            @Override
            protected Void doInBackground() {
                try {
                    File tmp = File.createTempFile(this.ts.getName(),".pdb");
                    PDBFileIO.writePDBFile(this.ts.getResidues3D(), true,tmp);
                    for (TertiaryViewerDriver driver:drivers) {
                        driver.loadTertiaryStructure(tmp);
                        driver.addTertiaryStructureDisplayed(this.ts);
                    }
                } catch (IOException e) {
                    this.finished();
                    JOptionPane.showMessageDialog(window,
                            "Problem to create a temporary PDB file",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);
                }
                catch (InterruptedException e) {

                }
                this.finished();
                return null;
            }
        }

    }

    private class LWMatricesPanel extends JXPanel {
        public LWMatricesPanel() throws HeadlessException {
            this.setLayout(new VerticalLayout(10));
            this.setBackground(Color.WHITE);
            this.add(new SymetricFamilyPanel("CWW",new ImageIcon(RessourcesUtils.getImage("CisWCWC.png"))));
            this.add(new SymetricFamilyPanel("TWW",new ImageIcon(RessourcesUtils.getImage("TransWCWC.png"))));
            this.add(new SymetricFamilyPanel("CHH",new ImageIcon(RessourcesUtils.getImage("CisHH.png"))));
            this.add(new SymetricFamilyPanel("THH",new ImageIcon(RessourcesUtils.getImage("TransHH.png"))));
            this.add(new FamilyPanel("CWH",new ImageIcon(RessourcesUtils.getImage("CisHWC.png"))));
            this.add(new FamilyPanel("TWH",new ImageIcon(RessourcesUtils.getImage("TransHWC.png"))));
            this.add(new FamilyPanel("CHS",new ImageIcon(RessourcesUtils.getImage("CisHSE.png"))));
            this.add(new FamilyPanel("THS",new ImageIcon(RessourcesUtils.getImage("TransHSE.png"))));
            this.add(new FamilyPanel("CWS",new ImageIcon(RessourcesUtils.getImage("CisSEWC.png"))));
            this.add(new FamilyPanel("TWS",new ImageIcon(RessourcesUtils.getImage("TransSEWC.png"))));
            this.add(new SymetricFamilyPanel("CSS",new ImageIcon(RessourcesUtils.getImage("CisSESE.png"))));
            this.add(new SymetricFamilyPanel("TSS",new ImageIcon(RessourcesUtils.getImage("TransSESE.png"))));
        }

        private class FamilyPanel extends JXPanel {
            public FamilyPanel(String lwFamily,final javax.swing.Icon icon) {
                this.setLayout(new GridLayout(5,5));
                this.setBackground(Color.WHITE);
                JLabel f = new JLabel(icon,JLabel.CENTER);
                f.setBackground(Color.WHITE);
                this.add(f);
                //columns
                this.add(new JLabel("A",JLabel.CENTER));
                this.add(new JLabel("C",JLabel.CENTER));
                this.add(new JLabel("G",JLabel.CENTER));
                this.add(new JLabel("U",JLabel.CENTER));
                //rows
                this.add(new JLabel("A",JLabel.CENTER));
                this.add(new IsostericityFamilyButton("A",lwFamily,"A"));
                this.add(new IsostericityFamilyButton("A",lwFamily,"C"));
                this.add(new IsostericityFamilyButton("A",lwFamily,"G"));
                this.add(new IsostericityFamilyButton("A",lwFamily,"U"));
                this.add(new JLabel("C",JLabel.CENTER));
                this.add(new IsostericityFamilyButton("C",lwFamily,"A"));
                this.add(new IsostericityFamilyButton("C",lwFamily,"C"));
                this.add(new IsostericityFamilyButton("C",lwFamily,"G"));
                this.add(new IsostericityFamilyButton("C",lwFamily,"U"));
                this.add(new JLabel("G",JLabel.CENTER));
                this.add(new IsostericityFamilyButton("G",lwFamily,"A"));
                this.add(new IsostericityFamilyButton("G",lwFamily,"C"));
                this.add(new IsostericityFamilyButton("G",lwFamily,"G"));
                this.add(new IsostericityFamilyButton("G",lwFamily,"U"));
                this.add(new JLabel("U",JLabel.CENTER));
                this.add(new IsostericityFamilyButton("U",lwFamily,"A"));
                this.add(new IsostericityFamilyButton("U",lwFamily,"C"));
                this.add(new IsostericityFamilyButton("U",lwFamily,"G"));
                this.add(new IsostericityFamilyButton("U",lwFamily,"U"));
            }
        }

        private class SymetricFamilyPanel extends JXPanel {
            public SymetricFamilyPanel(String lwFamily,final javax.swing.Icon icon) {
                this.setLayout(new GridLayout(5,5));
                this.setBackground(Color.WHITE);
                JLabel f = new JLabel(icon,JLabel.CENTER);
                f.setBackground(Color.WHITE);
                this.add(f);
                //columns
                this.add(new JLabel("A",JLabel.CENTER));
                this.add(new JLabel("C",JLabel.CENTER));
                this.add(new JLabel("G",JLabel.CENTER));
                this.add(new JLabel("U",JLabel.CENTER));
                //rows
                this.add(new JLabel("A",JLabel.CENTER));
                IsostericityFamilyButton    AC = new IsostericityFamilyButton("A",lwFamily,"C"),
                        CA = new IsostericityFamilyButton("C",lwFamily,"A"),
                        AG = new IsostericityFamilyButton("A",lwFamily,"G"),
                        GA = new IsostericityFamilyButton("G",lwFamily,"A"),
                        AU = new IsostericityFamilyButton("A",lwFamily,"U"),
                        UA = new IsostericityFamilyButton("U",lwFamily,"A"),
                        CG = new IsostericityFamilyButton("C",lwFamily,"G"),
                        GC = new IsostericityFamilyButton("G",lwFamily,"C"),
                        CU = new IsostericityFamilyButton("C",lwFamily,"U"),
                        UC = new IsostericityFamilyButton("U",lwFamily,"C"),
                        GU = new IsostericityFamilyButton("G",lwFamily,"U"),
                        UG = new IsostericityFamilyButton("U",lwFamily,"G");
                AC.button = CA;
                CA.button = AC;
                AG.button = GA;
                GA.button = AG;
                AU.button = UA;
                UA.button = AU;
                CG.button = GC;
                GC.button = CG;
                CU.button = UC;
                UC.button = CU;
                GU.button = UG;
                UG.button = GU;

                this.add(new IsostericityFamilyButton("A",lwFamily,"A"));
                this.add(AC);
                this.add(AG);
                this.add(AU);
                this.add(new JLabel("C",JLabel.CENTER));
                this.add(CA);
                this.add(new IsostericityFamilyButton("C",lwFamily,"C"));
                this.add(CG);
                this.add(CU);
                this.add(new JLabel("G",JLabel.CENTER));
                this.add(GA);
                this.add(GC);
                this.add(new IsostericityFamilyButton("G",lwFamily,"G"));
                this.add(GU);
                this.add(new JLabel("U",JLabel.CENTER));
                this.add(UA);
                this.add(UC);
                this.add(UG);
                this.add(new IsostericityFamilyButton("U",lwFamily,"U"));
            }
        }

        private class IsostericityFamilyButton extends JButton {
            private IsostericityFamilyButton button;
            private String family;

            private IsostericityFamilyButton(final String base1, final String family, final String base2) {
                if (BaseBaseInteraction.isostericFamiliesInstances.containsKey(base1+family+base2))
                    this.family = base1+family+base2;
                else if (BaseBaseInteraction.isostericFamiliesInstances.containsKey(base2+family+base1))
                    this.family = base2+family+base1;
                if (this.family != null)
                    this.setText(BaseBaseInteraction.isostericFamiliesInstances.get(this.family).split("-")[1]);
                this.setBackground(Color.WHITE);
                this.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (IsostericityFamilyButton.this.family != null) {
                            String newValue = JOptionPane.showInputDialog("Enter new value", IsostericityFamilyButton.this.getText());
                            if (newValue != null) {
                                BaseBaseInteraction.isostericFamiliesInstances.put(IsostericityFamilyButton.this.family,IsostericityFamilyButton.this.family.substring(1,IsostericityFamilyButton.this.family.length()-1)+"-"+newValue);
                                if (IsostericityFamilyButton.this.family.matches("SS")) {
                                    BaseBaseInteraction.isostericFamiliesInstances.put(IsostericityFamilyButton.this.family.replaceFirst("SS","sS"), IsostericityFamilyButton.this.family.substring(1,IsostericityFamilyButton.this.family.length()-1)+"-"+newValue);
                                    BaseBaseInteraction.isostericFamiliesInstances.put(IsostericityFamilyButton.this.family.replaceFirst("SS","Ss"), IsostericityFamilyButton.this.family.substring(1,IsostericityFamilyButton.this.family.length()-1)+"-"+newValue);
                                }
                                IsostericityFamilyButton.this.setText(newValue);
                                IsostericityFamilyButton.this.revalidate();
                                if (IsostericityFamilyButton.this.button != null) {
                                    IsostericityFamilyButton.this.button.setText(newValue);
                                    IsostericityFamilyButton.this.button.revalidate();
                                }
                                S2S.this.getAlignmentCanvas().recalculateConservationScores();
                            }
                        }
                    }
                });
            }
        }
    }

    public void setPreferencesPanel(boolean areSecondaryInteractionsDisplayed, boolean areTertiaryInteractionsDisplayed, boolean isLinkedToNextView, int sequencesDisplayed) {
        this.preferencesPanel.secondaryInteractionsDisplayed.setSelected(areSecondaryInteractionsDisplayed);
        this.preferencesPanel.tertiaryInteractionsDisplayed.setSelected(areTertiaryInteractionsDisplayed);
        this.preferencesPanel.linkedToNextView.setSelected(isLinkedToNextView);
        this.preferencesPanel.spinner.setValue(sequencesDisplayed);
    }

    public class PreferencesPanel extends JXTaskPaneContainer {

        private JCheckBox secondaryInteractionsDisplayed, tertiaryInteractionsDisplayed, linkedToNextView;
        private SequencesToDisplaySpinner spinner;

        public PreferencesPanel() {

            JXTaskPane navigationPane = new JXTaskPane();
            navigationPane.setTitle("Alignment View Navigation");
            navigationPane.add(new NavigatorPanel());
            this.add(navigationPane);

            JXTaskPane viewOptions = new JXTaskPane();
            viewOptions.setTitle("Alignment View Display");

            final JTextField location = new JTextField(10);
            JButton b = new JButton("Filter");
            b.setBackground(Color.WHITE);
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mediator.getSelectedAlignmentView() !=null )
                        try {
                            if (location.getText().trim().length() == 0)
                                mediator.getSelectedAlignmentView().getSelectedPositionsToDrawInteractions().clear();
                            else {
                                Location l = new Location(location.getText().trim());
                                //the user gives the location according to the alignment, we're searching the corresponding positions in the reference molecule
                                for (int i=0;i<mediator.getSelectedAlignmentView().getAlignment().getLength();i++) {
                                    int pos = mediator.getSelectedAlignmentView().getAlignment().getBiologicalReferenceSequence().getSymbol(i).getPositionInSequence();
                                    //if the position in alignment is inside the location given by the user and if the position in the reference sequence is not -1
                                    if (l.hasPosition(i+1) && pos != -1)
                                        mediator.getSelectedAlignmentView().getSelectedPositionsToDrawInteractions().add(pos);
                                }
                            }
                            mediator.getAlignmentCanvas().repaint();
                        } catch (BiologicalSymbolException e1) {
                            e1.printStackTrace();
                        }
                }
            });

            JXPanel locationPanel = new JXPanel(new GridBagLayout());
            locationPanel.add(location, new GridBagConstraints(0,1,1, 1, 1.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
            locationPanel.add(b, new GridBagConstraints(1,1,1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 5, 0, 5), 0, 0));

            viewOptions.add(locationPanel);

            b = new JButton("Restrict to the selection");
            b.setBackground(Color.WHITE);
            b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mediator.getSelectedAlignmentView() != null) {
                        BiologicalSequence ref = mediator.getSelectedAlignmentView().getAlignment().getBiologicalReferenceSequence();
                        for (int pos:ref.getSelectedPositions())
                            mediator.getSelectedAlignmentView().getSelectedPositionsToDrawInteractions().add(pos);
                        mediator.getAlignmentCanvas().repaint();
                    }

                }
            });

            JXPanel selectionPanel = new JXPanel(new GridBagLayout());
            selectionPanel.add(b, new GridBagConstraints(0,1,1, 1, 1.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
            viewOptions.add(selectionPanel);

            JXPanel checkBoxesPanel = new JXPanel();
            checkBoxesPanel.setLayout(new VerticalLayout());

            this.secondaryInteractionsDisplayed = new JCheckBox("Secondary Interactions") ;
            this.secondaryInteractionsDisplayed.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mediator.getSelectedAlignmentView() !=null )
                        mediator.getSelectedAlignmentView().setDisplaySecondaryInteractions(secondaryInteractionsDisplayed.isSelected());
                }
            });
            checkBoxesPanel.add(this.secondaryInteractionsDisplayed);

            this.tertiaryInteractionsDisplayed = new JCheckBox("Tertiary Interactions");
            this.tertiaryInteractionsDisplayed.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mediator.getSelectedAlignmentView() !=null )
                        mediator.getSelectedAlignmentView().setDisplayTertiaryInteractions(tertiaryInteractionsDisplayed.isSelected());
                }
            });
            checkBoxesPanel.add(this.tertiaryInteractionsDisplayed);

            this.linkedToNextView = new JCheckBox("Linked to next view");
            this.linkedToNextView.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mediator.getSelectedAlignmentView() !=null)
                        if (linkedToNextView.isSelected()) {
                            int index = mediator.getAlignmentCanvas().getAlignmentViews().indexOf(mediator.getSelectedAlignmentView());
                            //if the view is not the last one in the structural alignment, we link it with the next one
                            if (index <= mediator.getAlignmentCanvas().getAlignmentViews().size()-2) {
                                AlignmentView nextView = mediator.getAlignmentCanvas().getAlignmentViews().get(index+1);
                                mediator.getSelectedAlignmentView().setNextView(nextView);
                            }
                        }
                        else {
                            mediator.getSelectedAlignmentView().setNextView(null);
                        }
                }
            });
            checkBoxesPanel.add(this.linkedToNextView);



            viewOptions.add(checkBoxesPanel);

            this.spinner = new SequencesToDisplaySpinner();
            JXPanel spinnerPanel = new JXPanel(new GridBagLayout());
            spinnerPanel.add(spinner,new GridBagConstraints(0,1,1, 1, 0.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
            spinnerPanel.add(new JLabel("sequences displayed"), new GridBagConstraints(1,1,1, 1, 1.0, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
            viewOptions.add(spinnerPanel);

            this.add(viewOptions);

            JXTaskPane lwPanel = new JXTaskPane();
            lwPanel.setTitle("Base-Base Interactions Display");

            JXPanel allButtonsPanel = new JXPanel();
            allButtonsPanel.setLayout(new VerticalLayout());
            LWFamilyButton lwFamilyButton;

            JXPanel lwFamilyButtonsPanel = new JXPanel();
            lwFamilyButton = new LWFamilyButton("CisWCWC");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            lwFamilyButton = new LWFamilyButton("TransWCWC");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            allButtonsPanel.add(lwFamilyButtonsPanel);

            lwFamilyButtonsPanel = new JXPanel();
            lwFamilyButton = new LWFamilyButton("CisSEWC");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            lwFamilyButton = new LWFamilyButton("TransSEWC");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            allButtonsPanel.add(lwFamilyButtonsPanel);

            lwFamilyButtonsPanel = new JXPanel();
            lwFamilyButton = new LWFamilyButton("CisHH");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            lwFamilyButton = new LWFamilyButton("TransHH");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            allButtonsPanel.add(lwFamilyButtonsPanel);

            lwFamilyButtonsPanel = new JXPanel();
            lwFamilyButton = new LWFamilyButton("CisHSE");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            lwFamilyButton = new LWFamilyButton("TransHSE");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            allButtonsPanel.add(lwFamilyButtonsPanel);

            lwFamilyButtonsPanel = new JXPanel();
            lwFamilyButton = new LWFamilyButton("CisSESE");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            lwFamilyButton = new LWFamilyButton("TransSESE");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            allButtonsPanel.add(lwFamilyButtonsPanel);

            lwFamilyButtonsPanel = new JXPanel();
            lwFamilyButton = new LWFamilyButton("CisHWC");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            lwFamilyButton = new LWFamilyButton("TransHWC");
            lwFamilyButtonsPanel.add(lwFamilyButton);
            allButtonsPanel.add(lwFamilyButtonsPanel);

            lwPanel.add(allButtonsPanel);

            this.add(lwPanel);

        }

        private class NavigatorPanel extends JXPanel {

            private NavigatorPanel() {
                JXPanel buttons = new JXPanel(new BorderLayout());
                buttons.setBackground(Color.WHITE);
                JButton b =  new JButton(new ImageIcon(RessourcesUtils.getImage("left.png")));
                b.setBackground(Color.WHITE);
                b.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (mediator.getAlignmentCanvas().getSelectedView()!=null)
                            mediator.getAlignmentCanvas().getSelectedView().moveViewOnLeft();
                    }
                });
                buttons.add(b, BorderLayout.WEST);

                b =  new JButton(new ImageIcon(RessourcesUtils.getImage("up.png")));
                b.setBackground(Color.WHITE);
                b.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (mediator.getAlignmentCanvas().getSelectedView()!=null)
                            mediator.getAlignmentCanvas().getSelectedView().setViewY(mediator.getAlignmentCanvas().getSelectedView().getViewY()+mediator.getAlignmentCanvas().getGraphicContext().getVerticalAdvance());
                    }
                });
                buttons.add(b, BorderLayout.NORTH);

                b =  new JButton(new ImageIcon(RessourcesUtils.getImage("down.png")));
                b.setBackground(Color.WHITE);
                b.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (mediator.getAlignmentCanvas().getSelectedView()!=null)
                            mediator.getAlignmentCanvas().getSelectedView().setViewY(mediator.getAlignmentCanvas().getSelectedView().getViewY()-mediator.getAlignmentCanvas().getGraphicContext().getVerticalAdvance());
                    }
                });
                buttons.add(b, BorderLayout.SOUTH);

                b =  new JButton(new ImageIcon(RessourcesUtils.getImage("right.png")));
                b.setBackground(Color.WHITE);
                b.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (mediator.getAlignmentCanvas().getSelectedView()!=null)
                            mediator.getAlignmentCanvas().getSelectedView().moveViewOnRight();
                    }
                });
                buttons.add(b, BorderLayout.EAST);
                buttons.setAlignmentY(Component.CENTER_ALIGNMENT);
                this.add(buttons);
            }
        }

        private class SequencesToDisplaySpinner extends JSpinner {

            private SequencesToDisplaySpinner() {
                this.addChangeListener(new ChangeListener() {
                    public void stateChanged(ChangeEvent e) {
                        if (mediator.getSelectedAlignmentView() != null)
                            mediator.getSelectedAlignmentView().setNumberOfSequencesToDisplay((Integer)SequencesToDisplaySpinner.this.getValue());
                    }
                });
            }
        }

        private class LWFamilyButton extends JButton {

            private String lwFamily;
            private boolean display = true;


            private LWFamilyButton(final String lwFamily) {
                this.lwFamily = lwFamily;
                this.setIcon(new ImageIcon(RessourcesUtils.getImage(this.lwFamily+".png")));
                this.setBackground(Color.WHITE);
                this.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        display = !display;
                        for (AlignmentView view:mediator.getAlignmentCanvas().getAlignmentViews()) {
                            try {
                                Method set = view.getClass().getMethod("setDisplay"+lwFamily,new Class[]{Boolean.class}),
                                        get =view.getClass().getMethod("isDisplay"+lwFamily,new Class[]{});
                                set.invoke(view,new Object[]{display});
                            } catch (NoSuchMethodException e1) {
                                e1.printStackTrace();
                            } catch (IllegalAccessException e1) {
                                e1.printStackTrace();
                            } catch (InvocationTargetException e1) {
                                e1.printStackTrace();
                            }
                        }
                        if (rna2dViewer.getSecondaryCanvas() != null) {
                            if (lwFamily.equals("CisSESE"))
                                rna2dViewer.getSecondaryCanvas().addFamily("CSS");
                            else if (lwFamily.equals("TransSESE"))
                                rna2dViewer.getSecondaryCanvas().addFamily("TSS");
                            else if (lwFamily.equals("CisHWC")) {
                                rna2dViewer.getSecondaryCanvas().addFamily("CHW");
                                rna2dViewer.getSecondaryCanvas().addFamily("CWH");
                            }
                            else if (lwFamily.equals("TransHWC")) {
                                rna2dViewer.getSecondaryCanvas().addFamily("THW");
                                rna2dViewer.getSecondaryCanvas().addFamily("TWH");
                            }
                            else if (lwFamily.equals("CisHSE")) {
                                rna2dViewer.getSecondaryCanvas().addFamily("CHS");
                                rna2dViewer.getSecondaryCanvas().addFamily("CSH");
                            }
                            else if (lwFamily.equals("THSE")) {
                                rna2dViewer.getSecondaryCanvas().addFamily("THS");
                                rna2dViewer.getSecondaryCanvas().addFamily("TSH");
                            }
                            else if (lwFamily.equals("CisSEWC")) {
                                rna2dViewer.getSecondaryCanvas().addFamily("CSW");
                                rna2dViewer.getSecondaryCanvas().addFamily("CWS");
                            }
                            else if (lwFamily.equals("TransSEWC")) {
                                rna2dViewer.getSecondaryCanvas().addFamily("TSW");
                                rna2dViewer.getSecondaryCanvas().addFamily("TWS");
                            }
                            else if (lwFamily.equals("TransSEWC")) {
                                rna2dViewer.getSecondaryCanvas().addFamily("TSW");
                                rna2dViewer.getSecondaryCanvas().addFamily("TWS");
                            }
                            else if (lwFamily.equals("CisWCWC")) {
                                rna2dViewer.getSecondaryCanvas().addFamily("CWW");
                            }
                            else if (lwFamily.equals("TransWCWC")) {
                                rna2dViewer.getSecondaryCanvas().addFamily("TWW");
                            }
                        }
                        if (display)
                            LWFamilyButton.this.setIcon(new ImageIcon(RessourcesUtils.getImage(LWFamilyButton.this.lwFamily+".png")));
                        else
                            LWFamilyButton.this.setIcon(new ImageIcon(RessourcesUtils.getImage(LWFamilyButton.this.lwFamily+"_hidden.png")));
                    }
                });
            }
        }
    }

    private class NewsPanel extends JXPanel {
        private static final String feedChannel = "http://serialized-thoughts.blogspot.com/feeds/posts/default";
        private List<Post> posts;
        private String lastPostID;
        private AbstractTableModel dataModel;

        private NewsPanel() {
            this.posts = new ArrayList<Post>();
            this.setLayout(new BorderLayout());
            this.dataModel = new AbstractTableModel() {
                public int getColumnCount() {
                    return 2;
                }

                public int getRowCount() {
                    return posts.size();
                }

                public Object getValueAt(int row, int col) {
                    Post p = NewsPanel.this.posts.get(row);
                    switch (col) {
                        case 0 : return p.title;
                        case 1 : return new SimpleDateFormat("yyyy.MM.dd HH:mm").format(p.date);
                        default : return null;
                    }
                }

                public String getColumnName(int col) {
                    switch (col) {
                        case 0 : return "Post Title";
                        case 1 : return "Publication Date";
                        default : return "?";
                    }
                }
            };
            final JXTable table =  new JXTable(dataModel);
            table.setEditable(false);
            table.setSortable(false);
            table.setColumnControlVisible(false);
            table.setHighlighters(new Highlighter[]{AlternateRowHighlighter.quickSilver});
            table.getColumnModel().getColumn(1).setMinWidth(200);
            table.getColumnModel().getColumn(1).setMaxWidth(200);
            table.getColumnModel().getColumn(1).setPreferredWidth(200);

            table.addMouseListener(new MouseAdapter() {

                public void mouseClicked(MouseEvent event) {
                    if (event.getClickCount() == 2)
                        GeneralUtils.openBrowser(posts.get(table.getSelectedRow()).url.toString());
                }

            });


            this.add(new JScrollPane(table), BorderLayout.CENTER);
            new SwingWorker() {
                protected Object doInBackground() throws Exception {
                    fetchRSSChannel();
                    new Timer(30*60000, new ActionListener() { //each 30 minutes
                        public void actionPerformed(ActionEvent actionEvent) {
                            try {
                                fetchRSSChannel();
                            } catch (FeedException e) {
                                e.printStackTrace();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }).start();
                    return null;
                }
            }.execute();
        }

        private void fetchRSSChannel() throws FeedException, IOException {
            SyndFeedInput sfi=new SyndFeedInput();
            SyndFeed feed= sfi.build(new XmlReader(new URL(feedChannel)));
            if (lastPostID == null)
                for (Object o : feed.getEntries()) {
                    SyndEntry entry = (SyndEntry)o;
                    Post p = new Post();
                    p.id = entry.getUri();
                    if (lastPostID == null)
                        lastPostID = p.id;
                    p.title = entry.getTitle();
                    p.url = new URL(entry.getLink());
                    p.date = entry.getPublishedDate();
                    posts.add(p);
                    dataModel.fireTableRowsInserted(posts.size()-1,posts.size()-1);
                }
            else {
                int i = 0;
                for (Object o : feed.getEntries()) {
                    SyndEntry entry = (SyndEntry)o;
                    if (lastPostID.equals(entry.getUri()))
                        break;
                    Post p = new Post();
                    p.id = entry.getUri();
                    if (lastPostID == null)
                        lastPostID = p.id;
                    p.title = entry.getTitle();
                    p.url = new URL(entry.getLink());
                    p.date = entry.getPublishedDate();
                    posts.add(i++,p);
                    dataModel.fireTableRowsInserted(i-1,i-1);
                }
                if (i > 0) {//meaning that fresh news have been posted
                    lastPostID = posts.get(0).id;
                    toolWindowManager.getToolWindow("News").setFlashing(true);
                }
            }
        }

        private class Post {
            private String title, id;
            private URL url;
            private Date date;
        }
    }

    private class OpenWebPageItem extends JMenuItem implements ActionListener {

        private String url;

        public OpenWebPageItem(String webPageTitle, String url) {
            super(webPageTitle);
            this.url = url;
            this.addActionListener(this);
        }

        public void actionPerformed(ActionEvent actionEvent) {
            GeneralUtils.openBrowser(this.url);
        }
    }

    public static String getUserDirPath() {
        return new StringBuffer(System.getProperty("user.home")).append(System.getProperty("file.separator")).append(".s2s").append(System.getProperty("file.separator")).toString();
    }

    public FileParsingTask parseFile(File f, ParadiseFileIO parser) {
        FileParsingTask task = new FileParsingTask(this,f,parser,this.messageBar,this.workingSession);
        task.execute();
        return task;
    }


    public void save(final File paradiseProject) {
        final List<MainParadiseFeature> features = new ArrayList<MainParadiseFeature>();
        features.add(this.getCurrentFeature());
        ParadiseTask t = new ParadiseTask(this,messageBar) {
            protected Object doInBackground() {
                try {
                    this.setMessage("Synchronize data");
                    synchronize();
                    this.setMessage("End of synchronization");
                    this.setMessage("Save data");
                    ParadiseProjectIO.saveWorkingSession(paradiseProject,features);
                    this.finished();
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                } catch (InterruptedException e) {
                }
                return null;
            }
        };
        t.execute();
    }

    public void saveAs(final File paradiseProject) {
        String name = JOptionPane.showInputDialog("Please choose a name for your alignment", getCurrentFeature().getName());
        if (name != null && name.length()!=0) {
            final File f = getCurrentFeature().getAssociatedFile();  //we receover this information to reload the data into the working session
            getCurrentFeature().setName(name);
            final List<MainParadiseFeature> features = new ArrayList<MainParadiseFeature>();
            features.add(getCurrentFeature());
            new ParadiseTask(S2S.this, messageBar) {
                protected Object doInBackground() {
                    /*
                   try {
                       this.setMessage("Synchronize data");
                       synchronize();
                       this.setMessage("End of synchronization");
                       this.setMessage("Save data");
                       ParadiseProjectIO.saveWorkingSession(paradiseProject,features);
                       if (f!=null) { // if this feature was already attached to a file in the current project
                           //since the user can only open one paradise project at a time, the paradis project parser provided by paradise has kept the memory of all the molecules loaded for the current project
                           //consequently, we can use it to reload the file describing the current main paradise feature before the renaming and the generation of the new file
                           //the molecules of this file should not be reinjected into the working session
                           Paradise.getParadise().getParadiseProjectParser().parseRnamlFile(f);
                       }
                       this.finished();
                   } catch (FileNotFoundException e) {
                       e.printStackTrace();
                   } catch (InterruptedException e) {

                   } catch (FileParsingException e) {
                       e.printStackTrace();
                   } catch (IOException e) {
                       e.printStackTrace();
                   } catch (BiologicalSymbolException e) {
                       e.printStackTrace();
                   } catch (JDOMException e) {
                       e.printStackTrace();
                   } */
                    return null;
                }
            }.execute();
        }
        else
            JOptionPane.showMessageDialog(null, "This name is not correct, please retry", "Error", JOptionPane.ERROR_MESSAGE);
    }

    class MessageBar extends JToolBar implements fr.unistra.ibmc.paradise.tools.component.ProgressMonitor {

        private javax.swing.JTextField message;
        private JProgressBar progressBar;
        private MemoryMonitor memoryMonitor;
        private JLabel warning, connectionStatus, state;
        private Exception currentException;
        private String messageAssociatedToTheException;
        private Timer timer;
        private Icon exclamation = new ImageIcon(RessourcesUtils.getImage("exclamation.png")), noExclamation =  new ImageIcon(RessourcesUtils.getImage("no-exclamation.png"));

        MessageBar() {
            this.setFloatable(false);
            this.setBackground(Color.white);
            this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
            this.timer = new Timer(500,new ActionListener() {
                private boolean b = true;
                public void actionPerformed(ActionEvent actionEvent) {
                    if (this.b)
                        warning.setIcon(exclamation);
                    else
                        warning.setIcon(noExclamation);
                    this.b = !this.b;
                }
            });
            this.message = new javax.swing.JTextField("S2S ready...");
            this.message.setBackground(Color.white);
            this.message.setEditable(false);
            this.add(this.message);
            this.add(Box.createRigidArea(new Dimension(20,0)));
            this.progressBar = new JProgressBar() {
                public Dimension getMaximumSize() {
                    return new Dimension(150, 20);
                }

                public Dimension getMinimumSize() {
                    return this.getMaximumSize();
                }

                public Dimension getPreferredSize() {
                    return this.getMaximumSize();
                }
            };
            this.add(this.progressBar);
            this.add(Box.createRigidArea(new Dimension(20,0)));
            this.state = new JLabel(new ImageIcon(RessourcesUtils.getImage("move-view.png")));
            this.add(this.state);
            this.warning = new JLabel(noExclamation);
            this.warning.addMouseListener(new MouseAdapter() {

                public void mouseClicked(MouseEvent e) {
                    ErrorInfo errorInfo = new ErrorInfo(
                            "Error",
                            messageAssociatedToTheException,
                            null,
                            null,
                            currentException,
                            Level.SEVERE,
                            null);
                    timer.stop();
                    warning.setIcon(null);
                    currentException = null;
                    messageAssociatedToTheException = null;
                    JXErrorPane.showDialog(null, errorInfo);
                }
            });
            this.add(this.warning);
            if (this.currentException == null)
                this.warning.setToolTipText("No errors");
            else
                this.warning.setToolTipText("Click for error details");
            this.add(Box.createRigidArea(new Dimension(5,0)));
            this.connectionStatus = new JLabel(new ImageIcon(S2S.this.isConnected() ? RessourcesUtils.getImage("connect.png"):RessourcesUtils.getImage("disconnect.png")));
            if (S2S.this.isConnected())
                this.connectionStatus.setToolTipText("Connected to "+S2S.this.workingSession.getPlatformAddress());
            else
                this.connectionStatus.setToolTipText("Not connected");
            this.add(this.connectionStatus);
            this.add(Box.createRigidArea(new Dimension(20,0)));
            this.memoryMonitor = new MemoryMonitor();
            this.add(this.memoryMonitor);
            this.memoryMonitor.start();
        }

        public void printMessage(String message) {
            timer.stop();
            warning.setIcon(null);
            currentException = null;
            messageAssociatedToTheException = null;
            this.message.setText(message);
        }

        public void printException(Exception e) {
            timer.stop();
            warning.setIcon(null);
            currentException = null;
            messageAssociatedToTheException = null;
            if (NoSolutionException.class.isInstance(e)) {
                JOptionPane.showMessageDialog(S2S.this.window, e.getMessage());
            }
            else {
                this.message.setText(e.getMessage());
                this.currentException = e;
                this.messageAssociatedToTheException = e.getMessage();
                this.timer.start();
            }
        }

        public void printState(final String state) {
            if ("insert".equals(state))
                this.state.setIcon(new ImageIcon(RessourcesUtils.getImage("insert-gap.png")));
            else if ("delete".equals(state))
                this.state.setIcon(new ImageIcon(RessourcesUtils.getImage("delete-gap.png")));
            else if ("move".equals(state))
                this.state.setIcon(new ImageIcon(RessourcesUtils.getImage("move-gap.png")));
            else if ("standard".equals(state))
                this.state.setIcon(new ImageIcon(RessourcesUtils.getImage("move-view.png")));
            else if ("user-defined-interactions".equals(state))
                this.state.setIcon(new ImageIcon(RessourcesUtils.getImage("CisHSE.png")));
        }

        public void startAnimation() {
            this.progressBar.setIndeterminate(true);
        }

        public void stopAnimation() {
            this.progressBar.setIndeterminate(false);
        }
    }

    private class ActionButton extends javax.swing.JButton implements MouseListener {

        private ActionButton(final String name, final String toolTipText, final javax.swing.Icon image, final ActionListener listener) {
            super(name, image);
            this.setMargin(new Insets(0, 0, 0, 0));
            this.setBorderPainted(false);
            this.setBackground(Color.WHITE);
            this.addActionListener(listener);
            this.addMouseListener(this);
            this.setBorderPainted(false);
            this.setToolTipText(toolTipText);
            this.setBackground(Color.WHITE);
        }

        public void mouseClicked(MouseEvent e) {
        }

        public void mousePressed(MouseEvent e) {
        }

        public void mouseReleased(MouseEvent e) {
        }

        public void mouseEntered(MouseEvent e) {
            this.setBackground(Color.lightGray);
            this.setBorderPainted(true);
        }

        public void mouseExited(MouseEvent e) {
            this.setBackground(Color.white);
            this.setBorderPainted(false);
        }
    }

    private class S2SToolBar extends JToolBar {

        private Mediator mediator;

        S2SToolBar(final Mediator mediator) {
            this.mediator = mediator;
            int height = new ImageIcon(RessourcesUtils.getImage("plus.png")).getIconHeight();
            this.setBackground(Color.WHITE);
            this.setFloatable(false);
            JLabel toolBarName = new JLabel();
            toolBarName.setIcon(new VTextIcon(toolBarName, "AL", VTextIcon.ROTATE_LEFT));
            this.add(toolBarName);
            this.add(javax.swing.Box.createHorizontalStrut(5));

            this.add(new ActionButton(null, "Create a view", new ImageIcon(RessourcesUtils.getImage("new-view.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (mediator.getAlignmentCanvas().getMainAlignment()!=null)
                        mediator.getAlignmentCanvas().addView();
                }
            }));
            this.add(new ActionButton(null, "Delete last view", new ImageIcon(RessourcesUtils.getImage("delete-view.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (mediator.getAlignmentCanvas().getAlignmentViews().size() > 0) {
                        AlignmentView lastView = mediator.getAlignmentCanvas().getAlignmentViews().get(mediator.getAlignmentCanvas().getAlignmentViews().size()-1);
                        //remove the potential linked with the previous AlignmentView
                        if (mediator.getAlignmentCanvas().getAlignmentViews().size() > 1 ) {
                            AlignmentView beforeLastView = mediator.getAlignmentCanvas().getAlignmentViews().get(mediator.getAlignmentCanvas().getAlignmentViews().size()-2);
                            beforeLastView.setNextView(null);
                        }
                        mediator.getAlignmentCanvas().removeLastView();
                    }
                }
            }));
            this.add(new ActionButton(null, "Store current selection", new ImageIcon(RessourcesUtils.getImage("user.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    Map<BiologicalSequence, List<Integer>> selectedPositions = mediator.getAlignmentCanvas().getSelectedPositions();
                    if (!selectedPositions.isEmpty()) {
                        String name = JOptionPane.showInputDialog(S2S.this.window, "Choose a selection name");
                        if ( name == null || name.length() == 0)
                            name ="No Name";
                        UserSelection us = new UserSelection(mediator,name);
                        for (BiologicalSequence s:mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequences())
                            for (Integer i:s.getSelectedPositions())
                                us.addResidue(new Residue(i,s.getMolecule()));
                        mediator.addUserSelection(us);

                    }
                }
            }));
            this.add(new ActionButton(null, "Clear selected positions", new ImageIcon(RessourcesUtils.getImage("unselect.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (mediator.getAlignmentCanvas().getMainAlignment() != null) {
                        mediator.getAlignmentCanvas().clearSelectedPositions();
                        mediator.getAlignmentCanvas().fireResiduesSelected(new ArrayList<Residue>(), false);
                    }
                }
            }));
            JSeparator separator = new JSeparator(SwingConstants.VERTICAL);
            separator.setBackground(Color.GRAY);
            separator.setMaximumSize(new Dimension(2, height));
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(separator);
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(new ActionButton(null, "Zoom in", new ImageIcon(RessourcesUtils.getImage("plus.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (mediator.getAlignmentCanvas()!=null)
                        mediator.getAlignmentCanvas().modifyFontSize(2);
                }
            }));
            this.add(new JLabel(new ImageIcon(RessourcesUtils.getImage("zoom.png"))));
            this.add(new ActionButton(null, "Zoom out", new ImageIcon(RessourcesUtils.getImage("minus.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (mediator.getAlignmentCanvas()!=null)
                        mediator.getAlignmentCanvas().modifyFontSize(-2);
                }
            }));

            this.add(new ActionButton(null, "Capture view in a PNG file", new ImageIcon(RessourcesUtils.getImage("capture.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(fr.unistra.ibmc.paradise.Paradise.getWorkingDirectory());
                    fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {

                        public boolean accept(File file) {
                            return file.getName().endsWith(".png") || file.isDirectory();
                        }

                        public String getDescription() {
                            return "PNG files";
                        }
                    });
                    if (fileChooser.showSaveDialog(S2SToolBar.this) == javax.swing.JFileChooser.APPROVE_OPTION) {
                        java.io.File file = null;
                        String extension = null;
                        java.io.PrintWriter pw = null;
                        try {
                            file = fileChooser.getSelectedFile();
                            extension = IoUtils.getExtension(file);
                            if (extension.equals("png")) {
                                file.createNewFile();
                                Paradise.setWorkingDirectory(file.getAbsolutePath());
                                ImageIO.write(mediator.getAlignmentCanvas().getImage(), "png", file);
                                JOptionPane.showMessageDialog(null,"Saving successful");
                            } else {
                                JOptionPane.showMessageDialog(null,
                                        "Not supported file type",
                                        "Warning",
                                        JOptionPane.WARNING_MESSAGE);
                            }
                        }
                        catch (FileNotFoundException ex) {
                            ex.printStackTrace();
                        }
                        catch (IOException ex) {
                            ex.printStackTrace();
                        }
                        finally {
                            if (pw != null)
                                pw.close();
                        }
                    }
                }
            }));
            final ActionButton undoButton =  new ActionButton(null, "Reject current alignment", new ImageIcon(RessourcesUtils.getImage("reject.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (mediator.getAlignmentCanvas()!=null) {
                        mediator.getAlignmentCanvas().undo();
                        mediator.setRedoUndoButtonsState();
                    }
                }
            });
            //this.add(undoButton);
            mediator.setUndoButton(undoButton);
            final ActionButton redoButton =  new ActionButton(null, "Accept current alignment", new ImageIcon(RessourcesUtils.getImage("accept.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (mediator.getAlignmentCanvas()!=null) {
                        mediator.getAlignmentCanvas().redo();
                        mediator.setRedoUndoButtonsState();
                    }
                }
            });
            //this.add(redoButton);
            mediator.setRedoButton(redoButton);
            JLabel l = new JLabel(new ImageIcon(RessourcesUtils.getImage("select.png")));
            l.setToolTipText("Type of RNA object to select");
            this.add(l);
            this.add(new SelectionMode(height));
            separator = new JSeparator(SwingConstants.VERTICAL);
            separator.setBackground(Color.GRAY);
            separator.setMaximumSize(new Dimension(2, height));
            this.add(javax.swing.Box.createHorizontalStrut(10));
            this.add(separator);
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(new Generate3DButton());
            //this.add(new DisplayButton());
        }

        void setMediator(final Mediator mediator) {
            this.mediator = mediator;
        }

        private class Generate3DButton extends JButton implements MouseListener {

            private Generate3DButton() {
                super(new ImageIcon(RessourcesUtils.getImage("generate3D.png")));
                this.setBackground(Color.white);
                this.setBorderPainted(false);
                this.addMouseListener(this);
                this.setToolTipText("Infer Assemble Model from Alignment");
                this.addActionListener(new ActionListener() {
                    public void actionPerformed(final ActionEvent e) {
                        new ParadiseTask(S2S.this,messageBar) {
                            @Override
                            protected Void doInBackground() throws InterruptedException {
                                if (mediator.getAlignmentCanvas()!= null && mediator.getAlignmentCanvas().getMainAlignment() != null ) {
                                    List<TertiaryStructure> tertiaryStructures = mediator.getAlignmentCanvas().getMainAlignment().getReferenceStructure().getParentFeatures(TertiaryStructure.class);
                                    if (mediator.getSequencesList().getSelectedRows().length != 1)  {
                                        JOptionPane.showMessageDialog(S2S.this.window,"Select a single sequence in the Sequences panel to use this option","Warning",JOptionPane.WARNING_MESSAGE);
                                        return null;
                                    }
                                    int s = mediator.getSequencesList().getSelectedRows()[0];
                                    if (s == 0) {//we don't do the job for the reference sequence
                                        JOptionPane.showMessageDialog(S2S.this.window,"You cannot use this option for the reference sequence","Warning",JOptionPane.WARNING_MESSAGE);
                                        return null;
                                    }
                                    S2S.this.getProgressMonitor().startAnimation();
                                    S2S.this.getProgressMonitor().printMessage("Assemble Model inference");
                                    Alignment alignment = mediator.getAlignmentCanvas().getMainAlignment();
                                    //we're updating the structural alignment model according to the current view
                                    alignment.updateStructuralAlignment();
                                    int start = 0, end = alignment.getLength()-1;
                                    /*
                              if (dialog.startField.getText().trim().length() != 0)
                                   start = Integer.parseInt(dialog.startField.getText().trim())-1;
                              if (dialog.endField.getText().trim().length() != 0)
                                  end = Integer.parseInt(dialog.endField.getText().trim())-1;*/
                                    BiologicalSequence  referenceBiologicalSequence = alignment.getBiologicalReferenceSequence(),
                                            targetBiologicalSequence = alignment.getBiologicalSequenceAt(s);
                                    Molecule targetMolecule = targetBiologicalSequence.getMolecule(),
                                            moleculeOfReference = referenceBiologicalSequence.getMolecule();
                                    StructuralFeatureFactory f = mediator.getS2s().createStructuralFeatureFactory();

                                    try {
                                        SecondaryStructure newSecondaryStructure = alignment.deduceNewSecondaryStructure(targetBiologicalSequence);
                                        TertiaryStructure newTertiaryStructure = f.createTertiaryStructure(new ParadiseID(), "Tertiary Structure", Source.getGraphicalToolAsSource(mediator.getS2s().getDescription()));
                                        if (newSecondaryStructure != null && !tertiaryStructures.isEmpty()) {
                                            TertiaryStructure referenceTertiaryStructure =  tertiaryStructures.get(0);
                                            if (!tertiaryStructures.isEmpty()) {
                                                for (int i=start; i<= end; i++) {
                                                    Symbol targetSymbol = targetBiologicalSequence.getSymbol(i),
                                                            referenceSymbol = referenceBiologicalSequence.getSymbol(i);
                                                    if (!targetSymbol.isGap() && !referenceSymbol.isGap()) {
                                                        Residue3D referenceResidue3D = referenceTertiaryStructure.getResidue3DAt(referenceSymbol.getPositionInSequence(),moleculeOfReference);
                                                        if (referenceResidue3D != null)
                                                            try {
                                                                newTertiaryStructure.getFactory().createResidue3D(newTertiaryStructure, referenceResidue3D, targetBiologicalSequence.getMolecule(), targetSymbol.getPositionInSequence());
                                                            } catch (BiologicalSymbolException e1) {
                                                                e1.printStackTrace();
                                                            }
                                                    };
                                                }
                                            }
                                        }
                                        //else the new tertiary structure will be empty
                                        newTertiaryStructure.linkSecondaryStructure(newSecondaryStructure);
                                        JFileChooser jFileChooser = new JFileChooser(getS2SLastSavePath());
                                        jFileChooser.setFileHidingEnabled(true);
                                        jFileChooser.setAcceptAllFileFilterUsed(false);
                                        jFileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {

                                            public boolean accept(File file) {
                                                return file.isDirectory() || file.getName().endsWith(".assemble");
                                            }

                                            public String getDescription() {
                                                return "Assemble models (binary data)";
                                            }
                                        });
                                        jFileChooser.setFileView(new FileView() {
                                            public Icon getIcon(File f) {
                                                if (IOUtils.isParadiseProject(f) || f.getName().endsWith(".assemble"))
                                                    return new ImageIcon(RessourcesUtils.getImage("paradise-project.png"));
                                                else return null;
                                            }

                                            public Boolean isTraversable(File f) {
                                                return f.isDirectory() && !IOUtils.isParadiseProject(f);
                                            }
                                        });
                                        if (jFileChooser.showSaveDialog(S2S.this.window) == JFileChooser.APPROVE_OPTION) {
                                            final File saveFile = jFileChooser.getSelectedFile();
                                            setS2SLastSavePath(jFileChooser.getSelectedFile());
                                            File _f = new File(saveFile.getAbsolutePath()+".assemble");
                                            saveFile.renameTo(_f);
                                            newTertiaryStructure.setAssociatedFile(_f);
                                            synchronize();
                                            ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(_f));
                                            oos.writeObject(newTertiaryStructure);
                                            oos.flush();
                                            oos.close();
                                            JOptionPane.showMessageDialog(S2S.this.window,"Assemble Model generated successfully","Great!!",JOptionPane.PLAIN_MESSAGE);
                                        }
                                    }
                                    catch (Exception e) {
                                        e.printStackTrace();
                                    }
                                    S2S.this.getProgressMonitor().stopAnimation();
                                    this.finished();
                                }
                                return null;
                            }

                        }.execute();
                    }
                });
            }

            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                this.setBackground(Color.lightGray);
                this.setBorderPainted(true);
            }

            public void mouseExited(MouseEvent e) {
                this.setBackground(Color.white);
                this.setBorderPainted(false);
            }
        }

        private class SelectionMode extends JComboBox {

            private SelectionItem[] selectionItems = {
                    new SelectionItem("Structural Domains", S2S.SELECT_STRUCTURAL_DOMAIN),
                    new SelectionItem("Interactions", S2S.SELECT_INTERACTION),
                    new SelectionItem("Residues", S2S.SELECT_RESIDUE)
            };

            SelectionMode(int height) {
                this.setMaximumSize(new Dimension(150, height));
                for (SelectionItem item :this.selectionItems)
                    this.addItem(item);
                this.setBackground(Color.WHITE);
                this.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        mediator.setSelectionMode(((SelectionItem) SelectionMode.this.getSelectedItem()).selectionMode);
                        mediator.getSecondaryCanvas().setState(((SelectionItem) SelectionMode.this.getSelectedItem()).selectionMode);
                    }
                });
                this.setEditable(false);
            }

            private class SelectionItem {

                private String label;
                private byte selectionMode;

                private SelectionItem(String label, byte selectionMode) {
                    this.label = label;
                    this.selectionMode = selectionMode;
                }

                public String toString() {
                    return this.label;
                }
            }
        }

    }

    public static String getInstallPath() {
        return S2S.class.getProtectionDomain().getCodeSource().getLocation().getPath().split("s2s.jar")[0];
    }

    public static void main(String[] args) throws BackingStoreException, IOException, JDOMException {
        try {
            UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
        } catch (InstantiationException e) {
        } catch (ClassNotFoundException e) {
        } catch (UnsupportedLookAndFeelException e) {
        } catch (IllegalAccessException e) {
        }
        S2SConfig.parseConfigFile();
        new SplashScreen();
    }

    static private class UpdateFrame extends JFrame {

        private JProgressBar progressBar;
        private JLabel title;
        private SwingWorker currentTask;
        private List<String> newFiles, filesToDelete;

        private UpdateFrame(List<Object> versions) throws IOException {
            super("S2S Update");
            this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            JPanel panel = new JPanel();
            panel.setLayout(new BorderLayout());
            this.progressBar = new JProgressBar(0, 100);
            this.progressBar.setValue(0);
            this.progressBar.setStringPainted(true);
            this.title = new JLabel("Searching first update...");
            panel.add(this.title, BorderLayout.NORTH);
            panel.add(this.progressBar, BorderLayout.CENTER);
            panel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
            panel.setPreferredSize(new Dimension(400,75));
            this.add(panel);
            this.pack();
            this.setResizable(false);
            GeneralUtils.centerOnScreen(this);
            this.setVisible(true);
            boolean foundCurrentVersion = false;
            for (int i = versions.size()-1; i >=0 ; i--) { // from the oldest to the most recent
                Element newVersion = (Element)versions.get(i);
                if (!foundCurrentVersion && newVersion.getChild("name").getText().equals(S2S.releaseName)) {
                    foundCurrentVersion = true;
                    continue;
                }
                else if (foundCurrentVersion) {
                    if (newVersion.getAttribute("password") != null) {
                        String  password = newVersion.getAttribute("password").getValue(),
                                userPassword = JOptionPane.showInputDialog(null,"Please enter a password to update to "+newVersion.getValue());
                        if (userPassword == null || !password.equals(userPassword)) {
                            JOptionPane.showMessageDialog(null, "Wrong password");
                            this.dispose();
                            return;
                        }
                    }
                    this.newFiles = new ArrayList<String>();
                    this.filesToDelete = new ArrayList<String>();
                    this.setTitle("Update to "+newVersion.getChild("name").getText());
                    for (Object _o:newVersion.getChild("files").getChildren("file")) {
                        Element file = (Element)_o;
                        if ("add".equals(file.getAttributeValue("action"))) {
                            this.title.setText("Add "+file.getChild("destination").getText());
                            this.updateFile(file.getChild("url").getText(), S2S.getInstallPath()+file.getChild("destination").getText().replace('/', System.getProperty("file.separator").charAt(0)));
                        }
                        else if ("delete".equals(file.getAttributeValue("action"))) {
                            this.title.setText("Delete "+file.getChild("destination").getText());
                            this.filesToDelete.add(S2S.getInstallPath()+file.getChild("destination").getText().replace('/', System.getProperty("file.separator").charAt(0)));
                        }
                    }
                    if (this.currentTask != null)
                        while (!this.currentTask.isDone()) {

                        }
                    //if everything done with success, we substitute the old files with the new ones just downloaded
                    for (String newFile:this.newFiles) {
                        new File(newFile).delete();
                        new File(newFile+"_tmp").renameTo(new File(newFile));
                        if (newFile.endsWith(".zip")) { //if the file is a zip one, we extract it in its current directory (not a jar, otherwise the update system will extract the jar libraries)
                            File f = new File(newFile);
                            IOUtils.extractArchivedFile(f.getAbsolutePath(),f);
                        }
                    }
                    for (String fileToDelete:filesToDelete) {
                        File f = new File(fileToDelete);
                        if (f.isDirectory())
                            IOUtils.deleteDirectory(f);
                        else
                            f.delete();
                    }
                }
            }
            JOptionPane.showMessageDialog(null, "S2S has been successfully updated to \""+((Element)versions.get(0)).getChild("name").getText()+"\"." +
                    "\nRestart it to use the new version." +
                    "\nRead the CHANGELOG file in the S2S directory for details about this update.");
            this.dispose();
        }

        private void updateFile(final String fileToDownload, final String destination) {
            if (this.currentTask != null)
                while (!this.currentTask.isDone()) {

                }
            this.currentTask = new SwingWorker() {
                public Void doInBackground() {

                    URLConnection connection = null;
                    InputStream is = null;
                    FileOutputStream destinationFile = null;

                    try {
                        URL url = new URL(fileToDownload);
                        connection = url.openConnection( );
                        int length = connection.getContentLength();
                        progressBar.setMinimum(0);
                        progressBar.setMaximum(length);
                        progressBar.setValue(0);

                        if(length == -1)
                            throw new IOException("Empty File");

                        is = new BufferedInputStream(connection.getInputStream());

                        byte[] data = new byte[length];

                        int currentBit = 0;
                        int currentPos = 0;

                        while(currentPos < length){
                            progressBar.setValue(currentPos);
                            currentBit = is.read(data, currentPos, data.length-currentPos);
                            if(currentBit == -1)break;
                            currentPos += currentBit;
                        }

                        if(currentPos != length)
                            throw new IOException("The file has not been recovered completely: "
                                    + currentPos + " on " + length + ")");

                        destinationFile = new FileOutputStream(destination+"_tmp");

                        destinationFile.write(data);

                        destinationFile.flush();

                        newFiles.add(destination);

                    } catch (MalformedURLException e) {
                        System.err.println("Malformed URL : " + fileToDownload);
                    } catch (IOException e) {
                        e.printStackTrace();
                    } finally{
                        try {
                            is.close();
                            destinationFile.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                    return null;
                }
            };
            this.currentTask.execute();
        }

    }

    private class RNAMotifDescrFileFrame extends JFrame {
        private JXTable table;

        private RNAMotifDescrFileFrame(final List<String> databasesSelected) {
            super("Searching Criteria");
            final SecondaryStructure ss = mediator.getAlignmentCanvas().getMainAlignment().getStructure2D3D().getReferenceSecondaryStructure();
            this.setLayout(new BorderLayout());
            this.table = new JXTable();
            this.table.setBackground(Color.WHITE);
            this.table.setHighlighters(new Highlighter[]{AlternateRowHighlighter.quickSilver});
            this.table.setSortable(false);
            this.add(new JScrollPane(this.table), BorderLayout.CENTER);

            List<StructuralDomain> domains=new ArrayList<StructuralDomain>();
            //construction of the table model
            Molecule m = mediator.getAlignmentCanvas().getMainAlignment().getBiologicalReferenceSequence().getMolecule();
            for (int i=1; i<=m.getLength();) {
                StructuralDomain sd = ss.getEnclosingStructuralDomain(new Residue(i,m));
                if (!domains.contains(sd))
                    domains.add(sd);
                i = i+sd.getLength();
            }

            Object [][] objects = new Object [domains.size()][3];

            for (int i=0 ; i< domains.size() ; i++) {
                objects[i][0] = domains.get(i);
                objects[i][1] = domains.get(i).getLength();
                objects[i][2] = domains.get(i).getLength();
            }

            table.setModel(new javax.swing.table.DefaultTableModel(
                    objects,
                    new String [] {
                            "Structural Domain", "Min Length", "Max Length"
                    }
            ) {
                Class[] types = new Class [] {
                        StructuralDomain.class, Integer.class, Integer.class
                };
                boolean[] canEdit = new boolean [] {
                        false, true,true
                };

                public Class getColumnClass(int columnIndex) {
                    return types [columnIndex];
                }

                public boolean isCellEditable(int rowIndex, int columnIndex) {
                    return canEdit [columnIndex];
                }


            });

            table.addMouseListener(new MouseAdapter() {
                @Override
                public void mouseClicked(MouseEvent mouseEvent) {
                    StructuralDomain sdSelected = (StructuralDomain)table.getModel().getValueAt(table.getSelectedRow(),0);
                    S2S.this.mediator.getAlignmentCanvas().structuralDomainSelected(sdSelected, false);
                    S2S.this.mediator.getRna2dViewer().structuralDomainSelected(sdSelected, false);
                    S2S.this.mediator.getSecondaryStructureNavigator().structuralDomainSelected(sdSelected, false);
                }
            });

            table.getColumn("Structural Domain").setCellRenderer(new TableCellRenderer() {
                public Component getTableCellRendererComponent(JTable table, Object o, boolean isSelected, boolean hasFocus, int row, int column) {
                    JLabel label = null;
                    if (Helix.class.isInstance(o))
                        label = new JLabel("Helix "+((StructuralDomain)o).getName());
                    else
                        label = new JLabel("SingleStrand "+((StructuralDomain)o).getName());
                    label.setOpaque(true);
                    if (isSelected) {
                        label.setForeground(table.getSelectionForeground());
                        label.setBackground(table.getSelectionBackground());
                    }
                    else {
                        label.setForeground(table.getForeground());
                        label.setBackground(table.getBackground());
                    }
                    return label;
                }
            });



            JPanel buttonsPanel = new JPanel();
            buttonsPanel.setLayout(new FlowLayout());
            JButton button = new JButton("Search");
            buttonsPanel.add(button);
            button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent actionEvent) {
                    for (String databaseSelected: databasesSelected) { //one message send to the Service for each database selected
                        Parameters parameters = new Parameters();
                        Parameter p = new Parameter("database");
                        p.setValue(databaseSelected);
                        parameters.addParameter(p);

                        for (int i = 0; i< table.getRowCount() ; i++) {
                            int minLength = (Integer)table.getValueAt(i,1), maxLength = (Integer)table.getValueAt(i,2);
                            minLength = minLength < 1 ? 1 : minLength;
                            maxLength = maxLength < minLength ? minLength : maxLength;
                            p = new Parameter("minlength");
                            p.setFeature((ParadiseFeature)table.getValueAt(i,0));
                            p.setValue(""+minLength);
                            parameters.addParameter(p);
                            p = new Parameter("maxlength");
                            p.setFeature((ParadiseFeature)table.getValueAt(i,0));
                            p.setValue(""+maxLength);
                            parameters.addParameter(p);
                        }

                        GuiEvent ev = new GuiEvent(this, fr.unistra.ibmc.paradise.Paradise.NC_RNAS_CANDIDATES_SEARCHING);
                        ArrayList<Molecule> molecules = new ArrayList<Molecule>();
                        for (Molecule m:ss.getMolecules()) {
                            m.removeAllSelectedFeatures();
                            m.addSelectedFeature(ss);
                            molecules.add(m);
                        }
                        ev.addParameter(new RNAMotifRequestAnswerBehaviour(molecules,parameters));
                        ((S2SAgent)S2S.this.getAgent()).postGuiEvent(ev);
                    }

                }
            });
            button = new JButton("Cancel");
            button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent actionEvent) {
                    RNAMotifDescrFileFrame.this.dispose();
                }
            });
            buttonsPanel.add(button);
            this.add(buttonsPanel,BorderLayout.SOUTH);
            this.pack();
            GeneralUtils.centerOnScreen(this);
        }

    }

    private class SelectDatabasesFrame extends JFrame {
        private JXTable table;
        private Map<String,String> ncbiDatabases;

        private SelectDatabasesFrame() {
            super("Select Databases");
            this.setLayout(new BorderLayout());
            this.ncbiDatabases = new HashMap<String,String>();
            this.ncbiDatabases.put("Caenorhabditis_elegans", "Caenorhabditis Elegans");

            Map<String,String> ncbiDatabasesToDownload = new HashMap<String,String>(this.ncbiDatabases);

            this.table = new JXTable();
            this.table.setBackground(Color.WHITE);
            this.table.setHighlighters(new Highlighter[]{AlternateRowHighlighter.quickSilver});
            this.table.setSortable(false);

            List<String> directoriesNames = new ArrayList<String>();

            for (File dir:Paradise.getDatabasesDirectory().listFiles(new FileFilter() {
                public boolean accept(File file) {
                    return file.isDirectory();
                }})) {
                directoriesNames.add(dir.getName());
                ncbiDatabasesToDownload.remove(dir.getName());
            }

            Collections.sort(directoriesNames, new Comparator(){

                public int compare(Object o, Object o1) {
                    return ((String)o).compareTo((String)o1);
                }
            });

            List<String> ncbiDatabasesNames = new ArrayList<String>(this.ncbiDatabases.values());

            Collections.sort(ncbiDatabasesNames, new Comparator(){

                public int compare(Object o, Object o1) {
                    return ((String)o).compareTo((String)o1);
                }
            });

            //Object [][] objects = new Object [directoriesNames.size()+ncbiDatabasesToDownload.size()][3];//TODO reactivate this part when the automatic download data from NCBI will be ready

            Object [][] objects = new Object [directoriesNames.size()][3];
            //first the databases already created on the user computer
            for (int i =0; i< directoriesNames.size() ; i++) {
                objects[i][0] = false;
                if (this.ncbiDatabases.keySet().contains(directoriesNames.get(i))) {
                    objects[i][1] = this.ncbiDatabases.get(i);
                    objects[i][2] = "Update";
                }
                //this is a directory created by the user, => no get/update option
                else {
                    objects[i][1] = directoriesNames.get(i);
                    objects[i][2] = "N/A";
                }
            }

            //the databases that the user can still download
            if (false) { //TODO reactivate this part when the automatic download data from NCBI will be ready
                int i = directoriesNames.size();
                for (String databaseName:ncbiDatabasesToDownload.values()) {
                    objects[i][0] = false;
                    objects[i][1] = databaseName;
                    objects[i][2] = "Get";
                    i++;
                }
            }

            table.setModel(new javax.swing.table.DefaultTableModel(
                    objects,
                    new String [] {
                            "Search", "Database Name", "Data"
                    }
            ) {
                Class[] types = new Class [] {
                        java.lang.Boolean.class, java.lang.String.class, String.class
                };
                boolean[] canEdit = new boolean [] {
                        true, false,true
                };

                public Class getColumnClass(int columnIndex) {
                    return types [columnIndex];
                }

                public boolean isCellEditable(int rowIndex, int columnIndex) {
                    if (columnIndex == 0) { //cannot select a row for a search if the Data button is at the "Get" value
                        return !"Get".equals(this.getValueAt(rowIndex,2));
                    }
                    return canEdit [columnIndex];
                }
            });
            table.getColumn("Data").setCellRenderer(new ButtonRenderer());
            table.getColumn("Data").setCellEditor(new ButtonEditor(new JCheckBox()));
            table.getColumn("Data").setMaxWidth(150);

            this.add(new JScrollPane(this.table), BorderLayout.CENTER);
            JPanel buttonsPanel = new JPanel();
            buttonsPanel.setLayout(new FlowLayout());
            JButton button = new JButton("Next Step");
            buttonsPanel.add(button);
            button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent actionEvent) {
                    List<String> databaseSelected = new ArrayList<String>();
                    for (int i=0; i< table.getRowCount(); i++) {
                        if (((Boolean)table.getModel().getValueAt(i,0)))
                            databaseSelected.add((String)table.getModel().getValueAt(i,1));
                    }
                    if (databaseSelected.isEmpty()) {
                        JOptionPane.showMessageDialog(S2S.this.window, "No Database Selected!!");
                        SelectDatabasesFrame.this.setVisible(true);
                    }
                    else {
                        RNAMotifDescrFileFrame f = new RNAMotifDescrFileFrame(databaseSelected);
                        SelectDatabasesFrame.this.dispose();
                        f.setVisible(true);
                    }

                }
            });
            button = new JButton("Cancel");
            button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent actionEvent) {
                    //TODO stop any download process
                    SelectDatabasesFrame.this.dispose();
                }
            });
            buttonsPanel.add(button);
            this.add(buttonsPanel,BorderLayout.SOUTH);
            this.pack();
            GeneralUtils.centerOnScreen(this);
            this.setVisible(true);
        }

        public class ButtonRenderer extends JButton implements TableCellRenderer {

            public ButtonRenderer() {
                setOpaque(true);
            }

            public Component getTableCellRendererComponent(JTable table, Object value,
                                                           boolean isSelected, boolean hasFocus, int row, int column) {
                if (isSelected) {
                    setForeground(table.getSelectionForeground());
                    setBackground(table.getSelectionBackground());
                } else{
                    setForeground(table.getForeground());
                    setBackground(UIManager.getColor("Button.background"));
                }
                setText( (value ==null) ? "" : value.toString() );
                return this;
            }
        }

        public class ButtonEditor extends DefaultCellEditor {
            protected JButton button;
            private String    label;
            private boolean   isPushed;
            private int selectedRow;

            public ButtonEditor(JCheckBox checkBox) {
                super(checkBox);
                button = new JButton();
                button.setOpaque(true);
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        fireEditingStopped();
                    }
                });
            }

            public Component getTableCellEditorComponent(JTable table, Object value,
                                                         boolean isSelected, int row, int column) {
                if (isSelected) {
                    button.setForeground(table.getSelectionForeground());
                    button.setBackground(table.getSelectionBackground());
                } else{
                    button.setForeground(table.getForeground());
                    button.setBackground(table.getBackground());
                }
                label = (value == null) ? "" : value.toString();
                button.setText( label );
                isPushed = true;
                selectedRow = row;
                return button;
            }

            public Object getCellEditorValue() {
                if (isPushed)  {
                    if (!"N/A".equals(table.getValueAt(selectedRow, 2))) { //if it's not a user defined directory
                        for (Map.Entry<String,String> e:ncbiDatabases.entrySet()) {
                            if (e.getValue().equals(table.getValueAt(selectedRow, 1))) {
                                System.out.println("ftp://ftp.ncbi.nih.gov/genomes/"+e.getKey());
                                break;
                            }
                        }
                    }
                }
                isPushed = false;
                return new String( label ) ;
            }

            public boolean stopCellEditing() {
                isPushed = false;
                return super.stopCellEditing();
            }

            protected void fireEditingStopped() {
                super.fireEditingStopped();
            }
        }
    }

}
