package fr.unistra.ibmc.s2s;

import fr.unistra.ibmc.paradise.core.io.AbstractParadiseFileIO;
import fr.unistra.ibmc.paradise.core.io.FileParsingException;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.MoleculeFactory;
import fr.unistra.ibmc.paradise.core.features.StructuralAlignment;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.features.TertiaryStructure;
import fr.unistra.ibmc.paradise.tools.component.ProgressMonitor;
import fr.unistra.ibmc.paradise.tools.ParadiseWorkingSession;

import javax.swing.filechooser.FileFilter;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;
import java.io.File;
import java.io.ObjectInputStream;
import java.io.FileInputStream;


public class S2SAlignmentParser extends AbstractParadiseFileIO {

    private List<Molecule> molecules;
    private StructuralAlignment alignment;

    public S2SAlignmentParser(MoleculeFactory moleculeFactory) {
        super("S2S structural alignement parser", moleculeFactory);
    }

    public List<Molecule> getMolecules() {
        return this.molecules;
    }

    public List<StructuralAlignment> getStructuralAlignments() {
        return Arrays.asList(new StructuralAlignment[]{this.alignment});
    }

    public List<SecondaryStructure> getSecondaryStructures() {
        return new ArrayList<SecondaryStructure>();
    }

    public List<TertiaryStructure> getTertiaryStructures() {
        return new ArrayList<TertiaryStructure>();
    }

    public void parseFile(File f, ProgressMonitor monitor) throws FileParsingException, InterruptedException {
        this.molecules = new ArrayList<Molecule>();
        try {
            ObjectInputStream ois = new ObjectInputStream(new FileInputStream(f));
            Object o = ois.readObject();
            if (StructuralAlignment.class.isInstance(o)) {
                this.alignment = (StructuralAlignment)o;
                this.molecules.addAll(this.alignment.getMolecules());
                ((ParadiseWorkingSession)this.moleculeFactory).addAllMolecules(this.molecules);
            }
            ois.close();
        }
        catch (Exception e) {
            throw new FileParsingException(e);
        }

    }

    public FileFilter getFileFilter() {
        return new javax.swing.filechooser.FileFilter() {
            public boolean accept(File file) {
                return file.isDirectory() || file.getName().endsWith("s2s");
            }

            public String getDescription() {
                return "S2S alignments";
            }
        };
    }
}
