package fr.unistra.ibmc.s2s;

import org.jdesktop.swingx.JXPanel;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.*;

import fr.unistra.ibmc.paradise.core.features.*;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.ParadiseFeature;
import fr.unistra.ibmc.s2s.rnalign.utils.RessourcesUtils;

import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.*;

public class SecondaryStructureNavigator extends JXPanel implements SelectionListener {

    private Mediator mediator;
    private Explorer explorer;
    private JPopupMenu popupMenu;
    private java.util.List<SelectionListener> selectionListeners;

    public SecondaryStructureNavigator(Mediator mediator) {
        this.setBackground(Color.WHITE);
        this.mediator = mediator;
        this.mediator.setSecondaryStructureNavigator(this);
        this.popupMenu = new JPopupMenu();
    }

    private void maybeShowPopup(MouseEvent e) {
        if (e.isPopupTrigger()) {
            this.popupMenu.show(e.getComponent(),
                    e.getX(), e.getY());
        }
    }

    public void updateNavigator() {
        this.removeAll();
        if (mediator.getSecondaryCanvas().getModel2D() != null) {
            this.explorer = new Explorer(new ExplorerModel(mediator.getSecondaryCanvas().getModel2D().getSecondaryStructure()));
            this.add(this.explorer);
            this.explorer.addMouseListener(new MouseListener() {
                public void mouseClicked(MouseEvent event) {
                    mediator.getSecondaryCanvas().clearAllSelections();
                    if (explorer.selectedNodes != null)
                        for (TreePath path:explorer.selectedNodes) {
                            ExplorerNode nodeSelected = (ExplorerNode)path.getLastPathComponent();
                            if (nodeSelected.wrappedObject instanceof SecondaryStructure ) {
                                for  (Helix h:((SecondaryStructure)nodeSelected.wrappedObject).getHelices())
                                    for (SelectionListener l:selectionListeners)
                                        l.structuralDomainSelected(h,event.isShiftDown() || event.isControlDown());
                                for  (SingleStrand ss:((SecondaryStructure)nodeSelected.wrappedObject).getSingleStrands())
                                    for (SelectionListener l:selectionListeners)
                                        l.structuralDomainSelected(ss,event.isShiftDown() || event.isControlDown());
                            }
                            else if (nodeSelected.wrappedObject instanceof StructuralDomain)
                                for (SelectionListener l:selectionListeners)
                                    l.structuralDomainSelected((StructuralDomain)nodeSelected.wrappedObject,event.isShiftDown() || event.isControlDown());
                            else if (nodeSelected.wrappedObject instanceof BaseBaseInteraction)
                                for (SelectionListener l:selectionListeners)
                                    l.interactionSelected((BaseBaseInteraction)nodeSelected.wrappedObject,event.isShiftDown() || event.isControlDown());
                            else if (nodeSelected.wrappedObject instanceof UserSelection)
                                for (SelectionListener l:selectionListeners)
                                    l.residuesSelected(((UserSelection)nodeSelected.wrappedObject).getResidues(),event.isShiftDown() || event.isControlDown());
                        }
                }

                public void mousePressed(MouseEvent event) {
                    if (explorer.selectedNodes != null && explorer.selectedNodes.length == 1 && UserSelection.class.isInstance(((ExplorerNode)explorer.selectedNodes[0].getLastPathComponent()).wrappedObject)) {
                        popupMenu.removeAll();
                        JMenuItem item = new JMenuItem("Rename");
                        popupMenu.add(item);
                        item.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent actionEvent) {
                                String name = JOptionPane.showInputDialog(null,"New name");
                                if (name != null && name.length() != 0) {
                                    ((UserSelection)((ExplorerNode)explorer.selectedNodes[0].getLastPathComponent()).wrappedObject).setName(name);
                                    updateNode(((ExplorerNode)explorer.selectedNodes[0].getParentPath().getLastPathComponent()).wrappedObject,((ExplorerNode)explorer.selectedNodes[0].getLastPathComponent()).wrappedObject);
                                }
                            }
                        });
                        item = new JMenuItem("Delete");
                        item.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent actionEvent) {
                                if (JOptionPane.OK_OPTION == JOptionPane.showConfirmDialog(null,"Are you sure to delete this selection?")) {
                                    removeNode(((ExplorerNode)explorer.selectedNodes[0].getParentPath().getLastPathComponent()).wrappedObject,((ExplorerNode)explorer.selectedNodes[0].getLastPathComponent()).wrappedObject);
                                }
                            }
                        });
                        popupMenu.add(item);
                        maybeShowPopup(event);
                    }
                }

                public void mouseReleased(MouseEvent event) {
                    maybeShowPopup(event);
                }

                public void mouseEntered(MouseEvent event) {
                }

                public void mouseExited(MouseEvent event) {
                }
            });
        }
    }

    public void addUserSelection(UserSelection selection) {
        this.insertNode("User Selections", selection);
    }

    public void insertNode(Object parentContent, Object newContent) {
        this.explorer.insertNode(parentContent, newContent);
    }

    public void removeNode(Object parentContent, Object newContent) {
        this.explorer.removeNode(parentContent, newContent);
    }

    public void updateNode(Object parentContent, Object newContent) {
        this.explorer.updateNode(parentContent, newContent);
    }

    public void selectNode(Object content) {
        ExplorerNode hit = this.explorer.search(content,(ExplorerNode)this.explorer.getModel().getRoot());
        if (hit != null)
            this.explorer.setSelectionPath(hit.getTreePath());
    }

    public void addNode(Object content) {
        ExplorerNode hit = this.explorer.search(content,(ExplorerNode)this.explorer.getModel().getRoot());
        if (hit != null)
            this.explorer.addSelectionPath(hit.getTreePath());
    }

    public void clearSelections() {
        explorer.clearSelection();
    }

    public void addSelectionListener(SelectionListener l) {
        if (this.selectionListeners == null)
            this.selectionListeners = new ArrayList<SelectionListener>();
        this.selectionListeners.add(l);
    }

    public void residuesSelected(Collection<fr.unistra.ibmc.paradise.core.utils.Residue> residues, boolean isShiftDown) {
        if (!isShiftDown)
            this.clearSelections();
    }

    public void interactionSelected(BaseBaseInteraction interaction, boolean isShiftDown) {
        if (!isShiftDown) {
            this.clearSelections();
            this.selectNode(interaction);
        }
        else
            this.addNode(interaction);
    }

    public void structuralDomainSelected(StructuralDomain structuralDomain, boolean isShiftDown) {
        if (!isShiftDown) {
            this.clearSelections();
            this.selectNode(structuralDomain);
        }
        else
            this.addNode(structuralDomain);
    }

    public void clear() {
        this.removeAll();
    }


    private class Explorer extends javax.swing.JTree implements javax.swing.event.TreeSelectionListener {

        private TreePath[] selectedNodes;

        Explorer(final ExplorerModel treeModel) {
            super(treeModel);
            putClientProperty("JTree.lineStyle", "Angled");
            setEditable(false);
            getSelectionModel().setSelectionMode(javax.swing.tree.TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
            this.addTreeSelectionListener(this);
            this.setCellRenderer(new ExplorerRenderer());
            this.setExpandsSelectedPaths(true);
        }

        public void valueChanged(final javax.swing.event.TreeSelectionEvent e) {
            this.selectedNodes = this.getSelectionPaths();
        }

        private void insertNode(Object parentContent, Object newContent) {
            ExplorerNode parentNode=this.search(parentContent,((ExplorerModel)treeModel).root);
            if (parentNode != null) {
                ExplorerNode _newNode = new ExplorerNode(newContent);
                ((ExplorerModel)treeModel).addNode(parentNode,_newNode);
                this.setSelectionPath(_newNode.getTreePath());
            }
        }

        private void removeNode(Object parentContent, Object content) {
            ExplorerNode _parentNode=this.search(parentContent,((ExplorerModel)treeModel).root),
                    _childNode=this.search(content,((ExplorerModel)treeModel).root)  ;
            if (_parentNode != null && _childNode != null)
                ((ExplorerModel)treeModel).removeNode(_parentNode,_childNode);
        }

        private void updateNode(Object parentContent, Object content) {
            ExplorerNode _parentNode=this.search(parentContent,((ExplorerModel)treeModel).root),
                    _childNode=this.search(content,((ExplorerModel)treeModel).root)  ;
            if (_parentNode != null && _childNode != null)
                ((ExplorerModel)treeModel).updateNode(_parentNode,_childNode);
        }

        private ExplorerNode search(Object content,ExplorerNode node) {
            ExplorerNode hit = null;
            if (node.wrappedObject == content)
                hit = node;
            else
                for (int i=0;i<node.children.size() && hit == null ;i++)
                    hit = this.search(content,node.getChild(i));
            return hit;
        }

    }

    private class ExplorerModel implements javax.swing.tree.TreeModel {

        private ExplorerNode root;
        private ExplorerNode userSelections;
        private ExplorerNode secondaryStructure;
        private java.util.List<TreeModelListener> listeners;

        private ExplorerModel(final SecondaryStructure secondaryStructure) {
            this.root = new ExplorerNode("RNA Alignment");
            this.userSelections = new ExplorerNode("User Selections");
            this.root.addChild(this.userSelections);
            this.secondaryStructure = new ExplorerNode(secondaryStructure);
            this.root.addChild(this.secondaryStructure);
            this.listeners = new ArrayList<TreeModelListener>();
            for (Molecule m:secondaryStructure.getMolecules()) {
                ExplorerNode moleculeNode = new ExplorerNode(m);
                this.secondaryStructure.addChild(moleculeNode);
                for (Helix h:secondaryStructure.getHelices())
                    if (h.getMolecules().size() == 1 && h.annotates(m)) {
                        ExplorerNode helixNode = new ExplorerNode(h);
                        moleculeNode.addChild(helixNode);
                        for (BaseBaseInteraction interaction:h.getBaseBaseInteractions())
                            helixNode.addChild(new ExplorerNode(interaction));
                    }
                for (SingleStrand ss:secondaryStructure.getSingleStrands())
                    if (ss.annotates(m))
                        moleculeNode.addChild(new ExplorerNode(ss));
                for (BaseBaseInteraction interaction:secondaryStructure.getTertiaryInteractions())
                    if (interaction.getMolecules().size() == 1 && interaction.annotates(m))
                        moleculeNode.addChild(new ExplorerNode(interaction));
            }
            for (Helix h:secondaryStructure.getHelices())
                if (h.getMolecules().size() == 2) {
                    ExplorerNode n = new ExplorerNode(h);
                    this.secondaryStructure.addChild(n);
                    for (BaseBaseInteraction interaction:h.getBaseBaseInteractions())
                        n.addChild(new ExplorerNode(interaction));
                }
            for (BaseBaseInteraction interaction:secondaryStructure.getTertiaryInteractions())
                if (interaction.getMolecules().size() == 2)
                    this.secondaryStructure.addChild(new ExplorerNode(interaction));
        }

        public Object getRoot() {
            return this.root;
        }

        public Object getChild(final Object parent, final int index) {
            return ((ExplorerNode) parent).getChild(index);
        }

        public int getChildCount(final Object parent) {
            return ((ExplorerNode) parent).getChildrenCount();
        }

        public boolean isLeaf(final Object node) {
            return ((ExplorerNode) node).isLeaf();
        }

        public void valueForPathChanged(final javax.swing.tree.TreePath path, final Object newValue) {
        }

        public int getIndexOfChild(final Object parent, final Object child) {
            return ((ExplorerNode) parent).getIndexOfChild(child);
        }

        public void addTreeModelListener(TreeModelListener treeModelListener) {
            this.listeners.add(treeModelListener);
        }

        public void removeTreeModelListener(TreeModelListener treeModelListener) {
            this.listeners.remove(treeModelListener);
        }

        private void addNode(ExplorerNode parent, ExplorerNode child) {
            parent.addChild(child);
            for (TreeModelListener l:listeners)
                l.treeNodesInserted(new TreeModelEvent(this,parent.getTreePath(),new int[]{parent.children.indexOf(child)},new Object[]{child}));
        }

        private void removeNode(ExplorerNode parent, ExplorerNode child) {
            int index = parent.children.indexOf(child);
            parent.removeChild(child);
            for (TreeModelListener l:listeners)
                l.treeNodesRemoved(new TreeModelEvent(this,parent.getTreePath(),new int[]{index},new Object[]{child}));
        }

        private void updateNode(ExplorerNode parent, ExplorerNode child) {
            for (TreeModelListener l:listeners)
                l.treeNodesChanged(new TreeModelEvent(this,parent.getTreePath(),new int[]{parent.children.indexOf(child)},new Object[]{child}));
        }


    }

    private class ExplorerRenderer extends DefaultTreeCellRenderer {

        private ExplorerRenderer() {
            setFont(new java.awt.Font("Monospaced", java.awt.Font.PLAIN, 14));
            setForeground(Color.black);
            setVisible(true);
        }

        public java.awt.Component getTreeCellRendererComponent(final javax.swing.JTree tree, final Object value, final boolean selected, final boolean expanded, final boolean leaf, final int row, final boolean hasFocus) {
            if (selected) {
                this.setOpaque(true);
                this.setBackground(Color.ORANGE);
            } else {
                this.setOpaque(false);
            }
            /*
            if (NS == S2SViewer.ABSOLUTE_NS) {
                this.setText(value.toString());
            } else {
                this.setText(((ExplorerNode) value).toString2());
            }*/
            this.setText(value.toString());
            if (((ExplorerNode) value).wrappedObject instanceof SingleStrand) {
                this.setIcon(new ImageIcon(RessourcesUtils.getImage("sstrand-node.png")));
            } else if (((ExplorerNode) value).wrappedObject instanceof Helix) {
                this.setIcon(new ImageIcon(RessourcesUtils.getImage("helix-node.png")));
            } else if (((ExplorerNode) value).wrappedObject instanceof BaseBaseInteraction) {
                this.setIcon(new ImageIcon(RessourcesUtils.getImage("interaction-node.png")));
            } else if (((ExplorerNode) value).wrappedObject instanceof Molecule) {
                this.setIcon(new ImageIcon(RessourcesUtils.getImage("molecule-node.png")));
            } else if (((ExplorerNode) value).wrappedObject instanceof SecondaryStructure) {
                this.setIcon(new ImageIcon(RessourcesUtils.getImage("secondary-structure-node.png")));
            }
            else if (((ExplorerNode) value).wrappedObject instanceof String && (((ExplorerNode) value).wrappedObject).equals("User Selections")) {
                this.setIcon(new ImageIcon(RessourcesUtils.getImage("workspace.gif")));
            }
            else if (((ExplorerNode) value).wrappedObject instanceof UserSelection ) {
                this.setIcon(new ImageIcon(RessourcesUtils.getImage("user.gif")));
            }
            else {
                this.setIcon(new ImageIcon(RessourcesUtils.getImage("node.png")));
            }
            return this;
        }
    }

    private class ExplorerNode implements Comparable {
        private ExplorerNode parent;
        private java.util.List<ExplorerNode> children;
        private Object wrappedObject;

        private ExplorerNode(Object o) {
            this.children = new ArrayList<ExplorerNode>();
            this.wrappedObject = o;
        }

        private int getIndexOfChild(Object child) {
            return this.children.indexOf(child);
        }

        private boolean isLeaf() {
            return this.children.size() == 0;
        }

        private int getChildrenCount() {
            return this.children.size();
        }

        private ExplorerNode getChild(int index) {
            return this.children.get(index);
        }

        private void addChild(ExplorerNode explorerNode) {
            this.children.add(explorerNode);
            explorerNode.parent = this;
            Collections.sort(this.children);
        }

        private void removeChild(ExplorerNode explorerNode) {
            this.children.remove(explorerNode);
            explorerNode.parent = null;
            Collections.sort(this.children);
        }

        public String toString() {
            StringBuffer location = new StringBuffer();
            if (ParadiseFeature.class.isInstance(wrappedObject)) {
                Collection<Molecule> molecules = ((ParadiseFeature)wrappedObject).getMolecules();
                for (Molecule m : molecules) {
                    if (molecules.size() != 1 )
                        location.append(m.getName()).append(":").append(((ParadiseFeature)wrappedObject).getFullLocation(m)).append(" ");
                    else
                        location.append(((ParadiseFeature)wrappedObject).getFullLocation(m));
                }
                if (StructuralDomain.class.isInstance(wrappedObject))
                    return wrappedObject.getClass().getSimpleName()+" "+((StructuralDomain)wrappedObject).getName()+" "+location.toString();
                else
                    return wrappedObject.getClass().getSimpleName()+" "+location.toString();
            }
            else if (Molecule.class.isInstance(wrappedObject))
                return "Molecule "+((Molecule)wrappedObject).getName();
            else if (SecondaryStructure.class.isInstance(wrappedObject))
                return "SecondaryStructure";
            else if (UserSelection.class.isInstance(wrappedObject))
                return ((UserSelection)wrappedObject).getName();
            else if (String.class.isInstance(wrappedObject))
                return (String)wrappedObject;
            else
                return "";
        }

        public int compareTo(Object o) {
            if (Helix.class.isInstance(this.wrappedObject) && SingleStrand.class.isInstance(((ExplorerNode)o).wrappedObject))
                return -1;
            else if (SingleStrand.class.isInstance(this.wrappedObject) && Helix.class.isInstance(((ExplorerNode)o).wrappedObject))
                return 1;
            else if (Helix.class.isInstance(this.wrappedObject) && BaseBaseInteraction.class.isInstance(((ExplorerNode)o).wrappedObject))
                return -1;
            else if (BaseBaseInteraction.class.isInstance(this.wrappedObject) && Helix.class.isInstance(((ExplorerNode)o).wrappedObject))
                return 1;
            else if (BaseBaseInteraction.class.isInstance(this.wrappedObject) && SingleStrand.class.isInstance(((ExplorerNode)o).wrappedObject))
                return 1;
            else if (SingleStrand.class.isInstance(this.wrappedObject) && BaseBaseInteraction.class.isInstance(((ExplorerNode)o).wrappedObject))
                return -1;
            else if (Helix.class.isInstance(this.wrappedObject) && Helix.class.isInstance(((ExplorerNode)o).wrappedObject)) {
                Helix h1 = (Helix)this.wrappedObject, h2 = (Helix)((ExplorerNode)o).wrappedObject;
                if (h1.getMolecules().size() == 1 && h2.getMolecules().size() == 1) {
                    int diff = h1.getFullLocation(h1.getMolecules().iterator().next()).getStart() - h2.getFullLocation(h2.getMolecules().iterator().next()).getStart();
                    if (diff < 0)
                        return -1;
                    else
                        return 1;
                }
                else return 0;
            }
            else if (BaseBaseInteraction.class.isInstance(this.wrappedObject) && BaseBaseInteraction.class.isInstance(((ExplorerNode)o).wrappedObject)) {
                BaseBaseInteraction bb1 = (BaseBaseInteraction)this.wrappedObject, bb2 = (BaseBaseInteraction)((ExplorerNode)o).wrappedObject;
                if (bb1.getMolecules().size() == 1 && bb2.getMolecules().size() == 1) {
                    int diff = bb1.getResidue().getAbsolutePosition() - bb2.getResidue().getAbsolutePosition();
                    if (diff < 0)
                        return -1;
                    else
                        return 1;
                }
                else return 0;
            }
            else if (SingleStrand.class.isInstance(this.wrappedObject) && SingleStrand.class.isInstance(((ExplorerNode)o).wrappedObject)) {
                SingleStrand ss1 = (SingleStrand)this.wrappedObject, ss2 = (SingleStrand)((ExplorerNode)o).wrappedObject;
                if (ss1.getMolecules().size() == 1 && ss2.getMolecules().size() == 1) {
                    int diff = ss1.getFullLocation(ss1.getMolecules().iterator().next()).getStart() - ss2.getFullLocation(ss2.getMolecules().iterator().next()).getStart();
                    if (diff < 0)
                        return -1;
                    else
                        return 1;
                }
                else return 0;
            }
            return 0;
        }

        private TreePath getTreePath() {
            java.util.List<ExplorerNode> parents = new ArrayList<ExplorerNode>();
            parents.add(this);
            ExplorerNode _parent = this.parent;
            while (_parent != null) {
                parents.add(0,_parent);
                _parent = _parent.parent;
            }
            return new TreePath(parents.toArray(new ExplorerNode[]{}));
        }
    }
}
