package fr.unistra.ibmc.s2s.behaviours;

import fr.unistra.ibmc.paradise.tools.AnswerBehaviour;
import fr.unistra.ibmc.paradise.tools.ParadiseToolAgent;
import fr.unistra.ibmc.paradise.tools.AbstractParadiseToolAgent;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import fr.unistra.ibmc.paradise.Paradise;
import fr.unistra.ibmc.s2s.S2S;

import java.util.ArrayList;
import java.util.Arrays;

import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import jade.lang.acl.MessageTemplate;
import jade.gui.GuiEvent;


public class RNAMotifRequestAnswerBehaviour extends AnswerBehaviour {

    public RNAMotifRequestAnswerBehaviour(ArrayList<Molecule> selectedMolecules, Parameters selectedParameters) {
        super(selectedMolecules, selectedParameters);
    }

    public void action() {
        ACLMessage message = myAgent.receive(MessageTemplate.MatchConversationId("" + this.conversationId));
        if (message != null && message.getPerformative() == ACLMessage.PROPOSE) {
            try {
                for (Molecule m : ((ArrayList<Molecule>) message.getContentObject())) {
                    //the new molecule has to be registered in the local working session
                    S2S s2s = (S2S)((ParadiseToolAgent)myAgent).getParadiseTool();
                    s2s.getWorkingSession().addAllMolecules(Arrays.asList(new Molecule[]{m}));
                    SecondaryStructure ss = m.getSelectedFeatures(SecondaryStructure.class).get(0);
                    ss.getFactory().copySecondaryStructure(s2s.getWorkingSession().getMolecules(),ss,false);
                    GuiEvent ev = new GuiEvent(this, Paradise.STRUCTURAL_ALIGNMENT);
                    ArrayList<Molecule> molecules = new ArrayList<Molecule>();
                    Molecule referenceMolecule = s2s.getMediator().getAlignmentCanvas().getMainAlignment().getBiologicalReferenceSequence().getMolecule();
                    SecondaryStructure refSS = s2s.getMediator().getAlignmentCanvas().getMainAlignment().getStructure2D3D().getReferenceSecondaryStructure();
                    referenceMolecule.removeAllSelectedFeatures();
                    referenceMolecule.addSelectedFeature(refSS);
                    molecules.add(referenceMolecule);
                    m.removeAllSelectedFeatures();
                    m.addSelectedFeature(ss);
                    molecules.add(m);
                    ev.addParameter(new StructuralAlignmentRequestAnswerBehaviour(molecules,new Parameters()));
                    ((AbstractParadiseToolAgent)myAgent).postGuiEvent(ev);
                }
            }
            catch (Exception e) {
                monitor.printException(e);
            }
            monitor.stopAnimation();
            this.done = true;
        } else if (message != null && message.getPerformative() == ACLMessage.INFORM) {
            monitor.printMessage(message.getContent());
        } else if (message != null && message.getPerformative() == ACLMessage.FAILURE) {
            try {
                monitor.printException((Exception) message.getContentObject());
            } catch (UnreadableException e) {
                e.printStackTrace();
            }
            monitor.stopAnimation();
            this.done = true;
        }
    }
}
