package fr.unistra.ibmc.s2s.behaviours;

import fr.unistra.ibmc.paradise.tools.AnswerBehaviour;
import fr.unistra.ibmc.paradise.tools.AbstractParadiseToolAgent;
import fr.unistra.ibmc.paradise.tools.ParadiseToolAgent;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import fr.unistra.ibmc.paradise.Paradise;
import fr.unistra.ibmc.s2s.S2S;

import java.util.ArrayList;

import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;
import jade.gui.GuiEvent;


public class SecondaryStructurePredictionBeforeStructuralAlignmentAnswerBehaviour extends AnswerBehaviour {

    private String structuralAlignmentAgentName;

    public SecondaryStructurePredictionBeforeStructuralAlignmentAnswerBehaviour(ArrayList<Molecule> selectedMolecules, Parameters selectedParameters, String structuralAlignmentAgentName) {
        super(selectedMolecules, selectedParameters);
        this.structuralAlignmentAgentName = structuralAlignmentAgentName;
    }

    public void action() {
        ACLMessage message = myAgent.receive(MessageTemplate.MatchConversationId("" + this.conversationId));
        if (message != null && message.getPerformative() == ACLMessage.PROPOSE) {
            try {
                S2S s2s = (S2S)((ParadiseToolAgent)myAgent).getParadiseTool();
                SecondaryStructure ss = ((ArrayList<Molecule>) message.getContentObject()).get(0).getSelectedFeatures(SecondaryStructure.class).get(0);
                SecondaryStructure copySS = ss.getFactory().copySecondaryStructure(selectedMolecules, ss, false);
                copySS.setName("Secondary Structure");
                monitor.printMessage("End of Analysis");
                //now we launch the structural alignment between the reference 2D of the alignment and the new secondary structure just calculated
                GuiEvent ev = new GuiEvent(this, Paradise.STRUCTURAL_ALIGNMENT);
                ArrayList<Molecule> molecules = new ArrayList<Molecule>();
                Molecule refMol = s2s.getMediator().getAlignmentCanvas().getMainAlignment().getBiologicalReferenceSequence().getMolecule();
                refMol.addSelectedFeature(s2s.getAlignmentCanvas().getMainAlignment().getReferenceStructure());
                selectedMolecules.get(0).addSelectedFeature(copySS);
                molecules.add(refMol);
                molecules.add(selectedMolecules.get(0));
                ev.addParameter(new StructuralAlignmentRequestAnswerBehaviour(molecules,new Parameters()));
                ev.addParameter(this.structuralAlignmentAgentName);
                ((AbstractParadiseToolAgent)myAgent).postGuiEvent(ev);
            }
            catch (Exception e) {
                monitor.printException(e);
            }
            this.done = true;
        } else if (message != null && message.getPerformative() == ACLMessage.INFORM) {
            monitor.printMessage(message.getContent());
        } else if (message != null && message.getPerformative() == ACLMessage.FAILURE) {
            try {
                monitor.printException((Exception) message.getContentObject());
            } catch (UnreadableException e) {
                e.printStackTrace();
            }
            monitor.stopAnimation();
            this.done = true;

        }
    }
}
