package fr.unistra.ibmc.s2s.behaviours;

import fr.unistra.ibmc.paradise.tools.AnswerBehaviour;
import fr.unistra.ibmc.paradise.tools.ParadiseToolAgent;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.features.StructuralAlignment;
import fr.unistra.ibmc.paradise.core.features.StructuralIdentity;
import fr.unistra.ibmc.paradise.core.analysis.Parameter;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import fr.unistra.ibmc.s2s.S2S;
import fr.unistra.ibmc.s2s.rnalign.features.BiologicalSequence;
import fr.unistra.ibmc.s2s.rnalign.features.Symbol;

import java.util.ArrayList;

import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;


public class StructuralAlignmentRequestAnswerBehaviour extends AnswerBehaviour {
    public StructuralAlignmentRequestAnswerBehaviour(ArrayList<Molecule> selectedMolecules, Parameters selectedParameters) {
        super(selectedMolecules, selectedParameters);
    }

    public void action() {
        ACLMessage message = myAgent.receive(MessageTemplate.MatchConversationId(""+this.conversationId));
        if (message != null && message.getPerformative() == ACLMessage.PROPOSE) {
            try {
                ArrayList<Molecule> remoteMolecules = (ArrayList<Molecule>)message.getContentObject();
                S2S s2S = (S2S)((ParadiseToolAgent)myAgent).getParadiseTool();
                StructuralAlignment remoteAlignment = null;
                Molecule remoteReferenceMolecule = null,
                        localReferenceMolecule = s2S.getAlignmentCanvas().getMainAlignment().getBiologicalReferenceSequence().getMolecule();
                //########### here starts the code to update the current alignment with a precomputed structural alignment
                for (Molecule remoteMolecule :remoteMolecules) {
                    if (remoteMolecule.getParadiseID().equals(localReferenceMolecule.getParadiseID())) {
                        //we recover the structural alignment attached to the reference molecule
                        remoteAlignment = remoteMolecule.getSelectedFeatures(StructuralAlignment.class).get(0);
                        remoteReferenceMolecule = remoteMolecule;
                    }
                    else if (remoteAlignment != null && remoteReferenceMolecule != null && !remoteMolecule.getParadiseID().equals(localReferenceMolecule.getParadiseID())) {
                        Molecule newMolecule = s2S.getWorkingSession().getMolecule(remoteMolecule.getParadiseID());
                        BiologicalSequence newBiologicalSequence = new BiologicalSequence(newMolecule);
                        int alongLocalAlignmentCounter = 0;
                        StringBuffer buffer = new StringBuffer();
                        int newMoleculePosition = 1;
                        for (StructuralIdentity si: remoteAlignment.getStructuralIdentities(true)) {
                            Location remoteReferenceGaps = si.getGapsLocation(remoteReferenceMolecule), remoteNewMoleculeGaps = si.getGapsLocation(remoteMolecule) ;
                            for (int i=1; i <= si.getLength();i++,alongLocalAlignmentCounter++) {
                                if (alongLocalAlignmentCounter >= s2S.getMediator().getAlignmentCanvas().getMainAlignment().getLength()) {
                                    s2S.getMediator().getAlignmentCanvas().getMainAlignment().insertGaps(alongLocalAlignmentCounter);
                                }
                                else {
                                    boolean isLocalGap = s2S.getMediator().getAlignmentCanvas().getMainAlignment().getBiologicalReferenceSequence().getSymbol(alongLocalAlignmentCounter).isGap(),
                                            isRemoteGap = (remoteReferenceGaps != null && remoteReferenceGaps.hasPosition(i));
                                    if (isLocalGap && !isRemoteGap) {
                                        buffer.append("-");
                                        i--;
                                        continue;
                                    }
                                    else if (!isLocalGap && isRemoteGap) { //the local alignment needs a new gap to be synchronized
                                        s2S.getMediator().getAlignmentCanvas().getMainAlignment().insertGaps(alongLocalAlignmentCounter); //the local alignment is shifted by 1
                                    }
                                }
                                if (remoteNewMoleculeGaps.hasPosition(i))
                                    buffer.append("-");
                                else
                                    buffer.append(newMolecule.getResidueAt(newMoleculePosition++));
                            }
                        }
                        BiologicalSequence seq =  new BiologicalSequence(s2S.getMediator(),newMolecule.getName(),buffer.toString());
                        //we remove from the working sesssion the Molecule object that has been used in the message
                        s2S.getWorkingSession().removeMolecule(newMolecule);
                        s2S.getAlignmentCanvas().getMainAlignment().addBiologicalSequence(seq);
                        s2S.getMediator().getSequencesList().addRow(seq.getMolecule());
                        s2S.getAlignmentCanvas().repaint();
                        monitor.printMessage("End of Analysis");
                    }
                }
            }
            catch (Exception e) {
                monitor.printException(e);
            }
            monitor.stopAnimation();
            this.done = true;
        }else if (message != null && message.getPerformative() == ACLMessage.INFORM) {
            monitor.printMessage(message.getContent());
        }
        else if (message != null && message.getPerformative() == ACLMessage.FAILURE) {
            try {
                monitor.printException((Exception) message.getContentObject());
            } catch (UnreadableException e) {
                e.printStackTrace();
            }
            monitor.stopAnimation();
            this.done = true;
        }
    }
}

