package fr.unistra.ibmc.s2s.behaviours;

import fr.unistra.ibmc.paradise.tools.AnswerBehaviour;
import fr.unistra.ibmc.paradise.tools.ParadiseToolAgent;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.features.TertiaryStructure;
import fr.unistra.ibmc.paradise.core.analysis.Parameter;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import fr.unistra.ibmc.s2s.S2S;

import java.util.ArrayList;

import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;

public class TertiaryStructureAnnotationRequestAnswerBehaviour extends AnswerBehaviour {

    public TertiaryStructureAnnotationRequestAnswerBehaviour(ArrayList<Molecule> selectedMolecules, Parameters selectedParameters) {
        super(selectedMolecules, selectedParameters);
    }

    public void action() {
        ACLMessage message = myAgent.receive(MessageTemplate.MatchConversationId("" + this.conversationId));
        if (message != null && message.getPerformative() == ACLMessage.PROPOSE) {
            try {
                SecondaryStructure remoteSecondaryStructure = (SecondaryStructure) (((ArrayList<Molecule>) message.getContentObject()).get(0)).getSelectedFeatures().get(0);
                SecondaryStructure copySecondaryStructure = remoteSecondaryStructure.getFactory().copySecondaryStructure(selectedMolecules, remoteSecondaryStructure, false);
                //we add the secondaryStructure selected to the tertiaryStructure (which is the parent feature of the first Residue3D selected)
                TertiaryStructure ts = selectedMolecules.get(0).getSelectedFeatures().get(0).getParentFeatures(TertiaryStructure.class).get(0);
                ts.linkSecondaryStructure(copySecondaryStructure);
                //we copy the display produced by RNAVIEW
                remoteSecondaryStructure.getFactory().copySecondaryStructureDisplay(selectedMolecules, remoteSecondaryStructure.getSecondaryStructureDisplay(), copySecondaryStructure);
                ((S2S)((ParadiseToolAgent)myAgent).getParadiseTool()).loadRNASecondaryStructure(copySecondaryStructure);
                monitor.printMessage("End of Analysis");
            }
            catch (Exception e) {
                monitor.printException(e);
            }
            monitor.stopAnimation();
            this.done = true;
            this.selectedMolecules.clear();
            this.selectedParameters.clear();
        } else if (message != null && message.getPerformative() == ACLMessage.INFORM) {
            monitor.printMessage(message.getContent());
        } else if (message != null && message.getPerformative() == ACLMessage.FAILURE) {
            try {
                   monitor.printException((Exception) message.getContentObject());
                } catch (UnreadableException e) {
                    e.printStackTrace();
                }
            monitor.stopAnimation();
            this.done = true;
            this.selectedMolecules.clear();
            this.selectedParameters.clear();
        }
    }
}

