package fr.unistra.ibmc.s2s.rnalign.features;

import fr.unistra.ibmc.paradise.core.features.BaseBaseInteraction;
import fr.unistra.ibmc.paradise.core.utils.Residue;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.awt.*;


public class StructuralSymbol extends Symbol {

    private Residue residue;
    //one base can establish several interactions
    //these interactions are stored in a Map to store the "paired" symbols for each interaction established by the current symbol
    protected Map<BaseBaseInteraction, StructuralSymbol> referenceStructureInteractions;
    protected Map<ConsensusBaseBaseInteraction, StructuralSymbol> consensusInteractions;
    private Structure2D3D structure2D3D;


    protected StructuralSymbol(final Structure2D3D structure2D3D,char symbol) {
        super(symbol);
        this.structure2D3D = structure2D3D;
        this.referenceStructureInteractions = new HashMap<BaseBaseInteraction,StructuralSymbol>();
        this.consensusInteractions = new HashMap<ConsensusBaseBaseInteraction, StructuralSymbol>();
    }

    public StructuralSymbol(final Structure2D3D structure2D3D, final Residue residue) {
        this(structure2D3D,'.');
        this.residue = residue;
    }

    public boolean isGap() {
        return this.getSymbol() == '-';
    }

    public boolean isSingleStrand() {
        return this.getSymbol() == '.';
    }

    public void addReferenceBaseBaseInteraction(BaseBaseInteraction interaction, StructuralSymbol pairedSymbol) {
        this.referenceStructureInteractions.put(interaction, pairedSymbol);
        if (interaction.isSecondaryInteraction()) {
            if (interaction.getResidue().equals(this.residue) && interaction.isCanonical())
                this.symbol = '(';
            else if (interaction.getPartnerResidue().equals(this.residue) && interaction.isCanonical())
                this.symbol = ')';
            else if (interaction.getResidue().equals(this.residue) && !interaction.isCanonical())
                this.symbol = '<';
            else if (interaction.getPartnerResidue().equals(this.residue) && !interaction.isCanonical())
                this.symbol = '>';
                //if inter-molecular base-base interaction
            else if (interaction.getResidue().equals(this.residue) && interaction.getMolecules().size() == 2 )
                this.symbol = '{';
            else if (interaction.getPartnerResidue().equals(this.residue) && interaction.getMolecules().size() == 2 )
                this.symbol = '}';
        }
        if (!pairedSymbol.referenceStructureInteractions.containsKey(interaction))
            pairedSymbol.addReferenceBaseBaseInteraction(interaction,this);
    }

    public void addConsensusBaseBaseInteraction(ConsensusBaseBaseInteraction interaction, StructuralSymbol pairedSymbol) {
        this.consensusInteractions.put(interaction, pairedSymbol);
        this.symbol = this.structure2D3D.getIndex(this) < this.structure2D3D.getIndex(pairedSymbol) ? '(' : ')';
        if (!pairedSymbol.consensusInteractions.containsKey(interaction))
            pairedSymbol.addConsensusBaseBaseInteraction(interaction,this);
    }


    public List<BaseBaseInteraction> getReferenceBaseBaseInteractions() {
        return new ArrayList<BaseBaseInteraction>(this.referenceStructureInteractions.keySet());
    }

    public Residue getResidue() {
        return residue;
    }

    public char getEdge(BaseBaseInteraction interaction) {
        if (this.referenceStructureInteractions.containsKey(interaction)) {
            if (interaction.getResidue().equals(this.residue))
                return interaction.getEdge1();
            else if (interaction.getPartnerResidue().equals(this.residue))
                return interaction.getEdge2();
        }
        //[fjossinet] TODO improve that
        throw new RuntimeException();
    }

    public StructuralSymbol getPairedSymbol(BaseBaseInteraction interaction) {
        return this.referenceStructureInteractions.get(interaction);
    }

    public StructuralSymbol getPairedSymbol(ConsensusBaseBaseInteraction interaction) {
        return this.consensusInteractions.get(interaction);
    }

    public BaseBaseInteraction getSecondaryInteraction() {
        for (BaseBaseInteraction interaction:this.getReferenceBaseBaseInteractions())
            if (interaction.isSecondaryInteraction())
                return interaction;
        return null;
    }

    public List<BaseBaseInteraction> getTertiaryInteractions() {
        List<BaseBaseInteraction> interactions = new ArrayList<BaseBaseInteraction>();
        for (BaseBaseInteraction interaction:this.getReferenceBaseBaseInteractions())
            if (!interaction.isSecondaryInteraction())
                interactions.add(interaction);
        return interactions;
    }

    public void removeBaseBaseInteraction(BaseBaseInteraction interaction) {
        if (this.referenceStructureInteractions.containsKey(interaction)) {
            StructuralSymbol pairedSymbol = (StructuralSymbol)this.referenceStructureInteractions.get(interaction);
            pairedSymbol.referenceStructureInteractions.remove(interaction);
            this.referenceStructureInteractions.remove(interaction);
            if (interaction.isSecondaryInteraction()) {
                this.symbol = '.';
                pairedSymbol.symbol = '.';
            }
            return;
        }
    }

    public Color getColor() {
        return Color.BLACK;
    }

    public static StructuralSymbol createGap(Structure2D3D structure2D3D) {
        return new StructuralSymbol(structure2D3D, '-');
    }

}
