package fr.unistra.ibmc.s2s.rnalign.features;

import fr.unistra.ibmc.paradise.core.features.*;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.s2s.rnalign.graphics.AlignmentView;
import fr.unistra.ibmc.s2s.rnalign.graphics.GraphicContext;
import fr.unistra.ibmc.s2s.rnalign.graphics.ReversedAlignmentView;

import java.awt.*;
import java.util.*;
import java.util.List;


public class Structure2D3D extends SymbolSequence {

    int helixIndex;
    List<BasePair> basePairs, pknotBasePairs;
    List<Domain> domains;
    BiologicalSequence referenceSequence;
    SecondaryStructure referenceSecondaryStructure;

    Structure2D3D(final BiologicalSequence referenceSequence, final SecondaryStructure referenceSecondaryStructure) {
        super(referenceSequence.size());
        this.referenceSequence = referenceSequence;
        this.referenceSecondaryStructure = referenceSecondaryStructure;
        this.domains = new ArrayList<Domain>();
        //first step, construction of the symbols list without the gaps in the reference sequence
        List<BaseBaseInteraction> interactions = this.referenceSecondaryStructure.getBaseBaseInteractions();
        List<Helix> helices = this.referenceSecondaryStructure.getHelices();
        List<SingleStrand> sstrands = this.referenceSecondaryStructure.getSingleStrands();
        Symbol symbol = null;
        //we are using the BiologicalReferenceSequence object just constructed to speed the construction of the Structure2D3D object
        int moleculePos = 1;
        for (int i=0; i < referenceSequence.size();i++) {
            if (referenceSequence.isGap(i))
                symbol = StructuralSymbol.createGap(this);
            else {
                Residue r = new Residue(moleculePos, referenceSequence.getMolecule());
                symbol = new StructuralSymbol(this,r) ;
                symbol.setPositionInSequence(moleculePos);
                boolean found = false;
                HELICES: for (Helix h:helices) {
                    if (!h.annotates(this.referenceSequence.getMolecule()))
                        continue;
                    Residue[] _5 = h.get5PrimeEnds();
                    if (_5[0].equals(r) || _5[1].equals(r)) {
                        Domain d = new Domain(h);
                        d.start = symbol;
                        this.domains.add(d);
                        found = true;
                        break HELICES;
                    }
                    Residue[] _3 = h.get3PrimeEnds();
                    if (_3[0].equals(r)) {
                        for (Domain _d:domains)
                            if (_d.structuralDomain == h && _d.start.getPositionInSequence() == h.get5PrimeEnd(_3[0]).getAbsolutePosition())  {
                                _d.end = symbol;
                                found = true;
                                break HELICES;
                            }
                    }
                    else if (_3[1].equals(r)) {
                        for (Domain _d:domains)
                            if (_d.structuralDomain == h && _d.start.getPositionInSequence() == h.get5PrimeEnd(_3[1]).getAbsolutePosition())  {
                                _d.end = symbol;
                                found = true;
                                break HELICES;
                            }
                    }
                }
                if (!found)
                    SSTRANDS: for (SingleStrand ss:sstrands) {
                        if (!ss.annotates(this.referenceSequence.getMolecule()))
                            continue;
                        if (ss.get5PrimeEnd() == moleculePos) {
                            Domain d = new Domain(ss);
                            d.start = symbol;
                            this.domains.add(d);
                            if (ss.getLength()==1)
                                d.end = symbol;
                            break SSTRANDS;
                        }
                        else if (ss.get3PrimeEnd() == moleculePos)
                            for (Domain _d:domains)
                                if (_d.structuralDomain == ss)  {
                                    _d.end = symbol;
                                    break SSTRANDS;
                                }
                    }
                List<BaseBaseInteraction> interactionsRegistered = new ArrayList<BaseBaseInteraction>();
                INTERACTIONS:for (BaseBaseInteraction interaction:interactions) {
                    if (!interactionsRegistered.contains(interaction)) {
                        if (interaction.getResidue().equals(r)) {
                            for (Symbol s:this.symbols) {
                                if (((StructuralSymbol)s).getResidue()!= null && ((StructuralSymbol)s).getResidue().equals(interaction.getPartnerResidue())) {
                                    ((StructuralSymbol)symbol).addReferenceBaseBaseInteraction(interaction, (StructuralSymbol)s);
                                    interactionsRegistered.add(interaction);
                                    continue INTERACTIONS;
                                }
                            }
                        }
                        else if (interaction.getPartnerResidue().equals(r)) {
                            for (Symbol s:this.symbols) {
                                if (((StructuralSymbol)s).getResidue()!= null && ((StructuralSymbol)s).getResidue().equals(interaction.getResidue())) {
                                    ((StructuralSymbol)symbol).addReferenceBaseBaseInteraction(interaction, (StructuralSymbol)s);
                                    interactionsRegistered.add(interaction);
                                    continue INTERACTIONS;
                                }
                            }
                        }
                    }
                }
                interactions.removeAll(interactionsRegistered);
                moleculePos++;
            }
            this.symbols.add(symbol);
        }
        Collections.sort(this.domains, new Comparator() {
            public int compare(Object o, Object o1) {
                return ((Domain)o).getStart().getPositionInSequence()-((Domain)o1).getStart().getPositionInSequence();
            }
        });
    }

    public SecondaryStructure getReferenceSecondaryStructure() {
        return referenceSecondaryStructure;
    }

    public boolean removeGap(final int index) {
        Symbol s = this.getSymbol(index);
        if (s.isGap()) {
            this.symbols.remove(index);
            return true;
        }
        return false;
    }

    public void insertGap(final int index) {
        this.symbols.add(index, StructuralSymbol.createGap(this));
    }

    public boolean isGap(final int index) {
        return this.getSymbol(index).isGap();
    }

    public void increaseSize() {
        this.symbols.add(this.symbols.size(), StructuralSymbol.createGap(this));
    }

    public Symbol getSymbol(final int index) {
        return this.symbols.get(index);
    }

    public void drawSymbol(final Alignment alignment, Graphics2D g, int currentX, int viewX, int currentY, int viewY, GraphicContext gc, int index) {
        g.setColor(this.symbols.get(index).getColor());
        g.drawString(new StringBuffer().append(this.getSymbol(index).getSymbol()).toString(), currentX - viewX, currentY);
        g.setColor(Color.BLACK);
    }

    //TODO dirty trick to avoid the inversion of brackets with the reversed view
    public void drawSymbol(final AlignmentView view, final Alignment alignment, Graphics2D g, int currentX, int viewX, int currentY, int viewY, GraphicContext gc, int index) {
        g.setColor(this.symbols.get(index).getColor());
        char s = this.getSymbol(index).getSymbol();
        if (ReversedAlignmentView.class.isInstance(view)) {
            switch (s) {
                case '(' : s = ')'; break;
                case ')' : s = '('; break;
                case '<' : s = '>'; break;
                case '>' : s = '<'; break;
            }
            g.drawString(new StringBuffer().append(s).toString(), currentX - viewX, currentY);
        }
        else {
            g.drawString(new StringBuffer().append(s).toString(), currentX - viewX, currentY);
        }
        g.setColor(Color.BLACK);
    }

    public void setSelected(final boolean selected) {
        this.selected = false;
    }

    public int size() {
        return this.symbols.size();
    }

    public String getSequence() {
        final StringBuffer buff = new StringBuffer(this.symbols.size());
        for (Iterator i = this.symbols.iterator(); i.hasNext();)
            buff.append(((Symbol) i.next()).getSymbol());
        return buff.toString();

    }

    /**
     * Define the new structure of reference. Each gap in the
     * BiologicalReferenceSequence will be added at the same position in the reference structure
     */
    public void setSecondaryStructure(final SecondaryStructure secondaryStructure) {
        /*
        this.interactions = secondaryStructure.getTertiaryInteractions();
        this.symbols = new ArrayList<Symbol>();
        for (int i=0; i < this.alignment.getBiologicalReferenceSequence().getMolecule().getLength();i++) {
            for (BaseBaseInteraction interaction:this.interactions)
                if (interaction.getResidue().getAbsolutePosition() == i) {
                    this.symbols.add(Symbol.leftH);
                    continue;
                }
                else if (interaction.getPartnerResidue().getAbsolutePosition() == i) {
                    this.symbols.add(Symbol.rightH);
                    continue;
                }
            this.symbols.add(Symbol.dot);
        }
        for (int i = 0; i < this.alignment.getBiologicalReferenceSequence().size(); i++)
            if (this.alignment.getBiologicalReferenceSequence().isGap(i)) {
                if (i >= this.symbols.size())
                    this.symbols.add(this.symbols.size(), MutableSymbol.createGap());
                else
                    this.symbols.add(i, MutableSymbol.createGap());
            }*/
    }

    public void modifySequence(String name, List sequence) {
        //Structure2D3D cannot be modified
    }

    /**
     *
     * @param absolutePosition
     * @return -1 if no partner for the absolutePosition given as argument
     */
    public int getPartnerBase(int absolutePosition) {
        for (BaseBaseInteraction interaction: this.referenceSecondaryStructure.getBaseBaseInteractions()) {
            if (interaction.getResidue().getAbsolutePosition() == absolutePosition)
                return interaction.getPartnerResidue().getAbsolutePosition();
            else if (interaction.getPartnerResidue().getAbsolutePosition() == absolutePosition)
                return interaction.getResidue().getAbsolutePosition();
        }
        return -1;
    }

    public void setSymbolAt(Symbol s, int i) {
        //does nothing
    }

    void calculateBasePairs() {
        this.basePairs = new ArrayList<BasePair>();
        this.pknotBasePairs = new ArrayList<BasePair>();
        this.helixIndex = 0;
        List tmpBasePairs = new ArrayList(), tmpPknotBasePairs = new ArrayList();
        BasePair tmpBasePair = null, lastBasePair = null;
        PkBasePair tmpPknotBasePair = null;
        Symbol l = null;
        int absPos = 0;
        for (int i = 0; i < this.symbols.size(); i++) {
            l = (Symbol) this.symbols.get(i);
            if (l.getSymbol() != '-')
                absPos++;
            if (l.getSymbol() == '(' || l.getSymbol() == '<')
                tmpBasePairs.add(new BasePair(i, absPos));
            else if (l.getSymbol() == ')' || l.getSymbol() == '>') {
                if (tmpBasePairs.size() > 0) {
                    tmpBasePair = (BasePair) tmpBasePairs.get(tmpBasePairs.size() - 1);
                    tmpBasePairs.remove(tmpBasePairs.size() - 1);
                    tmpBasePair.partnerBaseAbs = i;
                    tmpBasePair.partnerBaseReal = absPos;
                    if (this.basePairs.size() == 0) {
                        tmpBasePair.helixIndex = this.helixIndex++;
                    } else {
                        lastBasePair = (BasePair) this.basePairs.get(this.basePairs.size() - 1);
                        if (tmpBasePair.baseReal + 1 != lastBasePair.baseReal || tmpBasePair.partnerBaseReal - 1 != lastBasePair.partnerBaseReal)
                            tmpBasePair.helixIndex = this.helixIndex++;
                        else
                            //its the same Helix
                            tmpBasePair.helixIndex = lastBasePair.helixIndex;
                    }
                    this.basePairs.add(tmpBasePair);
                }
            } else if ('A' <= l.getSymbol() && l.getSymbol() <= 'Z')
                tmpPknotBasePairs.add(new PkBasePair(i, absPos, l.getSymbol()));
            else if ('a' <= l.getSymbol() && l.getSymbol() <= 'z') {
                if (tmpPknotBasePairs.size() > 0) {
                    for (int j = tmpPknotBasePairs.size() - 1; j >= 0; j--) {
                        tmpPknotBasePair = (PkBasePair) tmpPknotBasePairs.get(j);
                        if (tmpPknotBasePair.pkId == (l.getSymbol() - 32)) {
                            tmpPknotBasePairs.remove(j);
                            tmpPknotBasePair.partnerBaseAbs = i;
                            tmpPknotBasePair.partnerBaseReal = absPos;
                            tmpPknotBasePair.helixIndex = l.getSymbol() - 32 - 'A';
                            this.pknotBasePairs.add(tmpPknotBasePair);
                            break;
                        }
                    }
                }
            }
        }
        tmpBasePairs = null;
        tmpPknotBasePairs = null;
    }


    class BasePair {
        int baseAbs = -1, baseReal = -1, partnerBaseAbs = -1, partnerBaseReal = -1;
        int helixIndex;
        float conservationScore=-1f;

        BasePair(int baseAbs, int baseReal) {
            this.baseAbs = baseAbs;
            this.baseReal = baseReal;
        }

        BasePair(int baseAbs, int baseReal, int partnerBaseAbs, int partnerBaseReal, int helixIndex) {
            this.baseAbs = baseAbs;
            this.baseReal = baseReal;
            this.partnerBaseAbs = partnerBaseAbs;
            this.partnerBaseReal = partnerBaseReal;
            this.helixIndex = helixIndex;
        }
    }

    class PkBasePair extends BasePair {
        char pkId;

        PkBasePair(int baseAbs, int baseReal, char pkId) {
            super(baseAbs, baseReal);
            this.pkId = pkId;
        }

    }

    public class Domain {
        private StructuralDomain structuralDomain;
        private Symbol start, end;

        private Domain(StructuralDomain structuralDomain) {
            this.structuralDomain = structuralDomain;
        }

        public Symbol getEnd() {
            return end;
        }

        public Symbol getStart() {
            return start;
        }

        public StructuralDomain getStructuralDomain() {
            return structuralDomain;
        }
    }

}
