package fr.unistra.ibmc.s2s.rnalign.features;

import fr.unistra.ibmc.s2s.rnalign.graphics.AlignmentCanvas;

import java.awt.*;

/**
 * This class stores a single character for a biological residue or symbol. Within S2S, for each kind of Residue,
 * a Symbol object is created once but can be used/displayed several times.
 */
public class Symbol {

    protected char symbol;
    //from  1 to sequence length
    protected int positionInSequence = -1;

    public static final Symbol A = new Symbol('A'),
            U = new Symbol('U'),
            T = new Symbol('T'),
            G = new Symbol('G'),
            C = new Symbol('C'),
            a = new Symbol('a'),
            u = new Symbol('u'),
            t = new Symbol('t'),
            g = new Symbol('g'),
            c = new Symbol('c'),
            Any = new Symbol('N'),
            Gap = new Symbol('-'),
            LockedGap = new Symbol('-',true),
            Unknown = new Symbol('?'),
            rightH = new Symbol('('),
            leftH = new Symbol(')'),
            rightInterH = new Symbol('{'),
            leftInterH = new Symbol('}'),
            rightCanonical = new Symbol('<'),
            leftCanonical = new Symbol('>'),
            rightPKnot = new Symbol('['),
            leftPKnot = new Symbol(']'),
            dot = new Symbol('.');
    private boolean locked;


    protected Symbol() {
        this('?');
    }

    public Symbol(final char symbol) {
        this.symbol = symbol;
        this.positionInSequence = -1;
    }

    protected Symbol(final char symbol, final boolean locked) {
        this(symbol);
        this.locked = locked;
    }

    public static Symbol createGap() {
        return Gap;
    }

    public boolean isGap() {
        return this.getSymbol() == Gap.getSymbol();
    }

    public int getPositionInSequence() {
        return positionInSequence;
    }

    public void setPositionInSequence(int positionInSequence) {
        this.positionInSequence = positionInSequence;
    }

    /**
     * Return the character stored
     *
     * @return the charcter stored
     */
    public char getSymbol() {
        return this.symbol;
    }

    /**
     * Return the Symbol object corresponding to a character
     *
     * @param symbol the character for which we want the Symbol object in memory
     * @return the corresponding Symbol object
     */
    public static Symbol getSymbol(final char symbol) {
        switch (symbol) {
            case 'A':
            case 'T':
            case 'U':
            case 'G':
            case 'C':
            case 'N':
                //we create a new object since each object will be attached to its own positionInSequence
                return new Symbol(symbol);
                //for all the other possibilities, we can use the same object for all the symbols to display
            case '-':
                return createGap();
            case '(':
                return rightH;
            case ')':
                return leftH;
            case '{':
                return rightInterH;
            case '}':
                return leftInterH;
            case '<':
                return rightCanonical;
            case '>':
                return leftCanonical;
            case '[':
                return rightPKnot;
            case ']':
                return leftPKnot;
            case '.':
                return dot;
                //if we have an unusual character
            default :
                return new Symbol(symbol);
        }
    }

    public void nextLetter() {

    }

    public boolean isLocked() {
        return this.locked;
    }

    public void setLocked(boolean locked) {
        this.locked = locked;
    }

    public Color getColor() {
        switch (this.symbol) {
            case 'A':
                return AlignmentCanvas.A_Color;
            case 'U':
                return AlignmentCanvas.U_Color;
            case 'G':
                return AlignmentCanvas.G_Color;
            case 'C':
                return AlignmentCanvas.C_Color;
            default :
                return Color.BLACK;
        }
    }
}
