package fr.unistra.ibmc.s2s.rnalign.features;

import fr.unistra.ibmc.s2s.rnalign.graphics.GraphicContext;

import java.awt.*;
import java.util.ArrayList;


abstract public class SymbolSequence {

    Color color = Color.BLACK;
    boolean selected;
    private int countingFrom = 1;
    protected java.util.List<Symbol> symbols;

    SymbolSequence() {
        this.symbols = new ArrayList<Symbol>();
    }

    SymbolSequence(int size) {
        this.symbols = new ArrayList<Symbol>(size);
    }

    public void setColor(Color c) {
        this.color = c;
    }

    /**
     * Return the List of Symbol objects as a String
     *
     * @return the symbols as a String
     */
    abstract public String getSequence();

    /**
     * Return the moleculesCount of the symbols
     *
     * @return
     */
    abstract public int size();

    /**
     * Insert a gap character at the multiple alignment's index position
     *
     * @param index
     */
    abstract public void insertGap(int index);

    /**
     * Remove a gap character at the multiple alignment's index position
     *
     * @param index
     * @return
     */
    abstract public boolean removeGap(int index);

    abstract public void increaseSize();

    abstract public boolean isGap(int index);

    /**
     *
     * @param index from 0 to length-1
     * @return
     */
    abstract public Symbol getSymbol(int index);

    public java.util.List<Symbol> getSymbols() {
        return new ArrayList<Symbol>(this.symbols);
    }

    public void addSymbol(Symbol s) {
        this.symbols.add(s);
    }

    abstract public void drawSymbol(final Alignment alignment, final Graphics2D g, int currentX, int viewX, int currentY, int viewY, final GraphicContext gc, final int index);

    public Color getColor() {
        return this.color;
    }

    public boolean isSelected() {
        return selected;
    }

    public void setSelected(final boolean selected) {
        this.selected = selected;
    }

    public void setCountingFrom(int countingFrom) {
        this.countingFrom = countingFrom;
    }

    public int getCountingFrom() {
        return countingFrom;
    }

    /**
     * Return the position of a Symbol in this SymbolSequence
     * @param s
     * @return the position from 0 to length-1
     */
    public int getIndex(Symbol s) {
        return this.symbols.indexOf(s);
    }

    abstract public void setSymbolAt(Symbol s, int i);
}
