package fr.unistra.ibmc.s2s.rnalign.graphics;

import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.features.*;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.paradise.tools.ParadiseWorkingSession;
import fr.unistra.ibmc.s2s.rnalign.features.*;
import fr.unistra.ibmc.s2s.rnalign.utils.DrawingUtils;
import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.s2s.SelectionListener;
import fr.unistra.ibmc.s2s.S2SConfig;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.util.*;
import java.util.List;
import java.util.prefs.BackingStoreException;
import java.io.IOException;

public class AlignmentCanvas extends JPanel implements SelectionListener, MouseWheelListener, ActionListener, java.awt.event.MouseListener, java.awt.event.MouseMotionListener {

    public static final char STANDARD = 's',  INSERT = 'i',  DELETE = 'd',  USER_DEFINED_INTERACTIONS = 'u';
    GraphicContext gc;
    int pressedX, pressedY, mouseX, mouseY;

    public static Color A_Color = Color.BLUE, U_Color = Color.MAGENTA, G_Color = Color.GREEN, C_Color = Color.GRAY, SecondaryInteraction_Color = new Color(192,128,0), TertiaryInteraction_Color = new Color(0,192,128);

    //**** the parameters for the user defined interactions ****
    int firstUserDefinedInteractionX = -1, firstUserDefinedInteractionY = -1, secondUserDefinedInteractionX = -1, secondUserDefinedInteractionY = -1;
    int firstPositionInAlignment = -1, secondPositionInAlignment = -1;
    Alignment firstAlignment, secondAlignment;
    //*************
    char mode = AlignmentCanvas.STANDARD;
    public static final int MAXLABELSIZE = 20;
    private ParadiseWorkingSession workingSession;
    private AlignmentView mouseOverAlignmentView;
    int leftMarge = 1, topMarge = 1, bottomMarge = 0;
    private Rectangle drawingArea;
    Mediator mediator;
    private List<AlignmentView> alignmentViews;
    private AlignmentView selectedView, pressedView;
    private Alignment mainAlignment,  alignmentDiff;
    private Molecule moleculeToCompare;
    private List<SelectionListener> selectionListeners;
    private JPopupMenu popupMenu;

    public AlignmentCanvas(Mediator mediator, ParadiseWorkingSession workingSession, int screenWidth, int screenHeight) {
        this.workingSession = workingSession;
        this.gc = new GraphicContext();
        this.mediator = mediator;
        this.mediator.setAlignmentCanvas(this);
        this.setPreferredSize(new Dimension((int) (screenWidth * 0.8), (int) (screenHeight * 0.8)));
        this.setBackground(java.awt.Color.WHITE);
        this.addMouseListener(this);
        this.addMouseMotionListener(this);
        this.addMouseWheelListener(this);
        this.registerKeyboardAction(this, "standard", KeyStroke.getKeyStroke('s'), JComponent.WHEN_IN_FOCUSED_WINDOW);
        this.registerKeyboardAction(this, "insert", KeyStroke.getKeyStroke('i'), JComponent.WHEN_IN_FOCUSED_WINDOW);
        this.registerKeyboardAction(this, "delete", KeyStroke.getKeyStroke('d'), JComponent.WHEN_IN_FOCUSED_WINDOW);
        this.registerKeyboardAction(this, "user-defined-interactions", KeyStroke.getKeyStroke('u'), JComponent.WHEN_IN_FOCUSED_WINDOW);
        this.registerKeyboardAction(this, "move-view-right", KeyStroke.getKeyStroke('r'), JComponent.WHEN_IN_FOCUSED_WINDOW);
        this.registerKeyboardAction(this, "move-view-left", KeyStroke.getKeyStroke('e'), JComponent.WHEN_IN_FOCUSED_WINDOW);
        this.registerKeyboardAction(this, "move-residues-right", KeyStroke.getKeyStroke('p'), JComponent.WHEN_IN_FOCUSED_WINDOW);
        this.registerKeyboardAction(this, "move-residues-left", KeyStroke.getKeyStroke('o'), JComponent.WHEN_IN_FOCUSED_WINDOW);
        this.alignmentViews = new ArrayList<AlignmentView>();
        this.selectionListeners = new ArrayList<SelectionListener>();
        this.popupMenu = new JPopupMenu();
        this.popupMenu.add(new ColorMenu("Secondary Interactions Color",ColorMenu.SECONDARY_INTERACTIONS));
        this.popupMenu.add(new ColorMenu("Tertiary Interactions Color",ColorMenu.TERTIARY_INTERACTIONS));
        JMenu residues = new JMenu("Residues Color");
        this.popupMenu.add(residues);
        residues.add(new ColorMenu("Adenine",ColorMenu.ADENINE));
        residues.add(new ColorMenu("Uracil",ColorMenu.URACIL));
        residues.add(new ColorMenu("Guanine",ColorMenu.GUANINE));
        residues.add(new ColorMenu("Cytosine",ColorMenu.CYTOSINE));
        JMenuItem setAsDefault = new JMenuItem("Set Current Colors as Default");
        setAsDefault.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent actionEvent) {
                S2SConfig.saveUserColors();
                try {
                    S2SConfig.save();
                } catch (BackingStoreException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
        this.popupMenu.add(setAsDefault);
    }

    private void maybeShowPopup(MouseEvent e) {
        if (e.isPopupTrigger()) {
            this.popupMenu.show(e.getComponent(),
                    e.getX(), e.getY());
        }
    }

    public void removeBaseBaseInteraction(BaseBaseInteraction interaction) {
        for (Symbol s:this.mainAlignment.getStructure2D3D().getSymbols()) {
            if (s.getPositionInSequence() == interaction.getResidue().getAbsolutePosition() && this.mainAlignment.getBiologicalReferenceSequence().getMolecule().equals(interaction.getResidue().getMolecule())) {
                ((StructuralSymbol)s).removeBaseBaseInteraction(interaction);
                this.repaint();
                return;
            }
        }
    }

    public void addBaseBaseInteraction(BaseBaseInteraction interaction) {
        StructuralSymbol symbol = null, pairedSymbol=null;
        for (Symbol s:this.mainAlignment.getStructure2D3D().getSymbols()) {
            if (s.getPositionInSequence() == interaction.getResidue().getAbsolutePosition() && this.mainAlignment.getBiologicalReferenceSequence().getMolecule().equals(interaction.getResidue().getMolecule()))
                symbol = (StructuralSymbol)s;
            else if (s.getPositionInSequence() == interaction.getPartnerResidue().getAbsolutePosition() && this.mainAlignment.getBiologicalReferenceSequence().getMolecule().equals(interaction.getPartnerResidue().getMolecule())) {
                pairedSymbol = (StructuralSymbol)s;
            }
            if (symbol!= null && pairedSymbol != null) {
                symbol.addReferenceBaseBaseInteraction(interaction, pairedSymbol);
                this.repaint();
                return;
            }

        }
    }

    public void addSelectionListener(SelectionListener l) {
        this.selectionListeners.add(l);
    }

    public GraphicContext getGraphicContext() {
        return gc;
    }

    public void clearDisplay() {
        this.mainAlignment = null;
        this.selectedView = null;
        this.repaint();
    }

    public List<AlignmentView> getAlignmentViews() {
        return new ArrayList<AlignmentView>(alignmentViews);
    }

    public AlignmentView getSelectedView() {
        return this.selectedView;
    }

    public Alignment getMainAlignment() {
        return mainAlignment;
    }

    public Alignment getAlignmentDiff() {
        return alignmentDiff;
    }

    public void addView() {
        if (this.alignmentDiff == null) {
            this.alignmentViews.add(new AlignmentView(this, this.mainAlignment));
            this.alignmentViews.add(new ReversedAlignmentView(this, this.mainAlignment));
        } else {
            this.alignmentViews.add(new AlignmentView(this, this.alignmentDiff));
            this.alignmentViews.add(new AlignmentView(this, this.mainAlignment));
            this.alignmentViews.add(new ReversedAlignmentView(this, this.mainAlignment));
            this.alignmentViews.add(new ReversedAlignmentView(this, this.alignmentDiff));
        }
        this.repaint();
    }

    public void removeLastView() {
        if (this.alignmentDiff == null) {
            this.alignmentViews.remove(this.alignmentViews.get(this.alignmentViews.size() - 1));
            this.alignmentViews.remove(this.alignmentViews.get(this.alignmentViews.size() - 1));
        } else {
            this.alignmentViews.remove(this.alignmentViews.get(this.alignmentViews.size() - 1));
            this.alignmentViews.remove(this.alignmentViews.get(this.alignmentViews.size() - 1));
            this.alignmentViews.remove(this.alignmentViews.get(this.alignmentViews.size() - 1));
            this.alignmentViews.remove(this.alignmentViews.get(this.alignmentViews.size() - 1));
        }
        this.repaint();
    }

    void removeBiologicalSequenceAt(int index) {
        this.mainAlignment.removeBiologicalSequenceAt(index);
        this.mainAlignment.updateConservationScores();
        //now, the "number of sequences to display" parameter can be greater than the total number of sequences in the alignment, so...
        for (AlignmentView v:this.alignmentViews)
            if (v.alignment == this.mainAlignment)
                v.setNumberOfSequencesToDisplay(v.getNumberOfSequencesToDisplay() > this.mainAlignment.getBiologicalSequenceCount() ? this.mainAlignment.getBiologicalSequenceCount() : v.getNumberOfSequencesToDisplay());
        this.repaint();
    }

    public void setMainAlignment(final Alignment alignment) {
        this.mainAlignment = alignment;
        this.moleculeToCompare = this.mainAlignment.getBiologicalReferenceSequence().getMolecule();
        this.alignmentDiff = null;
        this.alignmentViews = new ArrayList<AlignmentView>();
        this.mainAlignment.updateConservationScores();
        this.addView();
        this.repaint();
    }

    /**
     * Set a new alignment to make the difference with the main one
     * @param alignment
     */
    public void setSecondAlignment(final Alignment alignment) {
        if (this.alignmentDiff != null) {
            this.removeAlignmentDiff();
        }
        this.alignmentDiff = alignment;
        List<AlignmentView> views = new ArrayList<AlignmentView>();
        for (int i = 0; i < this.alignmentViews.size(); i++) {
            if (!ReversedAlignmentView.class.isInstance(this.alignmentViews.get(i))) {
                views.add(new AlignmentView(this, this.alignmentDiff));
                views.add(this.alignmentViews.get(i));
            } else {
                views.add(this.alignmentViews.get(i));
                views.add(new ReversedAlignmentView(this, this.alignmentDiff));
            }
        }
        this.alignmentViews = views;
        this.alignmentDiff.updateConservationScores();
        this.repaint();
    }

    public void removeAlignmentDiff() {
        List<AlignmentView> views = new ArrayList<AlignmentView>();
        for (int i = 0; i < this.alignmentViews.size(); i++) {
            if (this.alignmentViews.get(i).getAlignment() != this.alignmentDiff) {
                views.add(this.alignmentViews.get(i));
            }
        }
        this.alignmentViews = views;
        this.alignmentDiff = null;
        this.repaint();
    }

    void setMediator(final Mediator mediator) {
        this.mediator = mediator;
    }

    public void setQuickDraw(final boolean quickDraw) {
        this.gc.setQuickDraw(quickDraw);
        this.repaint();
    }

    public void modifyFontSize(final int level) {
        final Font f = this.gc.getFont();
        this.gc.setFont(f.deriveFont((float) (f.getSize() + level)));
        this.repaint();
    }

    public void paintComponent(final java.awt.Graphics g) {
        super.paintComponent(g);
        if (this.mainAlignment != null) {
            final java.awt.Graphics2D g2 = (java.awt.Graphics2D) g;
            g2.setRenderingHint(java.awt.RenderingHints.KEY_TEXT_ANTIALIASING, java.awt.RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            g2.setRenderingHint(java.awt.RenderingHints.KEY_ANTIALIASING, java.awt.RenderingHints.VALUE_ANTIALIAS_ON);
            g2.setFont(this.gc.getFont());
            //we start with the BLACK color
            g2.setColor(Color.BLACK);
            final FontMetrics fm = g2.getFontMetrics();
            int previousHorizontalAdvance = this.gc.getHorizontalAdvance();
            this.gc.setHorizontalAdvance(fm.stringWidth("A"));
            float ratio = (float)previousHorizontalAdvance / (float)this.gc.getHorizontalAdvance();
            for (AlignmentView view : this.alignmentViews)
                view.viewX = ((int)(view.viewX/ratio));
            this.gc.setVerticalAdvance(fm.getHeight());
            this.gc.setLetterHeight(fm.getAscent() + fm.getDescent());
            this.gc.setLetterWidth(fm.charWidth('A'));
            this.gc.currentX = this.leftMarge * this.gc.getHorizontalAdvance();
            this.gc.currentY = this.topMarge * this.gc.getVerticalAdvance();
            int totalHeight = this.getTotalHeight(gc);
            this.setPreferredSize(new Dimension(this.getSize().width, totalHeight));
            this.revalidate();
            this.gc.currentWidth = this.getSize().width - 2 * this.leftMarge * this.gc.getHorizontalAdvance();
            this.drawingArea = new Rectangle(gc.currentX, gc.currentY, gc.currentWidth, this.getTotalHeight(gc));
            //g2.setColor(Color.RED);
            //g2.draw(this.drawingArea);
            //g2.setColor(Color.BLACK);
            for (AlignmentView view : new ArrayList<AlignmentView>(this.alignmentViews)) {
                gc.currentY += view.getTopMarge() * gc.getVerticalAdvance();
                gc.currentX += view.getLeftMarge() * gc.getHorizontalAdvance();
                gc.currentWidth -= 2 * view.getLeftMarge() * gc.getHorizontalAdvance();
                view.draw(g2, gc);
                gc.currentY += view.getBottomMarge() * gc.getVerticalAdvance();
                gc.currentX -= view.getLeftMarge() * gc.getHorizontalAdvance();
                gc.currentWidth += 2 * view.getLeftMarge() * gc.getHorizontalAdvance();
            }
            gc.setCurrentX(gc.getCurrentX() - this.leftMarge * gc.getHorizontalAdvance());
            gc.setCurrentY(gc.getCurrentY() + this.bottomMarge * gc.getVerticalAdvance());
            gc.currentWidth += 2 * this.leftMarge * gc.getHorizontalAdvance();
            int alignmentViewNb = this.alignmentViews.size();
            for (int i = 0; i < alignmentViewNb - 1; i++) {
                if (this.alignmentViews.get(i).alignment == this.mainAlignment && this.alignmentViews.get(i + 1).alignment == this.mainAlignment) {
                    this.drawBaseBaseInteractions(g2, gc, this.alignmentViews.get(i), this.alignmentViews.get(i + 1));
                } else if (this.alignmentViews.get(i).alignment == this.alignmentDiff && this.alignmentViews.get(i + 1).alignment == this.alignmentDiff) {
                    this.drawBaseBaseInteractions(g2, gc, this.alignmentViews.get(i), this.alignmentViews.get(i + 1));
                }
            }
            if (this.alignmentDiff != null) {
                for (int i = 0; i < this.alignmentViews.size(); i++) {
                    if (this.alignmentViews.get(i).alignment == this.alignmentDiff) {
                        if (i > 0 && this.alignmentViews.get(i - 1).alignment == this.mainAlignment) {
                            this.drawAlignmentsDiff(g2, gc, this.alignmentViews.get(i - 1), this.alignmentViews.get(i));
                        }
                        if (i < this.alignmentViews.size() - 1 && this.alignmentViews.get(i + 1).alignment == this.mainAlignment) {
                            this.drawAlignmentsDiff(g2, gc, this.alignmentViews.get(i), this.alignmentViews.get(i + 1));
                        }
                    }
                }
            }
            if (this.mode == AlignmentCanvas.USER_DEFINED_INTERACTIONS && this.firstUserDefinedInteractionX != -1 && this.firstUserDefinedInteractionY != -1 && this.secondUserDefinedInteractionX != -1 && this.secondUserDefinedInteractionY != -1) {
                g.drawLine(this.firstUserDefinedInteractionX, this.firstUserDefinedInteractionY, this.secondUserDefinedInteractionX, this.secondUserDefinedInteractionY);
            }
        }
    }

    private void drawAlignmentsDiff(final java.awt.Graphics2D g, GraphicContext gc, AlignmentView upperView, AlignmentView lowerView) {
        BiologicalSequence upperSequence = null,
                lowerSequence = null;
        for (BiologicalSequence s : upperView.alignment.getBiologicalSequences()) {
            if (s.getMolecule().getParadiseID().equals(this.moleculeToCompare.getParadiseID())) {
                upperSequence = s;
                break;
            }
        }
        if (upperSequence != null) {
            for (BiologicalSequence s : lowerView.alignment.getBiologicalSequences()) {
                if (s.getMolecule().getParadiseID().equals(this.moleculeToCompare.getParadiseID())) {
                    lowerSequence = s;
                    break;
                }
            }
        }
        if (upperSequence != null && lowerSequence != null) {
            //if the diff is between two reversed views
            if (ReversedAlignmentView.class.isInstance(upperView)) {
                UPPERVIEW:
                for (int j = upperView.getLastPos(); j >= upperView.getFirstPos(); j--) {
                    Symbol upperStructuralSymbol = upperView.getAlignment().getStructure2D3D().getSymbol(j),
                            upperBiologicalSymbol = upperSequence.getSymbol(j);
                    if (!upperBiologicalSymbol.isGap()) {
                        int upperPositionInSequence = upperBiologicalSymbol.getPositionInSequence();
                        LOWERVIEW:
                        for (int k = lowerView.getLastPos(); k >= lowerView.getFirstPos(); k--) {
                            Symbol lowerStructuralSymbol = lowerView.getAlignment().getStructure2D3D().getSymbol(k),
                                    lowerBiologicalSymbol = lowerSequence.getSymbol(k);
                            if (!lowerBiologicalSymbol.isGap()) {
                                int lowerPositionInSequence = lowerBiologicalSymbol.getPositionInSequence();
                                if (upperPositionInSequence == lowerPositionInSequence) {
                                    int upperStartX = upperView.getCurrentColumnCoordinate(j),
                                            upperEndX = upperView.getCurrentColumnCoordinate(j),
                                            upperY = (int) upperView.getDrawingArea().getMaxY() + gc.getVerticalAdvance(),
                                            lowerStartX = lowerView.getCurrentColumnCoordinate(k),
                                            lowerEndX = lowerView.getCurrentColumnCoordinate(k),
                                            lowerY = (int) lowerView.getDrawingArea().getMinY();
                                    while (--j >= upperView.getFirstPos() && !upperSequence.getSymbol(j).isGap() &&
                                            --k >= lowerView.getFirstPos() && !lowerSequence.getSymbol(k).isGap() &&
                                            upperStructuralSymbol.getSymbol() == upperView.getAlignment().getStructure2D3D().getSymbol(j).getSymbol() &&
                                            lowerStructuralSymbol.getSymbol() == lowerView.getAlignment().getStructure2D3D().getSymbol(k).getSymbol()) {
                                        upperEndX = upperView.getCurrentColumnCoordinate(j);
                                        lowerEndX = lowerView.getCurrentColumnCoordinate(k);
                                        upperStructuralSymbol = upperView.getAlignment().getStructure2D3D().getSymbol(j);
                                        lowerStructuralSymbol = lowerView.getAlignment().getStructure2D3D().getSymbol(k);
                                    }


                                    //p.lineTo(lowerEndX+gc.getHorizontalAdvance()/2, lowerY);
                                    GeneralPath p = new java.awt.geom.GeneralPath();
                                    p.moveTo(upperStartX - gc.getHorizontalAdvance() / 2, upperY);
                                    double distance = DrawingUtils.getDistance(upperEndX + gc.getHorizontalAdvance() / 2, upperY, lowerEndX + gc.getHorizontalAdvance() / 2, upperY);
                                    int middleEndX = (upperEndX + gc.getHorizontalAdvance() / 2 + lowerEndX + gc.getHorizontalAdvance() / 2) / 2,
                                            middleY = (upperY + lowerY) / 2,
                                            ctrl1X,
                                            ctrl2X;
                                    if (upperEndX < lowerEndX) {
                                        ctrl1X = middleEndX - (int) distance / 2;
                                        ctrl2X = middleEndX + (int) distance / 2;
                                    } else {
                                        ctrl1X = middleEndX + (int) distance / 2;
                                        ctrl2X = middleEndX - (int) distance / 2;
                                    }
                                    p.append(new CubicCurve2D.Float(upperEndX + gc.getHorizontalAdvance() / 2, (float) upperY, ctrl1X, middleY, ctrl2X, middleY, lowerEndX + gc.getHorizontalAdvance() / 2, lowerY), true);

                                    distance = DrawingUtils.getDistance(upperStartX - gc.getHorizontalAdvance() / 2, upperY, lowerStartX - gc.getHorizontalAdvance() / 2, upperY);
                                    int middleStartX = (upperStartX - gc.getHorizontalAdvance() / 2 + lowerStartX - gc.getHorizontalAdvance() / 2) / 2;
                                    if (upperEndX < lowerEndX) {
                                        ctrl1X = middleStartX + (int) distance / 2;
                                        ctrl2X = middleStartX - (int) distance / 2;
                                    } else {
                                        ctrl1X = middleStartX - (int) distance / 2;
                                        ctrl2X = middleStartX + (int) distance / 2;
                                    }
                                    p.append(new CubicCurve2D.Float(lowerStartX - gc.getHorizontalAdvance() / 2, (float) lowerY, ctrl1X, middleY, ctrl2X, middleY, upperStartX - gc.getHorizontalAdvance() / 2, upperY), true);
                                    g.setPaint(new GradientPaint((upperStartX + upperEndX) / 2, upperY, this.getStructuralAlignmentRelation(upperStructuralSymbol.getSymbol()),
                                            (lowerStartX + lowerEndX) / 2, lowerY, this.getStructuralAlignmentRelation(lowerStructuralSymbol.getSymbol())));
                                    g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.3f));
                                    g.fill(p);
                                    g.setColor(Color.BLACK);
                                    g.draw(p);
                                    //why the process has been stopped ?
                                    if (j < upperView.getFirstPos() || k < lowerView.getFirstPos()) {
                                        break UPPERVIEW;
                                    }
                                    if (upperSequence.getSymbol(j).isGap()) {
                                        continue UPPERVIEW;
                                    } else if (lowerSequence.getSymbol(k).isGap()) {
                                        j++;
                                        continue LOWERVIEW;
                                    } //if the problem is due to a difference of the structural symbol, we restart from the beginning
                                    else {
                                        //since its not due to a gap, we reduce the current positions by one since the loops will increase them
                                        j++;
                                        k++;
                                        continue UPPERVIEW;
                                    }

                                }

                            }
                        }
                    }
                }
            } //if the diff is between two non-reversed views
            else {
                UPPERVIEW:
                for (int j = upperView.getFirstPos(); j <= upperView.getLastPos(); j++) {
                    Symbol upperStructuralSymbol = upperView.getAlignment().getStructure2D3D().getSymbol(j),
                            upperBiologicalSymbol = upperSequence.getSymbol(j);
                    if (!upperBiologicalSymbol.isGap()) {
                        int upperPositionInSequence = upperBiologicalSymbol.getPositionInSequence();
                        LOWERVIEW:
                        for (int k = lowerView.getFirstPos(); k <= lowerView.getLastPos(); k++) {
                            Symbol lowerStructuralSymbol = lowerView.getAlignment().getStructure2D3D().getSymbol(k),
                                    lowerBiologicalSymbol = lowerSequence.getSymbol(k);
                            if (!lowerBiologicalSymbol.isGap()) {
                                int lowerPositionInSequence = lowerBiologicalSymbol.getPositionInSequence();
                                if (upperPositionInSequence == lowerPositionInSequence) {
                                    int upperStartX = upperView.getCurrentColumnCoordinate(j),
                                            upperEndX = upperView.getCurrentColumnCoordinate(j),
                                            upperY = (int) upperView.getDrawingArea().getMaxY() + gc.getVerticalAdvance(),
                                            lowerStartX = lowerView.getCurrentColumnCoordinate(k),
                                            lowerEndX = lowerView.getCurrentColumnCoordinate(k),
                                            lowerY = (int) lowerView.getDrawingArea().getMinY();
                                    while (++j <= upperView.getLastPos() && !upperSequence.getSymbol(j).isGap() &&
                                            ++k <= lowerView.getLastPos() && !lowerSequence.getSymbol(k).isGap() &&
                                            upperStructuralSymbol.getSymbol() == upperView.getAlignment().getStructure2D3D().getSymbol(j).getSymbol() &&
                                            lowerStructuralSymbol.getSymbol() == lowerView.getAlignment().getStructure2D3D().getSymbol(k).getSymbol()) {
                                        upperEndX = upperView.getCurrentColumnCoordinate(j);
                                        lowerEndX = lowerView.getCurrentColumnCoordinate(k);
                                        upperStructuralSymbol = upperView.getAlignment().getStructure2D3D().getSymbol(j);
                                        lowerStructuralSymbol = lowerView.getAlignment().getStructure2D3D().getSymbol(k);
                                    }


                                    //p.lineTo(lowerEndX+gc.getHorizontalAdvance()/2, lowerY);
                                    GeneralPath p = new java.awt.geom.GeneralPath();
                                    p.moveTo(upperStartX - gc.getHorizontalAdvance() / 2, upperY);
                                    double distance = DrawingUtils.getDistance(upperEndX + gc.getHorizontalAdvance() / 2, upperY, lowerEndX + gc.getHorizontalAdvance() / 2, upperY);
                                    int middleEndX = (upperEndX + gc.getHorizontalAdvance() / 2 + lowerEndX + gc.getHorizontalAdvance() / 2) / 2,
                                            middleY = (upperY + lowerY) / 2,
                                            ctrl1X,
                                            ctrl2X;
                                    if (upperEndX < lowerEndX) {
                                        ctrl1X = middleEndX - (int) distance / 2;
                                        ctrl2X = middleEndX + (int) distance / 2;
                                    } else {
                                        ctrl1X = middleEndX + (int) distance / 2;
                                        ctrl2X = middleEndX - (int) distance / 2;
                                    }
                                    p.append(new CubicCurve2D.Float(upperEndX + gc.getHorizontalAdvance() / 2, (float) upperY, ctrl1X, middleY, ctrl2X, middleY, lowerEndX + gc.getHorizontalAdvance() / 2, lowerY), true);

                                    distance = DrawingUtils.getDistance(upperStartX - gc.getHorizontalAdvance() / 2, upperY, lowerStartX - gc.getHorizontalAdvance() / 2, upperY);
                                    int middleStartX = (upperStartX - gc.getHorizontalAdvance() / 2 + lowerStartX - gc.getHorizontalAdvance() / 2) / 2;
                                    if (upperEndX < lowerEndX) {
                                        ctrl1X = middleStartX + (int) distance / 2;
                                        ctrl2X = middleStartX - (int) distance / 2;
                                    } else {
                                        ctrl1X = middleStartX - (int) distance / 2;
                                        ctrl2X = middleStartX + (int) distance / 2;
                                    }
                                    p.append(new CubicCurve2D.Float(lowerStartX - gc.getHorizontalAdvance() / 2, (float) lowerY, ctrl1X, middleY, ctrl2X, middleY, upperStartX - gc.getHorizontalAdvance() / 2, upperY), true);
                                    g.setPaint(new GradientPaint((upperStartX + upperEndX) / 2, upperY, this.getStructuralAlignmentRelation(upperStructuralSymbol.getSymbol()),
                                            (lowerStartX + lowerEndX) / 2, lowerY, this.getStructuralAlignmentRelation(lowerStructuralSymbol.getSymbol())));
                                    g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.3f));
                                    g.fill(p);
                                    g.setColor(Color.BLACK);
                                    g.draw(p);
                                    //why the process has been stopped ?
                                    if (j > upperView.getLastPos() || k > lowerView.getLastPos()) {
                                        break UPPERVIEW;
                                    }
                                    if (upperSequence.getSymbol(j).isGap()) {
                                        continue UPPERVIEW;
                                    } else if (lowerSequence.getSymbol(k).isGap()) {
                                        j--;
                                        continue LOWERVIEW;
                                    } //if the problem is due to a difference of the structural symbol, we restart from the beginning
                                    else {
                                        //since its not due to a gap, we reduce the current positions by one since the loops will increase them
                                        j--;
                                        k--;
                                        continue UPPERVIEW;
                                    }

                                }

                            }
                        }
                    }
                }
            }
        }
    }

    private void drawBaseBaseInteractions(final java.awt.Graphics2D g, GraphicContext gc, AlignmentView upperView, AlignmentView lowerView) {
        for (int j = upperView.getFirstPos(); j <= upperView.getLastPos(); j++) {
            if (upperView.selectedPositionsToDrawInteractions.isEmpty() || upperView.selectedPositionsToDrawInteractions.contains(upperView.getAlignment().getBiologicalReferenceSequence().getSymbol(j).getPositionInSequence())) {
                StructuralSymbol s = (StructuralSymbol)upperView.getAlignment().getStructure2D3D().getSymbol(j);
                //if the position is not a gap and not a MutatableSymbol (meaning not a non-gap symbol corresponding a structural position of the structure of reference)
                if (!s.isGap()) {
                    for (BaseBaseInteraction interaction : s.getReferenceBaseBaseInteractions()) {
                        if ((interaction.isSecondaryInteraction() && upperView.displaySecondaryInteractions || (!interaction.isSecondaryInteraction() && upperView.displayTertiaryInteractions))) {
                            StructuralSymbol pairedSymbol = s.getPairedSymbol(interaction);
                            int partnerPositionInAlignment = upperView.getAlignment().getStructure2D3D().getIndex(pairedSymbol);
                            //[fjossinet] the first part of the next test is to avoid to draw twice the interaction if the two residues establishing this interaction are displayed both in the upperView and lowerView
                            if ( /*(partnerPositionInAlignment < j && partnerPositionInAlignment >= upperView.firstPos && partnerPositionInAlignment <= upperView.lastPos) && */partnerPositionInAlignment >= lowerView.getFirstPos() && partnerPositionInAlignment <= lowerView.getLastPos()) {
                                int x1 = upperView.getCurrentColumnCoordinate(j),
                                        y1 = (int) upperView.getDrawingArea().getMaxY(),
                                        x2 = lowerView.getCurrentColumnCoordinate(partnerPositionInAlignment),
                                        y2 = (int) lowerView.getDrawingArea().getMinY();


                                //****************************
                                Color c = (interaction.isSecondaryInteraction()) ? AlignmentCanvas.SecondaryInteraction_Color : AlignmentCanvas.TertiaryInteraction_Color;

                                g.setColor(c.darker());
                                List<BaseEdgeShape> shapes = new ArrayList<BaseEdgeShape>();
                                final double distance = DrawingUtils.getDistance(x1, y1, x2, y2);
                                //using the VerticalAdvance for the symbolSize, this size will be fit the zoom level (since the VerticalAdvance is updated each time the font size change)
                                double symbolSize = 2 * gc.getVerticalAdvance();
                                //the points outside the symbol
                                Point2D[] ip1 = null;
                                if (distance >= symbolSize) {
                                    ip1 = DrawingUtils.fit(x1, y1, x2, y2, (distance - symbolSize) / 2);
                                }
                                if (ip1 != null) {
                                    String type = s.getEdge(interaction) + "" + pairedSymbol.getEdge(interaction);
                                    if ("HH".equals(type)) {
                                        if (!upperView.displayCisHH && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_CIS) {

                                        } else if (!upperView.displayTransHH && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_TRANS) {

                                        } else {
                                            shapes.add(new Line(new Point2D.Double(x1, y1), ip1[0]));
                                            shapes.add(new Line(ip1[1], new Point2D.Double(x2, y2)));
                                            this.drawHHSymbol(shapes, ip1[0], ip1[1], symbolSize, interaction.getOrientation());
                                        }
                                    } else if ("SS".equals(type)) {
                                        if (!upperView.displayCisSESE && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_CIS) {

                                        } else if (!upperView.displayTransSESE && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_TRANS) {

                                        } else {
                                            shapes.add(new Line(new Point2D.Double(x1, y1), ip1[0]));
                                            shapes.add(new Line(ip1[1], new Point2D.Double(x2, y2)));
                                            this.drawSESESymbol(shapes, ip1[0], ip1[1], symbolSize, interaction.getOrientation());
                                        }
                                    } else if ("WW".equals(type)) {
                                        if (!upperView.displayCisWCWC && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_CIS) {

                                        } else if (!upperView.displayTransWCWC && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_TRANS) {

                                        } else {
                                            shapes.add(new Line(new Point2D.Double(x1, y1), ip1[0]));
                                            shapes.add(new Line(ip1[1], new Point2D.Double(x2, y2)));
                                            this.drawWCWCSymbol(shapes, ip1[0], ip1[1], symbolSize, interaction.getOrientation());
                                        }
                                    } else if ("SH".equals(type) || "HS".equals(type)) {
                                        if (!upperView.displayCisHSE && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_CIS) {

                                        } else if (!upperView.displayTransHSE && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_TRANS) {

                                        } else {
                                            shapes.add(new Line(new Point2D.Double(x1, y1), ip1[0]));
                                            shapes.add(new Line(ip1[1], new Point2D.Double(x2, y2)));
                                            if ("SH".equals(type)) {
                                                this.drawSEHSymbol(shapes, ip1[0], ip1[1], symbolSize, interaction.getOrientation());
                                            } else {
                                                this.drawHSESymbol(shapes, ip1[0], ip1[1], symbolSize, interaction.getOrientation());
                                            }
                                        }
                                    } else if ("HW".equals(type) || "WH".equals(type)) {
                                        if (!upperView.displayCisHWC && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_CIS) {

                                        } else if (!upperView.displayTransHWC && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_TRANS) {

                                        } else {
                                            shapes.add(new Line(new Point2D.Double(x1, y1), ip1[0]));
                                            shapes.add(new Line(ip1[1], new Point2D.Double(x2, y2)));
                                            if ("HW".equals(type)) {
                                                this.drawHWCSymbol(shapes, ip1[0], ip1[1], symbolSize, interaction.getOrientation());
                                            } else {
                                                this.drawWCHSymbol(shapes, ip1[0], ip1[1], symbolSize, interaction.getOrientation());
                                            }
                                        }
                                    } else if ("WS".equals(type) || "SW".equals(type)) {
                                        if (!upperView.displayCisSEWC && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_CIS) {

                                        } else if (!upperView.displayTransSEWC && interaction.getOrientation() == BaseBaseInteraction.ORIENTATION_TRANS) {

                                        } else {
                                            shapes.add(new Line(new Point2D.Double(x1, y1), ip1[0]));
                                            shapes.add(new Line(ip1[1], new Point2D.Double(x2, y2)));
                                            if ("WS".equals(type)) {
                                                this.drawWCSESymbol(shapes, ip1[0], ip1[1], symbolSize, interaction.getOrientation());
                                            } else {
                                                this.drawSEWCSymbol(shapes, ip1[0], ip1[1], symbolSize, interaction.getOrientation());
                                            }
                                        }
                                    }
                                } //if we cannot draw the symbol (not enough space), we draw only a line
                                else {
                                    shapes.add(new Line(new Point2D.Double(x1, y1), new Point2D.Double(x2, y2)));
                                }
                                for (BaseEdgeShape shape : shapes) {
                                    shape.draw(g);
                                }
                                //******************
                            }
                        }
                    }
                }
            }
        }
        g.setColor(Color.BLACK);
    }

    private void drawHHSymbol(List<BaseEdgeShape> shapes, final Point2D point1, final Point2D point2, final double distance, char orientation) {
        //we divide the symbol in three part each enclosing 1/3 of the distance
        final Point2D[] points = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 3);
        //we draw the lines
        shapes.add(new Line(point1, points[0]));
        shapes.add(new Line(point2, points[1]));
        //we draw the squarre
        shapes.add(new Squarre(point1, point2, points[0], points[1], distance, orientation));
    }

    private void drawWCWCSymbol(List<BaseEdgeShape> shapes, final Point2D point1, final Point2D point2, final double distance, char orientation) {
        //we divide the symbol in three part each enclosing 1/3 of the distance
        final Point2D[] points = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 3);
        //we draw the lines
        shapes.add(new Line(point1, points[0]));
        shapes.add(new Line(point2, points[1]));
        //we draw the circle
        //the center of the circle
        final Point2D center = new Point2D.Double((point1.getX() + point2.getX()) / 2, (point1.getY() + point2.getY()) / 2);
        shapes.add(new Circle(points[0], center, orientation));
    }


    //TODO fjossinet the symbol is symetric but the interaction is asymetric, so fix that (take a look at how it is done in the RNA2DViewer tool)

    private void drawSESESymbol(List<BaseEdgeShape> shapes, final Point2D point1, final Point2D point2, final double distance, char orientation) {
        final Point2D[] points = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 3);
        //we draw the lines
        shapes.add(new Line(point1, points[0]));
        shapes.add(new Line(point2, points[1]));
        shapes.add(new Triangle(point1, point2, points[0], points[1], distance, orientation));
    }

    private void drawSEHSymbol(List<BaseEdgeShape> shapes, final Point2D point1, final Point2D point2, final double distance, char orientation) {
        final Point2D[] points = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 3);
        //we construct the square
        shapes.add(new Triangle(point1, point2, point1, points[0], distance, orientation));
        //we construct the triangle
        shapes.add(new Squarre(point1, point2, points[1], point2, distance, orientation));
        //we draw the line between the two forms
        shapes.add(new Line(points[0], points[1]));
    }

    private void drawHSESymbol(List<BaseEdgeShape> shapes, final Point2D point1, final Point2D point2, final double distance, char orientation) {
        final Point2D[] points = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 3);
        //we construct the square
        shapes.add(new Squarre(point1, point2, point1, points[0], distance, orientation));
        //we construct the triangle
        shapes.add(new Triangle(point1, point2, points[1], point2, distance, orientation));
        //we draw the line between the two forms
        shapes.add(new Line(points[0], points[1]));
    }

    private void drawHWCSymbol(List<BaseEdgeShape> shapes, final Point2D point1, final Point2D point2, final double distance, char orientation) {
        //we divide the symbol in three part each enclosing 1/3 of the distance
        final Point2D[] points = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 3);
        //we draw the circle
        //the center of the circle
        final Point2D center = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 6)[1];
        //we construct the squarre
        shapes.add(new Squarre(point1, point2, point1, points[0], distance, orientation));
        shapes.add(new Circle(point2, center, orientation));
        //we draw the line between the two forms
        shapes.add(new Line(points[0], points[1]));
    }

    private void drawWCHSymbol(List<BaseEdgeShape> shapes, final Point2D point1, final Point2D point2, final double distance, char orientation) {
        //we divide the symbol in three part each enclosing 1/3 of the distance
        final Point2D[] points = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 3);
        //we draw the circle
        //the center of the circle
        final Point2D center = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 6)[0];
        //we construct the squarre
        shapes.add(new Circle(point1, center, orientation));
        shapes.add(new Squarre(point1, point2, point2, points[1], distance, orientation));
        //we draw the line between the two forms
        shapes.add(new Line(points[0], points[1]));
    }

    private void drawWCSESymbol(List<BaseEdgeShape> shapes, final Point2D point1, final Point2D point2, final double distance, char orientation) {
        final Point2D[] points = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 3);
        final Point2D center = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 6)[0];
        //we construct the circle
        shapes.add(new Circle(point1, center, orientation));
        //we construct the triangle
        shapes.add(new Triangle(point1, point2, points[1], point2, distance, orientation));
        //we draw the line between the two forms
        shapes.add(new Line(points[0], points[1]));
    }

    private void drawSEWCSymbol(List<BaseEdgeShape> shapes, final Point2D point1, final Point2D point2, final double distance, char orientation) {
        final Point2D[] points = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 3);
        final Point2D center = DrawingUtils.fit(point1.getX(), point1.getY(), point2.getX(), point2.getY(), distance / 6)[1];
        //we construct the triangle
        shapes.add(new Triangle(point1, point2, points[0], point1, distance, orientation));
        //we construct the circle
        shapes.add(new Circle(point2, center, orientation));
        //we draw the line between the two forms
        shapes.add(new Line(points[0], points[1]));
    }

    private Color getStructuralAlignmentRelation(char symbol) {
        switch (symbol) {
            case '.':
                return Color.GREEN;
            case '{':
            case '}':
            case ')':
            case '(':
            case '>':
            case '<':
                return Color.BLUE;
            default:
                return Color.BLACK;
        }
    }

    private int getTotalHeight(GraphicContext gc) {
        int height = 0;
        for (AlignmentView view : this.alignmentViews) {
            height += view.getTopMarge() * gc.getVerticalAdvance();
            height += view.getHeight(gc);
            height += view.getBottomMarge() * gc.getVerticalAdvance();
        }
        height += this.topMarge * gc.getVerticalAdvance();
        height += this.bottomMarge * gc.getVerticalAdvance();
        return height;
    }

    public void mousePressed(final MouseEvent e) {
        maybeShowPopup(e);
        this.pressedX = e.getX();
        this.pressedY = e.getY();
        for (AlignmentView v : this.alignmentViews) {
            if (v.getDrawingArea().contains(e.getX(), e.getY())) {
                v.mousePressed(e);
                this.pressedView = v;
                break;
            }
        }
        this.repaint();
    }

    public void mouseDragged(final java.awt.event.MouseEvent e) {
        if (this.pressedView != null ) //to be able to move the residues even if the mouse is dragged outside the pressedView
            this.pressedView.mouseDragged(e);
        this.repaint();
    }

    public void mouseClicked(final MouseEvent e) {
        /*if (this.mode == AlignmentCanvas.USER_DEFINED_INTERACTIONS && this.firstUserDefinedInteractionX == -1 && this.firstUserDefinedInteractionY == -1) {
            this.firstUserDefinedInteractionX = e.getX();
            this.firstUserDefinedInteractionY = e.getY();
            for (AlignmentView v : this.alignmentViews) {
                if (v.contains(e.getX(), e.getY())) {
                    this.firstPositionInAlignment = v.mouseOverPositionInAlignment;
                    this.firstAlignment = v.alignment;
                    break;
                }
            }
        } else if (this.mode == AlignmentCanvas.USER_DEFINED_INTERACTIONS) {
            //can we construct a user defined interaction ??
            for (AlignmentView v : this.alignmentViews) {
                if (v.contains(e.getX(), e.getY())) {
                    if (this.firstAlignment == v.alignment && this.firstPositionInAlignment != -1 && v.mouseOverPositionInAlignment != -1) {
                        Symbol s1 = this.firstAlignment.getStructure2D3D().getSymbol(this.firstPositionInAlignment),
                                s2 = this.firstAlignment.getStructure2D3D().getSymbol(v.mouseOverPositionInAlignment);
                        if (MutableSymbol.class.isInstance(s1) && MutableSymbol.class.isInstance(s2)) {
                            ((MutableSymbol) s1).setUserDefinedInteraction(BaseBaseInteraction.ORIENTATION_CIS, BaseBaseInteraction.EDGE_WATSON_CRICK, (MutableSymbol) s2, BaseBaseInteraction.EDGE_WATSON_CRICK);
                            this.firstAlignment.getBiologicalReferenceSequence().setSymbolAt(Symbol.LockedGap, this.firstPositionInAlignment);
                            this.firstAlignment.getBiologicalReferenceSequence().setSymbolAt(Symbol.LockedGap, v.mouseOverPositionInAlignment);
                        }
                    }
                    break;
                }
            }
            this.firstUserDefinedInteractionX = -1;
            this.firstUserDefinedInteractionY = -1;
            this.secondUserDefinedInteractionX = -1;
            this.secondUserDefinedInteractionY = -1;
            this.repaint();
        } else {*/
        for (AlignmentView v : this.alignmentViews) {
            if (v.selectionBox.contains(e.getX(), e.getY())) {
                v.setSelected(true);
                this.selectedView = v;
                mediator.setPreferencesPanel(v.isDisplaySecondaryInteractions(), v.isDisplayTertiaryInteractions(), v.isLinkedToNextView(), v.getNumberOfSequencesToDisplay());
                //only one view can be selected
                for (AlignmentView _v : this.alignmentViews)
                    if (_v != v)
                        _v.setSelected(false);
                this.repaint();
                return;
            } else if (v.contains(e.getX(), e.getY())) {
                v.mouseClicked(e);
                this.repaint();
                return;
            }
        }
        //}
    }

    public void mouseReleased(final MouseEvent e) {
        maybeShowPopup(e);
    }

    public void mouseMoved(final MouseEvent e) {
        this.mouseX = e.getX();
        this.mouseY = e.getY();
        if (this.mode == AlignmentCanvas.USER_DEFINED_INTERACTIONS) {
            this.secondUserDefinedInteractionX = e.getX();
            this.secondUserDefinedInteractionY = e.getY();
        } else {
            this.mouseOverAlignmentView = null;
            for (AlignmentView v : this.alignmentViews) {
                if (v.contains(e.getX(), e.getY())) {
                    this.mouseOverAlignmentView = v;
                    break;
                }
            }
        }
        this.repaint();
    }

    public void mouseEntered(MouseEvent e) {
        this.requestFocus();
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseWheelMoved(MouseWheelEvent e) {
        int notches = e.getWheelRotation();
        if (notches < 0) {
            if (this.mouseOverAlignmentView != null) {
                this.mouseOverAlignmentView.setViewY(this.mouseOverAlignmentView.getViewY() + e.getScrollAmount() * gc.getVerticalAdvance());
            }
        } else {
            if (this.mouseOverAlignmentView != null) {
                this.mouseOverAlignmentView.setViewY(this.mouseOverAlignmentView.getViewY() - e.getScrollAmount() * gc.getVerticalAdvance());
            }
        }
    }

    public BufferedImage getImage() {
        BufferedImage bufferedImage = new BufferedImage(this.getSize().width, this.getSize().height, BufferedImage.TYPE_BYTE_INDEXED);
        Graphics2D g2 = bufferedImage.createGraphics();
        this.paintComponent(g2);
        g2.dispose();
        return bufferedImage;
    }

    void unselectAllSequences() {
        for (int i = 0; i < this.mainAlignment.getBiologicalSequenceCount(); i++) {
            this.mainAlignment.getBiologicalSequenceAt(i).setSelected(false);
        }
        this.repaint();
    }

    void setMode(char mode) {
        this.mode = mode;
    }

    public void actionPerformed(ActionEvent event) {
        if ("standard".equals(event.getActionCommand())) {
            this.setMode(AlignmentCanvas.STANDARD);
            if (this.mediator != null) {
                this.mediator.getS2s().printState(event.getActionCommand());
            }
        } else if ("insert".equals(event.getActionCommand())) {
            this.setMode(AlignmentCanvas.INSERT);
            if (this.mediator != null) {
                this.mediator.getS2s().printState(event.getActionCommand());
            }
        } else if ("delete".equals(event.getActionCommand())) {
            this.setMode(AlignmentCanvas.DELETE);
            if (this.mediator != null) {
                this.mediator.getS2s().printState(event.getActionCommand());
            }
        } else if ("user-defined-interactions".equals(event.getActionCommand())) {
            this.setMode(AlignmentCanvas.USER_DEFINED_INTERACTIONS);
            this.firstUserDefinedInteractionX = -1;
            this.firstUserDefinedInteractionY = -1;
            if (this.mediator != null) {
                this.mediator.getS2s().printState(event.getActionCommand());
            }
        }
        else if ("move-view-right".equals(event.getActionCommand())) {
            if (this.selectedView != null) {
                this.selectedView.setViewX(this.getSelectedView().getViewX()+mediator.getHorizontalStep()*gc.getHorizontalAdvance());
                this.repaint();
            }
        }
        else if ("move-view-left".equals(event.getActionCommand())) {
            if (this.selectedView != null) {
                this.selectedView.setViewX(this.getSelectedView().getViewX()-mediator.getHorizontalStep()*gc.getHorizontalAdvance());
                this.repaint();
            }
        }
        else if ("move-residues-right".equals(event.getActionCommand())) {
            if (this.selectedView != null) {
                this.selectedView.moveResidues(true);
            }
        }
        else if ("move-residues-left".equals(event.getActionCommand())) {
            if (this.selectedView != null) {
                this.selectedView.moveResidues(false);
            }
        }
    }

    public void clearSelectedPositions() {
        if (this.mainAlignment != null) {
            for (BiologicalSequence s : this.mainAlignment.getBiologicalSequences())
                s.clearSelectedPositions();
            this.repaint();
        }
    }

    public Map<BiologicalSequence, List<Integer>> getSelectedPositions() {
        Map<BiologicalSequence, List<Integer>> selectedPositions = new HashMap<BiologicalSequence, List<Integer>>();
        if (this.getMainAlignment() != null)
            for (BiologicalSequence s:this.getMainAlignment().getBiologicalSequences())
                selectedPositions.put(s,s.getSelectedPositions());
        return selectedPositions;
    }

    public void undo() {
        if (this.mainAlignment != null) {
            this.mainAlignment.undo(this.gc);
            this.repaint();
        }
    }

    public void redo() {
        if (this.mainAlignment != null) {
            this.mainAlignment.redo(this.gc);
            this.repaint();
        }
    }

    public void recalculateConservationScores() {
        if (this.mainAlignment != null) {
            this.mainAlignment.updateConservationScores();
            this.repaint();
        }
    }

    public void setMoleculeToCompare(Molecule molecule) {
        this.moleculeToCompare = molecule;
    }

    /*
private class ConservationsView {

   public Color[] conservationColors = new Color[5];
   private int height = 35;

   public ConservationsView() {
       conservationColors[0] = Color.BLUE;
       conservationColors[1] = Color.CYAN;
       conservationColors[2] = Color.GREEN;
       conservationColors[3] = Color.ORANGE;
       conservationColors[4] = Color.RED;
   }

   private void draw(Graphics2D g) {
       gc.setCurrentX(gc.getLeftMarge() + (gc.getFirstPos()-gc.notDisplayedResiduesCollapsed) * gc.getHorizontalAdvance());
       gc.setCurrentY(gc.getCurrentY() + this.height+(int)(0.5*gc.getLetterHeight()));
       double y;
       //if we have drawn more than the first symbols
       if (gc.getLastSeq() > 1)
           y = gc.getCurrentY()- gc.getViewY();
       else
           y = 3 * gc.getVerticalAdvance()+ this.height+(int)(0.5*gc.getLetterHeight());
       for (int i = gc.getFirstPos(); i <= gc.getLastPos(); i++) {
           if (gc.isCollapsedPosition(i))
               continue;
           if (!MutableSymbol.class.isInstance(alignment.getStructure2D3D().getSymbol(i))) {
               Symbol s = alignment.getStructure2D3D().getSymbol(i);
               char letterFromStruct = s.getSymbol();
               if ((letterFromStruct != '.' && letterFromStruct != '-' && letterFromStruct != '{' && letterFromStruct != '}')) {
                   StructuralSymbol symbol = (StructuralSymbol)s;
                   //We check only the secondaryInteraction
                   BaseBaseInteraction inter = symbol.getSecondaryInteraction();
                   double score = alignment.getConservationScore(inter);
                   double width = gc.getHorizontalAdvance()-1, height = this.height*score-0.25*gc.getLetterHeight();
                   double x = gc.getCurrentX()-gc.getViewX()+1;
                   GradientPaint paint = new GradientPaint((float)(x+width/4), (float)y, Color.WHITE, (float)(x+width/4), (float)(y-height),this.conservationColors[(int)(Math.round(score*(this.conservationColors.length-1)))]);
                   g.setPaint(paint);
                   g.fill(new Rectangle2D.Double(x,y-height,width/2,height));

                   double mutualinformation = alignment.calculateMutualInformation(i,alignment.getStructure2D3D().getIndex(symbol.getPairedSymbol(inter)));
                   //System.out.println(mutualinformation);
                   x = gc.getCurrentX()-gc.getViewX()+1+width/2;
                   height = this.height*mutualinformation-0.25*gc.getLetterHeight();
                   paint = new GradientPaint((float)(x+width/4), (float)y, Color.WHITE, (float)(x+width/4), (float)(y-height),Color.BLACK);
                   g.setPaint(paint);
                   //g.fill(new Rectangle2D.Double(x,getCurrentY()-gc.getViewY()-height,width/2,height));
               }
           }
           gc.setCurrentX(gc.getCurrentX() + gc.getHorizontalAdvance());
       }
       g.setColor(Color.BLACK);
   }

} */

    public void residuesSelected(Collection<Residue> residues, boolean isShiftDown) {
        Iterator<Residue> it = residues.iterator();
        if (!isShiftDown)
            this.clearSelectedPositions();
        while (it.hasNext()) {
            Residue r = it.next();
            for (BiologicalSequence seq:this.mainAlignment.getBiologicalSequences())
                if (seq.getMolecule().equals(r.getMolecule())) {
                    seq.addSelectedPosition(r.getAbsolutePosition());
                    break;
                }
            this.repaint();
        }
    }

    public void fireResiduesSelected(Collection<Residue> residues, boolean isShiftDown) {
        for (SelectionListener l:this.selectionListeners)
            l.residuesSelected(residues,isShiftDown);
    }

    public void structuralDomainSelected(StructuralDomain structuralDomain, boolean isShiftDown) {
        this.residuesSelected(structuralDomain.getResidues().getResidues(), isShiftDown);
    }

    public void fireStructuralDomainSelected(StructuralDomain structuralDomain, boolean isShiftDown) {
        for (SelectionListener l:this.selectionListeners)
            l.structuralDomainSelected(structuralDomain,isShiftDown);
    }

    public void interactionSelected(BaseBaseInteraction interaction, boolean isShiftDown) {
        this.residuesSelected(Arrays.asList(new Residue[]{interaction.getResidue(),interaction.getPartnerResidue()}), isShiftDown);
    }

    public void fireInteractionSelected(BaseBaseInteraction interaction, boolean isShiftDown) {
        for (SelectionListener l:this.selectionListeners)
            l.interactionSelected(interaction,isShiftDown);
    }

    private  class ColorMenu extends JMenu {

        private static final int SECONDARY_INTERACTIONS = 0, TERTIARY_INTERACTIONS = 2, ADENINE = 3, GUANINE = 4, CYTOSINE = 5, URACIL = 6 ;
        protected int mode;
        protected Border unselectedBorder;

        protected Border selectedBorder;

        protected Border activeBorder;

        protected Hashtable paneTable;

        protected ColorPane colorPane;

        public ColorMenu(String name, int mode) {
            super(name);
            this.mode = mode;
            unselectedBorder = new CompoundBorder(new MatteBorder(1, 1, 1, 1,
                    getBackground()), new BevelBorder(BevelBorder.LOWERED,
                    Color.white, Color.gray));
            selectedBorder = new CompoundBorder(new MatteBorder(1, 1, 1, 1,
                    Color.red), new MatteBorder(1, 1, 1, 1, getBackground()));
            activeBorder = new CompoundBorder(new MatteBorder(1, 1, 1, 1,
                    Color.blue), new MatteBorder(1, 1, 1, 1, getBackground()));

            JPanel p = new JPanel();
            p.setBorder(new EmptyBorder(5, 5, 5, 5));
            p.setLayout(new GridLayout(8, 8));
            paneTable = new Hashtable();

            int[] values = new int[] { 0 ,128, 192, 255 };

            for (int r = 0; r < values.length; r++) {
                for (int g = 0; g < values.length; g++) {
                    for (int b = 0; b < values.length; b++) {
                        Color c = new Color(values[r], values[g], values[b]);
                        ColorPane pn = new ColorPane(c);
                        p.add(pn);
                        paneTable.put(c, pn);
                    }
                }
            }
            add(p);
        }

        public void setColor(Color c) {
            Object obj = paneTable.get(c);
            if (obj == null)
                return;
            if (colorPane != null)
                colorPane.setSelected(false);
            colorPane = (ColorPane) obj;
            colorPane.setSelected(true);
        }

        public Color getColor() {
            if (colorPane == null)
                return null;
            return colorPane.getColor();
        }

        public void doSelection() {
            fireActionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
                    getActionCommand()));
        }

        private class ColorPane extends JPanel implements MouseListener {
            protected Color color;

            protected boolean isSelected;

            public ColorPane(Color c) {
                color = c;
                setBackground(c);
                setBorder(unselectedBorder);
                String msg = "R " + c.getRed() + ", G " + c.getGreen() + ", B "
                        + c.getBlue();
                setToolTipText(msg);
                addMouseListener(this);
            }

            public Color getColor() {
                return color;
            }

            public Dimension getPreferredSize() {
                return new Dimension(15, 15);
            }

            public Dimension getMaximumSize() {
                return getPreferredSize();
            }

            public Dimension getMinimumSize() {
                return getPreferredSize();
            }

            public void setSelected(boolean selected) {
                isSelected = selected;
                if (isSelected)
                    setBorder(selectedBorder);
                else
                    setBorder(unselectedBorder);
            }

            public boolean isSelected() {
                return isSelected;
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseClicked(MouseEvent e) {

            }

            public void mouseReleased(MouseEvent e) {
                switch (mode) {
                    case ColorMenu.SECONDARY_INTERACTIONS :
                        AlignmentCanvas.SecondaryInteraction_Color = this.color;
                        AlignmentCanvas.this.repaint();
                        mediator.getRna2dViewer().getSecondaryCanvas().repaint();
                        break;
                    case ColorMenu.TERTIARY_INTERACTIONS :
                        AlignmentCanvas.TertiaryInteraction_Color = this.color;
                        AlignmentCanvas.this.repaint();
                        mediator.getRna2dViewer().getSecondaryCanvas().repaint();
                        break;
                    case ColorMenu.ADENINE:
                        AlignmentCanvas.A_Color = this.color;
                        AlignmentCanvas.this.repaint();
                        mediator.getRna2dViewer().getSecondaryCanvas().repaint();
                        break;
                    case ColorMenu.URACIL:
                        AlignmentCanvas.U_Color = this.color;
                        AlignmentCanvas.this.repaint();
                        mediator.getRna2dViewer().getSecondaryCanvas().repaint();
                        break;
                    case ColorMenu.GUANINE :
                        AlignmentCanvas.G_Color = this.color;
                        AlignmentCanvas.this.repaint();
                        mediator.getRna2dViewer().getSecondaryCanvas().repaint();
                        break;
                    case ColorMenu.CYTOSINE :
                        AlignmentCanvas.C_Color = this.color;
                        AlignmentCanvas.this.repaint();
                        mediator.getRna2dViewer().getSecondaryCanvas().repaint();
                        break;
                }
            }

            public void mouseEntered(MouseEvent e) {
                setBorder(activeBorder);
            }

            public void mouseExited(MouseEvent e) {
                setBorder(isSelected ? selectedBorder : unselectedBorder);
            }
        }
    }
}
