package fr.unistra.ibmc.s2s.rnalign.graphics;

import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.paradise.core.features.StructuralDomain;
import fr.unistra.ibmc.paradise.core.features.BaseBaseInteraction;
import fr.unistra.ibmc.s2s.rnalign.features.*;
import fr.unistra.ibmc.s2s.S2S;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.util.*;
import java.util.List;


public class ReversedAlignmentView extends AlignmentView {

    public ReversedAlignmentView(AlignmentCanvas canvas, Alignment alignment) {
        super(canvas, alignment);
    }

    public void setNumberOfSequencesToDisplay(int nb) {
        this.numberOfSequencesToDisplay = nb;
        this.canvas.repaint();
    }

    public int getNumberOfSequencesToDisplay() {
        return this.numberOfSequencesToDisplay;
    }

    public int getLeftMarge() {
        return leftMarge;
    }

    public int getTopMarge() {
        return topMarge;
    }

    public int getBottomMarge() {
        return bottomMarge;
    }

    public void moveViewOnLeft() {
        //do nothing if the first position of the alignment is already displayed
        if (this.lastPos == this.alignment.getLength()-1)
            return;
        //test to avoid to go beyond the last position
        int length = (this.alignment.getLength()-this.lastPos)*canvas.gc.getVerticalAdvance(),
                width = (int)this.getDrawingArea().getWidth();
        this.viewX -= (length < width ? length : width);
        if (this.viewX < 0)
            this.viewX = 0;
        if (this.nextView != null)
            this.nextView.moveViewOnLeft();
        this.canvas.repaint();

    }

    public void moveViewOnRight() {
        //do nothing is the first position of the alignment is already displayed (important for the ReversedAlignmentView)
        if (this.firstPos == 0)
            return;
        int length = this.firstPos*this.canvas.gc.getVerticalAdvance(),
                width = (int)this.getDrawingArea().getWidth();
        //test to avoid to go beyond the first position
        this.viewX += (length < width ? length : width);
        if (this.viewX > this.alignment.getLength()*this.canvas.gc.getVerticalAdvance())
            this.viewX = this.alignment.getLength()*this.canvas.gc.getVerticalAdvance()-width;
        if (this.nextView != null)
            this.nextView.moveViewOnRight();
        this.canvas.repaint();
    }


    public void draw(Graphics2D g2, GraphicContext gc) {
        SymbolSequence seq = null;
        Rectangle2D r = null;
        boolean found = false;
        this.currentX = 0;
        this.currentY = 0;
        this.firstPos = -1;
        this.lastPos = -2;
        this.drawingArea = new Rectangle(gc.currentX, gc.currentY, gc.currentWidth, this.getHeight(gc));
        this.selectionBox = new Rectangle(gc.currentX-this.leftMarge*gc.getHorizontalAdvance(),gc.currentY-this.leftMarge*gc.getVerticalAdvance()/4,this.leftMarge*gc.getVerticalAdvance()/2,this.leftMarge*gc.getVerticalAdvance()/2);
        if (this.selected)
            g2.setColor(Color.RED);
        if (this.alignment == canvas.getMainAlignment())
            g2.fillOval((int)this.selectionBox.getMinX(), (int)this.selectionBox.getMinY(),(int)this.selectionBox.getWidth(), (int)this.selectionBox.getHeight());
        else
            g2.fill(this.selectionBox);
        g2.setColor(Color.BLACK);
        this.currentColumnCoordinates.clear();
        seq = this.alignment.getSequenceMeter();
        this.currentX = this.alignment.getMaxLabelSize()*gc.getHorizontalAdvance();
        this.currentY += gc.getVerticalAdvance();
        for (int j = 0; j < seq.size(); j++) {
            if (this.currentX - this.viewX < this.alignment.getMaxLabelSize()*gc.getHorizontalAdvance()) {
                this.currentX += gc.getHorizontalAdvance();
                continue;
            }
            if (this.drawingArea.getWidth() < (this.currentX - this.viewX + gc.getHorizontalAdvance()))
                break;
            if (this.firstPos == -1)
                this.firstPos = j;
            this.lastPos = j;
            seq.drawSymbol(this.alignment, g2, currentX+gc.currentX, viewX, this.currentY+gc.getCurrentY(), viewY, gc, seq.size()-1-j);
            this.currentColumnCoordinates.put(seq.size()-1-j,currentX+gc.currentX - viewX+gc.getLetterWidth()/2);
            if (!found) {
                r = new Rectangle2D.Float(this.currentX+gc.currentX - this.viewX, this.currentY+gc.getCurrentY()- gc.getLetterHeight()+(int)(0.25*gc.getLetterHeight()),gc.getLetterWidth() , gc.getLetterHeight());
                if (r.contains(canvas.mouseX, canvas.mouseY)) {
                    found = true;
                    mouseOverSequence = seq;
                    mouseOverPositionInAlignment = seq.size()-1-j;
                    if (!this.alignment.getBiologicalReferenceSequence().isGap(seq.size()-1-j)) {
                        mouseOverResidueFromRefSeq = new Residue(this.alignment.getBiologicalReferenceSequence().getSymbol(seq.size()-1-j).getPositionInSequence(),this.alignment.getBiologicalReferenceSequence().getMolecule());
                    }
                }
            }
            this.currentX += gc.getHorizontalAdvance();
        }
        seq = this.alignment.getStructure2D3D();
        this.currentY += gc.getVerticalAdvance();
        this.currentX = 0;
        this.currentX = this.alignment.getMaxLabelSize()*gc.getHorizontalAdvance() + this.firstPos * gc.getHorizontalAdvance();
        for (int j = this.firstPos; j <= this.lastPos; j++) {
            ((Structure2D3D)seq).drawSymbol(this,this.alignment, g2,currentX+gc.currentX, viewX, this.currentY+gc.getCurrentY(), viewY, gc, seq.size()-1-j);
            if (!found) {
                r = new Rectangle2D.Float(this.currentX+gc.currentX - this.viewX, this.currentY+gc.getCurrentY()- gc.getLetterHeight()+(int)(0.25*gc.getLetterHeight()),gc.getLetterWidth() , gc.getLetterHeight());
                if (r.contains(canvas.mouseX, canvas.mouseY)) {
                    found = true;
                    mouseOverSequence = seq;
                    mouseOverPositionInAlignment = seq.size()-1-j;
                    if (!this.alignment.getBiologicalReferenceSequence().isGap(seq.size()-1-j)) {
                        mouseOverResidueFromRefSeq = new Residue(this.alignment.getBiologicalReferenceSequence().getSymbol(seq.size()-1-j).getPositionInSequence(),this.alignment.getBiologicalReferenceSequence().getMolecule());
                    }
                }
            }
            this.currentX += gc.getHorizontalAdvance();
        }

        seq = this.alignment.getBiologicalSequenceAt(0);
        this.currentY += gc.getVerticalAdvance();
        this.currentX = 0;
        g2.drawString("S1", gc.currentX, gc.currentY+currentY);
        this.currentX = this.alignment.getMaxLabelSize()*gc.getHorizontalAdvance() + this.firstPos * gc.getHorizontalAdvance();
        for (int j = this.firstPos; j <= this.lastPos; j++) {
            seq.drawSymbol(this.alignment, g2,currentX+gc.currentX, viewX, this.currentY+gc.getCurrentY(), viewY, gc, seq.size()-1-j);
            if (!found) {
                r = new Rectangle2D.Float(this.currentX+gc.currentX - this.viewX, this.currentY+gc.getCurrentY()- gc.getLetterHeight()+(int)(0.25*gc.getLetterHeight()),gc.getLetterWidth() , gc.getLetterHeight());
                if (r.contains(canvas.mouseX, canvas.mouseY)) {
                    found = true;
                    mouseOverSequence = seq;
                    mouseOverPositionInAlignment = seq.size()-1-j;
                    if (!this.alignment.getBiologicalReferenceSequence().isGap(seq.size()-1-j)) {
                        mouseOverResidueFromRefSeq = new Residue(this.alignment.getBiologicalReferenceSequence().getSymbol(seq.size()-1-j).getPositionInSequence(),this.alignment.getBiologicalReferenceSequence().getMolecule());
                        realPositionFromMouseOverSequence = mouseOverSequence.getSymbol(mouseOverPositionInAlignment).getPositionInSequence();
                    }
                }
            }
            this.currentX += gc.getHorizontalAdvance();
            this.firstSeq = 1;
            this.lastSeq = 1;
        }
        int totalSeq = this.alignment.getBiologicalSequenceCount();
        SEQLOOP : for (int i = 1; i < totalSeq; i++) {
            seq = this.alignment.getBiologicalSequenceAt(i);
            this.currentY += gc.getVerticalAdvance();
            //the 4 parameter is for : SequenceMeter+Structure 2D/3D + Sequence of reference + first non-reference to display
            // if not enough place then the current sequence is not the first non-reference sequence to display
            if (this.currentY-this.viewY < 4*gc.getVerticalAdvance())
                continue SEQLOOP;
            // the "2" parameter is for the SequenceMeter and the Structure 2D/3D of reference
            if ((numberOfSequencesToDisplay+2)*gc.getVerticalAdvance()< (this.currentY- this.viewY))
                break SEQLOOP;
            if (this.firstSeq == 1)
                this.firstSeq = i;
            this.lastSeq = i;
            this.currentX = 0;
            g2.drawString("S"+(i+1), gc.currentX, gc.currentY+this.currentY-this.viewY);
            this.currentX = this.alignment.getMaxLabelSize()*gc.getHorizontalAdvance() + this.firstPos * gc.getHorizontalAdvance();
            for (int j = this.firstPos; j <= this.lastPos; j++) {
                seq.drawSymbol(this.alignment, g2,currentX+gc.currentX, viewX, this.currentY+gc.getCurrentY(), viewY, gc, seq.size()-1-j);
                if (!found) {
                    r = new Rectangle2D.Float(this.currentX+gc.currentX - this.viewX, this.currentY+gc.getCurrentY() - this.viewY - gc.getLetterHeight()+(int)(0.25*gc.getLetterHeight()),gc.getLetterWidth() , gc.getLetterHeight());
                    if (r.contains(canvas.mouseX, canvas.mouseY)) {
                        found = true;
                        mouseOverSequence = seq;
                        mouseOverPositionInAlignment = seq.size()-1-j;
                        if (!this.alignment.getBiologicalReferenceSequence().isGap(seq.size()-1-j)) {
                            mouseOverResidueFromRefSeq = new Residue(this.alignment.getBiologicalReferenceSequence().getSymbol(seq.size()-1-j).getPositionInSequence(),this.alignment.getBiologicalReferenceSequence().getMolecule());
                            realPositionFromMouseOverSequence = mouseOverSequence.getSymbol(mouseOverPositionInAlignment).getPositionInSequence();
                        }
                    }
                }
                this.currentX += gc.getHorizontalAdvance();
            }
        }
        // the "2" parameter is for the SequenceMeter and the Structure 2D/3D of reference
        gc.currentY += this.getHeight(gc);
        int tmp = this.firstPos;
        this.firstPos = seq.size()-1-this.lastPos;
        this.lastPos =  seq.size()-1-tmp;
    }

    public int getCurrentColumnCoordinate(int column) {
        return this.currentColumnCoordinates.get(column);
    }

    public int getHeight(GraphicContext gc) {
        return (numberOfSequencesToDisplay+2)*gc.getVerticalAdvance(); // the "2" parameter is for the SequenceMeter and the Structure 2D/3D of reference
    }

    public boolean contains(int x, int y) {
        if (this.drawingArea == null)
            return false;
        return this.drawingArea.contains(x,y);
    }

    public void moveResidues(boolean right) {
        if (this.selected ) {  //only if the view is selected
            for (BiologicalSequence seq:this.canvas.getMainAlignment().getBiologicalSequences()) {
                if (seq.hasSelectedPositions()) {
                    for (int i=this.firstPos;i<= this.lastPos;i++) {  // only if the residue selected is displayed
                        if (seq.isSelectedPosition(seq.getSymbol(i).getPositionInSequence()) ) {
                            int start = i, currentPosition = i;
                            if (!right) {
                                while (currentPosition < this.canvas.getMainAlignment().getLength() && (seq.getSymbol(currentPosition) != Symbol.Gap || seq.getSymbol(currentPosition).isLocked()) )
                                    currentPosition++;
                                if (currentPosition < this.canvas.getMainAlignment().getLength()) {
                                    MoveStep step = new MoveStep();
                                    seq.removeGap(currentPosition);
                                    seq.insertGap(start);
                                    step.add(seq,start, currentPosition);
                                    if (alignment.getBiologicalReferenceSequence() == seq) {
                                        canvas.getMainAlignment().getStructure2D3D().removeGap(currentPosition);
                                        canvas.getMainAlignment().getStructure2D3D().insertGap(start);
                                        step.add(canvas.getMainAlignment().getStructure2D3D(),start, currentPosition);
                                    }
                                    this.canvas.getMainAlignment().addStep(step);
                                    canvas.getMainAlignment().updateConservationScores(start, currentPosition);
                                    canvas.repaint();
                                }
                                i+=currentPosition;
                            }
                            else {
                                while (currentPosition >=0 && (seq.getSymbol(currentPosition) != Symbol.Gap || seq.getSymbol(currentPosition).isLocked()) )
                                    currentPosition--;
                                if (currentPosition >= 0 ) {
                                    MoveStep step = new MoveStep();
                                    seq.removeGap(currentPosition);
                                    seq.insertGap(start);
                                    step.add(seq,start, currentPosition);
                                    if (alignment.getBiologicalReferenceSequence() == seq) {
                                        canvas.getMainAlignment().getStructure2D3D().removeGap(currentPosition);
                                        canvas.getMainAlignment().getStructure2D3D().insertGap(start);
                                        step.add(canvas.getMainAlignment().getStructure2D3D(),currentPosition, start);
                                    }
                                    this.canvas.getMainAlignment().addStep(step);
                                    canvas.getMainAlignment().updateConservationScores(currentPosition,start);
                                    canvas.repaint();
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    public void mouseClicked(MouseEvent e) {
        //since the insertion or deletion of gaps are made for all the sequences => no modification in the conservation scores
        //not necessary to recalculate them
        if (alignment.getSequenceMeter() == this.mouseOverSequence) {
            if (canvas.mode == AlignmentCanvas.INSERT) {
                for (int i = 0; i < alignment.getSymbolSequenceCount(); i++)
                    alignment.getLetterSequenceAt(i).insertGap(this.mouseOverPositionInAlignment+1);
                alignment.addStep(new InsertionStep(this.mouseOverPositionInAlignment));
                //cannot use a redo/undo SymbolSequenceStep objects generated before the insert/delete gaps (the positions stored in the SymbolSequenceStep objects are unusable)
                //this.alignment.clearSteps();
            } else if (canvas.mode == AlignmentCanvas.DELETE) {
                for (int i = 0; i < alignment.getSymbolSequenceCount(); i++)
                    if (!alignment.getLetterSequenceAt(i).isGap(this.mouseOverPositionInAlignment))
                        return;
                for (int i = 0; i < alignment.getSymbolSequenceCount(); i++)
                    alignment.getLetterSequenceAt(i).removeGap(this.mouseOverPositionInAlignment);
                alignment.addStep(new DeletionStep(this.mouseOverPositionInAlignment));
                //cannot use a redo/undo SymbolSequenceStep objects generated before the insert/delete gaps (the positions stored in the SymbolSequenceStep objects are unusable)
                //this.alignment.clearSteps();
            }
        }
        else if (alignment.getStructure2D3D() == this.mouseOverSequence) {
            Symbol b = this.mouseOverSequence.getSymbol(this.mouseOverPositionInAlignment);
            if (e.isShiftDown()) {
                b.nextLetter();
                //if the current symbol is not a gap, then the corresponding position in the reference molecule has to be a locked gap to move along with the MutableSymbol
                if (b.getSymbol() != '-')
                    alignment.getBiologicalReferenceSequence().setSymbolAt(Symbol.LockedGap,this.mouseOverPositionInAlignment);
                else
                    alignment.getBiologicalReferenceSequence().setSymbolAt(Symbol.Gap,this.mouseOverPositionInAlignment);
            }
            else if (canvas.mode == AlignmentCanvas.INSERT) {
                for (int i = 0; i < alignment.getSymbolSequenceCount(); i++)
                    alignment.getLetterSequenceAt(i).insertGap(this.mouseOverPositionInAlignment+1);
                alignment.addStep(new InsertionStep(this.mouseOverPositionInAlignment));
                //cannot use a redo/undo SymbolSequenceStep objects generated before the insert/delete gaps (the positions stored in the SymbolSequenceStep objects are unusable)
                //this.alignment.clearSteps();
            } else if (canvas.mode == AlignmentCanvas.DELETE) {
                for (int i = 0; i < alignment.getSymbolSequenceCount(); i++)
                    if (!alignment.getLetterSequenceAt(i).isGap(this.mouseOverPositionInAlignment))
                        return;
                for (int i = 0; i < alignment.getSymbolSequenceCount(); i++)
                    alignment.getLetterSequenceAt(i).removeGap(this.mouseOverPositionInAlignment);
                alignment.addStep(new DeletionStep(this.mouseOverPositionInAlignment));

                //cannot use a redo/undo SymbolSequenceStep objects generated before the insert/delete gaps (the positions stored in the SymbolSequenceStep objects are unusable)
                //this.alignment.clearSteps();
            }
        }
        else if (this.mouseOverSequence != null) { //its a biological sequence
            Symbol b = this.mouseOverSequence.getSymbol(this.mouseOverPositionInAlignment);
            if (!b.isGap()) {
                List<Residue> residues = new ArrayList<Residue>();
                int posInSequence = b.getPositionInSequence();
                if (!e.isShiftDown())
                    canvas.clearSelectedPositions();
                //we select the residue clicked and all the residues in the same structural domain
                if (alignment.getBiologicalReferenceSequence() == this.mouseOverSequence) {
                    switch (canvas.mediator.getSelectionMode()) {
                        case S2S.SELECT_STRUCTURAL_DOMAIN:
                            StructuralDomain domain = canvas.getMainAlignment().getReferenceStructure().getEnclosingStructuralDomain(new Residue(posInSequence,((BiologicalSequence)this.mouseOverSequence).getMolecule()));
                            for (Residue r:domain.getResidues().getResidues()) {
                                residues.add(new Residue(r.getAbsolutePosition(),((BiologicalSequence)this.mouseOverSequence).getMolecule()));
                                ((BiologicalSequence)this.mouseOverSequence).addSelectedPosition(r.getAbsolutePosition());
                            }
                            canvas.fireStructuralDomainSelected(domain,e.isShiftDown());
                            break;
                        case S2S.SELECT_INTERACTION:
                            Residue r = new Residue(posInSequence,((BiologicalSequence)this.mouseOverSequence).getMolecule());
                            for (BaseBaseInteraction bbi:canvas.getMainAlignment().getReferenceStructure().getBaseBaseInteractions())
                                if (bbi.getResidue().equals(r) || bbi.getPartnerResidue().equals(r)) {
                                    residues.add(bbi.getResidue());
                                    residues.add(bbi.getPartnerResidue());
                                    ((BiologicalSequence)this.mouseOverSequence).addSelectedPosition(bbi.getResidue().getAbsolutePosition());
                                    ((BiologicalSequence)this.mouseOverSequence).addSelectedPosition(bbi.getPartnerResidue().getAbsolutePosition());
                                    canvas.fireInteractionSelected(bbi,e.isShiftDown());
                                }
                            break;
                        case S2S.SELECT_RESIDUE:
                            Residue _r = new Residue(posInSequence,((BiologicalSequence)this.mouseOverSequence).getMolecule());
                            residues.add(_r);
                            ((BiologicalSequence)this.mouseOverSequence).addSelectedPosition(posInSequence);
                            canvas.fireResiduesSelected(Arrays.asList(new Residue[]{_r}), e.isShiftDown());
                            break;
                    }

                }
                else
                    ((BiologicalSequence)this.mouseOverSequence).addSelectedPosition(posInSequence);
                if (e.isShiftDown()) {
                    int i=posInSequence+1;
                    while (i <= ((BiologicalSequence)this.mouseOverSequence).getMolecule().getLength() &&  !((BiologicalSequence)this.mouseOverSequence).isSelectedPosition(i)) {
                        ((BiologicalSequence)this.mouseOverSequence).addSelectedPosition(i);
                        i++;
                    }
                }
            }
        }
    }

    void translateViewX(final int transX) {
        this.viewX += transX;
        if (this.viewX < 0)
            this.viewX = 0;
        if (this.previousView != null)
            this.previousView.translateViewX(transX);
        if (this.nextView != null)
            this.nextView.translateViewX(transX);
    }

    void translateViewY(final int transY) {
        this.viewY += transY;
        if (this.viewY < 0)
            this.viewY = 0;
    }

    public Rectangle getDrawingArea() {
        return this.drawingArea;
    }
}
