package fr.unistra.ibmc.s2s.rnalign.graphics;

import fr.unistra.ibmc.paradise.tools.ParadiseWorkingSession;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.ParadiseID;
import fr.unistra.ibmc.paradise.core.Source;
import fr.unistra.ibmc.paradise.BiologicalSymbolException;
import fr.unistra.ibmc.s2s.Mediator;
import org.jdesktop.swingx.JXTable;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.*;
import java.awt.event.*;
import java.io.*;

public class SequencesList extends JXTable implements MouseListener, DragSourceListener, DropTargetListener, DragGestureListener {

    private Mediator mediator;
    private ParadiseWorkingSession workingSession;
    private DefaultTableModel model;
    private JPopupMenu popupMenu;
    private DragSource dragSource;
    private DropTarget dropTarget;
    private int startIndex = -1,  overIndex = -1;
    private static DataFlavor localObjectFlavor;
    static {
        try {
            localObjectFlavor = new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }
    private DataFlavor[] supportedFlavors = {localObjectFlavor};

    public SequencesList(final Mediator mediator, final ParadiseWorkingSession workingSession) {
        this.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        this.setSortable(false);
        this.dragSource = new DragSource();
        DragGestureRecognizer dgr = dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_MOVE, this);
        this.dropTarget = new DropTarget(this, this);
        this.mediator = mediator;
        this.workingSession = workingSession;
        this.model = new DefaultTableModel();
        this.model.addColumn("Label");
        this.model.addColumn("Sequence Name");
        this.addMouseListener(this);
        this.setModel(this.model);
        this.popupMenu = new JPopupMenu();
        JMenuItem menuItem;
        //this.popupMenu.add(menuItem);
        menuItem = new JMenuItem("Create new molecules from selection");
        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                for (int i : SequencesList.this.getSelectedRows()) {
                    java.util.List<Integer> selectedPositions = mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequenceAt(i).getSelectedPositions();
                    StringBuffer newSequence = null;
                    int previousPos = -1;
                    for (int pos:selectedPositions) {
                        if (pos-1 != previousPos) {
                            if (newSequence != null)
                                try {
                                    mediator.getS2s().getWorkingSession().createRNA(new ParadiseID(),mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequenceAt(i).getMolecule().getName(), Source.getGraphicalToolAsSource(mediator.getS2s().getDescription()), newSequence.toString());
                                } catch (BiologicalSymbolException e1) {
                                    e1.printStackTrace();
                                }
                            newSequence = new StringBuffer();
                        }
                        newSequence.append(mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequenceAt(i).getMolecule().getResidueAt(pos));
                        previousPos = pos;
                    }
                    if (newSequence != null)
                        try {
                            mediator.getS2s().getWorkingSession().createRNA(new ParadiseID(),mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequenceAt(i).getMolecule().getName(),Source.getGraphicalToolAsSource(mediator.getS2s().getDescription()), newSequence.toString());
                        } catch (BiologicalSymbolException e1) {
                            e1.printStackTrace();
                        }
                }
            }
        });
        //this.popupMenu.add(menuItem);
        menuItem = new JMenuItem("Remove selected sequences");
        menuItem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {

                int numRows = SequencesList.this.getRowCount();
                //the user cannot remove the first sequence (the reference molecule)
                for (int i = numRows - 1; i >= 1; i--) {
                    if (SequencesList.this.isCellSelected(i, 1)) {
                        mediator.getAlignmentCanvas().removeBiologicalSequenceAt(i);
                        SequencesList.this.model.removeRow(i);
                    }
                }
            }
        });
        this.popupMenu.add(menuItem);

        menuItem = new JMenuItem("Search motif");
        menuItem.addActionListener(new ActionListener() {

            private String lastMotif = "";

            public void actionPerformed(ActionEvent e) {
                String motif = JOptionPane.showInputDialog("Enter your motif", this.lastMotif);
                if (motif != null && motif.length() != 0) {
                    for (int i : SequencesList.this.getSelectedRows()) {
                        mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequenceAt(i).clearSelectedPositions();
                        mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequenceAt(i).searchFor(motif);
                    }
                    mediator.getAlignmentCanvas().repaint();
                    this.lastMotif = motif;
                }
            }
        });
        this.popupMenu.add(menuItem);

        menuItem = new JMenuItem("Show diff");
        menuItem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                for (int i = 0; i < SequencesList.this.getRowCount(); i++) {
                    if (SequencesList.this.isCellSelected(i, 1)) {
                        mediator.getAlignmentCanvas().setMoleculeToCompare(mediator.getMoleculeAt(i));
                        mediator.getAlignmentCanvas().repaint();
                        break;
                    }
                }
            }
        });
        //this.popupMenu.add(menuItem);
    }

    public void addRow(Molecule m) {
        this.model.addRow(new Object[]{"", m.getName()});
    }

    public void clearList() {
        int size = this.model.getRowCount();
        for (int i = 0; i < size; i++) {
            this.model.removeRow(0);
        }
    }

    public boolean isCellEditable(int row, int column) {
        if (column==1)
            return true;
        else
            return false;
    }

    public TableCellRenderer getCellRenderer(int row, int column) {
        if (row == overIndex && column == 1) {
            return new ReorderTableRenderer();
        } else if (column == 0) {
            return new FirstColumnTableRenderer();
        }
        return super.getCellRenderer(row, column);
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
        maybeShowPopup(e);
    }

    public void mouseReleased(MouseEvent e) {
        maybeShowPopup(e);
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    private void maybeShowPopup(MouseEvent e) {
        if (e.isPopupTrigger()) {
            this.popupMenu.show(e.getComponent(),
                    e.getX(), e.getY());
        }
    }

    public void dragEnter(DragSourceDragEvent dsde) {

    }

    public void dragOver(DragSourceDragEvent dsde) {
    }

    public void dropActionChanged(DragSourceDragEvent dsde) {
    }

    public void dragExit(DragSourceEvent dse) {
    }

    public void dropActionChanged(DropTargetDragEvent dtde) {
    }

    public void dragExit(DropTargetEvent dte) {
    }

    public void dragEnter(DropTargetDragEvent dtde) {
        if (dtde.getSource() != dropTarget) {
            dtde.rejectDrag();
        } else {
            dtde.acceptDrag(DnDConstants.ACTION_MOVE);
        }
    }

    public void dragOver(DropTargetDragEvent dtde) {
        if (dtde.getSource() != dropTarget) {
            dtde.rejectDrag();
        } else {
            Point dragPoint = dtde.getLocation();
            int index = this.rowAtPoint(dragPoint);
            if (index == -1 || index == 0) {
                return;
            } else {
                overIndex = index;
                repaint();
            }
        }
    }

    public void drop(DropTargetDropEvent dtde) {
        if (dtde.getSource() != dropTarget) {
            dtde.rejectDrop();
        } else {
            if (overIndex == -1 || overIndex == 0 || overIndex == startIndex) {
                return;
            } else {
                dtde.acceptDrop(DnDConstants.ACTION_MOVE);
                model.moveRow(startIndex, startIndex, overIndex);
                mediator.getAlignmentCanvas().getMainAlignment().moveSequence(overIndex, mediator.getAlignmentCanvas().getMainAlignment().getBiologicalSequenceAt(startIndex));
                mediator.getAlignmentCanvas().repaint();
            }
        }
    }

    public void dragGestureRecognized(DragGestureEvent dge) {
        Point clickPoint = dge.getDragOrigin();
        int index = this.rowAtPoint(clickPoint);
        //the user cannot drag the first sequence (the reference molecule)
        if (index == -1 || index == 0) {
            return;
        } else {
            startIndex = index;
            dragSource.startDrag(dge, Cursor.getDefaultCursor(), new MyTransferable(null), this);
        }
    }

    public void dragDropEnd(DragSourceDropEvent dsde) {
        startIndex = -1;
        overIndex = -1;
        repaint();
    }

    private class MyTransferable implements Transferable {

        private Object object;

        private MyTransferable(Object object) {
            this.object = object;
        }

        public DataFlavor[] getTransferDataFlavors() {
            return supportedFlavors;
        }

        public boolean isDataFlavorSupported(DataFlavor flavor) {
            return flavor.equals(localObjectFlavor);
        }

        public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
            if (isDataFlavorSupported(flavor)) {
                return object;
            } else {
                throw new UnsupportedFlavorException(flavor);
            }
        }
    }

    private class ReorderTableRenderer extends JLabel implements TableCellRenderer {

        public ReorderTableRenderer() {
            setOpaque(true);
        }

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            return this;
        }

        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            g.setColor(Color.BLACK);
            g.drawLine(0, 0, getSize().width, 0);
        }
    }

    private class FirstColumnTableRenderer extends JLabel implements TableCellRenderer {

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            this.setText("S" + (row + 1));
            return this;
        }
    }

}
