package fr.unistra.ibmc.s2s.rnalign.utils;

import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.geom.AffineTransform;

public class DrawingUtils {

    final static public Color[] colors = new Color[]{
            new Color(0, 0, 255),
            new Color(210, 105, 30),
            new Color(0, 255, 255),
            new Color(135, 206, 235),
            new Color(165, 42, 42),
            new Color(255, 0, 255),
            new Color(238, 130, 238),
            new Color(64, 224, 208),
            new Color(255, 127, 80),
            new Color(192, 192, 192),
            new Color(255, 0, 255),
            new Color(218, 112, 214),
            new Color(144, 238, 144),
            new Color(160, 32, 240),
            new Color(255, 0, 0),
            new Color(0, 255, 0),
            Color.PINK
    };

    /**
     * Return the same color for the same index
     *
     * @param index
     * @return
     */
    public static Color getIndexedColor(int index) {
        int cursor = 0, i = 0;
        while (true) {
            for (; i < DrawingUtils.colors.length; i++, cursor++) {
                if (cursor == index)
                    return DrawingUtils.colors[i];
            }
            i = 0;
        }
        /*Color c=null;
          if (index%2==0)
              c=new Color((int) (95.0 * (Math.sin(index)+1.0)), (int) (95.0 * (Math.sin(index+2.0)+1.0)), (int) (95.0 *(Math.sin(index+3.0)+1.0)));
          else
              c=new Color((int) (95.0 * (Math.cos(index+Math.PI/4.0)+1.0)), (int) (95.0 * (Math.cos(index+2.0*Math.PI/4.0)+1.0)), (int) (95.0 * (Math.cos(index+3.0*Math.PI/4.0)+1.0)));
          return c;*/
    }

    public static double getDistance(final double x1, final double y1, final double x2, final double y2) {
        final double horizontal = x1 - x2;
        final double vertical = y1 - y2;
        return Math.sqrt(horizontal * horizontal + vertical * vertical);
    }

    public static Point2D[] fit(final double x1, final double y1, final double x2, final double y2, final double distance) {
        final Point2D[] drawingPoints = new Point2D[2];
        final double angle = getAngle(getDistance(x1, y1, x1, y2), getDistance(x1, y2, x2, y2));
        final double newx1;
        final double newy1;
        final double newx2;
        final double newy2;

        if (x1 >= x2) {
            newx2 = x2 + DrawingUtils.getAdjacentSide(angle, distance);
            newx1 = x1 - DrawingUtils.getAdjacentSide(angle, distance);
        } else {
            newx2 = x2 - DrawingUtils.getAdjacentSide(angle, distance);
            newx1 = x1 + DrawingUtils.getAdjacentSide(angle, distance);
        }
        if (y1 >= y2) {
            newy2 = y2 + DrawingUtils.getOppositeSide(angle, distance);
            newy1 = y1 - DrawingUtils.getOppositeSide(angle, distance);
        } else {
            newy2 = y2 - DrawingUtils.getOppositeSide(angle, distance);
            newy1 = y1 + DrawingUtils.getOppositeSide(angle, distance);
        }

        drawingPoints[0] = new Point2D.Double(newx1, newy1);
        drawingPoints[1] = new Point2D.Double(newx2, newy2);

        return drawingPoints;
    }

    public static double getAdjacentSide(final double angle, final double hypothenuse) {
        return Math.cos(angle) * hypothenuse;
    }

    public static double getAngle(final double oppositeSide, final double adjacentSide) {
        return Math.atan(oppositeSide / adjacentSide);
    }

    public static double getOppositeSide(final double angle, final double hypothenuse) {
        return Math.sin(angle) * hypothenuse;
    }

    /**
     * Calculate the two points defining a line perpendicular to the one passing through p1 and p2.
     * This perpendicular line cut the (p1,p2) line on the p0 point.
     *
     * @param p0       the location on the line defined by p1 and p2 where we want to place the perpendicular points
     * @param p1
     * @param p2
     * @param distance the distance between each perpendicular point and pO
     * @return the perpendicular points
     */

    public static Point2D[] getPerpendicular(final Point2D p0, final Point2D p1, final Point2D p2, final double distance) {
        final Point2D[] pp = new Point2D[2];
        pp[0] = (Point2D) p0.clone();
        pp[1] = (Point2D) p0.clone();
        final double angle = getAngle(p1.getY() - p2.getY(), p1.getX() - p2.getX());
        pp[0].setLocation(pp[0].getX() + DrawingUtils.getOppositeSide(angle, distance), pp[0].getY() - DrawingUtils.getAdjacentSide(angle, distance));
        pp[1].setLocation(pp[1].getX() - DrawingUtils.getOppositeSide(angle, distance), pp[1].getY() + DrawingUtils.getAdjacentSide(angle, distance));
        return pp;
    }

    /**
     * Calculate a new point from an original one after a rotation step
     * defined by a specific angle value and a center point.
     *
     * @param p the original point
     * @param a the angle of rotation in radians
     * @param c the center of rotation
     * @return the new point after the rotation step
     */
    //@todo can we return the original point rotated or a new one ?
    public static Point2D getRotatedPoint(final Point2D p, final double a, final Point2D c) {
        if (a != 0.0) {
            //we set the rotation
            final AffineTransform rot = new AffineTransform();
            rot.setToRotation(a, c.getX(), c.getY());
            //we get the rotated point with this transformation
            final Point2D pointRot = rot.transform(p, null);
            return new Point2D.Double(pointRot.getX(), pointRot.getY());
        }
        return p;
    }



}
