package fr.unistra.ibmc.s2s.ssviewer.features;

import fr.unistra.ibmc.paradise.core.features.BaseBaseInteraction;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.s2s.ssviewer.graphics.GraphicContext;
import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.s2s.rnalign.graphics.AlignmentCanvas;
import fr.unistra.ibmc.s2s.ssviewer.utils.DrawingUtils;

import java.awt.*;
import java.awt.geom.Point2D;


public abstract class BaseBaseInteraction2D extends Interaction2D {

    protected BaseBaseInteraction interaction;

    public static BaseBaseInteraction2D getInteraction(final BaseBaseInteraction i, final Residues2DRepository repository, final Mediator mediator, final GraphicContext gc) {
        if (i.getFamily().equals("CWW")) {
            final Residue b = i.getResidue();
            final Residue partnerBase = i.getPartnerResidue();
            if (b != null && partnerBase != null && (((b.getSymbol() == Residue.GUANINE && partnerBase.getSymbol() == Residue.URIDINE) || (b.getSymbol() == Residue.URIDINE && partnerBase.getSymbol() == Residue.GUANINE))))
                return new Wobble(i, repository, mediator,gc);
            else
            if (b != null && partnerBase != null && (((b.getSymbol() == Residue.ADENINE && partnerBase.getSymbol() == Residue.URIDINE) || (b.getSymbol() == Residue.URIDINE && partnerBase.getSymbol() == Residue.ADENINE))))
                return new AUWC(i, repository, mediator,gc);
            else
            if (b != null && partnerBase != null && (((b.getSymbol() == Residue.GUANINE && partnerBase.getSymbol() == Residue.CYTOSINE) || (b.getSymbol() == Residue.CYTOSINE && partnerBase.getSymbol() == Residue.GUANINE))))
                return new GCWC(i, repository, mediator,gc);
            else
                return new WCWC(i, repository, mediator,gc);
        } else if (i.getFamily().equals("TWW")) {
            return new WCWC(i, repository, mediator,gc);
        } else if (i.getFamily().equals("CHH"))
            return new HH(i, repository, mediator,gc);
        else if (i.getFamily().equals("THH"))
            return new HH(i, repository, mediator,gc);
        else if (i.getFamily().equals("CSs"))
            return new SESE(i, repository, 'S', 's', mediator,gc);
        else if (i.getFamily().equals("TSs"))
            return new SESE(i, repository, 'S', 's', mediator,gc);
        else if (i.getFamily().equals("CsS"))
            return new SESE(i, repository, 's', 'S', mediator,gc);
        else if (i.getFamily().equals("TsS"))
            return new SESE(i, repository, 's', 'S', mediator,gc);
        else if (i.getFamily().equals("CSS"))
            return new SESE(i, repository, 'S', 'S', mediator,gc);
        else if (i.getFamily().equals("TSS"))
            return new SESE(i, repository, 'S', 'S', mediator,gc);
        else if (i.getFamily().equals("CWH"))
            return new WCH(i, repository, mediator,gc);
        else if (i.getFamily().equals("CHW"))
            return new HWC(i, repository, mediator,gc);
        else if (i.getFamily().equals("TWH"))
            return new WCH(i, repository, mediator,gc);
        else if (i.getFamily().equals("THW"))
            return new HWC(i, repository, mediator,gc);
        else if (i.getFamily().equals("CHS"))
            return new HSE(i, repository, mediator,gc);
        else if (i.getFamily().equals("CSH"))
            return new SEH(i, repository, mediator,gc);
        else if (i.getFamily().equals("THS"))
            return new HSE(i, repository, mediator,gc);
        else if (i.getFamily().equals("TSH"))
            return new SEH(i, repository, mediator,gc);
        else if (i.getFamily().equals("CWS"))
            return new WCSE(i, repository, mediator,gc);
        else if (i.getFamily().equals("CSW"))
            return new SEWC(i, repository, mediator,gc);
        else if (i.getFamily().equals("TWS"))
            return new WCSE(i, repository, mediator,gc);
        else if (i.getFamily().equals("TSW"))
            return new SEWC(i, repository, mediator,gc);
        else {
            System.out.println("Unknow interaction found : " + i.getFamily());
            return new Unknown(i, repository, mediator,gc);
        }
    }

    BaseBaseInteraction2D(final BaseBaseInteraction interaction, final Residues2DRepository repository, final Mediator mediator, final GraphicContext gc) {
        super(repository, interaction.getResidue(), interaction.getPartnerResidue(), mediator, gc);
        this.interaction = interaction;
        this.defaultSymbol = new SymbolsArray();
        this.realSymbol = new SymbolsArray();
        if (interaction.isSecondaryInteraction()) {
            this.color = Interaction2D.DEFAULT_SECONDARY_INTERACTION_COLOR;
        } else {
            this.color = Interaction2D.DEFAULT_TERTIARY_INTERACTION_COLOR;
        }
        this.updated = true;
        this.getResidue().addInteraction(this);
        this.getPartnerResidue().addInteraction(this);
    }

    public boolean isDisplayed() {
        if (interaction.isSecondaryInteraction())
            return true && super.isDisplayed();
        else
            return this.gc.containFamily(interaction.getFamily()) && super.isDisplayed();
    }


    public BaseBaseInteraction getBaseBaseInteractionFeature() {
        return this.interaction;
    }

    /**
     * Check if the interaction is temporary displayed on the canvas
     *
     * @return
     */
    public boolean isTemporaryDisplayed() {
        return gc.containFamily(interaction.getFamily());
    }

    public void draw(Graphics2D g2) {
        if (this.interaction.getResidue() != null && this.interaction.getPartnerResidue() != null)
            if (this.interaction.isSecondaryInteraction() || gc.isTertiaryInteractionsDisplayed()) {
                if (this.isDisplayed() && this.isInsideDrawingArea()) {
                    if (this.updated) {
                        this.setOutsidePoints();
                        this.updated = false;
                    }
                    //TODO find how to put some calculation steps in the previous if statement to improve 2D draw
                    if (this.real_op != null) {
                        //calculate the outside points fitted with the current view and zoom level
                        this.op = new java.awt.geom.Point2D[2];
                        this.op[0] = new java.awt.geom.Point2D.Double(this.real_op[0]
                                .getX()
                                * this.gc.getFinalZoomLevel() + this.gc.getViewX(),
                                this.real_op[0].getY() * this.gc.getFinalZoomLevel()
                                        + this.gc.getViewY());
                        this.op[1] = new java.awt.geom.Point2D.Double(this.real_op[1]
                                .getX()
                                * this.gc.getFinalZoomLevel() + this.gc.getViewX(),
                                this.real_op[1].getY() * this.gc.getFinalZoomLevel()
                                        + this.gc.getViewY());
                    }
                    //we remove the precedent op points if no real_op
                    else
                        this.op = null;
                    if (this.isSelected())
                        g2.setColor(java.awt.Color.red);
                    else if (interaction.isSecondaryInteraction())
                        g2.setColor(AlignmentCanvas.SecondaryInteraction_Color);
                    else
                        g2.setColor(AlignmentCanvas.TertiaryInteraction_Color);
                    SymbolsArray.Iterator sc = null;
                    if ((this.drawRealSymbol() || gc.containFamily(interaction.getFamily()))
                            && this.getRealSymbol() != null) {
                        this.setRealSymbol();
                        sc = this.getRealSymbol().iterator();
                        while (sc.hasNext())
                            sc.next().draw(g2);
                    } else if (!this.drawRealSymbol() && this.getDefaultSymbol() != null) {
                        this.setDefaultSymbol();
                        sc = this.getDefaultSymbol().iterator();
                        while (sc.hasNext())
                            sc.next().draw(g2);
                    }
                }
            }
    }

    protected void setOutsidePoints() {
        final Residue2D residue = this.repository.getResidue(this.residue1);
        final Residue2D partnerResidue = this.repository.getResidue(this.residue2);
        //we calculate the points outside the bases
        if (this.interaction.isSecondaryInteraction() || (residue.isDisplayed() && partnerResidue.isDisplayed())) {
            final double dist = DrawingUtils.getDistance(residue.getRealX(), residue.getRealY(), partnerResidue.getRealX(), partnerResidue.getRealY());
            if (dist >= this.gc.getRealHalf() * 2)
                this.real_op = DrawingUtils.fit(residue.getRealCenterX(), residue.getRealCenterY(), partnerResidue.getRealCenterX(), partnerResidue.getRealCenterY(), this.gc.getRealHalf());
            else
                this.real_op = null;
        } else if (!residue.isDisplayed() && partnerResidue.isDisplayed()) {
            this.real_op = DrawingUtils.fit(residue.getRealCenterX(), residue.getRealCenterY(), partnerResidue.getRealCenterX(), partnerResidue.getRealCenterY(), this.gc.getRealHalf());
            this.real_op[0] = residue.getRealCenter();
        } else if (!partnerResidue.isDisplayed() && residue.isDisplayed()) {
            this.real_op = DrawingUtils.fit(residue.getRealCenterX(), residue.getRealCenterY(), partnerResidue.getRealCenterX(), partnerResidue.getRealCenterY(), this.gc.getRealHalf());
            this.real_op[1] = partnerResidue.getRealCenter();
        } else {
            this.real_op = new Point2D[2];
            this.real_op[0] = residue.getRealCenter();
            this.real_op[1] = partnerResidue.getRealCenter();
        }
    }

    protected void setRealSymbol() {
        this.realSymbol.clear();
        if (op != null) {
            //if we have a secondary interaction
            if (this.interaction.isSecondaryInteraction())
                this.setRealSymbol(op[0], op[1], this.gc.getCurrentSymbolSize());
                //if we have a tertiary interaction
            else {
                //we determine the distance between the two new points
                final double distance = DrawingUtils.getDistance(op[0], op[1]);
                //the points outside the symbol
                Point2D[] ip1 = null;
                if (distance >= this.gc.getCurrentSymbolSize())
                    ip1 = DrawingUtils.fit(op[0].getX(), op[0].getY(), op[1].getX(), op[1].getY(), (distance - this.gc.getCurrentSymbolSize()) / 2);
                if (ip1 != null) {
                    this.realSymbol.addSymbol(new Line(op[0], ip1[0], this.gc));
                    this.realSymbol.addSymbol(new Line(ip1[1], op[1], this.gc));
                    this.setRealSymbol(ip1[0], ip1[1], this.gc.getCurrentSymbolSize());
                }
                //if we cannot draw the symbol (not enough space), we draw only a line
                else
                    this.realSymbol.addSymbol(new Line(op[0], op[1], this.gc));
            }
        }
    }

    public boolean equals(Object obj) {
        if (!this.getClass().isInstance(obj))
            return false;
        BaseBaseInteraction2D inter = (BaseBaseInteraction2D) obj;
        return this.interaction.getOrientation() == inter.interaction.getOrientation() &&
                ((this.interaction.getResidue().equals(inter.interaction.getResidue()) && this.interaction.getPartnerResidue().equals(inter.interaction.getPartnerResidue()) && this.interaction.getEdge1() == inter.interaction.getEdge1() && this.interaction.getEdge2() == inter.interaction.getEdge2()) ||
                        (this.interaction.getResidue().equals(inter.interaction.getPartnerResidue()) && this.interaction.getPartnerResidue().equals(inter.interaction.getResidue()) && this.interaction.getEdge2() == inter.interaction.getEdge1() && this.interaction.getEdge1() == inter.interaction.getEdge2()));

    }
}
