/*
 * Circle.java
 *
 * Created on 29 janvier 2003, 10:20
 */

package fr.unistra.ibmc.s2s.ssviewer.features;

import fr.unistra.ibmc.s2s.ssviewer.graphics.GraphicContext;
import fr.unistra.ibmc.s2s.ssviewer.utils.DrawingUtils;

/**
 * @author fabricej
 */
class Circle extends Symbol {

    static private int steps = 30;
    private java.awt.geom.Point2D firstPoint, centerPoint;

    /**
     * Creates a new instance of Circle
     */
    Circle(final java.awt.geom.Point2D firstPoint, final java.awt.geom.Point2D centerPoint, final char orientation, GraphicContext gc) {
        super(orientation, gc);
        this.firstPoint = firstPoint;
        this.centerPoint = centerPoint;
        this.shape = new java.awt.geom.GeneralPath();
        final DoubleArray points = new DoubleArray(1);
        for (int i = 1; i <= steps; i++) {
            final java.awt.geom.Point2D point = DrawingUtils.getRotatedPoint(this.firstPoint, 2 * Math.PI / steps * i, this.centerPoint);
            points.add(point.getX());
            points.add(point.getY());
        }
        final double[] coordinates = points.getArray();
        ((java.awt.geom.GeneralPath) this.shape).moveTo((float) firstPoint.getX(), (float) firstPoint.getY());
        for (int i = 0; i < coordinates.length; i = i + 2) {
            ((java.awt.geom.GeneralPath) this.shape).lineTo((float) coordinates[i], (float) coordinates[i + 1]);
        }
        ((java.awt.geom.GeneralPath) this.shape).closePath();
    }


    public String toSVG() {
        final StringBuffer svg = new StringBuffer();
        svg.append("<circle " + super.getSVGStyle(this.getSVGColor()) + " cx=\"" +
                (this.centerPoint.getX() - gc.getViewX()) / gc.getFinalZoomLevel() +
                "\" cy=\"" + (this.centerPoint.getY() - gc.getViewY()) / gc.getFinalZoomLevel() +
                "\" r=\"" + DrawingUtils.getDistance(this.centerPoint, this.firstPoint) / gc.getFinalZoomLevel() + "\"/>\n");
        return svg.toString();
    }

    public String toString() {
        return "Circle";
    }

    static private class DoubleArray {

        private int position;
        private double[] array;

        private DoubleArray(final int size) {
            this.array = new double[size];
        }

        private void add(final double value) {
            if (position >= this.array.length) {
                final double[] tmp = new double[this.array.length + (position + 1 - this.array.length)];
                System.arraycopy(this.array, 0, tmp, 0, this.array.length);
                this.array = tmp;
            }
            this.array[position++] = value;
        }

        private int length() {
            return this.array.length;
        }

        private double[] getArray() {
            return this.array;
        }
    }

}
