package fr.unistra.ibmc.s2s.ssviewer.features;

import fr.unistra.ibmc.paradise.core.features.Helix;
import fr.unistra.ibmc.paradise.core.features.BaseBaseInteraction;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.s2s.ssviewer.utils.DrawingUtils;
import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.s2s.rnalign.graphics.AlignmentCanvas;
import fr.unistra.ibmc.s2s.ssviewer.graphics.GraphicContext;

import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.geom.Area;
import java.util.*;
import java.util.List;

public class Helix2D extends StructuralDomain2D {

    Set<BaseBaseInteraction2D> secondaryInteractions;
    public static final Color DEFAULT_COLOR = Color.BLUE;
    private Point2D middle, perpendicular;
    public double dx, dy;
    public boolean fixed;
    private Area area;

    public Helix2D(final Helix h, final Residues2DRepository repository, final Mediator mediator, final GraphicContext gc) {
        super(h,repository, mediator, gc);
        //the secondary interactions of an Helix are ordered and cannot accept redundancy
        this.secondaryInteractions = new TreeSet<BaseBaseInteraction2D>(new Comparator() {
            public int compare(Object o1, Object o2) {
                BaseBaseInteraction bb1 = ((BaseBaseInteraction2D)o1).getBaseBaseInteractionFeature(), bb2 = ((BaseBaseInteraction2D)o2).getBaseBaseInteractionFeature();
                return new Integer(bb1.getResidue().getAbsolutePosition()).compareTo(bb2.getResidue().getAbsolutePosition());
            }
        });
        for (BaseBaseInteraction inter:h.getBaseBaseInteractions())
            this.secondaryInteractions.add(BaseBaseInteraction2D.getInteraction(inter,this.repository, mediator,gc));
        this.setColor(Helix2D.DEFAULT_COLOR);
        for (Residue2D r:this.getResidues())
            r.setStructuralDomain(this);
    }

    public Helix getHelixFeature() {
        return (Helix)this.structuralDomain;
    }

    public List<Residue2D> getResidues() {
        List<Residue2D> residues = new ArrayList<Residue2D>(this.getLength());
        for (Residue r : this.getHelixFeature().getResidues().getResidues())
            residues.add(this.repository.getResidue(r));
        return residues;
    }

    public Residue2D get5PrimeEnd(Residue2D _3PrimeEnd) {
        return this.repository.getResidue(this.getHelixFeature().get5PrimeEnd(_3PrimeEnd.getResidue2DFeature().getResidues().iterator().next()));
    }

    public Residue2D get3PrimeEnd(Residue2D _5PrimeEnd) {
        return this.repository.getResidue(this.getHelixFeature().get3PrimeEnd(_5PrimeEnd.getResidue2DFeature().getResidues().iterator().next()));
    }

    public Residue2D getPairedResidue(Residue2D r) {
        for (BaseBaseInteraction2D interaction:this.secondaryInteractions)
            if (this.repository.getResidue(interaction.getBaseBaseInteractionFeature().getResidue()).equals(r))
                return this.repository.getResidue(interaction.getBaseBaseInteractionFeature().getPartnerResidue());
            else if (this.repository.getResidue(interaction.getBaseBaseInteractionFeature().getPartnerResidue()).equals(r))
                return this.repository.getResidue(interaction.getBaseBaseInteractionFeature().getResidue());
        return null;
    }

    public void draw(final java.awt.Graphics2D g2) {
        if (this.isInsideDrawingArea()) {
            if (this.isSelected()) {
                g2.setColor(java.awt.Color.red);
            }
            else
                g2.setColor(AlignmentCanvas.SecondaryInteraction_Color);
            for (BaseBaseInteraction2D inter:this.secondaryInteractions)
                inter.draw(g2);
            for (Residue2D r:this.getResidues())
                r.draw(g2);
            if (this.gc.isLabelsDisplayed()) {
                g2.setColor(Color.GRAY);
                //########## draw helix label ###################
                int middle = (this.getLength()+1)/2;
                //if the residues choosen can display their residues id (to avoid to overlap with the Helix label)
                Residue2D  _5PrimeEnd = this.get5PrimeEnds()[0],
                        _3PrimeEnd = this.get3PrimeEnd(_5PrimeEnd) ;
                if ((_5PrimeEnd.getAbsolutePosition()+middle-1)%5 == 0 && (_3PrimeEnd.getAbsolutePosition()-middle+1)%5 == 0) {
                    //if length enough long, we try to move more the Helix label
                    if (this.getLength()>3)
                        middle+=2;
                    else
                        middle++;
                }
                Residue2D r1 = this.getResidue(_5PrimeEnd.getAbsolutePosition()+middle-1),
                        r2 = this.getResidue(_3PrimeEnd.getAbsolutePosition()-middle+1);
                FontMetrics fm = g2.getFontMetrics(g2.getFont());
                Point2D[] points = DrawingUtils.fit(r1.getCurrentCenter(),r2.getCurrentCenter(),-(this.gc.getCurrentHalf()+fm.stringWidth(this.getLabel())/2));
                /*
                if (r1.getAbsolutePosition()%5 == 0 )
                    DrawingUtils.drawString(points[1],this.getLabel(),g2);
                else
                    DrawingUtils.drawString(points[0],this.getLabel(),g2);*/
                //########## draw residues'ids ###################
                Residue2D _5_0=this.get5PrimeEnds()[0],
                        _3_1=this.get3PrimeEnds()[1];
                int i=1;
                while(i<=this.getLength()) {
                    if (_5_0.getAbsolutePosition()%5 == 0 && _5_0.isDisplayed()) {
                        g2.setFont(this.gc.getFont().deriveFont(this.gc.getFont().getSize2D() / 1.5f));
                        fm = g2.getFontMetrics(g2.getFont());
                        Point2D[] lines = DrawingUtils.fit(_5_0.getCurrentCenter(),_3_1.getCurrentCenter(),-(this.gc.getCurrentHalf())),
                                _lines = DrawingUtils.fit(_5_0.getCurrentCenter(),_3_1.getCurrentCenter(),-(2*this.gc.getCurrentHalf()));
                        g2.drawLine((int)lines[0].getX(),(int)lines[0].getY(), (int)_lines[0].getX(), (int)_lines[0].getY());
                        points = DrawingUtils.fit(_5_0.getCurrentCenter(),_3_1.getCurrentCenter(),-(2*this.gc.getCurrentHalf()+fm.stringWidth(""+_5_0.getAbsolutePosition())/2));
                        DrawingUtils.drawString(points[0],""+_5_0.getAbsolutePosition(),g2);
                        g2.setFont(this.gc.getFont());
                    }
                    else if (_3_1.getAbsolutePosition()%5 == 0 && _3_1.isDisplayed()) {
                        g2.setFont(this.gc.getFont().deriveFont(this.gc.getFont().getSize2D() / 1.5f));
                        fm = g2.getFontMetrics(g2.getFont());
                        Point2D[] lines = DrawingUtils.fit(_5_0.getCurrentCenter(),_3_1.getCurrentCenter(),-(this.gc.getCurrentHalf())),
                                _lines = DrawingUtils.fit(_5_0.getCurrentCenter(),_3_1.getCurrentCenter(),-(2*this.gc.getCurrentHalf()));
                        g2.drawLine((int)lines[1].getX(),(int)lines[1].getY(), (int)_lines[1].getX(), (int)_lines[1].getY());
                        points = DrawingUtils.fit(_5_0.getCurrentCenter(),_3_1.getCurrentCenter(),-(2*this.gc.getCurrentHalf()+fm.stringWidth(""+_3_1.getAbsolutePosition())/2));
                        DrawingUtils.drawString(points[1],""+_3_1.getAbsolutePosition(), g2);
                        g2.setFont(this.gc.getFont());
                    }
                    _5_0 = _5_0.getNextResidue();
                    _3_1 = _3_1.getPreviousResidue();
                    i++;
                }
            }
        }
        //instead to calculate the area each time the getArea method is called, this area is calculated and stored now. This improve the time speed of the AutomaticLayout (see SecondaryCanvas class). This layout uses the helices area to reorganize automatically the helices.
        Polygon p = new Polygon();
        final Residue2D[] bases5 = this.get5PrimeEnds();
        final Residue2D[] bases3 = this.get3PrimeEnds();
        p.addPoint((int) bases5[0].getCurrentCenterX(), (int) bases5[0].getCurrentCenterY());
        p.addPoint((int) bases3[1].getCurrentCenterX(), (int) bases3[1].getCurrentCenterY());
        p.addPoint((int) bases5[1].getCurrentCenterX(), (int) bases5[1].getCurrentCenterY());
        p.addPoint((int) bases3[0].getCurrentCenterX(), (int) bases3[0].getCurrentCenterY());
        this.area = new Area(p);
    }

    public boolean contains(final double x, final double y) {
        return this.getArea().contains(x, y);
    }

    public Point2D getCenter() {
        Residue2D _5end = this.get5PrimeEnds()[0], _3end = this.get3PrimeEnds()[0];
        return new Point2D.Double((_5end.getCurrentCenterX()+_3end.getCurrentCenterX())/2,(_5end.getCurrentCenterY()+_3end.getCurrentCenterY())/2);
    }

    public String toSVG() {
        final StringBuffer svg = new StringBuffer();
        for (Interaction2D inter:this.secondaryInteractions)
            svg.append(inter.toSVG());
        for (Residue2D r:this.getResidues())
            svg.append(r.toSVG());
        return svg.toString();
    }

    public void setCoordinates() {
        //if no more than two bases, nothing to do
        if (this.getHelixFeature().getLength() > 2) {
            Residue2D base5 = this.get5PrimeEnds()[0], base3 = this.get3PrimeEnd(base5);
            double length = DrawingUtils.getDistance(base5.getRealCoordinates(),base3.getRealCoordinates());
            Residue2D base5_0 = base5.getNextResidue();
            int i = 1;
            while (base5_0.getAbsolutePosition() < base3.getAbsolutePosition()) {
                base5_0.setRealCoordinates(DrawingUtils.fit(base5.getRealCoordinates(), base3.getRealCoordinates(), length/(this.getLength()-1) * i)[0]);
                base5_0 = base5_0.getNextResidue();
                i++;
            }
            base5 = this.getPairedResidue(base3);
            base3 = this.get3PrimeEnd(base5);
            base5_0 = base5.getNextResidue();
            i = 1;
            while (base5_0.getAbsolutePosition() < base3.getAbsolutePosition()) {
                base5_0.setRealCoordinates(DrawingUtils.fit(base5.getRealCoordinates(), base3.getRealCoordinates(), length/(this.getLength()-1) * i)[0]);
                base5_0 = base5_0.getNextResidue();
                i++;
            }
        }

        for (Residue2D r:this.get5PrimeEnds())
            if (r.getPreviousResidue() != null && Sstrand2D.class.isInstance(r.getPreviousResidue().getStructuralDomain()))
                r.getPreviousResidue().getStructuralDomain().setCoordinates();
        for (Residue2D r:this.get3PrimeEnds())
            if (r.getNextResidue() != null && Sstrand2D.class.isInstance(r.getNextResidue().getStructuralDomain()))
                r.getNextResidue().getStructuralDomain().setCoordinates();
    }

    public void translateEnds(final double dragX, final double dragY) {
        final Residue2D[] bases5 = this.repository.getResidues(this.getHelixFeature().get5PrimeEnds());
        final Residue2D[] bases3 = this.repository.getResidues(this.getHelixFeature().get3PrimeEnds());
        bases5[0].translate(dragX, dragY);
        bases3[0].translate(dragX, dragY);
        bases5[1].translate(dragX, dragY);
        bases3[1].translate(dragX, dragY);
        this.setCoordinates();
    }

    public void rotateEnds(final java.awt.geom.Point2D centerPoint, final double angle) {
        final Residue2D[] bases5 = this.repository.getResidues(this.getHelixFeature().get5PrimeEnds());
        final Residue2D[] bases3 = this.repository.getResidues(this.getHelixFeature().get3PrimeEnds());
        bases5[0].rotate(centerPoint, angle);
        bases3[0].rotate(centerPoint, angle);
        bases5[1].rotate(centerPoint, angle);
        bases3[1].rotate(centerPoint, angle);
        this.setCoordinates();
    }

    public void flipEnds(final Point2D center) {
        final Residue2D[] bases5 = this.repository.getResidues(this.getHelixFeature().get5PrimeEnds());
        final Residue2D[] bases3 = this.repository.getResidues(this.getHelixFeature().get3PrimeEnds());
        bases5[0].flip(center);
        bases5[1].flip(center);
        bases3[0].flip(center);
        bases3[1].flip(center);
        this.setCoordinates();
    }

    public void setEnds(Residue2D r1, Point2D p1, Residue2D r2, Point2D p2, Residue2D r3, Point2D p3, Residue2D r4, Point2D p4) {
        r1.setRealCoordinates(p1);
        r2.setRealCoordinates(p2);
        r3.setRealCoordinates(p3);
        r4.setRealCoordinates(p4);
        this.setCoordinates();
    }

    /**
     * This method give a terminal loop with a perpendicular orientation. To do that we inverse the residue boundaries and the baseXBefore alternatively
     * The alternity is made by checking the loop.flipped Attribute of the TerminalLoop2D selected. So, we cannot inverse the ywp kinds of bases (boundaries and baseXBefore
     * in the same time).
     * to do : factorise the two if blocks
     */
    //@todo to implement
    public void flipTerminalLoop() {
        /*
        TerminalLoop2D loop = this.neighbors5.getTerminalLoop();
        Base2D[] bases5 = this.get5PrimeEnds();
		Base2D[] bases3 = this.get3PrimeEnds();
        if (loop != null && !loop.flipped) {
            if (loop.getBase5() == bases3[0]) {
                loop.base5Boundarie = bases5[0];
                loop.base3Boundarie = bases3[0];
            }
            else if (loop.getBase5() == bases5[0]) {
                loop.base5Boundarie = bases3[0];
                loop.base3Boundarie = bases5[0];
            }
            else if (loop.getBase5() == bases3[1]) {
                loop.base5Boundarie = bases5[1];
                loop.base3Boundarie = bases3[1];
            }
            else {
                loop.base5Boundarie = bases3[1];
                loop.base3Boundarie = bases5[1];
            }
            //we redraw the loop
            loop.setCoordinates();
        }

        if (loop != null && loop.flipped) {
            if (loop.base5Before == bases3[0].getPreviousBase()) {
                loop.base5Before = bases5[0].getNextBase();
            }

            else if (loop.base5Before == bases3[1].getPreviousBase()) {
                loop.base5Before = bases5[1].getNextBase();
            }

            else if (loop.base5Before == bases5[0].getNextBase()) {
                loop.base5Before = bases3[0].getPreviousBase();
            }

            else if (loop.base5Before == bases5[1].getNextBase()) {
                loop.base5Before = bases3[1].getPreviousBase();
            }

            //we redraw the loop
            loop.setBasesCoordinates();
        }

        if (loop != null) {
            loop.flipped = !loop.flipped;
        }

        loop = this.neighbors3.getTerminalLoop();

        if (loop != null && !loop.flipped) {
            if (loop.base5Boundarie == bases3[0]) {
                loop.base5Boundarie = bases5[0];
                loop.base3Boundarie = bases3[0];
            }
            else if (loop.base5Boundarie == bases5[0]) {
                loop.base5Boundarie = bases3[0];
                loop.base3Boundarie = bases5[0];
            }
            else if (loop.base5Boundarie == bases3[1]) {
                loop.base5Boundarie = bases5[1];
                loop.base3Boundarie = bases3[1];
            }
            else {
                loop.base5Boundarie = bases3[1];
                loop.base3Boundarie = bases5[1];
            }
            //we redraw the loop
            loop.setBasesCoordinates();
        }

        if (loop != null && loop.flipped) {
            if (loop.base5Before == bases3[0].getPreviousBase()) {
                loop.base5Before = bases5[0].getNextBase();
            }

            else if (loop.base5Before == bases3[1].getPreviousBase()) {
                loop.base5Before = bases5[1].getNextBase();
            }

            else if (loop.base5Before == bases5[0].getNextBase()) {
                loop.base5Before = bases3[0].getPreviousBase();
            }

            else if (loop.base5Before == bases5[1].getNextBase()) {
                loop.base5Before = bases3[1].getPreviousBase();
            }

            //we redraw the loop
            loop.setBasesCoordinates();
        }

        if (loop != null) {
            loop.flipped = !loop.flipped;
        }*/
    }

    public List<BaseBaseInteraction2D> getSecondaryInteractions() {
        return new ArrayList<BaseBaseInteraction2D>(this.secondaryInteractions);
    }

    public Residue2D[] get5PrimeEnds() {
        return this.repository.getResidues(this.getHelixFeature().get5PrimeEnds());
    }

    public Residue2D[] get3PrimeEnds() {
        return this.repository.getResidues(this.getHelixFeature().get3PrimeEnds());
    }

    public void setColor(final java.awt.Color color) {
        super.setColor(color);
        for (Interaction2D inter:this.secondaryInteractions)
            inter.setColor(color);
        for (Residue2D r:this.getResidues())
            r.setColor(color);
    }

    public Area getArea() {
        return this.area;
    }

}
