package fr.unistra.ibmc.s2s.ssviewer.features;

import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.s2s.ssviewer.graphics.GraphicContext;
import fr.unistra.ibmc.s2s.ssviewer.utils.DrawingUtils;
import fr.unistra.ibmc.paradise.core.utils.Residue;

import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.geom.Area;
import java.util.*;
import java.util.List;

abstract public class Interaction2D extends Rna2DElement {

    /**
     * This Attribute store the points outside the residue letters. This points are the boundaries to draw the symbol for the interaction.
     */
    java.awt.geom.Point2D[] op = null, real_op = new java.awt.geom.Point2D[2];
    SymbolsArray defaultSymbol, realSymbol;
    boolean drawRealSymbol = true, updated;
    public static final Color DEFAULT_TERTIARY_INTERACTION_COLOR = Color.MAGENTA,
            DEFAULT_SECONDARY_INTERACTION_COLOR = Helix2D.DEFAULT_COLOR  ;
    protected Residue residue1, residue2;

    Interaction2D(final Residues2DRepository repository,final Residue residue1, final Residue residue2, final Mediator mediator, final GraphicContext gc) {
        super(repository, mediator, gc);
        this.residue1 = residue1;
        this.residue2 = residue2;
        this.defaultSymbol = new SymbolsArray();
        this.realSymbol = new SymbolsArray();
        this.color = Color.GRAY;
        this.updated = true;
    }

    /**
     * Define if the interaction will draw its Leontis-Westhof symbol or only a single line.
     *
     * @param drawRealSymbol true if the Leontis-Westhof symbol is displayed, false if not
     */
    public void drawRealSymbol(final boolean drawRealSymbol) {
        this.drawRealSymbol = true;
    }

    public boolean isUpdated() {
        return updated;
    }

    public void isUpdated(boolean updated) {
        this.updated = updated;
    }

    public boolean isInsideDrawingArea(){
        return this.getResidue().isInsideDrawingArea() || this.getPartnerResidue().isInsideDrawingArea();
    }

    public boolean isStrictlyInsideDrawingArea(){
        return this.getResidue().isStrictlyInsideDrawingArea() && this.getPartnerResidue().isStrictlyInsideDrawingArea();
    }



    public void draw(final java.awt.Graphics2D g2) {
        //the op points can be null if the interaction was not drawn
        //the real_op points can be null if not enough space between the interacting bases
        if (this.isDisplayed() && this.isInsideDrawingArea()) {
            if (this.updated) {
                this.setOutsidePoints();
                this.updated = false;
            }
            //TODO find how to put some calculation steps in the previous if statement to improve 2D draw
            if (this.real_op != null) {
                //calculate the outside points fitted with the current view and zoom level
                this.op = new java.awt.geom.Point2D[2];
                this.op[0] = new java.awt.geom.Point2D.Double(this.real_op[0]
                        .getX()
                        * this.gc.getFinalZoomLevel() + this.gc.getViewX(),
                        this.real_op[0].getY() * this.gc.getFinalZoomLevel()
                                + this.gc.getViewY());
                this.op[1] = new java.awt.geom.Point2D.Double(this.real_op[1]
                        .getX()
                        * this.gc.getFinalZoomLevel() + this.gc.getViewX(),
                        this.real_op[1].getY() * this.gc.getFinalZoomLevel()
                                + this.gc.getViewY());
            }
            //we remove the precedent op points if no real_op
            else
                this.op = null;
            if (this.isSelected())
                g2.setColor(java.awt.Color.red);
            else if (this.isDisplayed())
                g2.setColor(this.color);
            else
                g2.setColor(Color.GRAY);
            SymbolsArray.Iterator sc = null;
            if (this.drawRealSymbol()
                    && this.getRealSymbol() != null) {
                this.setRealSymbol();
                sc = this.getRealSymbol().iterator();
                while (sc.hasNext())
                    sc.next().draw(g2);
            } else if (!this.drawRealSymbol() && this.getDefaultSymbol() != null) {
                this.setDefaultSymbol();
                sc = this.getDefaultSymbol().iterator();
                while (sc.hasNext())
                    sc.next().draw(g2);
            }
        }
    }


    /**
     * Return if the interaction will draw its Leontis-Westhof symbol or only a
     * single line
     *
     * @return true if the Leontis-Westhof symbol is displayed, false if not
     */
    public boolean drawRealSymbol() {
        return this.drawRealSymbol;
    }


    /**
     * Gets the defaultSymbol Attribute of the Interaction2D object
     *
     * @return The defaultSymbol value
     */
    protected SymbolsArray getDefaultSymbol() {
        return this.defaultSymbol;
    }


    /**
     * Gets the realSymbol Attribute of the Interaction2D object
     *
     * @return The realSymbol value
     */
    protected SymbolsArray getRealSymbol() {
        return this.realSymbol;
    }


    public Area getArea() {
        //if we want to center on an interaction not drawn, we need the surface but we need to calculate it before
        if (this.updated)
            this.setOutsidePoints();
        if (this.real_op != null) {
            this.op = new java.awt.geom.Point2D[2];
            this.op[0] = new java.awt.geom.Point2D.Double(this.real_op[0]
                    .getX()
                    * this.gc.getFinalZoomLevel() + this.gc.getViewX(),
                    this.real_op[0].getY() * this.gc.getFinalZoomLevel()
                            + this.gc.getViewY());
            this.op[1] = new java.awt.geom.Point2D.Double(this.real_op[1]
                    .getX()
                    * this.gc.getFinalZoomLevel() + this.gc.getViewX(),
                    this.real_op[1].getY() * this.gc.getFinalZoomLevel()
                            + this.gc.getViewY());
        }
        final java.awt.geom.GeneralPath shape = new java.awt.geom.GeneralPath();
        if (this.op != null) {
            final Point2D[] startPoints = DrawingUtils.getPerpendicular(this.op[0], this.getResidue().getCurrentCoordinates(), this.getPartnerResidue().getCurrentCoordinates(), gc.getCurrentSymbolSize() / 6);
            final Point2D[] endPoints = DrawingUtils.getPerpendicular(this.op[1], this.getResidue().getCurrentCoordinates(), this.getPartnerResidue().getCurrentCoordinates(), gc.getCurrentSymbolSize() / 6);
            shape.moveTo((float) startPoints[0].getX(), (float) startPoints[0].getY());
            shape.lineTo((float) endPoints[0].getX(), (float) endPoints[0].getY());
            shape.lineTo((float) endPoints[1].getX(), (float) endPoints[1].getY());
            shape.lineTo((float) startPoints[1].getX(), (float) startPoints[1].getY());
            shape.lineTo((float) startPoints[0].getX(), (float) startPoints[0].getY());
            shape.closePath();
            return new Area(shape);
        }
        //If we have an Interaction2D with no surface (two bases too close) we take the intersection of the two bases surfaces
        else {
            shape.moveTo((float)this.getResidue().getCurrentCenterX(),(float)(this.getResidue().getCurrentCenterY()-this.gc.getCurrentHeight()/2));
            shape.lineTo((float)this.getResidue().getCurrentCenterX(),(float)(this.getResidue().getCurrentCenterY()+this.gc.getCurrentHeight()/2));
            shape.lineTo((float)this.getPartnerResidue().getCurrentCenterX(),(float)(this.getPartnerResidue().getCurrentCenterY()+this.gc.getCurrentHeight()/2));
            shape.lineTo((float)this.getPartnerResidue().getCurrentCenterX(),(float)(this.getPartnerResidue().getCurrentCenterY()-this.gc.getCurrentHeight()/2));
            shape.lineTo((float)this.getResidue().getCurrentCenterX(),(float)(this.getResidue().getCurrentCenterY()-this.gc.getCurrentHeight()/2));
            shape.closePath();
        }
        return new Area(shape);
    }

    public Residue2D getResidue() {
        return this.repository.getResidue(this.residue1);
    }

    public Residue2D getPartnerResidue() {
        return this.repository.getResidue(this.residue2);
    }

    protected void setOutsidePoints() {
        final Residue2D residue = this.repository.getResidue(this.residue1);
        final Residue2D partnerResidue = this.repository.getResidue(this.residue2);
        //we calculate the points outside the bases
        if ((residue.isDisplayed() && partnerResidue.isDisplayed()))
        {
            final double dist = DrawingUtils.getDistance(residue.getRealX(), residue.getRealY(), partnerResidue.getRealX(), partnerResidue.getRealY());
            if (dist >= this.gc.getRealHalf() * 2)
                this.real_op = DrawingUtils.fit(residue.getRealCenterX(), residue.getRealCenterY(), partnerResidue.getRealCenterX(), partnerResidue.getRealCenterY(), this.gc.getRealHalf());
            else
                this.real_op = null;
        } else if (!residue.isDisplayed() && partnerResidue.isDisplayed()) {
            this.real_op = DrawingUtils.fit(residue.getRealCenterX(), residue.getRealCenterY(), partnerResidue.getRealCenterX(), partnerResidue.getRealCenterY(), this.gc.getRealHalf());
            this.real_op[0] = residue.getRealCenter();
        } else if (!partnerResidue.isDisplayed() && residue.isDisplayed()) {
            this.real_op = DrawingUtils.fit(residue.getRealCenterX(), residue.getRealCenterY(), partnerResidue.getRealCenterX(), partnerResidue.getRealCenterY(), this.gc.getRealHalf());
            this.real_op[1] = partnerResidue.getRealCenter();
        } else {
            this.real_op = new Point2D[2];
            this.real_op[0] = residue.getRealCenter();
            this.real_op[1] = partnerResidue.getRealCenter();
        }
    }

    protected void setDefaultSymbol() {
        this.defaultSymbol.clear();
        if (op != null)
            this.defaultSymbol.addSymbol(new Line(op[0], op[1], this.gc));
    }


    /**
     * Sets the realSymbol Attribute of the Interaction2D object
     */
    abstract protected void setRealSymbol();


    /**
     * Sets the realSymbol Attribute of the Interaction2D object
     *
     * @param point1   The new realSymbol value
     * @param point2   The new realSymbol value
     * @param distance The new realSymbol value
     */
    abstract protected void setRealSymbol(Point2D point1, Point2D point2, double distance);

    public void setColor(final java.awt.Color color) {
        super.setColor(color);
        this.getResidue().setColor(color);
        this.getPartnerResidue().setColor(color);
    }

    public String toSVG() {
        this.setOutsidePoints();
        if (this.real_op != null) {
            //calculate the outside points fitted with the current view and zoom level
            this.op = new java.awt.geom.Point2D[2];
            this.op[0] = new java.awt.geom.Point2D.Double(this.real_op[0]
                    .getX()
                    * this.gc.getFinalZoomLevel() + this.gc.getViewX(),
                    this.real_op[0].getY() * this.gc.getFinalZoomLevel()
                            + this.gc.getViewY());
            this.op[1] = new java.awt.geom.Point2D.Double(this.real_op[1]
                    .getX()
                    * this.gc.getFinalZoomLevel() + this.gc.getViewX(),
                    this.real_op[1].getY() * this.gc.getFinalZoomLevel()
                            + this.gc.getViewY());
        }
        //we remove the precedent op points if no real_op
        else
            this.op = null;
        if (this.drawRealSymbol) {
            this.setRealSymbol();
            return this.toSVGReal();
        }
        else  {
            this.setDefaultSymbol();
            return this.toSVGDefault();
        }
    }

    protected String toSVGDefault() {
        final StringBuffer svg = new StringBuffer();
        final SymbolsArray.Iterator sc = this.defaultSymbol.iterator();
        while (sc.hasNext())
            svg.append(sc.next().toSVG());
        return svg.toString();
    }

    protected String toSVGReal() {
        final StringBuffer svg = new StringBuffer();
        final SymbolsArray.Iterator sc = this.realSymbol.iterator();
        while (sc.hasNext())
            svg.append(sc.next().toSVG());
        return svg.toString();
    }

    public boolean contains(final double x, final double y) {
        final Area area = this.getArea();
        if (area == null)
            //if no surface since no symbols drawn (this.op==null) => not enough large distance between the bases
            return false;
        else
            //we check only if the interaction is visible (temporary or permanently shown)
            return area.contains(x, y);
    }

    public List<Residue2D> getResidues() {
        List<Residue2D> residues = new ArrayList<Residue2D>();
        residues.add(this.repository.getResidue(this.residue1));
        residues.add(this.repository.getResidue(this.residue2));
        return residues;
    }

    public boolean equals(Object obj) {
        if (!this.getClass().isInstance(obj))
            return false;
        Interaction2D inter = (Interaction2D)obj;
        return ( this.residue1.equals(inter.residue1) && this.residue2.equals(inter.residue2) ||
                this.residue1.equals(inter.residue2) && this.residue2.equals(inter.residue1));

    }

    class SymbolsArray {

        java.util.List symbols;

        SymbolsArray() {
            this.symbols = new ArrayList();
        }

        Iterator iterator() {
            return new Iterator();
        }

        void addSymbol(final Symbol symbol) {
            this.symbols.add(symbol);
        }

        void clear() {
            this.symbols.clear();
        }

        int size() {
            return this.symbols.size();
        }

        class Iterator {

            java.util.Iterator it;

            Iterator() {
                this.it = SymbolsArray.this.symbols.iterator();
            }

            boolean hasNext() {
                return this.it.hasNext();
            }

            Symbol next() {
                return (Symbol) this.it.next();
            }

        }

    }

}
