package fr.unistra.ibmc.s2s.ssviewer.features;

import fr.unistra.ibmc.s2s.ssviewer.graphics.GraphicContext;
import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.paradise.core.RNA;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.paradise.core.features.Helix;
import fr.unistra.ibmc.paradise.core.features.BaseBaseInteraction;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.features.SingleStrand;

import java.util.List;
import java.util.ArrayList;
import java.awt.geom.Area;

public class Rna2D extends Rna2DElement {

    private RNA rna;
    List<Helix2D> helices;
    List<Sstrand2D> strands;
    List<BaseBaseInteraction2D> tertiaryInteractions;
    private SecondaryStructure secondaryStructure;

    Rna2D(final RNA rna, final SecondaryStructure secondaryStructure, final Residues2DRepository repository, final Mediator mediator, final GraphicContext gc) {
        super(repository, mediator, gc);
        this.secondaryStructure = secondaryStructure;
        this.rna = rna;
        this.tertiaryInteractions = new ArrayList<BaseBaseInteraction2D>();
        this.helices = new ArrayList<Helix2D>();
        this.strands = new ArrayList<Sstrand2D>();
        for (SingleStrand ss:secondaryStructure.getSingleStrands()) {
            if (ss.getMolecules().iterator().next().getParadiseID().equals(this.rna.getParadiseID()))
                this.strands.add(new Sstrand2D(this,secondaryStructure,ss,repository, mediator,gc));
        }
        for (BaseBaseInteraction interaction:secondaryStructure.getTertiaryInteractions()) {
            if (!interaction.isSecondaryInteraction() && interaction.getMolecules().size() == 1 && interaction.getMolecules().iterator().next().getParadiseID().equals(this.rna.getParadiseID()))
                this.tertiaryInteractions.add(BaseBaseInteraction2D.getInteraction(interaction,repository, mediator,gc));
        }
        Helix2D h;
        for (Helix helix:secondaryStructure.getHelices()) {
            if (helix.getMolecules().size() == 1 && helix.getMolecules().iterator().next().getParadiseID().equals(this.rna.getParadiseID())) {
                h = new Helix2D(helix, repository, mediator,gc);
                this.helices.add(h);
            }
        }
        for (int i=1; i<= rna.getLength()-1;i++)
            new PhosphodiesterBond2D(new Residue(i,rna),new Residue(i+1,rna),repository,mediator,gc);
    }

    public void addTertiaryInteraction(BaseBaseInteraction2D interaction) {
        this.tertiaryInteractions.add(interaction);
    }

    public RNA getRnaMolecule() {
        return rna;
    }

    public List<Residue2D> getResidues() {
        List<Residue2D> residues = new ArrayList<Residue2D>(this.rna.getLength());
        for (int i=1;i<=this.rna.getLength();i++)
            residues.add(this.repository.getResidue(new Residue(i,this.rna)));
        return residues;
    }

    public boolean contains(final double x, final double y) {
        return this.getArea().contains(x, y);
    }

    public void draw(final java.awt.Graphics2D g2) {
        if (this.isSelected())
            g2.setColor(java.awt.Color.red);
        else
            g2.setColor(this.color);
        for (Helix2D h:this.helices)
            h.draw(g2);
        for (Sstrand2D ss:this.strands)
            ss.draw(g2);
        for (BaseBaseInteraction2D interaction:this.tertiaryInteractions)
            interaction.draw(g2);
    }

    public Area getArea() {
        return null;
    }

    public String toSVG() {
        final StringBuffer svg = new StringBuffer();
        for (BaseBaseInteraction2D interaction:this.tertiaryInteractions)
            svg.append(interaction.toSVG());
        for (Helix2D h:this.helices)
            svg.append(h.toSVG());
        for (Sstrand2D ss:this.strands)
            svg.append(ss.toSVG());
        for (BaseBaseInteraction2D interaction:this.tertiaryInteractions)
            svg.append(interaction.toSVG());
        return svg.toString();
    }

    public List<Sstrand2D> getStrands() {
        return new ArrayList<Sstrand2D>(this.strands);
    }

    public List<Helix2D> getHelices() {
        return new ArrayList<Helix2D>(this.helices);
    }

    public List<BaseBaseInteraction2D> getTertiaryInteractions() {
        return new ArrayList<BaseBaseInteraction2D>(this.tertiaryInteractions);
    }

    public int length() {
        return this.rna.getLength();
    }

    public boolean equals(Object obj) {
        if (this.getClass().isInstance(obj))
            return false;
        return this.rna.getParadiseID().equals(((Rna2D)obj).rna.getParadiseID());
    }

    public void removeTertiaryInteraction(BaseBaseInteraction2D interaction) {
        this.tertiaryInteractions.remove(interaction);
    }

    public void removeSstrand(Sstrand2D ss) {
        this.strands.remove(ss);
    }
}
