package fr.unistra.ibmc.s2s.ssviewer.features;

import fr.unistra.ibmc.s2s.ssviewer.graphics.GraphicContext;
import fr.unistra.ibmc.s2s.Mediator;

import java.awt.*;
import java.awt.geom.Area;

abstract public class Rna2DElement {

    Residues2DRepository repository;
    Color color;
    GraphicContext gc;
    private String label = "?";
    protected Mediator mediator;

    Rna2DElement(final Residues2DRepository repository, final Mediator mediator, final GraphicContext gc) {
        this.repository = repository;
        this.mediator = mediator;
        this.color = Color.BLACK;
        this.gc = gc;
    }

    public Residues2DRepository getRepository() {
        return repository;
    }

    public void setRepository(final Residues2DRepository repository) {
        this.repository = repository;
    }

    public void isSelected(boolean selected) {
        for (Residue2D r:this.getResidues())
            r.isSelected(selected);
    }

    public boolean isSelected() {
        for (Residue2D r:this.getResidues())
            if (!r.isSelected())
            return false;
        return true;
    }

    public boolean isInsideDrawingArea() {
        for (Residue2D r:this.getResidues())
            if (r.isInsideDrawingArea())
                return true;
        return false;
    }

    /**
     * Return is the RNA2Dfeature has been drawn (depending on several parameters: can be displayed, is inside the drawing area, .... )
     * @return
     */
    public boolean isDrawn() {
        return this.isDisplayed() && this.isInsideDrawingArea();
    }

    public void isDisplayed(boolean displayed) {
        for (Residue2D r:this.getResidues())
            r.isDisplayed(displayed);
    }

    public boolean isDisplayed() {
        for (Residue2D r:this.getResidues())
            if (!r.isDisplayed())
            return false;
        return true;
    }

    public java.awt.Color getColor() {
        return this.color;
    }

    public void setColor(final java.awt.Color color) {
        this.color = color;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String getLabel() {
        return this.label;
    }

    abstract public String toSVG();

    abstract public Area getArea();

    abstract public boolean contains(final double x, final double y);

    abstract public void draw(final java.awt.Graphics2D g2) ;
    
    /**
     * Return all the Residue2D contained in this Rna2DElement
     * @return
     */
    abstract public java.util.List<Residue2D> getResidues();

}
