package fr.unistra.ibmc.s2s.ssviewer.features;

import fr.unistra.ibmc.paradise.core.features.SingleStrand;
import fr.unistra.ibmc.paradise.core.features.SecondaryStructure;
import fr.unistra.ibmc.paradise.core.features.StructuralDomain;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.paradise.core.*;
import fr.unistra.ibmc.s2s.ssviewer.utils.DrawingUtils;
import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.s2s.ssviewer.graphics.GraphicContext;

import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.geom.Area;

/**
 * @author fjossinet
 */
public class Sstrand2D extends StructuralDomain2D {

    static final int VERSE = 0;
    public static final int REVERSE = 1;
    private boolean linear;
    int orientation = Sstrand2D.VERSE;
    private double crossProduct;
    Point2D centerPoint;
    private double rayon;
    private double midX, midY;
    protected boolean flipped;
    private double dragX, dragY;
    private Rna2D rna;

    /**
     * rotation angle between two bases
     */
    double rotationAngle;
    private Residue crossProductResidue;
    //Attribute to reverse the circular sstrand representation
    boolean reverse, automaticCoordinates = true;
    public static final Color DEFAULT_COLOR = Color.BLACK;
    private SecondaryStructure secondaryStructure;

    public Sstrand2D(final Rna2D rna, final SecondaryStructure secondaryStructure,final SingleStrand ss, final Residues2DRepository repository, final Mediator mediator, final GraphicContext gc) {
        super(ss, repository, mediator, gc);
        this.rna = rna;
        this.secondaryStructure = secondaryStructure;
        this.setColor(Sstrand2D.DEFAULT_COLOR);
        //we define the crossProduct residue necessary to give the good drawing orientation
        //the crossProductBase is the secondary partner of one of the boundaries residue
        if (ss.getLength() != this.rna.getRnaMolecule().getLength())
            this.setCrossProductResidue();
        for (Residue2D r:this.getResidues())
            r.setStructuralDomain(this);
    }

    public Rna2D getRna() {
        return rna;
    }

    public SingleStrand getSingleStrandFeature() {
        return (SingleStrand)this.structuralDomain;
    }

    protected double getRotationAngle() {
        return this.rotationAngle;
    }

    public boolean getReverse() {
        return this.reverse;
    }

    protected Residue2D getMoreExternalBase5() {
        Molecule m = this.structuralDomain.getMolecules().iterator().next();
        return this.repository.getResidue(new Residue(this.structuralDomain.getFullLocation(m).getStart(),m).getPreviousResidue().getPreviousResidue());
    }

    protected Residue2D getMoreExternalBase3() {
        Molecule m = this.structuralDomain.getMolecules().iterator().next();
        return this.repository.getResidue(new Residue(this.structuralDomain.getFullLocation(m).getEnd(),m).getNextResidue().getNextResidue());
    }

    public Residue2D getExternalBase5() {
        Molecule m = this.structuralDomain.getMolecules().iterator().next();
        return this.repository.getResidue(new Residue(this.structuralDomain.getFullLocation(m).getStart(),m).getPreviousResidue());
    }

    public Residue2D getExternalBase3() {
        Molecule m = this.structuralDomain.getMolecules().iterator().next();
        return this.repository.getResidue(new Residue(this.structuralDomain.getFullLocation(m).getEnd(),m).getNextResidue());
    }

    public int getBase5Id() {
        return this.structuralDomain.getFullLocation(this.structuralDomain.getMolecules().iterator().next()).getStart();
    }

    public int getExternalBase5Id() {
        return this.structuralDomain.getFullLocation(this.structuralDomain.getMolecules().iterator().next()).getStart()-1;
    }

    public int getBase3Id() {
        return this.structuralDomain.getFullLocation(this.structuralDomain.getMolecules().iterator().next()).getEnd();
    }

    public int getExternalBase3Id() {
        return this.structuralDomain.getFullLocation(this.structuralDomain.getMolecules().iterator().next()).getEnd()+1;
    }

    public ParadiseID getMoleculeId() {
        return this.structuralDomain.getMolecules().iterator().next().getParadiseID();
    }

    void setCenterPoint(final double x, final double y) {
        this.centerPoint = new Point2D.Double(x, y);
    }

    /*
    public void setReverse(boolean reverse) {
       this.reverse=reverse;
       this.setBasesCoordinates();
   }*/

    protected void setAngle(final double angle) {
        this.rotationAngle = angle;
    }

    public Residue2D getBase5() {
        Molecule m = this.structuralDomain.getMolecules().iterator().next();
        return this.repository.getResidue(new Residue(this.structuralDomain.getFullLocation(m).getStart(),m));
    }

    public Residue2D getBase3() {
        Molecule m = this.structuralDomain.getMolecules().iterator().next();
        return this.repository.getResidue(new Residue(this.structuralDomain.getFullLocation(m).getEnd(),m));
    }

    public void setOrientation(final int orientation) {
        this.orientation = orientation;
    }

    private void setCrossProductResidue() {
        if (this.getExternalBase3() != null) {
            this.crossProductResidue = this.structuralDomain.getParentFeatures(SecondaryStructure.class).iterator().next().getPairedResidueInSecondaryInteraction(this.getExternalBase3().getResidue2DFeature().getResidues().iterator().next());
        }
        else {
            this.crossProductResidue = this.structuralDomain.getParentFeatures(SecondaryStructure.class).iterator().next().getPairedResidueInSecondaryInteraction(this.getExternalBase5().getResidue2DFeature().getResidues().iterator().next());
        }
    }

    public void setCoordinates() {
        //if this single-strand is the only structural domain. Consequently, draw itself as a circle
        if (this.getLength() == this.rna.getRnaMolecule().getLength()) {
            double x = gc.getDrawingArea().getCenterX(), y = gc.getDrawingArea().getCenterY();
            double circonference = this.getLength()*gc.getRealHeight()+2 ; //the 2 parameter is for the 5' and 3' letters
            double rayon = (circonference/Math.PI)/2;
            Point2D startPoint = new Point2D.Double(x,y+rayon);
            Residue2D base5 = this.getBase5();
            int j = 0;
            double angle = (2*Math.PI)/(double)(this.getLength()+2);
            while (base5 != null && base5.getAbsolutePosition() <= this.getBase3Id()) {
                j++;
                base5.setRealCoordinates(DrawingUtils.getRotatedPoint(startPoint,angle*j,new Point2D.Double(x,y)));
                base5 = base5.getNextResidue();
            }
        }
        else {
            this.setCrossProductResidue();
            Molecule m = this.structuralDomain.getMolecules().iterator().next();
            Location l = this.structuralDomain.getFullLocation(m);
            Residue r1 = new Residue( l.getStart(),m).getPreviousResidue(), r2 = new Residue( l.getEnd(),m).getNextResidue();
            StructuralDomain sd1, sd2;
            if (r1 == null || r2 == null) {   //TerminalSstrand
                if (this.getExternalBase5Id() != 0) {
                    this.setDragDistances(this.getMoreExternalBase5(), this.getExternalBase5());
                    Residue2D base5 = this.getBase5();
                    while (base5 != null && base5.getAbsolutePosition() <= this.getBase3Id()) {
                        base5.setRealCoordinates(base5.getPreviousResidue().getRealX() + this.dragX, base5.getPreviousResidue().getRealY() + this.dragY);
                        base5 = base5.getNextResidue();
                    }
                } else {
                    this.setDragDistances(this.getExternalBase3(), this.getMoreExternalBase3());
                    //si cette methode est appelee, la sstrand est une 5'terminale
                    Residue2D base3 = this.getBase3();
                    while (base3 != null && base3.getAbsolutePosition() >= this.getBase5Id()) {
                        base3.setRealCoordinates(base3.getNextResidue().getRealX() + this.dragX, base3.getNextResidue().getRealY() + this.dragY);
                        base3 = base3.getPreviousResidue();
                    }
                }
            }
            else if (r1 != null && r2 != null) {
                sd1 = this.secondaryStructure.getEnclosingStructuralDomain(r1);
                sd2 = this.secondaryStructure.getEnclosingStructuralDomain(r2);
                if (sd1 != null && sd2 != null && sd1 == sd2 ) { //TerminalLoop
                    //we determine the center of the last basepair in the helix
                    this.midX = (this.getExternalBase5().getRealX() + this.getExternalBase3().getRealX()) / 2;
                    this.midY = (this.getExternalBase5().getRealY() + this.getExternalBase3().getRealY()) / 2;
                    //we determine the translation beetween the two last basepairs in the helix
                    final double trX = this.getExternalBase5().getRealX() - this.getMoreExternalBase5().getRealX();
                    final double trY = this.getExternalBase5().getRealY() - this.getMoreExternalBase5().getRealY();
                    //we apply the same translation to the center of the last basepair, this is the center of the terminal loop
                    //as an option we can fit the center proportionnal to the residue number inside the Loop (here this.moleculesCount()/4)
                    setCenterPoint(midX + trX * this.getLength() / 4, midY + trY * this.getLength() / 4);
                    //we determine the angles made with the centerPoint and the bases of the last bases in the helix
                    final double angle = DrawingUtils.getAngle(DrawingUtils.getDistance(centerPoint.getX(), centerPoint.getY(), midX, midY),
                            DrawingUtils.getDistance(midX, midY, this.getExternalBase5().getRealX(), this.getExternalBase5().getRealY()));
                    //we determine the angle beetween two bases in the loop
                    //we determine the orientation of the helix with the crossProduct
                    final double crossProduct = DrawingUtils.crossProduct(this.getExternalBase5().getRealCoordinates(), centerPoint, new Point2D.Double(this.midX, this.midY));
                    //calcul of the rotation angle
                    if (crossProduct <= 0.0)
                        rotationAngle = (Math.PI + 2 * angle) / (this.getLength() + 1);
                    else
                        rotationAngle = -(Math.PI + 2 * angle) / (this.getLength() + 1);
                    Residue2D base5 = this.getBase5();
                    int j = 0;
                    while (base5 != null && base5.getAbsolutePosition() <= this.getBase3Id()) {
                        j++;
                        base5.setRealCoordinates(setRotatedCoordinates(j * rotationAngle));
                        base5 = base5.getNextResidue();
                    }
                }
                else  {//regular Sstrand
                    if (this.automaticCoordinates) {
                        final Residue2D base5 = this.getExternalBase5();
                        final Residue2D base3 = this.getExternalBase3();
                        final double[] basePosition = new double[2];

                        basePosition[0] = base5.getRealX();
                        basePosition[1] = base5.getRealY();

                        //nombre de bases dans le Sstrand2D
                        final int basesNb = this.getLength();

                        //on calcule l'ecartement horizontal et vertical des bases aux extremites
                        final double horizontal = base5.getRealX() - base3.getRealX();
                        final double vertical = base5.getRealY() - base3.getRealY();

                        if (linear) {
                            for (int i = this.getBase5Id(), j = 1; i <= this.getBase3Id(); i++, j++) {
                                basePosition[0] -= j * horizontal / (basesNb + 1);
                                basePosition[1] -= j * vertical / (basesNb + 1);
                                this.repository.getResidue(this.structuralDomain.getMolecules().iterator().next().getParadiseID(), i).setRealCoordinates(basePosition[0], basePosition[1]);
                            }
                        } else {
                            //calcul de la distance entre les deux bases des helices
                            final double boundariesDistance = Math.sqrt(horizontal * horizontal + vertical * vertical);
                            //placement du centre du cercle en fonction de la distance entre les bases boundaries
                            this.setCenterPoint(boundariesDistance, horizontal, vertical);
                            for (int i = this.getBase5Id(), j = 1; i <= this.getBase3Id(); i++, j++)
                                this.repository.getResidue(this.structuralDomain.getMolecules().iterator().next().getParadiseID(), i).setRealCoordinates(setRotatedCoordinates((j) * this.rotationAngle));
                        }
                    }
                }
            }
        }
    }

    /**
     * Sets the dragDistances Attribute of the TerminalSstrand2D object
     *
     * @param base1 The new dragDistances value
     * @param base2 The new dragDistances value
     */
    private void setDragDistances(final Residue2D base1, final Residue2D base2) {
        if (this.getExternalBase5Id() != 0) {
            if (orientation == Sstrand2D.VERSE) {
                this.dragX = (base2.getRealX() - base1.getRealX());
                this.dragY = (base2.getRealY() - base1.getRealY());
            }
            /*
             *  else if (orientation==Sstrand2D.REVERSE) {
             *  this.dragX=(base2.getX()-base2.getPairedResidue().getX())/2;
             *  this.dragY=(base2.getY()-base2.getPairedResidue().getY())/2;
             *  }
             */
        } else {
            if (orientation == Sstrand2D.VERSE) {
                this.dragX = (base1.getRealX() - base2.getRealX());
                this.dragY = (base1.getRealY() - base2.getRealY());
            }
            /*
             *  else if (orientation==Sstrand2D.REVERSE) {
             *  this.dragX=(base1.getX()-base1.getPairedResidue().getX())/2;
             *  this.dragY=(base1.getY()-base1.getPairedResidue().getY())/2;
             *  }
             */
        }
    }

    private void setCenterPoint(final double boundariesDistance, final double horizontal, final double vertical) {

        final Residue2D base5 = this.getExternalBase5();
        final Residue2D base3 = this.getExternalBase3();

        //angle pour placer le centre du cercle, exprime en radians
        double angle;

        angle = Math.PI / 3;

        /*
        if (200<boundariesDistance) {angle=Math.PI/3;}
        else if (200>=boundariesDistance && boundariesDistance>150) {angle=Math.PI/10;}
        else if (150>=boundariesDistance && boundariesDistance>100) {angle=0;}
        else if (100>=boundariesDistance && boundariesDistance>50) {angle=-Math.PI/10;}
        else {angle=-Math.PI/7;}*/
        //calcul de l'angle pour faire la rotation d'une residue
        this.crossProduct = DrawingUtils.crossProduct(base5.getRealCoordinates(), base3.getRealCoordinates(), this.repository.getResidue(this.crossProductResidue).getRealCoordinates());
        //calcul de l'angle de rotation
        this.rotationAngle = (Math.PI - 2 * angle) / (this.getLength() + 1);
        if (this.crossProduct >= 0) this.rotationAngle = -this.rotationAngle;
        if (reverse) this.rotationAngle = -this.rotationAngle;

        //calcul des coordonnees du centre en fonction de l'extremite 5' et de l'angle qui vient d'etre fixe

        if (this.crossProduct >= 0) angle = -angle;

        if (reverse) this.rotationAngle = -this.rotationAngle;

        if (boundariesDistance == 0)
            this.centerPoint = new Point2D.Double(base5.getRealX(), base5.getRealY());
        else if (base5.getRealX() <= base3.getRealX())
            this.centerPoint = new Point2D.Double(base5.getRealX() + Math.sin(Math.PI / 2 + angle + Math.asin(vertical / boundariesDistance)) * boundariesDistance / (2 * Math.cos(-angle)),
                    base5.getRealY() + Math.cos(Math.PI / 2 + angle + Math.asin(vertical / boundariesDistance)) * boundariesDistance / (2 * Math.cos(-angle)));
        else
            this.centerPoint = new Point2D.Double(base5.getRealX() - Math.sin(Math.PI / 2 - angle + Math.asin(vertical / boundariesDistance)) * boundariesDistance / (2 * Math.cos(angle)),
                    base5.getRealY() + Math.cos(Math.PI / 2 - angle + Math.asin(vertical / boundariesDistance)) * boundariesDistance / (2 * Math.cos(angle)));

    }

    /**
     * cette methode rend un point ayant subit une rotation d'un certain "angle" autour du point centerPoint
     * partant du point base5Boundarie par translation-rotation-translation
     */

    Point2D setRotatedCoordinates(final double angle) {
        //l'extremite 5' est translatee pour que le centre du cercle soit en (0,0)
        final Point2D.Double translatedPoint = new Point2D.Double();
        translatedPoint.setLocation(this.getExternalBase5().getRealX() - this.centerPoint.getX(), this.getExternalBase5().getRealY() - this.centerPoint.getY());

        //calcul des coordonnees cartesiennes du point subissant la rotation "angle"
        double newX = translatedPoint.getX() * Math.cos(-angle) - translatedPoint.getY() * Math.sin(-angle);
        double newY = translatedPoint.getX() * Math.sin(-angle) + translatedPoint.getY() * Math.cos(-angle);

        //les nouvelles coordonnees sont retranslatees
        newX += this.centerPoint.getX();
        newY += this.centerPoint.getY();
        return new Point2D.Double(newX, newY);
    }

    public String toSVG() {
        final StringBuffer svg = new StringBuffer();
        for (Residue2D r:this.getResidues())
            svg.append(r.toSVG());
        return svg.toString();
    }

    public void draw(final java.awt.Graphics2D g2) {
        if (this.structuralDomain != null && this.structuralDomain.getMolecules().size() > 0 && this.isInsideDrawingArea()) {
            if (this.isSelected())
                g2.setColor(java.awt.Color.red);
            else
                g2.setColor(this.color);
            for (Residue2D r:this.getResidues())
                r.draw(g2);
        }
    }

    public boolean isInsideDrawingArea() {
        Residue2D base = this.getBase5(), previousR = base.getPreviousResidue(), nextR = base.getNextResidue();
        while (base != null && base.getAbsolutePosition() <= this.getBase3Id()) {
            if (base.isInsideDrawingArea())
                return true;
            base = base.getNextResidue();
        }
        if (nextR != null && nextR.isInsideDrawingArea())
            return true;
        if (previousR != null && previousR.isInsideDrawingArea())
            return true;
        return false;
    }

    public void flipEnds(final Point2D center) {
        /*
        if (this.orientation == Sstrand2D.VERSE) {
            this.orientation = Sstrand2D.REVERSE;
        }
        else {
            this.orientation = Sstrand2D.VERSE;
        }
        //we recalculate the coordinates
        this.setCoordinates(height);*/
    }

    public void rotateEnds(final Point2D centerPoint, final double angle) {
    }

    public void translateEnds(final double dragX, final double dragY) {
    }

    public boolean contains(final double x, final double y) {
        boolean hit = false;
        Residue2D base = this.getBase5();
        while (base != null && base.getAbsolutePosition() <= this.getBase3Id()) {
            hit = base.contains(x, y);
            if (hit)
                break;
            base = base.getNextResidue();
        }
        return hit;
    }


    public Area getArea() {
        Area area = null;
        for (Residue2D r:this.getResidues()) {
            if (area == null)
                area=r.getArea();
            else
                area.add(r.getArea());
        }
        return area;
    }

    public void isAutomaticCoordinates(boolean automaticCoordinates) {
        this.automaticCoordinates = automaticCoordinates;
    }

    public boolean isAutomaticCoordinates() {
        return this.automaticCoordinates;
    }

    public void setColor(final java.awt.Color color) {
        super.setColor(color);
        for (Residue2D r:this.getResidues())
            r.setColor(color);
    }

    public void extend(Location l) {
        Molecule m = this.structuralDomain.getMolecules().iterator().next();
        this.structuralDomain.addToLocation(l,m);
        for (int i=l.getStart(); i<=l.getEnd();i++)
            this.repository.getResidue(m.getParadiseID(),i).setStructuralDomain(this);
    }

    public void reduce(Location l) {
        Molecule m = this.structuralDomain.getMolecules().iterator().next();
        this.structuralDomain.removeFromLocation(l,m);
        for (int i=l.getStart(); i<=l.getEnd();i++)
            this.repository.getResidue(m.getParadiseID(),i).setStructuralDomain(null);
    }
}
