package fr.unistra.ibmc.s2s.ssviewer.features;

import fr.unistra.ibmc.s2s.ssviewer.graphics.GraphicContext;
import fr.unistra.ibmc.s2s.ssviewer.utils.DrawingUtils;
import fr.unistra.ibmc.paradise.core.features.BaseBaseInteraction;

import java.awt.*;
import java.awt.geom.Area;

/**
 * @author fjossinet
 */
abstract class Symbol {

    java.awt.Shape shape;
    private char orientation = '?';
    protected GraphicContext gc;

    Symbol(GraphicContext gc) {
        this.gc = gc;
    }

    Symbol(final char orientation, GraphicContext gc) {
        this(gc);
        this.orientation = orientation;
    }

    public boolean contains(final double x, final double y) {
        return this.getArea().contains(x, y);
    }

    public Area getArea() {
        return new Area(this.shape);
    }


    public void draw(final java.awt.Graphics2D g2) {
        this.drawShape(g2);
    }

    public void drawShape(final java.awt.Graphics2D g2) {
        if (this.orientation == BaseBaseInteraction.ORIENTATION_TRANS) {
            g2.draw(this.shape);
        } else {
            g2.fill(this.shape);
        }
    }

    abstract public String toSVG();


    java.lang.String getSVGStyle(final java.awt.Color color) {
        if (this.orientation == BaseBaseInteraction.ORIENTATION_CIS) {
            return "style=\" fill-rule : nonzero; fill :" + DrawingUtils.colorToString(color) + " ; stroke :" + DrawingUtils.colorToString(color) + "; \"";
        } else {
            return "style=\" fill :none ; stroke :" + DrawingUtils.colorToString(color) + "; \"";
        }
    }

    Color getSVGColor() {
        return Color.BLACK;
    }

}
