package fr.unistra.ibmc.s2s.ssviewer.graphics;


import fr.unistra.ibmc.s2s.ssviewer.features.*;
import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.paradise.core.utils.Residue;

import java.util.List;

/**
 * @author fabricej
 */
class InteractionSelect extends SelectState {

    InteractionSelect(Mediator mediator) {
        super(mediator);
    }

    void mouseClicked(final java.awt.event.MouseEvent e) {
        super.mouseClicked(e);
        Interaction2D interSelect = null;
        final Model2D model2D = this.mediator.getCurrentModel2D();
        if (model2D != null && e.getClickCount() == 1) {
            for (Interaction2D interaction: model2D.getInterMolecularTertiaryInteractions()) {
                if (interaction.isDrawn() && interaction.contains(e.getX(), e.getY())) {
                    interSelect = interaction;
                    break;
                }
            }
            if (interSelect == null)
                HELICES:for (Helix2D h:model2D.getInterMolecularHelices()) {
                    for (Interaction2D interaction: h.getSecondaryInteractions()) {
                        if (interaction.isDrawn() && interaction.contains(e.getX(), e.getY())) {
                            interSelect = interaction;
                            break HELICES;
                        }
                    }
                }
            if (interSelect == null)
                RNAS:for (Rna2D rna:model2D.getRnas()) {
                    for (Interaction2D interaction: rna.getTertiaryInteractions()) {
                        if (interaction.isDrawn() && interaction.contains(e.getX(), e.getY())) {
                            interSelect = interaction;
                            break RNAS;
                        }
                    }
                    for (Helix2D h:rna.getHelices()) {
                        for (Interaction2D interaction: h.getSecondaryInteractions()) {
                            if (interaction.isDrawn() && interaction.contains(e.getX(), e.getY())) {
                                interSelect = interaction;
                                break RNAS;
                            }
                        }
                    }
                }
            if (interSelect != null) {
                if (e.isShiftDown())
                    this.mediator.getSecondaryCanvas().addShiftCurrentInteraction(interSelect);
                else if (e.isControlDown())
                    this.mediator.getSecondaryCanvas().addCtrlCurrentInteraction(interSelect);
                else
                    this.mediator.getSecondaryCanvas().addCurrentInteraction(interSelect);
                this.mediator.getSecondaryCanvas().updateGravityPoint();
                if (BaseBaseInteraction2D.class.isInstance(interSelect))
                    this.mediator.getRna2dViewer().fireInteractionSelected(((BaseBaseInteraction2D)interSelect).getBaseBaseInteractionFeature(),e.isShiftDown());                                
            }

        }
    }

    //return the first interaction that wrap this residue
    Rna2DElement getWrapper(Residue residue) {
        if (this.mediator.getSecondaryCanvas() != null) {
            for (Rna2D rna:this.mediator.getSecondaryCanvas().getModel2D().getRnas()) {
                if (residue.getMoleculeId().equals(rna.getRnaMolecule().getParadiseID())) {
                    java.util.List interactions = this.mediator.getSecondaryCanvas().getModel2D().getInteractions(residue.getMoleculeId(), residue.getAbsolutePosition());
                    if (interactions.size() > 0)
                        return (Interaction2D) interactions.get(0);
                }
            }
        }
        return null;
    }

    public void select(SecondaryCanvas canvas, List<Residue2D> residues) {
        canvas.clearAllSelections();
        for (Residue2D r2D:residues)
            for (Interaction2D int2D:r2D.getInteractions()) {
                if (!PhosphodiesterBond2D.class.isInstance(int2D))
                    canvas.addShiftCurrentInteraction(int2D);
            }
    }

}
