package fr.unistra.ibmc.s2s.ssviewer.graphics;

import fr.unistra.ibmc.s2s.ssviewer.features.*;
import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.paradise.core.utils.Residue;

import java.util.*;

/**
 * @author fabricej
 */
class ResidueSelect extends SelectState {

    ResidueSelect(Mediator mediator) {
        super(mediator);
    }

    void mouseClicked(final java.awt.event.MouseEvent e) {
        super.mouseClicked(e);
        Residue2D residueSelected = null;
        final Model2D model2D = this.mediator.getCurrentModel2D();
        if (model2D != null && e.getClickCount() == 1) {
            HELICES : for (Helix2D h:model2D.getInterMolecularHelices()) {
                for (Residue2D r : h.getResidues()) {
                    if (r.isDrawn() && r.contains(e.getX(), e.getY())) {
                        residueSelected = r;
                        break HELICES;
                    }
                }
            }
            if (residueSelected == null)
                RNAS:for (Rna2D rna: model2D.getRnas()) {
                    for (Helix2D h:rna.getHelices()) {
                        for (Residue2D r : h.getResidues())
                            if (r.isDrawn() && r.contains(e.getX(), e.getY())) {
                                residueSelected = r;
                                break RNAS;
                            }
                    }
                    for (Sstrand2D ss:rna.getStrands()) {
                            for (Residue2D r : ss.getResidues())
                                if (r.isDrawn() && r.contains(e.getX(), e.getY())) {
                                    residueSelected = r;
                                    break RNAS;
                                }
                        }
                    //if no helix
                    if (rna.getHelices().size()== 0)
                        for (Residue2D r : rna.getStrands().get(0).getResidues())
                            if (r.isDrawn() && r.contains(e.getX(), e.getY())) {
                                residueSelected = r;
                                break;
                            }
                }
            if (residueSelected != null) {
                if (e.isShiftDown())
                    this.mediator.getSecondaryCanvas().addShiftCurrentResidue(residueSelected);
                else if (e.isControlDown())
                    this.mediator.getSecondaryCanvas().addCtrlCurrentResidue(residueSelected);
                else
                    this.mediator.getSecondaryCanvas().addCurrentResidue(residueSelected);
                List<Residue> selectedResidues = new ArrayList<Residue>();
                for (Residue2D r:this.mediator.getSecondaryCanvas().getCurrentResidues())
                    selectedResidues.add(r.getResidue2DFeature().getResidues().iterator().next());
                this.mediator.getRna2dViewer().fireResiduesSelected(selectedResidues,e.isShiftDown());
            }

        }
        //if two clicks on a residue, we give the details of the features in the Right features details panels
        else if (e.getClickCount() == 2) {
            //this.mediator.getCurrentCanvas().mediator.displayCanvasDetails((fr.unistra.ibmc.s2s.features.Node)structurePassed);
        }
    }

    void mouseDragged(final java.awt.event.MouseEvent e) {
        this.mediator.getSecondaryCanvas().updateGravityPoint();
        if (this.mediator.getSecondaryCanvas().getGravityPoint() != null) {
            final double trX = e.getX() - this.mediator.getSecondaryCanvas().getGravityPoint().getX();
            final double trY = e.getY() - this.mediator.getSecondaryCanvas().getGravityPoint().getY();
            this.mediator.getSecondaryCanvas().translateSelection(trX, trY);
        }
    }

    Rna2DElement getWrapper(Residue residue) {
        if (this.mediator.getSecondaryCanvas() != null) {
            return this.mediator.getSecondaryCanvas().getModel2D().getResidue(residue.getMoleculeId(), residue.getAbsolutePosition());
        }
        return null;
    }

    public void select(SecondaryCanvas canvas, List<Residue2D> residues) {
        canvas.clearAllSelections();
        for (Residue2D r2D:residues)
            canvas.addShiftCurrentResidue(r2D);
    }

}
