package fr.unistra.ibmc.s2s.ssviewer.graphics;

import fr.unistra.ibmc.paradise.ArchitectureException;
import fr.unistra.ibmc.paradise.BiologicalSymbolException;
import fr.unistra.ibmc.paradise.utils.GeneralUtils;
import fr.unistra.ibmc.paradise.utils.VTextIcon;
import fr.unistra.ibmc.paradise.core.*;
import fr.unistra.ibmc.paradise.core.features.*;
import fr.unistra.ibmc.paradise.core.io.CTFileIO;
import fr.unistra.ibmc.paradise.core.analysis.Parameter;
import fr.unistra.ibmc.paradise.core.analysis.Parameters;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.paradise.tools.*;
import fr.unistra.ibmc.s2s.rnalign.utils.RessourcesUtils;
import fr.unistra.ibmc.s2s.ssviewer.features.*;
import fr.unistra.ibmc.s2s.ssviewer.features.Residue2D;
import fr.unistra.ibmc.s2s.SelectionListener;
import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.s2s.behaviours.HelixReorganizationRequestAnswerBehaviour;
import jade.gui.GuiEvent;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;
import java.io.*;

/**
 * This class defines the widget displaying an extended secondary structure
 * @author fjossinet
 */
public class Rna2DViewer extends JPanel implements SelectionListener {

    private Mediator mediator;
    private Rna2DViewerToolBar toolbar;
    private javax.swing.JButton lastClicked;
    private SelectButton selectButton;
    public static Cursor selectCursor,  magnifyCursor,  magnifyPlusCursor,  magnifyMinusCursor,  moveviewCursor,  rotateCursor;
    private Cursor currentCursor;
    public static final int TERTIARY_STRUCTURE_ANNOTATION = 0, TERTIARY_STRUCTURE_PREDICTION = 1, SECONDARY_STRUCTURE_PREDICTION = 3, SECONDARY_STRUCTURE_DRAWING = 4, HELICES_REORGANIZATION = 5;
    private SelectState currentSelectionMode;
    private List<SelectionListener> selectionListeners;
    private SecondaryCanvas canvas;

    static {
        Toolkit tk = Toolkit.getDefaultToolkit();
        selectCursor = tk.createCustomCursor(RessourcesUtils.getImage("select-cursor.gif"), new Point(0, 0), "select");
        magnifyCursor = tk.createCustomCursor(RessourcesUtils.getImage("magnify-cursor.gif"), new Point(0, 0), "magnify");
        magnifyPlusCursor = tk.createCustomCursor(RessourcesUtils.getImage("magnify-plus-cursor.gif"), new Point(0, 0), "mplus");
        magnifyMinusCursor = tk.createCustomCursor(RessourcesUtils.getImage("magnify-minus-cursor.gif"), new Point(0, 0), "mminus");
        moveviewCursor = tk.createCustomCursor(RessourcesUtils.getImage("move-view-cursor.gif"), new Point(0, 0), "move view");
        rotateCursor = tk.createCustomCursor(RessourcesUtils.getImage("rotate-cursor.gif"), new Point(0, 0), "rotate");
    }

    public Rna2DViewer(fr.unistra.ibmc.s2s.Mediator mediator,ParadiseWorkingSession workingSession) {
        this.mediator = mediator;
        this.mediator.setRna2DViewer(this);
        this.toolbar = new Rna2DViewerToolBar();
        this.setLayout(new BorderLayout());
        this.canvas = new SecondaryCanvas(workingSession, this.mediator);
        this.add(this.canvas, java.awt.BorderLayout.CENTER);
        this.currentCursor = Rna2DViewer.selectCursor;
        this.selectionListeners = new ArrayList<SelectionListener>();
    }

    public Rna2DViewerToolBar getToolbar() {
        return this.toolbar;
    }

    public void addSelectionListener(SelectionListener l) {
        this.selectionListeners.add(l);
    }

    public Mediator getMediator() {
        return mediator;
    }

    public MainParadiseFeature getCurrentFeature() {
        if (this.mediator == null || this.mediator.getSecondaryCanvas() == null || this.mediator.getSecondaryCanvas().getModel2D() == null)
            return null;
        else
            return this.mediator.getSecondaryCanvas().getModel2D().getSecondaryStructure();
    }

    public void clearDisplay() {
        if (this.mediator.getSecondaryCanvas() != null)
            this.mediator.getSecondaryCanvas().clearDisplay();
    }

    private void setCursorForCanvas(Cursor cursor) {
        this.canvas.setCursor(cursor);
    }

    public Cursor getCurrentCursor() {
        return this.currentCursor;
    }

    public void loadRNASecondaryStructure(SecondaryStructure ss) {

    }

    /**
     * Return the current SecondaryCanvas
     *
     * @return the current SecondaryCanvas. Return null if no SecondaryCanvas available
     */
    public SecondaryCanvas getSecondaryCanvas() {
        return this.canvas;
    }

    void doClickOnSelectButton() {
        if (this.getSecondaryCanvas() != null) {
            if (this.lastClicked != null) {
                this.lastClicked.setEnabled(true);
            }
            this.currentCursor = Rna2DViewer.selectCursor;
            this.setCursorForCanvas(Rna2DViewer.selectCursor);
            this.lastClicked = this.selectButton;
        }
    }

    public void reorganizeHelices() {
        GuiEvent ev = new GuiEvent(this, Rna2DViewer.HELICES_REORGANIZATION);
        if (this.getSecondaryCanvas()!= null && this.getSecondaryCanvas().getModel2D() != null) {
            SecondaryStructure ss = this.getSecondaryCanvas().getModel2D().getSecondaryStructure();
            List<Molecule> molecules = new ArrayList<Molecule>();
            for (Molecule m:ss.getMolecules()) {
                m.removeAllSelectedFeatures();
                m.addSelectedFeature(ss);
                molecules.add(m);
            }
            ev.addParameter(molecules);
            ev.addParameter(new ArrayList<Parameter>());
            //((AbstractParadiseGUIToolAgent) agent).postGuiEvent(ev);
        }
    }

    public void residuesSelected(Collection<Residue> residues, boolean isShiftDown) {
        if (!isShiftDown)
            this.getSecondaryCanvas().clearAllSelections();
        Iterator<Residue> it = residues.iterator();
        while (it.hasNext()) {
            Residue2D r=this.getSecondaryCanvas().seekResidue2D(it.next());
            if (r!=null)
                this.getSecondaryCanvas().addShiftCurrentResidue(r);
        }
    }

    public void fireResiduesSelected(Collection<Residue> residues, boolean isShiftDown) {
        for (SelectionListener l:this.selectionListeners)
            l.residuesSelected(residues,isShiftDown);
    }

    public void structuralDomainSelected(StructuralDomain structuralDomain, boolean isShiftDown) {
        if (!isShiftDown)
            this.getSecondaryCanvas().clearAllSelections();
        this.getSecondaryCanvas().addShiftCurrentStructure((StructuralDomain2D)this.getSecondaryCanvas().seekRna2DNode(structuralDomain));
    }

    public void fireStructuralDomainSelected(StructuralDomain structuralDomain, boolean isShiftDown) {
        for (SelectionListener l:this.selectionListeners)
            l.structuralDomainSelected(structuralDomain,isShiftDown);
    }

    public void interactionSelected(BaseBaseInteraction interaction, boolean isShiftDown) {
        if (!isShiftDown)
            this.getSecondaryCanvas().clearAllSelections();
        this.getSecondaryCanvas().addShiftCurrentInteraction((Interaction2D)this.getSecondaryCanvas().seekRna2DNode(interaction));
    }

    public void fireInteractionSelected(BaseBaseInteraction interaction, boolean isShiftDown) {
        for (SelectionListener l:this.selectionListeners)
            l.interactionSelected(interaction,isShiftDown);
    }

    private class ColorsBar extends JToolBar {

        private static final int size = 20;
        private int counter;
        private ColorChooser cc;

        ColorsBar() {
            super();
            this.setOrientation(JToolBar.HORIZONTAL);
            this.setFloatable(false);
            this.setBackground(Color.WHITE);
            this.add(new ColorButton(Color.GREEN));
            this.add(new ColorButton(Color.ORANGE));
            this.add(new ColorButton(Color.PINK));
            this.add(new ColorButton(Color.YELLOW));
            this.add(new ColorButton(Color.BLUE));
            this.add(new ColorButton(Color.CYAN));
            this.add(new ColorButton(Color.RED));
            this.add(new ColorButton(Color.GRAY));
            this.counter = 8;
            for (int i = this.counter; i < size; i++) {
                this.add(new ColorButton());
            }
        }

        void registerColor(Color c) {
            for (int i = 0; i < this.getComponentCount(); i++) {
                if (((ColorButton) this.getComponent(i)).getColor().getRGB() == c.getRGB()) {
                    return;
                }
            }
            if (counter == ColorsBar.size) {
                counter = 0;
            }
            ((ColorButton) this.getComponent(counter)).setColor(c);
            counter++;
        }

        private class ColorButton extends JButton implements java.awt.event.ActionListener, MouseListener {

            private ColorIcon icon = new ColorIcon(Color.LIGHT_GRAY);

            public ColorButton(Color c) {
                this();
                this.setColor(c);
            }

            public ColorButton() {
                super();
                this.setMargin(new Insets(0, 0, 0, 0));
                this.setBorderPainted(false);
                this.setBackground(Color.WHITE);
                this.setIcon(icon);
                this.addActionListener(this);
                this.addMouseListener(this);
            }

            public void actionPerformed(ActionEvent event) {
                if (this.icon.getColor().getRGB() == Color.LIGHT_GRAY.getRGB()) {
                    if (cc == null) {
                        cc = new ColorChooser(new ChangeListener() {
                            public void stateChanged(final ChangeEvent e) {
                                getSecondaryCanvas().setColor(cc.getColor());
                                cc.dispose();
                            }
                        });
                    }
                    cc.toForeground();
                } else {
                    if (getSecondaryCanvas() != null) {
                        getSecondaryCanvas().setColor(this.icon.getColor());
                    }
                }
            }

            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                this.setBackground(Color.lightGray);
                this.setBorderPainted(true);
            }

            public void mouseExited(MouseEvent e) {
                this.setBackground(Color.white);
                this.setBorderPainted(false);
            }

            private void setColor(Color c) {
                this.icon.setColor(c);
                this.repaint();
            }

            private Color getColor() {
                return this.icon.getColor();
            }
        }

        private class ColorIcon implements Icon {

            private int height = 15;
            private int width = 15;
            private Color color;

            public ColorIcon(Color color) {
                this.color = color;
            }

            public int getIconHeight() {
                return height;
            }

            public int getIconWidth() {
                return width;
            }

            public void setColor(Color newValue) {
                this.color = newValue;
            }

            public Color getColor() {
                return this.color;
            }

            public void paintIcon(Component c, Graphics g, int x, int y) {
                g.setColor(color);
                g.fillRect(x, y, width, height);
            }
        }
    }

    private class Rna2DViewerToolBar extends JToolBar {

        private javax.swing.JLabel label;
        private StateButton zoomPlusButton,  zoomMinusButton,  noZoomButton,  rotButton,  moveView;
        private ActionButton addHelix, removeHelix, addTertiaryInteraction, removeTertiaryInteraction, modifyInteractionType;
        private ActionButton[] actions = new ActionButton[15];
        private BackboneSpinner backboneSpinner;

        private Rna2DViewerToolBar() {
            super();
            this.setFloatable(false);
            this.setBackground(Color.white);
            this.backboneSpinner = new BackboneSpinner();
            int height = new ImageIcon(RessourcesUtils.getImage("flip.png")).getIconHeight();
            actions[0] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("flip.png")), new ActionListener() {

                public void actionPerformed(final ActionEvent e) {
                    if (getSecondaryCanvas()!=null)
                        getSecondaryCanvas().flipSelection();
                }
            });
            actions[0].setToolTipText("Flip selection");
            actions[1] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("lock.png")), new ActionListener() {

                public void actionPerformed(final ActionEvent e) {

                }
            });
            actions[1].setToolTipText("Lock interaction display");
            actions[2] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("unlock.png")), new ActionListener() {

                public void actionPerformed(final ActionEvent e) {

                }
            });
            actions[2].setToolTipText("Unlock interaction display");
            actions[3] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("show-symbol.png")), new ActionListener() {

                public void actionPerformed(final ActionEvent e) {
                    if (getSecondaryCanvas()!=null) {

                        final Iterator ic = getSecondaryCanvas().getCurrentInteractions().iterator();
                        while (ic.hasNext()) {
                            ((Interaction2D) ic.next()).drawRealSymbol(true);
                        }
                        getSecondaryCanvas().repaint();
                    }
                }
            });
            actions[3].setToolTipText("Display LW symbol");
            actions[4] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("hide-symbol.png")), new ActionListener() {

                public void actionPerformed(final ActionEvent e) {
                    if (getSecondaryCanvas()!=null) {
                        final Iterator ic = getSecondaryCanvas().getCurrentInteractions().iterator();
                        while (ic.hasNext()) {
                            ((Interaction2D) ic.next()).drawRealSymbol(false);
                        }
                        getSecondaryCanvas().repaint();
                    }
                }
            });
            actions[4].setToolTipText("Hide LW symbol");
            actions[5] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("center-view.png")), new ActionListener() {

                public void actionPerformed(final ActionEvent e) {
                    if (getSecondaryCanvas()!=null)
                        getSecondaryCanvas().centerView();
                }
            });
            actions[5].setToolTipText("Center view on 2D structure");

            actions[8] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("plus.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (getSecondaryCanvas()!=null)
                        getSecondaryCanvas().increaseFontSize();
                }
            });
            actions[8].setToolTipText("Increase font");
            actions[9] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("minus.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (getSecondaryCanvas()!=null)
                        getSecondaryCanvas().decreaseFontSize();
                }
            });
            actions[9].setToolTipText("Decrease font");
            actions[12] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("ellipse.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    if (getSecondaryCanvas()!=null) {
                        List strs = getSecondaryCanvas().getCurrentStructures();
                        StructuralDomain2D str = null;
                        for (Iterator i = strs.iterator(); i.hasNext();) {
                            str = (StructuralDomain2D) i.next();
                            if (str instanceof Sstrand2D) {
                                ((Sstrand2D) str).isAutomaticCoordinates(true);
                            }
                        }
                        getSecondaryCanvas().repaint();
                    }
                }
            });
            actions[12].setToolTipText("Single-strand automatic drawing mode");
            actions[13] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("polygon.png")), new ActionListener() {

                public void actionPerformed(final ActionEvent e) {
                    if (getSecondaryCanvas()!=null) {
                        List strs = getSecondaryCanvas().getCurrentStructures();
                        StructuralDomain2D str = null;
                        for (Iterator i = strs.iterator(); i.hasNext();) {
                            str = (StructuralDomain2D) i.next();
                            if (str instanceof Sstrand2D) {
                                ((Sstrand2D) str).isAutomaticCoordinates(false);
                            }
                        }
                        getSecondaryCanvas().repaint();
                    }
                }
            });
            actions[13].setToolTipText("Single-strand user-defined drawing mode");
            actions[14] = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("clear.png")), new ActionListener() {

                public void actionPerformed(final ActionEvent e) {
                    if (mediator.getSecondaryCanvas() != null) {
                        mediator.getSecondaryCanvas().clearTertiaries();
                    }
                }
            });
            actions[14].setToolTipText("Clear tertiary interactions");
            selectButton = new SelectButton(null, new ImageIcon(RessourcesUtils.getImage("select.png")));
            selectButton.setToolTipText("Select/Translate selection");
            this.zoomPlusButton = new StateButton(null, new ImageIcon(RessourcesUtils.getImage("plus.png")), new MagnusZoomState(mediator), Rna2DViewer.magnifyPlusCursor);
            this.zoomPlusButton.setToolTipText("Zoom in");
            this.zoomMinusButton = new StateButton(null, new ImageIcon(RessourcesUtils.getImage("minus.png")), new MinusZoomState(mediator), Rna2DViewer.magnifyMinusCursor);
            this.zoomMinusButton.setToolTipText("Zoom out");
            this.noZoomButton = new StateButton(null, new ImageIcon(RessourcesUtils.getImage("zoom.png")), new ClearZoomState(mediator), Rna2DViewer.magnifyCursor);
            this.noZoomButton.setToolTipText("1:1");
            this.rotButton = new StateButton(null, new ImageIcon(RessourcesUtils.getImage("rotate.png")), new RotationState(mediator), Rna2DViewer.rotateCursor);
            this.rotButton.setToolTipText("Rotate selection");
            this.moveView = new StateButton(null, new ImageIcon(RessourcesUtils.getImage("move-view.png")), new MoveViewState(mediator), Rna2DViewer.moveviewCursor);
            this.moveView.setToolTipText("Move view");

            this.addHelix = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("new-secondary.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    SecondaryCanvas canvas = mediator.getSecondaryCanvas();
                    Model2D model2D = canvas.getModel2D();
                    if (model2D != null && canvas.getCurrentResidues().size() == 4) {
                        Iterator<Residue2D> it = canvas.getCurrentResidues().iterator();
                        Residue2D r1 = it.next(), r2 = it.next(), r3 = it.next(), r4 = it.next();
                        ParadiseID id1 = r1.getMoleculeId(), id2 = r2.getMoleculeId(), id3 = r3.getMoleculeId(), id4 = r4.getMoleculeId();
                        List<Residue2D> sortedResidues =  new ArrayList<Residue2D>(4);
                        if (id1 == id2 && id3 == id4) {
                            if (r1.getAbsolutePosition() < r2.getAbsolutePosition()) {
                                sortedResidues.add(r1);
                                sortedResidues.add(r2);
                            }
                            else {
                                sortedResidues.add(r2);
                                sortedResidues.add(r1);
                            }
                            if (r3.getAbsolutePosition() < r4.getAbsolutePosition()) {
                                sortedResidues.add(r3);
                                sortedResidues.add(r4);
                            }
                            else {
                                sortedResidues.add(r4);
                                sortedResidues.add(r3);
                            }
                        }
                        else if (id1 == id3 && id2 == id4) {
                            if (r1.getAbsolutePosition() < r3.getAbsolutePosition()) {
                                sortedResidues.add(r1);
                                sortedResidues.add(r3);
                            }
                            else {
                                sortedResidues.add(r3);
                                sortedResidues.add(r1);
                            }
                            if (r2.getAbsolutePosition() < r4.getAbsolutePosition()) {
                                sortedResidues.add(r2);
                                sortedResidues.add(r4);
                            }
                            else {
                                sortedResidues.add(r4);
                                sortedResidues.add(r2);
                            }
                        }
                        else if (id1 == id4 && id2 == id3) {
                            if (r1.getAbsolutePosition() < r4.getAbsolutePosition()) {
                                sortedResidues.add(r1);
                                sortedResidues.add(r4);
                            }
                            else {
                                sortedResidues.add(r4);
                                sortedResidues.add(r1);
                            }
                            if (r2.getAbsolutePosition() < r3.getAbsolutePosition()) {
                                sortedResidues.add(r2);
                                sortedResidues.add(r3);
                            }
                            else {
                                sortedResidues.add(r3);
                                sortedResidues.add(r2);
                            }
                        }
                        it = sortedResidues.iterator();
                        try {
                            model2D.createHelix(it.next(),it.next(),it.next(), it.next());
                            mediator.getSecondaryCanvas().clearAllSelections();
                        }
                        catch (Exception ex) {
                            JOptionPane.showMessageDialog(Rna2DViewer.this,ex.getMessage(),"Cannot create Helix", JOptionPane.ERROR_MESSAGE);
                        }
                        mediator.getSecondaryCanvas().repaint();
                    }
                    else {
                        JOptionPane.showMessageDialog(Rna2DViewer.this,"Select exactly four residues to use this option","Cannot create Helix", JOptionPane.ERROR_MESSAGE);
                    }
                }
            });
            this.addHelix.setToolTipText("Create new helix");

            this.removeHelix = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("delete-interaction.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    int transformAsTertiaryInteractions = JOptionPane.showConfirmDialog(Rna2DViewer.this,"Do you want to keep the secondary interactions as tertiary ones?");
                    if (transformAsTertiaryInteractions == JOptionPane.CANCEL_OPTION)
                        return;
                    Model2D model2D = mediator.getSecondaryCanvas().getModel2D();
                    if (model2D != null) {
                        List<StructuralDomain2D> domains = new ArrayList<StructuralDomain2D>(mediator.getSecondaryCanvas().getCurrentStructures());
                        for (StructuralDomain2D sd:domains)
                            if (Helix2D.class.isInstance(sd)) {
                                mediator.getSecondaryCanvas().removeCurrentStructure(sd);
                                try {
                                model2D.removeHelix((Helix2D)sd,transformAsTertiaryInteractions == JOptionPane.OK_OPTION);
                                } catch (BiologicalSymbolException e1) {
                                    JOptionPane.showMessageDialog(Rna2DViewer.this,e1.getMessage(),"Cannot remove Helix", JOptionPane.ERROR_MESSAGE);
                                    return;
                                }
                                }
                    }
                    mediator.getSecondaryCanvas().repaint();
                }
            });
            this.removeHelix.setToolTipText("Remove helix");

            this.addTertiaryInteraction = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("new-tertiary.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    List<Residue2D> residues = mediator.getSecondaryCanvas().getCurrentResidues();
                    if (residues.size() == 2) {
                        new InteractionSelector(residues.get(0), residues.get(1), true);
                        mediator.getSecondaryCanvas().repaint();
                    }
                    else
                        JOptionPane.showMessageDialog(Rna2DViewer.this,"You have to select two residues to create a new tertiary interaction","No residues selected!!", JOptionPane.ERROR_MESSAGE);
                }
            });
            this.addTertiaryInteraction.setToolTipText("Create new tertiary interaction");

            this.removeTertiaryInteraction = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("delete-interaction.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    List<Interaction2D> interactions = mediator.getSecondaryCanvas().getCurrentInteractions();
                    if (interactions.isEmpty())
                        JOptionPane.showMessageDialog(Rna2DViewer.this,"No interactions selected !!");
                    else {
                        List<BaseBaseInteraction> interactionsRemoved = new ArrayList<BaseBaseInteraction>();
                        mediator.getSecondaryCanvas().clearAllSelections();
                        for (Interaction2D i:interactions)
                            if (BaseBaseInteraction2D.class.isInstance(i) && !((BaseBaseInteraction2D)i).getBaseBaseInteractionFeature().isSecondaryInteraction() && ((BaseBaseInteraction2D)i).getBaseBaseInteractionFeature().getMolecules().size() == 1)  {
                                mediator.getSecondaryCanvas().getModel2D().removeTertiaryInteraction((BaseBaseInteraction2D)i);
                                interactionsRemoved.add(((BaseBaseInteraction2D)i).getBaseBaseInteractionFeature());
                            }
                        mediator.getSecondaryCanvas().repaint();
                    }
                }
            });
            this.removeTertiaryInteraction.setToolTipText("Remove tertiary interactions");

            this.modifyInteractionType = new ActionButton(null, new ImageIcon(RessourcesUtils.getImage("modify-interaction.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    List<Interaction2D> interactions = mediator.getSecondaryCanvas().getCurrentInteractions();
                    if (interactions.size() == 1 && BaseBaseInteraction2D.class.isInstance(interactions.get(0))) {
                        new InteractionSelector((BaseBaseInteraction2D)interactions.get(0));
                        mediator.getSecondaryCanvas().repaint();
                    }
                    else
                        JOptionPane.showMessageDialog(Rna2DViewer.this,"You have to select a single interaction!!");
                }
            });
            this.modifyInteractionType.setToolTipText("Modify interaction's type");


            JLabel toolBarName = new JLabel();
            toolBarName.setIcon(new VTextIcon(toolBarName, "2D", VTextIcon.ROTATE_LEFT));
            this.add(toolBarName);
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(this.moveView);
            this.add(selectButton);
            this.add(this.rotButton);
            ActionButton reorganizeHelices = new ActionButton(null,new ImageIcon(RessourcesUtils.getImage("magic.png")), new ActionListener() {
                public void actionPerformed(final ActionEvent e) {
                    GuiEvent ev = new GuiEvent(this, fr.unistra.ibmc.paradise.Paradise.HELICES_REORGANIZATION);
                    ArrayList<Molecule> molecules = new ArrayList<Molecule>();
                    if (getSecondaryCanvas().getModel2D() != null) {
                        SecondaryStructure ss = getSecondaryCanvas().getModel2D().getSecondaryStructure();
                        for (Molecule m : ss.getMolecules()) {
                            if (RNA.class.isInstance(m)) {
                                m.removeAllSelectedFeatures();
                                molecules.add(m);
                                m.addSelectedFeature(ss);
                            }
                        }
                        ev.addParameter(new HelixReorganizationRequestAnswerBehaviour(molecules,new Parameters()));
                        ((AbstractParadiseToolAgent) mediator.getS2s().getAgent()).postGuiEvent(ev);
                    }
                }
            });
            if (!mediator.getS2s().isConnected()) {
                reorganizeHelices.setToolTipText("Reorganize helices (need connection)");
                reorganizeHelices.setEnabled(false);
            }
            else
                reorganizeHelices.setToolTipText("Reorganize helices");
            this.add(reorganizeHelices);
            JSeparator separator = new JSeparator(SwingConstants.VERTICAL);
            separator.setBackground(Color.GRAY);
            separator.setMaximumSize(new Dimension(1, height));
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(separator);
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(actions[5]);
            this.add(actions[0]);

            separator = new JSeparator(SwingConstants.VERTICAL);
            separator.setBackground(Color.GRAY);
            separator.setMaximumSize(new Dimension(1, height));
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(separator);
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(addHelix);
            this.add(removeHelix);
            this.add(addTertiaryInteraction);
            this.add(removeTertiaryInteraction);
            this.add(modifyInteractionType);

            separator = new JSeparator(SwingConstants.VERTICAL);
            separator.setBackground(Color.GRAY);
            separator.setMaximumSize(new Dimension(1, height));
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(separator);
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(zoomPlusButton);
            this.add(noZoomButton);
            this.add(zoomMinusButton);

            /*
            separator = new JSeparator(SwingConstants.VERTICAL);
            separator.setBackground(Color.GRAY);
            separator.setMaximumSize(new Dimension(1, height));
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(separator);
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(new JLabel(new ImageIcon(RessourcesUtils.getImage("font.png"))));
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(actions[8]);
            this.add(actions[9]);*/

            separator = new JSeparator(SwingConstants.VERTICAL);
            separator.setBackground(Color.GRAY);
            separator.setMaximumSize(new Dimension(1, height));
            this.add(javax.swing.Box.createHorizontalStrut(5));
            this.add(separator);
            this.add(javax.swing.Box.createHorizontalStrut(5));
            //this.add(new BackboneButton());
            //this.add(this.backboneSpinner);
            this.add(new CaptureViewButton());
        }

        private class ActionButton extends javax.swing.JButton implements MouseListener {

            private ActionButton(final String name, final javax.swing.Icon image, final ActionListener listener) {
                super(name, image);
                this.addActionListener(listener);
                this.addMouseListener(this);
                this.setMargin(new Insets(0, 0, 0, 0));
                this.setBorderPainted(false);
                this.setBackground(Color.white);
            }

            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                this.setBackground(Color.lightGray);
                this.setBorderPainted(true);
            }

            public void mouseExited(MouseEvent e) {
                this.setBackground(Color.white);
                this.setBorderPainted(false);
            }
        }

        private class StateButton extends javax.swing.JButton implements java.awt.event.ActionListener, MouseListener {

            private State state;
            private Cursor cursor;

            private StateButton(final String name, final javax.swing.Icon image, final State state, final Cursor cursor) {
                super(name, image);
                this.setMargin(new Insets(0, 0, 0, 0));
                this.setBorderPainted(false);
                this.setBackground(Color.white);
                this.addActionListener(this);
                this.addMouseListener(this);
                this.state = state;
                this.cursor = cursor;
            }

            public void actionPerformed(final java.awt.event.ActionEvent e) {
                if (lastClicked != null) {
                    lastClicked.setEnabled(true);
                }
                lastClicked = this;
                if (getSecondaryCanvas().getModel2D() != null) {
                    getSecondaryCanvas().setState(this.state);
                }
                Rna2DViewer.this.setCursorForCanvas(this.cursor);
                Rna2DViewer.this.currentCursor = this.cursor;
            }

            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                this.setBackground(Color.lightGray);
                this.setBorderPainted(true);
            }

            public void mouseExited(MouseEvent e) {
                this.setBackground(Color.white);
                this.setBorderPainted(false);
            }
        }

        private class DisplayButton extends JButton implements MouseListener {

            private DisplayButton() {
                super(new ImageIcon(RessourcesUtils.getImage("display-unlocked.png")));
                this.setToolTipText("Lock/Unlock the display");
                this.setBackground(Color.white);
                this.setBorderPainted(false);
                this.addMouseListener(this);
                this.addActionListener(new ActionListener() {

                    public void actionPerformed(final ActionEvent e) {
                        mediator.isLinked(!mediator.isLinked());
                        if (mediator.isLinked()) {
                            setIcon(new ImageIcon(RessourcesUtils.getImage("display-unlocked.png")));
                        } else {
                            setIcon(new ImageIcon(RessourcesUtils.getImage("display-locked.png")));
                        }
                    }
                });
            }

            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                this.setBackground(Color.lightGray);
                this.setBorderPainted(true);
            }

            public void mouseExited(MouseEvent e) {
                this.setBackground(Color.white);
                this.setBorderPainted(false);
            }
        }

        private class CaptureViewButton extends JButton implements MouseListener {

            private CaptureViewButton() {
                super(new ImageIcon(RessourcesUtils.getImage("capture.png")));
                this.setToolTipText("Capture the 2D view");
                this.setBackground(Color.white);
                this.setBorderPainted(false);
                this.addMouseListener(this);
                this.addActionListener(new ActionListener() {

                    public void actionPerformed(final ActionEvent e) {
                        if (mediator.getCurrentModel2D()!= null){
                            final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(fr.unistra.ibmc.paradise.Paradise.getWorkingDirectory());
                            fileChooser.setFileFilter(new javax.swing.filechooser.FileFilter() {
                                public boolean accept(final java.io.File f) {
                                    return f.isDirectory() || f.getName().endsWith("svg");
                                }

                                public String getDescription() {
                                    return "PNG file";
                                }

                            });
                            if (fileChooser.showSaveDialog(Rna2DViewer.this) == javax.swing.JFileChooser.APPROVE_OPTION) {
                                java.io.File file = null;
                                java.io.PrintWriter pw = null;
                                try {
                                    file = fileChooser.getSelectedFile();
                                    file.createNewFile();
                                    fr.unistra.ibmc.paradise.Paradise.setWorkingDirectory(file.getAbsolutePath());
                                    ImageIO.write(mediator.getSecondaryCanvas().getImage(), "png", file);
                                    JOptionPane.showMessageDialog(Rna2DViewer.this, "Saving successful");
                                } catch (java.io.FileNotFoundException fnfe) {
                                    System.out.println("I cannot find the file : " + file.getPath());
                                    return;
                                } catch (java.io.IOException ioe) {
                                    System.out.println("Problem to save data in the file : " + file.getPath());
                                    return;
                                } finally {
                                    if (pw != null) {
                                        pw.close();
                                    }
                                }
                            }
                        }
                    }
                });
            }

            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                this.setBackground(Color.lightGray);
                this.setBorderPainted(true);
            }

            public void mouseExited(MouseEvent e) {
                this.setBackground(Color.white);
                this.setBorderPainted(false);
            }
        }

        private class BackboneSpinner extends JSpinner implements ChangeListener {

            private BackboneSpinner() {
                this.setModel(new SpinnerNumberModel(0,Integer.MIN_VALUE,Integer.MAX_VALUE,1));
                this.addChangeListener(this);
                this.setMinimumSize(new Dimension(50, 20));
                this.setPreferredSize(new Dimension(50, 20));
                this.setMaximumSize(new Dimension(50, 20));
            }

            public void stateChanged(ChangeEvent event) {
                int steps = (Integer) this.getModel().getValue();
                if (steps >= 0 && getSecondaryCanvas() != null) {
                    for (Residue2D r2D: getSecondaryCanvas().getCurrentResidues()) {
                        int _steps = steps;
                        while (_steps > 0 && r2D.getNextResidue() != null && !r2D.getNextResidue().isSelected()) {
                            r2D = r2D.getNextResidue();
                            r2D.isDisplayed(true);
                            _steps--;
                        }
                        r2D = r2D.getNextResidue();
                        //we hide the next residues until the next hidden or selected residues
                        while ( r2D != null && r2D.isDisplayed() && !r2D.isSelected()) {
                            r2D.isDisplayed(false);
                            r2D = r2D.getNextResidue();
                        }
                    }
                    getSecondaryCanvas().repaint();
                }
                if (steps <= 0 && getSecondaryCanvas() != null) {
                    for (Residue2D r2D: getSecondaryCanvas().getCurrentResidues()) {
                        int _steps = steps;
                        while (_steps < 0 && r2D.getPreviousResidue() != null && !r2D.getPreviousResidue().isSelected()) {
                            r2D = r2D.getPreviousResidue();
                            r2D.isDisplayed(true);
                            _steps++;
                        }
                        r2D = r2D.getPreviousResidue();
                        //we hide the next residues until the next hidden or selected residues
                        while ( r2D != null && r2D.isDisplayed() && !r2D.isSelected()) {
                            r2D.isDisplayed(false);
                            r2D = r2D.getPreviousResidue();
                        }
                    }
                    getSecondaryCanvas().repaint();
                }
            }
        }

        private class BackboneButton extends javax.swing.JButton implements java.awt.event.ActionListener, MouseListener {

            private BackboneButton() {
                super(null, new ImageIcon(RessourcesUtils.getImage("backbone.png")));
                this.setMargin(new Insets(0, 0, 0, 0));
                this.setBackground(Color.WHITE);
                this.setBorderPainted(false);
                this.addActionListener(this);
                this.addMouseListener(this);
                this.setToolTipText("Follow the backbone");
            }

            public void actionPerformed(final ActionEvent e) {
                int steps = (Integer) backboneSpinner.getModel().getValue();
                if (steps >= 0 && getSecondaryCanvas() != null) {
                    for (Residue2D r2D: getSecondaryCanvas().getCurrentResidues()) {
                        int _steps = steps;
                        while (_steps > 0 && r2D.getNextResidue() != null && !r2D.getNextResidue().isSelected()) {
                            r2D = r2D.getNextResidue();
                            r2D.isDisplayed(true);
                            _steps--;
                        }
                        r2D = r2D.getNextResidue();
                        //we hide the next residues until the next hidden or selected residues
                        while ( r2D != null && r2D.isDisplayed() && !r2D.isSelected() ) {
                            r2D.isDisplayed(false);
                            r2D = r2D.getNextResidue();
                        }
                    }
                    getSecondaryCanvas().repaint();
                }
                if (steps <= 0 && getSecondaryCanvas() != null) {
                    for (Residue2D r2D: getSecondaryCanvas().getCurrentResidues()) {
                        int _steps = steps;
                        while (_steps < 0 && r2D.getPreviousResidue() != null && !r2D.getPreviousResidue().isSelected()) {
                            r2D = r2D.getPreviousResidue();
                            r2D.isDisplayed(true);
                            _steps++;
                        }
                        r2D = r2D.getPreviousResidue();
                        //we hide the next residues until the next hidden or selected residues
                        while ( r2D != null && r2D.isDisplayed() && !r2D.isSelected()) {
                            r2D.isDisplayed(false);
                            r2D = r2D.getPreviousResidue();
                        }
                    }
                    getSecondaryCanvas().repaint();
                }
            }

            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                this.setBackground(Color.lightGray);
                this.setBorderPainted(true);
            }

            public void mouseExited(MouseEvent e) {
                this.setBackground(Color.white);
                this.setBorderPainted(false);
            }
        }
    }

    class SelectButton extends javax.swing.JButton implements java.awt.event.ActionListener, MouseListener {

        private SelectButton(final String name, final javax.swing.Icon image) {
            super(name, image);
            this.setMargin(new Insets(0, 0, 0, 0));
            this.setBorderPainted(false);
            this.setBackground(Color.white);
            this.addActionListener(this);
            this.addMouseListener(this);
        }

        public void actionPerformed(final java.awt.event.ActionEvent e) {
            if (lastClicked != null) {
                lastClicked.setEnabled(true);
            }
            lastClicked = this;
            if (getSecondaryCanvas().getModel2D() != null) {
                getSecondaryCanvas().setState(mediator.getSelectionMode());
            }
            Rna2DViewer.this.setCursorForCanvas(Rna2DViewer.selectCursor);
            Rna2DViewer.this.currentCursor = Rna2DViewer.selectCursor;
        }

        public void mouseClicked(MouseEvent e) {
        }

        public void mousePressed(MouseEvent e) {
        }

        public void mouseReleased(MouseEvent e) {
        }

        public void mouseEntered(MouseEvent e) {
            this.setBackground(Color.lightGray);
            this.setBorderPainted(true);
        }

        public void mouseExited(MouseEvent e) {
            this.setBackground(Color.white);
            this.setBorderPainted(false);
        }
    }

    private class ColorChooser extends JFrame {

        private JColorChooser jcc = new JColorChooser();

        ColorChooser(ChangeListener listener) {
            super("Color Chooser");
            this.getContentPane().add(jcc);
            jcc.getSelectionModel().addChangeListener(listener);
            this.pack();
            GeneralUtils.centerOnScreen(this);
        }

        public void toForeground() {
            this.setVisible(true);
            this.toFront();
        }

        public Color getColor() {
            return jcc.getColor();
        }
    }

    private class Rna2DMenu extends javax.swing.JMenuBar implements java.awt.event.ActionListener {

        final static int FILE_MENU = 0;
        final static int CLOSE_MENU = 1;
        final static int ABOUT_MENU = 2;
        private final static int EDIT_MENU = 3;
        private final static int TOOLS_MENU = 6;
        private final static int SELECTION_MENU = 7;
        private JMenu[] jmenus;
        private JMenuItem itemFile;

        /**
         * Constructor for the Rna2DMenu object
         */
        Rna2DMenu() {

            jmenus = new JMenu[11];

            //File Menu

            jmenus[FILE_MENU] = new JMenu("File");

            /*
            itemFile = new JMenuItem("Load a molecule");
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    java.util.List<Molecule> molecules = workingSession.getMolecules();
                    Collections.sort(molecules, new Comparator() {
                        public int compare(Object o1, Object o2) {
                            return o1.toString().compareTo(o2.toString());
                        }
                    });
                    if (molecules.size() == 0)
                        JOptionPane.showMessageDialog(Rna2DViewer.this,"No molecule available", "Warning", JOptionPane.WARNING_MESSAGE);
                    else {
                        Molecule m = (Molecule)JOptionPane.showInputDialog(Rna2DViewer.this "Please choose a molecule" , "Load a molecule",JOptionPane.PLAIN_MESSAGE,null,molecules.toArray(new Molecule[]{}),molecules.get(0));
                        if (m!=null) {
                            StructuralFeatureFactory f = Rna2DViewer.this.createStructuralFeatureFactory();
                            RNASecondaryStructure ss = f.createRNASecondaryStructure(new ParadiseID(), "??", Source.getGraphicalToolAsSource(Rna2DViewer.this.getDescription()), m , new Location(1,m.getLength()));
                            f.createSingleStrand("SS1",ss,m,new Location(1,m.getLength()));
                            SecondaryStructureDisplay display = f.createSecondaryStructureDisplay(ss,m, new Location(1, m.getLength()));
                            for (int i=1; i<=m.getLength();i++)
                                f.createResidue2D(display,0f,0f,m,i);
                            Rna2DViewer.this.loadRNASecondaryStructure(ss);
                        }
                    }
                }
            });
            jmenus[FILE_MENU].add(itemFile);

            itemFile = new JMenuItem("Fold a molecule");
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    java.util.List<RNA> rnas = workingSession.getRNAs();
                    Collections.sort(rnas, new Comparator() {
                        public int compare(Object o1, Object o2) {
                            return o1.toString().compareTo(o2.toString());
                        }
                    });
                    if (rnas.size() == 0)
                        JOptionPane.showMessageDialog(Rna2DViewer.this.window,"No molecule available", "Warning", JOptionPane.WARNING_MESSAGE);
                    else {
                        RNA rna = (RNA)JOptionPane.showInputDialog(Rna2DViewer.this.window, "Please choose a molecule" , "Fold a molecule",JOptionPane.PLAIN_MESSAGE,null,rnas.toArray(new Molecule[]{}),rnas.get(0));
                        if (rna!=null)
                            Rna2DViewer.this.foldMolecule(rna);
                    }
                }
            });
            jmenus[FILE_MENU].add(itemFile);

            itemFile = new JMenuItem("Load Tertiary Structure");
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    java.util.List<TertiaryStructure> allTertiaryStructures = new ArrayList<TertiaryStructure>();
                    for (Molecule m:workingSession.getMolecules())
                        for (Molecule.Annotation a :m.getAnnotations(TertiaryStructure.class)) {
                            if (!allTertiaryStructures.contains(a.getFeature()))
                                allTertiaryStructures.add((TertiaryStructure)a.getFeature());
                        }
                    Collections.sort(allTertiaryStructures, new Comparator() {
                        public int compare(Object o1, Object o2) {
                            return o1.toString().compareTo(o2.toString());
                        }
                    });
                    if (allTertiaryStructures.size() == 0)
                        JOptionPane.showMessageDialog(Rna2DViewer.this,"No Tertiary Structure available", "Warning", JOptionPane.WARNING_MESSAGE);
                    else {
                        TertiaryStructure ts = (TertiaryStructure)JOptionPane.showInputDialog(Rna2DViewer.this, "Please choose a tertiary structure" , "Load a tertiary structure",JOptionPane.PLAIN_MESSAGE,null,allTertiaryStructures.toArray(new TertiaryStructure[]{}),allTertiaryStructures.get(0));
                        if (ts!=null)
                            Rna2DViewer.this.loadTertiaryStructure(ts);
                    }
                }
            });
            jmenus[FILE_MENU].add(itemFile);

            itemFile = new JMenuItem("Load Secondary Structure");
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    java.util.List<RNASecondaryStructure> allSecondaryStructures = new ArrayList<RNASecondaryStructure>();
                    for (Molecule m:workingSession.getMolecules())
                        for (Molecule.Annotation a:m.getAnnotations(RNASecondaryStructure.class))
                            if (!allSecondaryStructures.contains(a.getFeature()))
                                allSecondaryStructures.add((RNASecondaryStructure)a.getFeature());
                    Collections.sort(allSecondaryStructures, new Comparator() {
                        public int compare(Object o1, Object o2) {
                            return (o1.toString()).compareTo(o2.toString());
                        }
                    });
                    if (allSecondaryStructures.size() == 0)
                        JOptionPane.showMessageDialog(Rna2DViewer.this,"No Secondary Structure available", "Warning", JOptionPane.WARNING_MESSAGE);
                    else {
                        RNASecondaryStructure ss = (RNASecondaryStructure)JOptionPane.showInputDialog(Rna2DViewer.this.window, "Please choose a secondary structure" , "Load a secondary structure",JOptionPane.PLAIN_MESSAGE,null,allSecondaryStructures.toArray(new RNASecondaryStructure[]{}),allSecondaryStructures.get(0));
                        if (ss!=null && !Rna2DViewer.this.isAlreadyOpened(ss))
                            Rna2DViewer.this.loadRNASecondaryStructure(ss);
                    }
                }
            });
            jmenus[FILE_MENU].add(itemFile);

            itemFile = new JMenuItem("Generate new Tertiary Structure") {
                public void paintComponent(Graphics graphics) {
                    this.setEnabled(mediator.getCurrentModel2D() != null);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    SecondaryStructure ss = mediator.getCurrentModel2D().getSecondaryStructure();
                    Collection<Molecule> molecules = ss.getMolecules();
                    ArrayList<Molecule> selectedMolecules = new ArrayList<Molecule>();
                    for(Molecule molecule : molecules) {
                        molecule.removeAllSelectedFeatures();
                        molecule.addSelectedFeature(ss);
                        selectedMolecules.add(molecule);
                    }
                    GuiEvent ev = new GuiEvent(this, Rna2DViewer.TERTIARY_STRUCTURE_PREDICTION);
                    ev.addParameter(selectedMolecules);
                    ev.addParameter(new ArrayList<Parameter>());
                    ((AbstractParadiseGUIToolAgent) agent).postGuiEvent(ev);
                }
            });
            jmenus[FILE_MENU].add(itemFile);

            itemFile = new JMenuItem("Save") {
                public void paintComponent(Graphics graphics) {
                    //enabled if the current feature is already assoiated to a PARADISE project file
                    this.setEnabled(getCurrentFeature()!= null && getCurrentFeature().getAssociatedFile() != null);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    Rna2DViewer.this.save(workingSession.getCurrentParadiseProject());
                }
            });
            jmenus[FILE_MENU].add(itemFile);

            itemFile = new JMenuItem("Save As") {
                public void paintComponent(Graphics graphics) {
                    this.setEnabled(getCurrentFeature()!= null);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    Rna2DViewer.this.saveAs(workingSession.getCurrentParadiseProject());
                }
            });
            jmenus[FILE_MENU].add(itemFile);

            itemFile = new JMenuItem("Load layout");
            itemFile.setActionCommand("loadLayout");
            itemFile.addActionListener(this);
            //jmenus[FILE_MENU].add(itemFile);

            itemFile = new JMenuItem("Save layout");
            itemFile.setActionCommand("saveLayout");
            itemFile.addActionListener(this);
            //jmenus[FILE_MENU].add(itemFile);

            jmenus[FILE_MENU].setEnabled(true);

            this.add(jmenus[FILE_MENU]);

            jmenus[SELECTION_MENU] = new JMenu("Selection");

            itemFile = new JMenuItem("Create new Molecules") {
                public void paintComponent(Graphics graphics) {
                    this.setEnabled(mediator.getCurrentModel2D() != null && mediator.getCurrentCanvas().getCurrentResidues().size() > 0);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    StringBuffer newSequence = null;
                    int previousPos = -1;
                    ParadiseID previousID = null;
                    Residue2D r = null;
                    List<Residue2D> residues = mediator.getCurrentCanvas().getCurrentResidues();
                    for (int i=0;i<residues.size();i++) {
                        r = residues.get(i);
                        if (r.getAbsolutePosition()-1 != previousPos || !r.getMoleculeId().equals(previousID)) {
                            if (newSequence != null)
                                try {
                                    mediator.getRna2DViewer().getWorkingSession().createRNA(new ParadiseID(),mediator.getRna2DViewer().getWorkingSession().getMolecule(r.getMoleculeId()).getName(),Source.getGraphicalToolAsSource(Rna2DViewer.this.getDescription()), newSequence.toString());
                                } catch (BiologicalSymbolException e1) {
                                    e1.printStackTrace();
                                }
                            newSequence = new StringBuffer();
                        }
                        newSequence.append(mediator.getRna2DViewer().getWorkingSession().getMolecule(r.getMoleculeId()).getResidueAt(r.getAbsolutePosition()));
                        previousPos = r.getAbsolutePosition();
                        previousID = r.getMoleculeId();
                    }
                    if (newSequence != null)
                        try {
                            mediator.getRna2DViewer().getWorkingSession().createRNA(new ParadiseID(),mediator.getRna2DViewer().getWorkingSession().getMolecule(r.getMoleculeId()).getName(),Source.getGraphicalToolAsSource(Rna2DViewer.this.getDescription()), newSequence.toString());
                        } catch (BiologicalSymbolException e1) {
                            e1.printStackTrace();
                        }
                }
            });
            jmenus[SELECTION_MENU].add(itemFile);

            itemFile = new JMenuItem("Export as PDB file") {
                public void paintComponent(Graphics graphics) {
                    this.setEnabled(mediator.getCurrentModel2D() != null  && mediator.getCurrentCanvas().getCurrentResidues().size() > 0 && mediator.getCurrentModel2D().getSecondaryStructure().getParentFeatures(TertiaryStructure.class).size() > 0);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    TertiaryStructure ts = Rna2DViewer.this.getCurrentCanvas().getModel2D().getSecondaryStructure().getParentFeatures(TertiaryStructure.class).get(0);
                    final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                    fileChooser.setFileHidingEnabled(true);
                    if (fileChooser.showSaveDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                        final java.io.File file = fileChooser.getSelectedFile();
                        try {
                            if (!file.exists())
                                file.createNewFile();
                            List<Residue3D> residues3D = new ArrayList<Residue3D>();
                            RESIDUES2D: for (Residue2D r:Rna2DViewer.this.getCurrentCanvas().getCurrentResidues()) {
                                fr.unistra.ibmc.paradise.core.features.Residue2D _r = r.getResidue2DFeature();
                                Molecule m = _r.getMolecules().iterator().next();
                                Location l = _r.getLocation(m);
                                for (Residue3D __r:ts.getResidues3D()) {
                                    Location _l = __r.getLocation(m);
                                    if (_l != null && _l.equals(l)) {
                                        residues3D.add(__r);
                                        continue RESIDUES2D;
                                    }
                                }
                            }
                            PDBFileIO.writePDBFile(residues3D,true,file);
                        }
                        catch (FileNotFoundException ex) {
                            ex.printStackTrace();
                        }
                        catch (IOException ex) {
                            ex.printStackTrace();
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                }
            });
            jmenus[SELECTION_MENU].add(itemFile); */

            itemFile = new JMenuItem("Export as CT file") {
                public void paintComponent(Graphics graphics) {
                    this.setEnabled(mediator.getCurrentModel2D() != null && mediator.getSecondaryCanvas().getCurrentResidues().size() > 0);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(fr.unistra.ibmc.paradise.Paradise.getWorkingDirectory());
                    fileChooser.setFileHidingEnabled(true);
                    if (fileChooser.showSaveDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                        final java.io.File file = fileChooser.getSelectedFile();
                        try {
                            if (!file.exists())
                                file.createNewFile();
                            List<Residue> residues = new ArrayList<Residue>();
                            Set<BaseBaseInteraction>  interactions = new HashSet<BaseBaseInteraction>();
                            for (Residue2D r2D:mediator.getSecondaryCanvas().getCurrentResidues()) {
                                residues.add(r2D.getResidue2DFeature().getResidues().iterator().next());
                                for (Interaction2D interaction:r2D.getInteractions())
                                    if (BaseBaseInteraction2D.class.isInstance(interaction) && ((BaseBaseInteraction2D)interaction).getBaseBaseInteractionFeature().isSecondaryInteraction()) /*a ct file stores only secondary interactions, not tertiary ones)*/
                                        interactions.add(((BaseBaseInteraction2D)interaction).getBaseBaseInteractionFeature());
                            }
                            Collections.sort(residues, new Comparator() {

                                public int compare(Object o1, Object o2) {
                                    return ((Residue)o1).getAbsolutePosition() - ((Residue)o2).getAbsolutePosition();
                                }
                            });
                            CTFileIO.writeCTFile(residues,new ArrayList(interactions),file);
                        }
                        catch (FileNotFoundException ex) {
                            ex.printStackTrace();
                        }
                        catch (IOException ex) {
                            ex.printStackTrace();
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }

                    }
                }
            });
            //jmenus[SELECTION_MENU].add(itemFile);

            //this.add(jmenus[SELECTION_MENU]);

            // 2D Menu
            jmenus[EDIT_MENU] = new JMenu("Edit");


            /*
            itemFile = new JMenuItem("Display all") {
                public void paintComponent(Graphics graphics) {
                    this.setEnabled(mediator.getCurrentModel2D() != null);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    mediator.getCurrentCanvas().showAll(true);
                }
            });
            jmenus[EDIT_MENU].add(itemFile);

            itemFile = new JMenuItem("Hide all") {
                public void paintComponent(Graphics graphics) {
                    this.setEnabled(mediator.getCurrentModel2D() != null);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    mediator.getCurrentCanvas().showAll(false);
                }
            });
            jmenus[EDIT_MENU].add(itemFile);*/

            itemFile = new JMenuItem("Reorganize helices") {
                public void paintComponent(Graphics graphics) {
                    this.setEnabled(mediator.getCurrentModel2D() != null);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    Rna2DViewer.this.reorganizeHelices();
                }
            });
            jmenus[EDIT_MENU].add(itemFile);

            //jmenus[EDIT_MENU].add(new JSeparator());

            itemFile = new JMenuItem("Hide unselected residues") {
                public void paintComponent(Graphics graphics) {
                    this.setEnabled(mediator.getCurrentModel2D() != null);
                    super.paintComponent(graphics);
                }
            };
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    for (Residue2D r2D:mediator.getCurrentModel2D().getResidues())
                        if (!r2D.isSelected())
                            r2D.isDisplayed(false);
                    mediator.getSecondaryCanvas().repaint();
                }
            });
            /*
            jmenus[EDIT_MENU].add(itemFile);

            jmenus[EDIT_MENU].add(new JSeparator());

            itemFile = new JMenuItem("Display tertiary partners for selected residues");
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    if (mediator.getCurrentCanvas() != null) {
                        mediator.getCurrentCanvas().showPeers();
                    }
                }
            });
            jmenus[EDIT_MENU].add(itemFile);

            itemFile = new JMenuItem("Display Helix/Sstrand containing selected residues");
            itemFile.addActionListener(new ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {
                    if (mediator.getCurrentCanvas() != null) {
                        for (Residue2D r:mediator.getCurrentCanvas().getCurrentResidues())
                            if (r.getStructuralDomain() != null)
                                r.getStructuralDomain().isDisplayed(true);
                        mediator.getCurrentCanvas().repaint();
                    }
                }
            });
            jmenus[EDIT_MENU].add(itemFile);*/

            this.add(jmenus[EDIT_MENU]);

        }

        /**
         * Description of the Method
         *
         * @param actionEvent Description of the Parameter
         */
        public void actionPerformed(final java.awt.event.ActionEvent actionEvent) {

            if (actionEvent.getActionCommand().equals("loadLayout")) {
                /*
                SecondaryCanvas c=mediator.getCurrentCanvas();
                if (c != null) {
                    final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                    fileChooser.setFileHidingEnabled(true);
                    fileChooser.setFileFilter(fr.unistra.ibmc.s2s.rna2d.utils.IoUtils.getS2SLayoutFilter());
                    if (fileChooser.showOpenDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
                        final java.io.File file = fileChooser.getSelectedFile();
                        Paradise.setWorkingDirectory(file.getAbsolutePath());
                        final SAXBuilder builder = new SAXBuilder(false);
                        try {
                            Element root = builder.build(file).getRootElement();
                            Element graphicContext = root.getChild("GraphicContext");
                            mediator.getCurrentCanvas().setGraphicContext(Double.parseDouble(graphicContext.getChild("width").getText().trim()), Double.parseDouble(graphicContext.getChild("height").getText().trim()));
                            String[] tokens = null;
                            if (graphicContext.getChild("zoom")!=null)
                                mediator.getCurrentCanvas().gc.finalZoomLevel = Double.parseDouble(graphicContext.getChild("zoom").getText().trim());
                            if (graphicContext.getChild("view")!=null) {
                                tokens = graphicContext.getChild("view").getText().trim().split("\\s");
                                mediator.getCurrentCanvas().gc.viewX = Double.parseDouble(tokens[0]);
                                mediator.getCurrentCanvas().gc.viewY = Double.parseDouble(tokens[1]);
                            }
                            for (Object h : root.getChildren("Helix")) {
                                for (Rna2DElement element :c.getSecondaryElementByLabel(((Element)h).getText().trim())) {
                                    if (((Element)h).getChild("Color") != null) {
                                        tokens=((Element)h).getChild("Color").getText().trim().split("\\s");
                                        Color new_color=new Color(Integer.parseInt(tokens[0]),
                                                Integer.parseInt(tokens[1]),
                                                Integer.parseInt(tokens[2]));
                                        element.setColor(new_color);
                                        List<Residue> residues=new ArrayList<Residue>();
                                        for (Residue2D r:element.getResidues())
                                            residues.add(r.getResidue2DFeature().getResidues().getAllResidues().iterator().next());
                                        //c.fireEvent(new S2SEvent(c, S2SEvent.NEW_COLOR, new Object[]{residues, new_color}));
                                    }
                                    List<Object> coordinates = ((Element)h).getChildren("Coordinates");
                                    for (int i=0 ; i<coordinates.size() ; i++) {
                                        tokens = ((Element)coordinates.get(0)).getText().trim().split("\\s");
                                        double _5_0_x = Double.parseDouble(tokens[0]), _5_0_y = Double.parseDouble(tokens[1]);
                                        tokens = ((Element)coordinates.get(1)).getText().trim().split("\\s");
                                        double _3_0_x = Double.parseDouble(tokens[0]), _3_0_y = Double.parseDouble(tokens[1]);
                                        tokens = ((Element)coordinates.get(2)).getText().trim().split("\\s");
                                        double _5_1_x = Double.parseDouble(tokens[0]), _5_1_y = Double.parseDouble(tokens[1]);
                                        tokens = ((Element)coordinates.get(3)).getText().trim().split("\\s");
                                        double _3_1_x = Double.parseDouble(tokens[0]), _3_1_y = Double.parseDouble(tokens[1]);
                                        double length1 = DrawingUtils.getSharedAncestorToRootDistance(_5_0_x, _5_0_y, _3_0_x, _3_0_y),
                                                length2 = DrawingUtils.getSharedAncestorToRootDistance(((Helix) element.getParadiseFeature()).get5PrimeEnds()[0].getCoordinates(),((Helix) element.getParadiseFeature()).get3PrimeEnds()[0].getCoordinates());
                                        double distance = (length1-length2)/2;
                                        Point2D[]   fittedPoints1 = DrawingUtils.fit(_5_0_x, _5_0_y, _3_0_x, _3_0_y, distance),
                                                fittedPoints2 = DrawingUtils.fit(_5_1_x, _5_1_y, _3_1_x, _3_1_y, distance);
                                        ((Helix2D) element).setEnds(fittedPoints1[0].getX(),fittedPoints1[0].getY(),
                                                fittedPoints2[0].getX(),fittedPoints2[0].getY(),
                                                fittedPoints1[1].getX(), fittedPoints1[1].getY(),
                                                fittedPoints2[1].getX(),fittedPoints2[1].getY());
                                    }
                                }
                            }
                            for (Object ss : root.getChildren("SStrand")) {
                                for (Rna2DElement element :c.getSecondaryElementByLabel(((Element)ss).getText().trim())) {
                                    if (((Element)ss).getChild("Color") != null) {
                                        tokens=((Element)ss).getChild("Color").getText().trim().split("\\s");
                                        Color new_color=new Color(Integer.parseInt(tokens[0]),
                                                Integer.parseInt(tokens[1]),
                                                Integer.parseInt(tokens[2]));
                                        element.setColor(new_color);
                                        List<Residue> residues=new ArrayList<Residue>();
                                        for (Residue2D r:element.getAllResidues())
                                            residues.add(r.getResidue2DFeature().getResidues().getAllResidues().iterator().next());
                                        //c.fireEvent(new S2SEvent(c, S2SEvent.NEW_COLOR, new Object[]{residues, new_color}));
                                    }
                                }
                            }

                            for (Object interaction : root.getChildren("Interaction")) {
                                Element residue5 = ((Element)interaction).getChild("residue5"),
                                        residue3 = ((Element)interaction).getChild("residue3");
                                for (Interaction2D interaction2D :c.getModel2D().getInteractions(new LSID(residue5.getAttributeValue("molecule-id")),Integer.parseInt(residue5.getAttributeValue("id")),new LSID(residue3.getAttributeValue("molecule-id")),Integer.parseInt(residue3.getAttributeValue("id")))) {
                                    if (((Element)interaction).getChild("isDisplayed") != null)
                                        interaction2D.isDisplayed(true);
                                    if (((Element)interaction).getChild("drawRealSymbol") != null)
                                        interaction2D.drawRealSymbol(true);
                                    if (((Element)interaction).getChild("Color") != null) {
                                        tokens=((Element)interaction).getChild("Color").getText().trim().split("\\s");
                                        Color new_color=new Color(Integer.parseInt(tokens[0]),
                                                Integer.parseInt(tokens[1]),
                                                Integer.parseInt(tokens[2]));
                                        interaction2D.setColor(new_color);
                                        List<Residue> residues=new ArrayList<Residue>();
                                        residues.add(interaction2D.getResidue().getResidue2DFeature().getResidues().getAllResidues().iterator().next());
                                        residues.add(interaction2D.getPartnerResidue().getResidue2DFeature().getResidues().getAllResidues().iterator().next());
                                        //c.fireEvent(new S2SEvent(c, S2SEvent.NEW_COLOR, new Object[]{residues, new_color}));
                                    }
                                }

                            }
                        } catch (JDOMException e) {
                            e.printStackTrace();
                            return;
                        } catch (IOException e) {
                            e.printStackTrace();
                            return;
                        }
                        c.repaint();
                    }
                }*/
            } else if (actionEvent.getActionCommand().equals("saveLayout")) {
                /*
                final javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(Paradise.getWorkingDirectory());
                fileChooser.setFileFilter(fr.unistra.ibmc.s2s.rna2d.utils.IoUtils.getS2SLayoutFilter());
                JPanel checkBoxes= new JPanel();
                checkBoxes.setLayout(new BoxLayout(checkBoxes,BoxLayout.Y_AXIS));
                JCheckBox saveCoordinates = new JCheckBox("Save coordinates");
                checkBoxes.add(saveCoordinates);
                JCheckBox saveView = new JCheckBox("Save view");
                checkBoxes.add(saveView);
                JCheckBox saveColors = new JCheckBox("Save colors");
                saveColors.setSelected(true);
                checkBoxes.add(saveColors);
                fileChooser.setAccessory(checkBoxes);
                if (fileChooser.showSaveDialog(this) == javax.swing.JFileChooser.APPROVE_OPTION) {
                    java.io.File file = null;
                    String extension = null;
                    java.io.PrintWriter pw = null;
                    Element rootElement = new Element("RNA2DViewer-Layout");
                    try {
                        file = fileChooser.getSelectedFile();
                        extension = fr.unistra.ibmc.s2s.rna2d.utils.IoUtils.getFileSuffix(file);
                        if (extension.equals("layout")) {
                            file.createNewFile();
                            Paradise.setWorkingDirectory(file.getAbsolutePath());
                            pw = new java.io.PrintWriter(new java.io.FileOutputStream(file));
                            Model2D model = mediator.getCurrentCanvas().getModel2D();
                            Document doc = new Document(rootElement);
                            Element graphicContext =  new Element("GraphicContext");
                            Element width = new Element("width");
                            width.setText(""+mediator.getCurrentCanvas().gc.getRealWidth());
                            graphicContext.addContent(width);
                            Element height = new Element("height");
                            height.setText(""+mediator.getCurrentCanvas().gc.getRealHeight());
                            graphicContext.addContent(height);
                            if (saveView.isSelected()) {
                                Element zoom = new Element("zoom");
                                zoom.setText(""+mediator.getCurrentCanvas().gc.getFinalZoomLevel());
                                graphicContext.addContent(zoom);
                                Element view = new Element("view");
                                view.setText(mediator.getCurrentCanvas().gc.getViewX()+" "+mediator.getCurrentCanvas().gc.getViewY());
                                graphicContext.addContent(view);
                            }
                            rootElement.addContent(graphicContext);
                            for (Helix2D h:model.getAllHelices())  {
                                Element hElement = null;
                                if (saveColors.isSelected() && h.getColor().getRGB() != Helix2D.DEFAULT_COLOR.getRGB()) {
                                    if (hElement == null) {
                                        hElement = new Element("Helix");
                                        hElement.setText(h.getLabel());
                                    }
                                    Element color = new Element("Color");
                                    color.setText(h.getColor().getRed()+" "+h.getColor().getGreen()+" "+h.getColor().getBlue());
                                    hElement.addContent(color);
                                }
                                if (saveCoordinates.isSelected()) {
                                    if (hElement == null) {
                                        hElement = new Element("Helix");
                                        hElement.setText(h.getLabel());
                                    }
                                    Element coordinates = new Element("Coordinates");
                                    coordinates.setText(h.get5PrimeEnds()[0].getRealX()+" "+h.get5PrimeEnds()[0].getRealY());
                                    hElement.addContent(coordinates);
                                    coordinates = new Element("Coordinates");
                                    coordinates.setText(h.get3PrimeEnds()[0].getRealX()+" "+h.get3PrimeEnds()[0].getRealY());
                                    hElement.addContent(coordinates);
                                    coordinates = new Element("Coordinates");
                                    coordinates.setText(h.get5PrimeEnds()[1].getRealX()+" "+h.get5PrimeEnds()[1].getRealY());
                                    hElement.addContent(coordinates);
                                    coordinates = new Element("Coordinates");
                                    coordinates.setText(h.get3PrimeEnds()[1].getRealX()+" "+h.get3PrimeEnds()[1].getRealY());
                                    hElement.addContent(coordinates);
                                    coordinates = new Element("Coordinates");
                                    Element length = new Element("Length");
                                    length.setText(""+h.getLength());
                                    hElement.addContent(length);
                                }
                                for (Sstrand2D ss:h.getSstrands()) {
                                    Element ssElement = null;
                                    if (saveColors.isSelected() && ss.getColor().getRGB() != Sstrand2D.DEFAULT_COLOR.getRGB()) {
                                        ssElement = new Element("SStrand");
                                        ssElement.setText(ss.getLabel());
                                        Element color = new Element("Color");
                                        color.setText(ss.getColor().getRed()+" "+ss.getColor().getGreen()+" "+ss.getColor().getBlue());
                                        ssElement.addContent(color);
                                    }
                                    if (ssElement != null)
                                        rootElement.addContent(ssElement);
                                }
                                if (hElement != null)
                                    rootElement.addContent(hElement);
                            }
                            for (Interaction2D interaction : model.getAllInteractions()) {
                                Element interactionElement = null;
                                //a secondary interaction is always displayed, so....
                                if (interaction.isDisplayed() && interaction.getParadiseFeature() instanceof TertiaryInteraction) {
                                    if (interactionElement == null)
                                        interactionElement = new Element("Interaction");
                                    interactionElement.addContent(new Element("isDisplayed"));
                                }
                                if (interaction.drawRealSymbol()) {
                                    if (interactionElement == null)
                                        interactionElement = new Element("Interaction");
                                    interactionElement.addContent(new Element("drawRealSymbol"));
                                }
                                if (saveColors.isSelected()) {
                                    if (interaction.getParadiseFeature() instanceof SecondaryInteraction && interaction.getColor().getRGB() != Interaction2D.DEFAULT_SECONDARY_INTERACTION_COLOR.getRGB() ||
                                            interaction.getParadiseFeature() instanceof TertiaryInteraction && interaction.getColor().getRGB() != Interaction2D.DEFAULT_TERTIARY_INTERACTION_COLOR.getRGB()) {
                                        Element color = new Element("Color");
                                        color.setText(interaction.getColor().getRed()+" "+interaction.getColor().getGreen()+" "+interaction.getColor().getBlue());
                                        if (interactionElement == null)
                                            interactionElement = new Element("Interaction");
                                        interactionElement.addContent(color);
                                    }
                                }
                                if (interactionElement !=null) {
                                    Element residue = new Element("residue5");
                                    residue.setAttribute("molecule-id",""+interaction.getResidue().getMoleculeId());
                                    residue.setAttribute("id",""+interaction.getResidue().getAbsolutePosition());
                                    interactionElement.addContent(residue);
                                    residue = new Element("residue3");
                                    residue.setAttribute("molecule-id",""+interaction.getPartnerResidue().getMoleculeId());
                                    residue.setAttribute("id",""+interaction.getPartnerResidue().getAbsolutePosition());
                                    interactionElement.addContent(residue);
                                    rootElement.addContent(interactionElement);
                                }
                            }
                            try {
                                XMLOutputter serializer = new XMLOutputter();
                                serializer.getFormat().setIndent(" ");
                                serializer.output(doc, pw);
                                pw.close();
                            }
                            catch (IOException e) {
                                System.err.println(e);
                            }
                            JOptionPane.showMessageDialog(Rna2DViewer.this.frame,"Saving successful");
                        } else {
                            System.out.println("Not supported file type");
                        }
                    }
                    catch (java.io.FileNotFoundException fnfe) {
                        System.out.println("I cannot find the file : " + file.getPath());
                        return;
                    }
                    catch (java.io.IOException ioe) {
                        System.out.println("Problem to save data in the file : " + file.getPath());
                        return;
                    }
                    finally {
                        if (pw != null)
                            pw.close();
                    }
                }*/
            } else if ("displayLabels".equals(actionEvent.getActionCommand())) {
                if (mediator.getSecondaryCanvas() != null) {
                    mediator.getSecondaryCanvas().setDisplayLabels(true);
                    mediator.getSecondaryCanvas().repaint();
                }
            } else if ("hideLabels".equals(actionEvent.getActionCommand())) {
                if (mediator.getSecondaryCanvas() != null) {
                    mediator.getSecondaryCanvas().setDisplayLabels(false);
                    mediator.getSecondaryCanvas().repaint();
                }
            } else if ("structurecol".equals(actionEvent.getActionCommand())) {
                if (mediator.getSecondaryCanvas() != null) {
                    mediator.getSecondaryCanvas().setColorMode(GraphicContext.STRUCTURECOLOR);
                    mediator.getSecondaryCanvas().repaint();
                }
            } else if ("intercol".equals(actionEvent.getActionCommand())) {
                if (mediator.getSecondaryCanvas() != null) {
                    mediator.getSecondaryCanvas().setColorMode(GraphicContext.INTERACTIONCOLOR);
                    mediator.getSecondaryCanvas().repaint();
                }
            } else if ("basecol".equals(actionEvent.getActionCommand())) {
                if (mediator.getSecondaryCanvas() != null) {
                    mediator.getSecondaryCanvas().setColorMode(GraphicContext.BASECOLOR);
                    mediator.getSecondaryCanvas().repaint();
                }
            } else if ("sequencecol".equals(actionEvent.getActionCommand())) {
                if (mediator.getSecondaryCanvas() != null) {
                    mediator.getSecondaryCanvas().setColorMode(GraphicContext.SEQUENCECOLOR);
                    mediator.getSecondaryCanvas().repaint();
                }
            }
        }

        void setEnableMenus(final boolean enable) {
            this.jmenus[FILE_MENU].setEnabled(enable);
            this.jmenus[EDIT_MENU].setEnabled(enable);
            this.jmenus[TOOLS_MENU].setEnabled(enable);
        }

        public void activate() {
            this.jmenus[FILE_MENU].setEnabled(true);
            this.jmenus[EDIT_MENU].setEnabled(true);
        }

    }

    public void residuesSelected(ParadiseTool source, Collection<Residue> residues) {
        if (source == this || !mediator.isLinked())
            return;
        //TODO seems to be called several times if some residues are selected => check how many times a Listener is registered
        //System.out.println("Residues Selected from Rna2DViewer");
        //for (int i = 0; i < this.getCanvasCount(); i++) {
        SecondaryCanvas canvas = this.getSecondaryCanvas();
        if (canvas != null && canvas.getModel2D() != null) {
            List<Residue2D> _residues = new ArrayList<Residue2D>();
            for (Rna2D rna2D : canvas.getModel2D().getRnas())
                for (Residue r : residues)
                    if (rna2D.getRnaMolecule().equals(r.getMolecule()))
                        for (Residue2D r2D : rna2D.getResidues())
                            if (r.getAbsolutePosition() == r2D.getAbsolutePosition())
                                _residues.add(r2D);
            currentSelectionMode.select(canvas,_residues);
        }
//}
    }

    private class InteractionSelector extends JDialog {
        private JButton ok = new JButton("Ok"), cancel = new JButton("Cancel");
        private JRadioButton    w1 = new JRadioButton("Watson Crick"),
                h1 = new JRadioButton("Hoogsteen"),
                s1 = new JRadioButton("Sugar Edge"),
                w2 = new JRadioButton("Watson Crick"),
                h2 = new JRadioButton("Hoogsteen"),
                s2 = new JRadioButton("Sugar Edge"),
                cis = new JRadioButton("Cis"),
                trans = new JRadioButton("Trans"),
                r1, r2;
        private Residue2D residue1, residue2;
        private Molecule m1, m2;
        private int pos1, pos2;
        private boolean generateTertiaryInteraction;

        private InteractionSelector(Residue2D residue1, Residue2D residue2) {
            this.residue1 = residue1;
            this.residue2 = residue2;
            this.pos1 = residue1.getAbsolutePosition();
            this.pos2 = residue2.getAbsolutePosition();
            this.m1 = residue1.getResidue2DFeature().getMolecules().iterator().next();
            this.m2 =  residue2.getResidue2DFeature().getMolecules().iterator().next();
            r1 = new JRadioButton(m1.getResidueAt(this.pos1)+" "+this.pos1+" ("+m1.getName()+")");
            r2 = new JRadioButton(m2.getResidueAt(this.pos2)+" "+this.pos2+" ("+m2.getName()+")");
            ButtonGroup be1 = new ButtonGroup();
            ButtonGroup be2 = new ButtonGroup();
            ButtonGroup bo = new ButtonGroup();
            ButtonGroup br = new ButtonGroup();
            be1.add(w1);
            be1.add(h1);
            be1.add(s1);

            be2.add(w2);
            be2.add(h2);
            be2.add(s2);

            bo.add(cis);
            bo.add(trans);

            br.add(r1);
            br.add(r2);

            this.getContentPane().setLayout(new BoxLayout(this.getContentPane(), BoxLayout.PAGE_AXIS));
            JPanel ed1 = new JPanel();
            ed1.setLayout(new BoxLayout(ed1, BoxLayout.LINE_AXIS));
            JPanel ed2 = new JPanel();
            ed2.setLayout(new BoxLayout(ed2, BoxLayout.LINE_AXIS));
            JPanel or = new JPanel();
            or.setLayout(new BoxLayout(or, BoxLayout.LINE_AXIS));
            JPanel re = new JPanel();
            re.setLayout(new BoxLayout(re, BoxLayout.LINE_AXIS));
            JPanel but = new JPanel();
            but.setLayout(new BoxLayout(but, BoxLayout.LINE_AXIS));

            ed1.add(new JLabel("Edge for "+m1.getResidueAt(this.pos1) + " " +this.pos1));
            ed1.add(w1);
            ed1.add(h1);
            ed1.add(s1);

            ed2.add(new JLabel("Edge for "+m2.getResidueAt(this.pos2) + " " +this.pos2));
            ed2.add(w2);
            ed2.add(h2);
            ed2.add(s2);

            or.add(new JLabel("Orientation of the Interaction"));
            or.add(cis);
            or.add(trans);

            re.add(new JLabel("Dominant Residue (if applicable)"));
            re.add(r1);
            re.add(r2);

            but.add(ok);
            but.add(cancel);

            this.setTitle("Describe the Interaction");
            this.getContentPane().add(ed1);
            this.getContentPane().add(ed2);
            this.getContentPane().add(or);
            this.getContentPane().add(re);
            this.getContentPane().add(but);

            cancel.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    InteractionSelector.this.dispose();
                }
            });

            this.pack();
            this.setResizable(false);
            GeneralUtils.centerOnScreen(this);
            this.setVisible(true);
        }

        public InteractionSelector(final BaseBaseInteraction2D interaction) {
            this(interaction.getResidue(), interaction.getPartnerResidue());
            switch (interaction.getBaseBaseInteractionFeature().getOrientation()) {
                case BaseBaseInteraction.ORIENTATION_CIS : cis.setSelected(true); break;
                case BaseBaseInteraction.ORIENTATION_TRANS : trans.setSelected(true); break;
            }
            switch (interaction.getBaseBaseInteractionFeature().getEdge1()) {
                case BaseBaseInteraction.EDGE_WATSON_CRICK: w1.setSelected(true);break;
                case BaseBaseInteraction.EDGE_HOOGSTEEN: h1.setSelected(true);break;
                case BaseBaseInteraction.EDGE_SUGAR_EDGE: s1.setSelected(true);break;
            }
            switch (interaction.getBaseBaseInteractionFeature().getEdge2()) {
                case BaseBaseInteraction.EDGE_WATSON_CRICK: w2.setSelected(true);break;
                case BaseBaseInteraction.EDGE_HOOGSTEEN: h2.setSelected(true);break;
                case BaseBaseInteraction.EDGE_SUGAR_EDGE: s2.setSelected(true);break;
            }
            this.ok.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    char    edge1 = BaseBaseInteraction.EDGE_WATSON_CRICK,
                            edge2 = BaseBaseInteraction.EDGE_WATSON_CRICK,
                            orientation = BaseBaseInteraction.ORIENTATION_CIS;
                    int dominant = BaseBaseInteraction.RESIDUE_1;
                    if(h1.isSelected())
                        edge1 = BaseBaseInteraction.EDGE_HOOGSTEEN;
                    if(s1.isSelected())
                        edge1 = BaseBaseInteraction.EDGE_SUGAR_EDGE;
                    if(h2.isSelected())
                        edge2 = BaseBaseInteraction.EDGE_HOOGSTEEN;
                    if(s2.isSelected())
                        edge2 = BaseBaseInteraction.EDGE_SUGAR_EDGE;
                    if(trans.isSelected())
                        orientation = BaseBaseInteraction.ORIENTATION_TRANS;
                    if(r2.isSelected())
                        dominant =BaseBaseInteraction.RESIDUE_2;
                    mediator.getCurrentModel2D().modifyInteractionType(interaction, residue1, edge1, residue2, edge2, orientation, dominant);
                    mediator.getSecondaryCanvas().repaint();
                    InteractionSelector.this.dispose();
                    mediator.getSecondaryCanvas().clearAllSelections();
                }
            });

        }

        public InteractionSelector(final Residue2D residue1, final Residue2D residue2, final boolean generateTertiaryInteraction) {
            this(residue1, residue2);
            w1.setSelected(true);
            w2.setSelected(true);
            cis.setSelected(true);
            r1.setSelected(true);
            this.generateTertiaryInteraction = generateTertiaryInteraction;
            this.ok.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if(e.getSource() == ok) {
                        char    edge1 = BaseBaseInteraction.EDGE_WATSON_CRICK,
                                edge2 = BaseBaseInteraction.EDGE_WATSON_CRICK,
                                orientation = BaseBaseInteraction.ORIENTATION_CIS,
                                dominant = BaseBaseInteraction.RESIDUE_1;

                        if(h1.isSelected())
                            edge1 = BaseBaseInteraction.EDGE_HOOGSTEEN;
                        if(s1.isSelected())
                            edge1 = BaseBaseInteraction.EDGE_SUGAR_EDGE;
                        if(h2.isSelected())
                            edge2 = BaseBaseInteraction.EDGE_HOOGSTEEN;
                        if(s2.isSelected())
                            edge2 = BaseBaseInteraction.EDGE_SUGAR_EDGE;
                        if(trans.isSelected())
                            orientation = BaseBaseInteraction.ORIENTATION_TRANS;
                        if(r2.isSelected())
                            dominant =BaseBaseInteraction.RESIDUE_2;
                        mediator.getCurrentModel2D().createTertiaryInteraction(residue1, edge1, residue2, edge2, orientation, dominant);
                        mediator.getSecondaryCanvas().repaint();
                        InteractionSelector.this.dispose();
                        mediator.getSecondaryCanvas().clearAllSelections();
                    }
                }
            });
        }


    }

}
