package fr.unistra.ibmc.s2s.ssviewer.graphics;

import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.s2s.ssviewer.features.*;
import fr.unistra.ibmc.s2s.Mediator;

import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;

/**
 * @author fjossinet
 */
public class StructSelect extends SelectState {

    /**
     * where to store temporary the structures selected with the rectangle selection
     */
    private List<StructuralDomain2D> structuresSelected;

    StructSelect(Mediator mediator) {
        super(mediator);
        this.structuresSelected = new ArrayList<StructuralDomain2D>();
    }

    void mouseClicked(final java.awt.event.MouseEvent e) {
        super.mouseClicked(e);
        StructuralDomain2D structSelect = null;
        final Model2D model2D = this.mediator.getSecondaryCanvas().getModel2D();
        if (model2D != null && e.getClickCount() == 1) {
            HELICES : for (Helix2D h : model2D.getInterMolecularHelices()) {
                if (h.isDrawn() && h.contains(e.getX(), e.getY())) {
                    structSelect = h;
                    break;
                }
            }
            RNAS : for (Rna2D rna:model2D.getRnas()) {
                for (Helix2D h : rna.getHelices()) {
                    if (h.isDrawn() && h.contains(e.getX(), e.getY())) {
                        structSelect = h;
                        break RNAS;
                    }

                }
                for (Sstrand2D ss:rna.getStrands())
                    if (ss.isDrawn() && ss.contains(e.getX(), e.getY())) {
                        structSelect = ss;
                        break RNAS;
                    }
            }
            if (structSelect != null) {
                if (e.isShiftDown())
                    this.mediator.getSecondaryCanvas().addShiftCurrentStructure(structSelect);
                else
                    this.mediator.getSecondaryCanvas().addCurrentStructure(structSelect);
                this.mediator.getRna2dViewer().fireStructuralDomainSelected(structSelect.getStructuralDomain(),e.isShiftDown());
                this.mediator.getSecondaryCanvas().updateGravityPoint();
            }
        }

    }

    void mouseDragged(final java.awt.event.MouseEvent e) {
        if (e.isShiftDown()) {
            Rectangle2D r = new Rectangle2D.Double(Math.min(this.corner.getX(),e.getX()), Math.min(this.corner.getY(),e.getY()), Math.abs(e.getX()-this.corner.getX()),Math.abs(e.getY()-this.corner.getY())),
                    previous = this.mediator.getSecondaryCanvas().selectionRectangle;
            this.mediator.getSecondaryCanvas().setSelectionRectangle(r);
            final Model2D model2D = this.mediator.getSecondaryCanvas().getModel2D();
            for (StructuralDomain2D s:this.structuresSelected)
                s.isSelected(false);
            this.structuresSelected.clear();
            if (model2D != null) {
                for (Helix2D h : model2D.getInterMolecularHelices()) {
                    if (h.isDrawn() && this.mediator.getSecondaryCanvas().selectionRectangle.contains(h.getArea().getBounds2D())) {
                        this.structuresSelected.add(h);
                        h.isSelected(true);
                    }
                }
                for (Rna2D rna:model2D.getRnas()) {
                    for (Helix2D h:rna.getHelices()) {
                        if (h.isDrawn() && this.mediator.getSecondaryCanvas().selectionRectangle.contains(h.getArea().getBounds2D())) {
                            this.structuresSelected.add(h);
                            h.isSelected(true);
                        }
                    }
                    for (Sstrand2D ss : rna.getStrands())
                        if (ss.isDrawn() && this.mediator.getSecondaryCanvas().selectionRectangle.contains(ss.getArea().getBounds2D())) {
                            this.structuresSelected.add(ss);
                            ss.isSelected(true);
                        }
                }
            }
            /*
            if (previous !=null)
                this.mediator.getCurrentCanvas().repaint(previous.getBounds());
            this.mediator.getCurrentCanvas().repaint(this.mediator.getCurrentCanvas().selectionRectangle.getBounds());*/
            this.mediator.getSecondaryCanvas().repaint();
        }
        else {
            this.mediator.getSecondaryCanvas().updateGravityPoint();
            if (this.mediator.getSecondaryCanvas().getGravityPoint() !=null) {
                final double trX = e.getX() - this.mediator.getSecondaryCanvas().getGravityPoint().getX();
                final double trY = e.getY() - this.mediator.getSecondaryCanvas().getGravityPoint().getY();
                this.mediator.getSecondaryCanvas().translateSelection(trX, trY);
            }
        }
    }

    void mouseReleased(final java.awt.event.MouseEvent e) {
        super.mouseReleased(e);
        if (this.structuresSelected != null && this.structuresSelected.size()>0) {
            List<StructuralDomain2D> structures=new ArrayList<StructuralDomain2D>();
            if (e.isShiftDown()) {
                //we add the structures already selected and stored
                for (StructuralDomain2D s:this.mediator.getSecondaryCanvas().getCurrentStructures())
                    structures.add(s);
                //we register the new ones
                for (StructuralDomain2D s:this.structuresSelected) {
                    this.mediator.getSecondaryCanvas().addShiftCurrentStructure(s);
                    structures.add(s);
                }
            }
            else {
                this.mediator.getSecondaryCanvas().addCurrentStructure(this.structuresSelected.get(0));
                structures.add(this.structuresSelected.get(0));
                for (int i=1;i<this.structuresSelected.size();i++) {
                    this.mediator.getSecondaryCanvas().addShiftCurrentStructure(this.structuresSelected.get(i));
                    structures.add(this.structuresSelected.get(i));
                }
            }
            this.structuresSelected.clear();
        }
    }

    Rna2DElement getWrapper(Residue residue) {
        if (this.mediator.getSecondaryCanvas() != null) {
            Model2D model2D = this.mediator.getSecondaryCanvas().getModel2D();
            for (Rna2D rna:model2D.getRnas()) {
                if (residue.getMoleculeId().equals(rna.getRnaMolecule().getParadiseID())) {
                    for (Helix2D h : rna.getHelices()) {
                        if ((residue.getAbsolutePosition() >= h.get5PrimeEnds()[0].getAbsolutePosition() && residue.getAbsolutePosition() < h.get5PrimeEnds()[0].getAbsolutePosition() + h.getLength())
                                || (residue.getAbsolutePosition() >= h.get5PrimeEnds()[1].getAbsolutePosition() && residue.getAbsolutePosition() < h.get5PrimeEnds()[1].getAbsolutePosition() + h.getLength()))
                            return h;

                    }
                    for (Sstrand2D ss :rna.getStrands())
                        if (residue.getAbsolutePosition() >= ss.getBase5().getAbsolutePosition() && residue.getAbsolutePosition() <= ss.getBase3().getAbsolutePosition())
                            return ss;
                }
            }
        }
        return null;
    }

    public void select(SecondaryCanvas canvas, List<Residue2D> residues) {
        canvas.clearAllSelections();
        for (Residue2D r2D:residues)
            if (r2D.getStructuralDomain() != null)
                canvas.addShiftCurrentStructure(r2D.getStructuralDomain());
    }

}
