package fr.unistra.ibmc.s2s.tertiaryviewers;

import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.features.TertiaryStructure;
import fr.unistra.ibmc.paradise.core.features.BaseBaseInteraction;
import fr.unistra.ibmc.paradise.core.features.StructuralDomain;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.s2s.tertiaryviewers.OnHangApplicationDriver;
import fr.unistra.ibmc.s2s.Mediator;
import fr.unistra.ibmc.s2s.SelectionListener;

import java.util.*;

public abstract class AbstractTertiaryViewerDriver extends OnHangApplicationDriver implements TertiaryViewerDriver {

    protected Map<Molecule, Location> molecules;
    protected List<TertiaryStructure> tertiaryStructuresDisplayed;
    protected Mediator mediator;
    private List<SelectionListener> selectionListeners;
    protected int currentState;
    protected List<Residue> previousResidues;


    public AbstractTertiaryViewerDriver(Mediator mediator, String tertiaryViewerCommand) {
        super(tertiaryViewerCommand);
        this.mediator = mediator;
        this.molecules = new HashMap<Molecule, Location>();
        this.tertiaryStructuresDisplayed = new ArrayList<TertiaryStructure>();
        this.previousResidues = new ArrayList<Residue>();
    }

    public void addSelectionListener(SelectionListener l) {
        if (this.selectionListeners == null)
            this.selectionListeners = new ArrayList<SelectionListener>();
        this.selectionListeners.add(l);
    }

    public List<TertiaryStructure> getTertiaryStructuresDisplayed() {
        return new ArrayList<TertiaryStructure>(this.tertiaryStructuresDisplayed);
    }

    public void addTertiaryStructureDisplayed(TertiaryStructure ts) {
        this.tertiaryStructuresDisplayed.add(ts);
    }

    public void setCurrentState(int state) {
        this.currentState = state;
    }

    /**
     * Close the tertiary viewer
     */
    public void close() {
        this.process.destroy();
    }

    protected void addResidue(Residue r) {
        Location location = this.molecules.get(r.getMolecule());
        if (location == null)
            location = new Location();
        this.molecules.put(r.getMolecule(), location.unionOf(new Location(r.getAbsolutePosition())));
    }


    public void residuesSelected(Collection<Residue> residues, boolean isShiftDown) {
        if (!isShiftDown)
            this.previousResidues.clear();
        this.previousResidues.addAll(residues);
        switch (this.currentState) {
            case TertiaryViewerDriver.SELECT:
                this.select(this.previousResidues);
                break;
            case TertiaryViewerDriver.ZOOM:
                this.zoomSelection(this.previousResidues);
                break;
            case TertiaryViewerDriver.HIDE :
                this.hideSelection(this.previousResidues);
                break;
            case TertiaryViewerDriver.CENTER :
                this.centerSelection(this.previousResidues);
                break;
            case TertiaryViewerDriver.SHOW :
                this.showSelection(this.previousResidues);
                break;
        }
    }

    public void structuralDomainSelected(StructuralDomain structuralDomain, boolean isShiftDown) {
        this.residuesSelected(structuralDomain.getResidues().getResidues(), isShiftDown);
    }

    public void interactionSelected(BaseBaseInteraction interaction, boolean isShiftDown) {
        this.residuesSelected(Arrays.asList(new Residue[]{interaction.getResidue(),interaction.getPartnerResidue()}), isShiftDown);
    }
}
