package fr.unistra.ibmc.s2s.tertiaryviewers;

import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.features.TertiaryStructure;
import fr.unistra.ibmc.s2s.S2SConfig;
import fr.unistra.ibmc.s2s.Mediator;

import java.io.File;
import java.util.*;


public class ChimeraDriver extends AbstractTertiaryViewerDriver {

    public ChimeraDriver(Mediator mediator) {
        super(mediator,S2SConfig.getChimeraPath());
        this.run(new String[]{}, null, null);
    }

    public void loadTertiaryStructure(File f) {
        this.evaluate("open "+f.getAbsolutePath());
    }

    public void select(Collection<Residue> residues) {
        //with the first residue of the list, we select the TertiaryStructures
        Iterator it = residues.iterator();
        if (it.hasNext()) {
            Residue r = residues.iterator().next();
            List<TertiaryStructure> selectedTS = new ArrayList<TertiaryStructure>();
            for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                if (ts.annotates(r.getMolecule()))
                    selectedTS.add(ts);
            final StringBuffer command = new StringBuffer();
            for (Residue _r:residues)
                this.addResidue(_r);
            for (Map.Entry<Molecule, Location> e:this.molecules.entrySet()) {
                String moleculeLabel = "?";
                for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                    if (ts.annotates(e.getKey())) {
                        moleculeLabel = ts.getMoleculeLabel(e.getKey());
                        break;
                    }
                int[] ends = e.getValue().getBoundaries();
                if (ends.length >= 2) {
                    for (int i=0;i<ends.length;i+=2) {
                        if (command.length() != 0)
                            command.append(",");
                        command.append(ends[i]+"-"+ends[i+1]);
                        command.append("."+moleculeLabel);
                    }
                    this.evaluate("select :"+command.toString());
                }
            }
            this.molecules.clear();
        }
    }

    public void showSelection(Collection<Residue> residues) {
        //with the first residue of the list, we select the TertiaryStructures
        Iterator it = residues.iterator();
        if (it.hasNext()) {
            Residue r = residues.iterator().next();
            List<TertiaryStructure> selectedTS = new ArrayList<TertiaryStructure>();
            for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                if (ts.annotates(r.getMolecule()))
                    selectedTS.add(ts);
            final StringBuffer command = new StringBuffer();
            for (Residue _r:residues)
                this.addResidue(_r);
            for (Map.Entry<Molecule, Location> e:this.molecules.entrySet()) {
                String moleculeLabel = "?";
                for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                    if (ts.annotates(e.getKey())) {
                        moleculeLabel = ts.getMoleculeLabel(e.getKey());
                        break;
                    }
                int[] ends = e.getValue().getBoundaries();
                if (ends.length >= 2) {
                    for (int i=0;i<ends.length;i+=2) {
                        if (command.length() != 0)
                            command.append(",");
                        command.append(ends[i]+"-"+ends[i+1]);
                        command.append("."+moleculeLabel);
                    }
                    this.evaluate(command.toString());
                    this.evaluate("select :"+command.toString());
                    this.evaluate("display :"+command.toString());
                }
            }
            this.molecules.clear();
        }
    }

    public void hideSelection(Collection<Residue> residues) {
        //with the first residue of the list, we select the TertiaryStructures
        Iterator it = residues.iterator();
        if (it.hasNext()) {
            Residue r = residues.iterator().next();
            List<TertiaryStructure> selectedTS = new ArrayList<TertiaryStructure>();
            for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                if (ts.annotates(r.getMolecule()))
                    selectedTS.add(ts);
            final StringBuffer command = new StringBuffer();
            for (Residue _r:residues)
                this.addResidue(_r);
            for (Map.Entry<Molecule, Location> e:this.molecules.entrySet()) {
                String moleculeLabel = "?";
                for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                    if (ts.annotates(e.getKey())) {
                        moleculeLabel = ts.getMoleculeLabel(e.getKey());
                        break;
                    }
                int[] ends = e.getValue().getBoundaries();
                if (ends.length >= 2) {
                    for (int i=0;i<ends.length;i+=2) {
                        if (command.length() != 0)
                            command.append(",");
                        command.append(ends[i]+"-"+ends[i+1]);
                        command.append("."+moleculeLabel);
                    }
                    this.evaluate(command.toString());
                    this.evaluate("select :"+command.toString());
                    this.evaluate("display :"+command.toString());
                }
            }
            this.molecules.clear();
        }
    }

    public void zoomSelection(Collection<Residue> residues) {
        //with the first residue of the list, we select the TertiaryStructures
        Iterator it = residues.iterator();
        if (it.hasNext()) {
            Residue r = residues.iterator().next();
            List<TertiaryStructure> selectedTS = new ArrayList<TertiaryStructure>();
            for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                if (ts.annotates(r.getMolecule()))
                    selectedTS.add(ts);
            final StringBuffer command = new StringBuffer();
            for (Residue _r:residues)
                this.addResidue(_r);
            for (Map.Entry<Molecule, Location> e:this.molecules.entrySet()) {
                String moleculeLabel = "?";
                for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                    if (ts.annotates(e.getKey())) {
                        moleculeLabel = ts.getMoleculeLabel(e.getKey());
                        break;
                    }
                int[] ends = e.getValue().getBoundaries();
                if (ends.length >= 2) {
                    for (int i=0;i<ends.length;i+=2) {
                        if (command.length() != 0)
                            command.append(",");
                        command.append(ends[i]+"-"+ends[i+1]);
                        command.append("."+moleculeLabel);
                    }
                    this.evaluate(command.toString());
                    this.evaluate("select :"+command.toString());
                    this.evaluate("focus :"+command.toString());
                }
            }
            this.molecules.clear();
        }
    }

    public void centerSelection(Collection<Residue> residues) {
        //with the first residue of the list, we select the TertiaryStructures
        Iterator it = residues.iterator();
        if (it.hasNext()) {
            Residue r = residues.iterator().next();
            List<TertiaryStructure> selectedTS = new ArrayList<TertiaryStructure>();
            for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                if (ts.annotates(r.getMolecule()))
                    selectedTS.add(ts);
            final StringBuffer command = new StringBuffer();
            for (Residue _r:residues)
                this.addResidue(_r);
            for (Map.Entry<Molecule, Location> e:this.molecules.entrySet()) {
                String moleculeLabel = "?";
                for (TertiaryStructure ts:this.tertiaryStructuresDisplayed)
                    if (ts.annotates(e.getKey())) {
                        moleculeLabel = ts.getMoleculeLabel(e.getKey());
                        break;
                    }
                int[] ends = e.getValue().getBoundaries();
                if (ends.length >= 2) {
                    for (int i=0;i<ends.length;i+=2) {
                        if (command.length() != 0)
                            command.append(",");
                        command.append(ends[i]+"-"+ends[i+1]);
                        command.append("."+moleculeLabel);
                    }
                    this.evaluate(command.toString());
                    this.evaluate("select :"+command.toString());
                    this.evaluate("center :"+command.toString());
                }
            }
            this.molecules.clear();
        }
    }

    public void clearDisplay() {
    }
}
