package fr.unistra.ibmc.s2s.tertiaryviewers;

import fr.unistra.ibmc.paradise.core.Location;
import fr.unistra.ibmc.paradise.core.Molecule;
import fr.unistra.ibmc.paradise.core.features.TertiaryStructure;
import fr.unistra.ibmc.paradise.core.utils.Residue;
import fr.unistra.ibmc.s2s.S2SConfig;
import fr.unistra.ibmc.s2s.Mediator;

import java.io.File;
import java.util.*;

public class PyMOLDriver extends AbstractTertiaryViewerDriver {

    public PyMOLDriver(Mediator mediator) {
        super(mediator,S2SConfig.getPyMOLPath());
        this.run(new String[]{"-px"}, null, null);
    }

    public void select(final Collection<Residue> residues) {
        //with the first residue of the list, we select the TertiaryStructures
        Iterator it = residues.iterator();
        if (it.hasNext()) {
            Residue r = residues.iterator().next();
            List<TertiaryStructure> selectedTS = new ArrayList<TertiaryStructure>();
            for (TertiaryStructure ts:this.tertiaryStructuresDisplayed) {
                if (ts.annotates(r.getMolecule()))
                    selectedTS.add(ts);
            }
            final StringBuffer command = new StringBuffer();
            for (Residue _r:residues)
                this.addResidue(_r);
            for (Map.Entry<Molecule, Location> e:this.molecules.entrySet()) {
                String moleculeLabel = "?";
                for (TertiaryStructure ts:this.tertiaryStructuresDisplayed) {
                    if (ts.annotates(e.getKey())) {
                        moleculeLabel = ts.getMoleculeLabel(e.getKey());
                        break;
                    }
                }
                int[] ends = e.getValue().getBoundaries();
                if (ends.length >= 2) {
                    for (int i=0;i<ends.length;i+=2) {
                        if (command.length() != 0)
                            command.append(" or ");
                        command.append("resi ");
                        command.append(ends[i]+"-"+ends[i+1]);
                        command.append(" and chain "+moleculeLabel);
                    }
                    this.evaluate(new StringBuffer("select s2s, ").append(command).toString());
                }
            }
            this.molecules.clear();
        }
    }

    public void showSelection(final Collection<Residue> residues) {
        this.select(residues);
        this.evaluate("show lines, (s2s)");
    }

    public void hideSelection(final Collection<Residue> residues) {
        this.select(residues);
        this.evaluate("hide (s2s)");
    }

    public void zoomSelection(final Collection<Residue> residues) {
        this.select(residues);
        this.evaluate("show lines, (s2s)");
        this.evaluate("zoom (s2s)");
    }

    public void centerSelection(final Collection<Residue> residues) {
        this.select(residues);
        this.evaluate("show lines, (s2s)");
        this.evaluate("center (s2s)");
    }

    public void clearDisplay() {
        this.evaluate("delete all");
    }

    public void loadTertiaryStructure(File f) {
        this.evaluate("load "+f.getAbsolutePath());
    }

}
